/*
 * Copyright (c) 2000 by Prosyst Software AG
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package demo.prosyst.profiler;

/**
 * Profiler demo.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class RunMe extends Thread {
  
  private final int NUMBER_OF_ITERATIONS = 10;
  private final int SLEEP_INTERVAL       = 1000;
  private final int DOUBLE_ARRAY_SIZE    = 1000;
  private final int INT_ARRAY_SIZE       = 50000;
  
  private int[] int_array = new int[INT_ARRAY_SIZE];
  
  /**
   * Constructor of RunMe thread.
   * Sets name and priority.
   */
  public RunMe() {
    super("RunMe-Thread");
    setPriority(Thread.NORM_PRIORITY - 1);
  }
  
  /**
   * Iterates ten times.
   * In each iteration allocates memory, sleeps one second
   * and prints iteration number in the beginning and "OK" at the end.
   */
  public void run() {
    for (int i = 1; i <= NUMBER_OF_ITERATIONS; ++i) {
      System.out.print("** Iteration #" + i + " ... ");
      
      makeAllocations(i);
      makeComputations();
      waitOneSecond();
      
      System.out.println("[ OK ]");
    }
  }
  
  /**
   * Allocates one instance of every primitive type and an array of doubles.
   *
   * @param   i  the number of the iteration used in constructor of each type
   */
  private void makeAllocations(int i) {
    new Boolean(i != 0);
    new Byte((byte) i);
    new Character((char) i);
    new Short((short) i);
    new Integer((int) i);
    new Long((long) i);
    new Float((float) i);
    new Double((double) i);
    
    double[] double_array = new double[DOUBLE_ARRAY_SIZE];
  }
  
  /**
   * Does intensive computations.
   */
  private void makeComputations() {
    for (int i = 0; i < INT_ARRAY_SIZE; i++) {
      int_array[(int) (Math.random() * INT_ARRAY_SIZE)] = (int) (Math.random() * Integer.MAX_VALUE);
    }
  }
  
  /**
   * Sleeps one second between each iteration.
   */
  private void waitOneSecond() {
    try {
      Thread.currentThread().sleep(SLEEP_INTERVAL);
    } catch (Exception _) {
    }
  }
   
  /**
   * Main method of Profiler demo.
   * Prints a greeting (the first command line argument or "World" if there is not),
   * prints all system properties,
   * starts RunMe thread.
   *
   * @param   args  
   */
  public static void main(String[] args) {
    String greeting = (args.length > 0) ? args[0] : "World";
    System.out.println("Hello, " + greeting + " !");
    
    java.util.Properties  p = System.getProperties();
    java.util.Enumeration e = p.keys();
    while (e.hasMoreElements()) {
      String key = (String) e.nextElement();
      System.out.println(key + " = " + p.getProperty(key));
    }
    
    new RunMe().start();
  }
}