/*
 * prof_class.h
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#ifndef _PROF_CLASS_H
#define _PROF_CLASS_H

#include "prof_hash.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

/* Methods */
typedef struct prof_method_t {
	void * next;
  jmethodID method_id;                 /* id from JVM, -1 on class unload */
  struct prof_class_t *class;          /* class to which this method belongs */
  struct prof_name_t *method_name;     /* name of method */
} prof_method_t;

/* Fields */
typedef struct {
  jint type;                           /* type */
  struct prof_name_t *name;            /* name of the field */
} prof_field_t;

/* Classes */
typedef struct prof_class_t {
	void * next;
  struct prof_objmap_t *class_id;      /* ptr to jvm<--->prof object map, -1 on unload */
  struct prof_class_t *super;          /* super class */
  struct prof_objmap_t *loader;        /* class loader */
  unsigned int serial_num;             /* unique serial number */
  struct prof_name_t *name;            /* class name */
  struct prof_name_t *src_name;        /* id of src file name */
  unsigned int num_interfaces;         /* number of interfaces */
  unsigned int num_statics;            /* number of static fields */
  prof_field_t *statics;               /* statics */
  unsigned int num_instances;          /* number of instances */
  prof_field_t *instances;             /* instances */
  unsigned int num_methods;            /* number of methods */
  prof_method_t **methods;             /* methods */
} prof_class_t;

extern prof_hash_t prof_class_table;
extern prof_class_t *java_lang_object_class;

void prof_class_table_init(void);
prof_class_t * prof_lookup_class_objmap(struct prof_objmap_t *objmap);
prof_class_t * prof_lookup_class(jobjectID class_id);
void prof_class_load_event(JNIEnv *env_id,
         char *name, char *src_name, unsigned int num_interfaces,
         unsigned int num_statics,   JVMPI_Field *statics,
         unsigned int num_instances, JVMPI_Field *instances,
         unsigned int num_methods,   JVMPI_Method *methods,
         jobjectID class_id, int requested);
void prof_class_unload_event(JNIEnv *env_id, jobjectID class_id);
void prof_superclass_link(jobjectID class_id, jobjectID super_id, jobjectID loader_id);

#endif /* _PROF_CLASS_H */
