/*
 * prof_gc.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#include "prof.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

static jint gc_last_active_time;
jint gc_active_time;

void prof_gc_start_event(JNIEnv *env_id)
{
  /* grab the data_access_lock here, because obj_move and obj_free come
   * in single threaded mode and we could run into deadlocks by grabbing
   * the lock in those handlers. */
  CALL(RawMonitorEnter)(data_access_lock);

  gc_last_active_time = prof_get_milliticks();
}

void prof_gc_finish_event(JNIEnv *env_id, jlong used_objects,
         jlong used_object_space, jlong total_object_space)
{
  gc_last_active_time = prof_get_milliticks() - gc_last_active_time;
  gc_active_time += gc_last_active_time;

  CALL(RawMonitorExit)(data_access_lock); /* we grabbed this in gc_start */
}
