/*
 * prof_global.h
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#ifndef _PROF_GLOBALS_H
#define _PROF_GLOBALS_H

#include "jvmpi.h"
#include <stdlib.h>

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

#define TRUE  1
#define FALSE 0

extern JavaVM *jvm;

extern int debug;
extern int prof_is_on;                 /* whether prof is enabled */
extern int prof_fd;                    /* non-zero file or socket descriptor */

extern int max_trace_depth;
extern int prof_trace_depth;
extern int thread_in_traces;
extern int lineno_in_traces;
extern float prof_cutoff_point;
extern int dump_on_exit;
extern int alloc_sites;
extern int cpu_sampling;
extern int heap_dump;
extern int send_data_on_exit;

/* triggers */
#define TRIGGER_METHOD_ENTRY 0
#define TRIGGER_METHOD_EXIT  1
extern int triggers;

extern jlong total_alloced_bytes;
extern jlong total_alloced_instances;
extern long total_live_bytes;
extern long total_live_instances;

/* Data access Lock */
extern JVMPI_RawMonitor data_access_lock;
/* Dump lock */
extern JVMPI_RawMonitor prof_dump_lock;

/* profiler interface */
extern JVMPI_Interface *prof_jvm_interface;
#define CALL(f) (prof_jvm_interface->f)

/* version */
#define PROF_HEADER "JAVA PROFILER AGENT 1.2"

void *prof_calloc(unsigned int size);

#ifdef _MEM_USAGE_
	extern unsigned int mem_usage_total;
	extern unsigned int mem_usage;
	extern unsigned int mem_usage_peak;

  void prof_free(void *memblock);
#else
  #define prof_free free
#endif

extern unsigned int micro_sec_ticks;

#endif /* _PROF_GLOBALS_H */
