/*
 * prof_hash.h
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#ifndef _PROF_HASH_H
#define _PROF_HASH_H

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

/* Generic support for hash tables */

typedef struct prof_bucket_t {
  struct prof_bucket_t *next;
  void *content;
} prof_bucket_t;

typedef struct {
  unsigned int n_entries;              /* total buckets in the table */
  unsigned int size;                   /* size of the top-level table */
  prof_bucket_t **entries;
  unsigned int (*hash_f)(void *);      /* hash function */
  unsigned int (*size_f)(void *);      /* content size */
  int (*compare_f)(void *, void *);    /* comparison function */
#ifdef _HASH_STAT_
  unsigned int num_calls_hash_f;
  unsigned int num_calls_compare_f;
#endif
} prof_hash_t;

void prof_hash_init(prof_hash_t *table, unsigned int size,
         unsigned int (*hash_f)(void *),
         unsigned int (*size_f)(void *),
         int (*compare_f)(void *, void *));
void * prof_hash_iterate(prof_hash_t *table, void * (*f)(void *, void *), void *arg);
void prof_hash_remove(prof_hash_t *table, int (*f)(void *, void *), void *arg);
void prof_hash_removeall(prof_hash_t *table);
void * prof_hash_lookup(prof_hash_t *table, void *new);
void * prof_hash_put(prof_hash_t *table, void *new);
void * prof_hash_intern(prof_hash_t *table, void *new);
void prof_hash_rehash(prof_hash_t *table);
void prof_print_tbl_hash_stats(prof_hash_t *table, char *tbl_name);

#endif /* _PROF_HASH_H */
