/*
 * prof_io.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#include <string.h>
#include "prof.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

/* Output buffer */
#define WRITE_BUF_SIZE 2048

static int buf_index = 0;
static char *write_buf;

static char *cur_ptr_in_dump;

void prof_dump_seek(char *ptr)
{
  cur_ptr_in_dump = ptr;
}

char * prof_dump_cur(void)
{
  return cur_ptr_in_dump;
}

void prof_dump_read(void *buf, int size)
{
  memcpy(buf, cur_ptr_in_dump, size);
  cur_ptr_in_dump += size;
}

void * prof_dump_read_id(void)
{
  void *p;
  prof_dump_read(&p, sizeof(void *));
  return p;
}

unsigned int prof_dump_read_u4(void)
{
  jint u4;
  prof_dump_read(&u4, sizeof(jint));
  return ntohl(u4);
}

unsigned short prof_dump_read_u2(void)
{
  unsigned short u2;
  prof_dump_read(&u2, sizeof(unsigned short));
  return ntohs(u2);
}

unsigned char prof_dump_read_u1(void)
{
  unsigned char u1;
  prof_dump_read(&u1, sizeof(unsigned char));
  return u1;
}

void prof_io_setup(void)
{
  write_buf = prof_calloc(WRITE_BUF_SIZE);
}

void prof_write_header(unsigned char type, jint length)
{
  prof_write_u1(type);
  prof_write_current_ticks();
  prof_write_u4(length);
}

void prof_write_dev(void *buf, int len)
{
  int res;
  res = prof_send(prof_fd, buf, len, 0);
  if (res < 0) {
    if (prof_is_on) {
      fprintf(stderr, "PROF ERROR: failed to write results, exiting .....\n");
      prof_is_on = FALSE;
    }
    CALL(ProfilerExit)((jint)1);
  }
}

void prof_write_raw(void *buf, int len)
{
  if (buf_index + len > WRITE_BUF_SIZE) {
    prof_flush();
    if (len > WRITE_BUF_SIZE) {
      prof_write_dev(buf, len);
      return;
    }
  }
  memcpy(write_buf + buf_index, buf, len);
  buf_index += len;
}

void prof_flush(void)
{
  if (buf_index) {
    prof_write_dev(write_buf, buf_index);
    buf_index = 0;
  }
}

void prof_write_current_ticks(void)
{
  prof_write_u4(prof_get_milliticks() * 1000 - micro_sec_ticks);
}

void prof_write_u4(unsigned int i)
{
  i = htonl(i);
  prof_write_raw(&i, sizeof(unsigned int));
}

void prof_write_u2(unsigned short i)
{
  i = htons(i);
  prof_write_raw(&i, sizeof(unsigned short));
}

void prof_write_u1(unsigned char i)
{
  prof_write_raw(&i, sizeof(unsigned char));
}

void prof_write_id(void *p)
{
  prof_write_raw(&p, sizeof(void *));
}
