/*
 * prof_jni.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#include "prof.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

static prof_globalref_t *prof_globalrefs = NULL;

static void prof_globalref_add(prof_objmap_t *obj_id, jobject ref_id, unsigned int trace_serial_num)
{
  prof_globalref_t *globalref = prof_calloc(sizeof(prof_globalref_t));
  globalref->obj_id = obj_id;
  globalref->ref_id = ref_id;
  globalref->trace_serial_num = trace_serial_num;
  globalref->next = prof_globalrefs;
  prof_globalrefs = globalref;
}

static void prof_globalref_del(jobject ref_id)
{
  prof_globalref_t **p = &prof_globalrefs;
  prof_globalref_t *globalref;
  while ((globalref = *p)) {
    if (globalref->ref_id == ref_id) {
      *p = globalref->next;
      prof_free(globalref);
      return;
    }
    p = &(globalref->next);
  }
}

prof_globalref_t * prof_globalref_find(jobject ref_id)
{
  prof_globalref_t *globalref = prof_globalrefs;
  while (globalref) {
    if (globalref->ref_id == ref_id) {
      return globalref;
    }
    globalref = globalref->next;
  }
  return NULL;
}

void prof_jni_globalref_alloc_event(JNIEnv *env_id, jobjectID obj_id, jobject ref_id)
{
  prof_trace_t *htrace;
  prof_objmap_t *objmap;

  CALL(RawMonitorEnter)(data_access_lock);
  htrace = prof_get_trace(env_id, max_trace_depth);
  if (htrace == NULL) {
    if (debug) fprintf(stderr, "PROF ERROR: got NULL trace in jni_globalref_alloc\n");
    goto globalref_alloc_done;
  }

  objmap = prof_fetch_object_info(obj_id);
  if (objmap == NULL) {
    if (debug) fprintf(stderr, "PROF ERROR: unable to map JVMPI obj ID to prof "
                       "obj ID in globalref_alloc \n");
    goto globalref_alloc_done;
  }

  prof_globalref_add(objmap, ref_id, htrace->serial_num);

globalref_alloc_done:
  CALL(RawMonitorExit)(data_access_lock);
}

void prof_jni_globalref_free_event(JNIEnv *env_id, jobject ref_id)
{
  CALL(RawMonitorEnter)(data_access_lock);
  prof_globalref_del(ref_id);
  CALL(RawMonitorExit)(data_access_lock);
}
