/*
 * prof_method.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#include <string.h>
#include "prof.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

#define PROF_METHOD_TABLE_SIZE 20011

prof_hash_t prof_method_table;

static unsigned int hash_method(void *_method)
{
  prof_method_t *method = _method;
  return (unsigned int)method->method_id;
}

static unsigned int size_method(void *_method)
{
  return sizeof(prof_method_t);
}

static int compare_method(void *_method1, void *_method2)
{
  prof_method_t *method1 = _method1;
  prof_method_t *method2 = _method2;

  return (int)((unsigned int)method1->method_id - (unsigned int)method2->method_id);
}

void prof_method_table_init(void)
{
  prof_hash_init(&prof_method_table, PROF_METHOD_TABLE_SIZE, hash_method, size_method, compare_method);
}

prof_method_t * prof_lookup_method(jmethodID method_id)
{
  prof_method_t method_tmp;
  method_tmp.method_id = method_id;
  return prof_hash_lookup(&prof_method_table, &method_tmp);
}

prof_method_t * prof_intern_method(JVMPI_Method *jmethod, prof_class_t *hclass)
{
  prof_method_t method_tmp;
  prof_method_t *method;

  method_tmp.method_id = jmethod->method_id;
  method_tmp.method_name = prof_intern_name(jmethod->method_name);
  method_tmp.class = hclass;
  method = prof_hash_intern(&prof_method_table, &method_tmp);
  if (method == NULL) {
    if (debug) fprintf(stderr, "PROF ERROR: failed to intern method\n");
  }
  return method;
}

void prof_method_entry_event(JNIEnv *env_id, jmethodID method_id)
{
  if (triggers) {
    prof_method_t *method = prof_lookup_method(method_id);
    prof_trigger_t *trigger = (prof_trigger_t *) prof_lookup_trigger_entry(method);
    if (trigger != NULL) {
      CALL(RawMonitorEnter)(data_access_lock);
      prof_output_sites(3, 0);
      prof_write_header(PROF_TRIGGER_EVENT, 9);
      prof_write_u1(TRIGGER_METHOD_ENTRY);
      prof_write_id(prof_lookup_name(method->class->name->name));
      prof_write_id(prof_lookup_name(method->method_name->name));
      CALL(RawMonitorExit)(data_access_lock);
    }
  }
}

void prof_method_exit_event(JNIEnv *env_id, jmethodID method_id)
{
  if (triggers) {
    prof_method_t *method = prof_lookup_method(method_id);
    prof_trigger_t *trigger = (prof_trigger_t *) prof_lookup_trigger_exit(method);
    if (trigger != NULL) {
      CALL(RawMonitorEnter)(data_access_lock);
      prof_output_sites(3, 0);
      prof_write_header(PROF_TRIGGER_EVENT, 9);
      prof_write_u1(TRIGGER_METHOD_EXIT);
      prof_write_id(prof_lookup_name(method->class->name->name));
      prof_write_id(prof_lookup_name(method->method_name->name));
      CALL(RawMonitorExit)(data_access_lock);
    }
  }
}
