/*
 * prof_name.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#include <string.h>
#include "prof.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

#define PROF_NAME_TABLE_SIZE 20011

prof_hash_t prof_name_table;

static unsigned int hash_name(void *_hname)
{
  prof_name_t *hname = _hname;
  unsigned int hash = 0;
  int len = strlen(hname->name);
  int i;

  for (i = 0; i < len; i++) {
    hash = 37 * hash + (unsigned int)hname->name[i];
  }
  return hash;
}

static unsigned int size_name(void *_name)
{
  return sizeof(prof_name_t);
}

static int compare_name(void *_hname1, void *_hname2)
{
  prof_name_t *hname1 = _hname1;
  prof_name_t *hname2 = _hname2;

  return (strcmp(hname1->name, hname2->name));
}

void prof_name_table_init(void)
{
  prof_hash_init(&prof_name_table, PROF_NAME_TABLE_SIZE, hash_name, size_name, compare_name);
}

void prof_output_name(prof_name_t *name)
{
  if (name->marked) {
    return;
  } else {
    name->marked = 1;
    if (name->name) {
      prof_write_header(PROF_UTF8, strlen(name->name) + sizeof(void *));
      prof_write_id(name);
      prof_write_raw(name->name, strlen(name->name));
    }
  }
}

prof_name_t * prof_intern_name(char *name)
{
  prof_name_t name_tmp;
  prof_name_t *result;

  if (name == NULL) {
    name = "<Unknown>";
  }
  name_tmp.name = name;
  result = prof_hash_lookup(&prof_name_table, &name_tmp);
  if (result == NULL) {
#ifdef _MEM_USAGE_
		unsigned int nsize = strlen(name);
		mem_usage_total += nsize;
		mem_usage += nsize;
#endif
#ifdef _VXWORKS_
    name_tmp.name = malloc(strlen(name) + 1);
    if (name_tmp.name != NULL) strcpy(name_tmp.name, name);
#else
#ifdef _WINCE_
    name_tmp.name = _strdup(name);
#else
    name_tmp.name = strdup(name);
#endif
#endif
    name_tmp.marked = 0;
    result = prof_hash_put(&prof_name_table, &name_tmp);
  }
  prof_output_name(result);
  return result;
}

prof_name_t * prof_lookup_name(char *name)
{
  prof_name_t name_tmp;

  name_tmp.name = name;
  return prof_hash_lookup(&prof_name_table, &name_tmp);
}
