/*
 * prof_thread.h
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#ifndef _PROF_THREAD_H
#define _PROF_THREAD_H

#include "prof_hash.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

/* Threads */
typedef struct prof_thread_t {
	void * next;
  JNIEnv *env_id;                      /* env of thread */
  struct prof_objmap_t *thread_id;     /* ptr to jvm<--->prof object map */
  unsigned int serial_num;             /* unique serial number */
} prof_thread_t;

extern prof_hash_t prof_thread_table;

/* recording an active thread */
typedef struct live_thread_t {
  struct live_thread_t *next;          /* next thread */
  struct live_thread_t *nextSuspended; /* next suspended thread */
  struct prof_objmap_t *tid;           /* thread object ID */
  JNIEnv *env;                         /* JNIEnv */
  int cpu_sampled:1;                   /* thread CPU profile on? */
} live_thread_t;

/* list of active threads */
extern live_thread_t *live_thread_list;
extern int num_live_threads;

/* recording a nonsystem thread */
typedef struct nonsys_thread_t {
  struct nonsys_thread_t *next;        /* next thread */
  JNIEnv *env;                         /* JNIEnv */
} nonsys_thread_t;

/* list of nonsystem threads */
extern nonsys_thread_t *nonsys_thread_list;
extern int num_nonsys_threads;

void prof_thread_table_init(void);
prof_thread_t * prof_intern_thread(JNIEnv *env_id);
void prof_remove_thread(JNIEnv *env_id);
prof_thread_t * prof_lookup_thread(JNIEnv *env_id);
void prof_thread_start_event(JNIEnv *env_id,
         char *t_name, char *g_name, char *p_name,
         jobjectID thread_id, int requested);
void prof_thread_end_event(JNIEnv *env_id);
prof_thread_t * prof_fetch_thread_info(JNIEnv *env);

#endif /* _PROF_THREAD_H */
