/*
 * prof_trace.h
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Vladislav Doshev,
 *            Alexi     Jordanov   (alexi91@yahoo.com),
 *            Ivo       Karabashev
 */

#ifndef _PROF_TRACE_H
#define _PROF_TRACE_H

#include "prof_hash.h"

/**
 * @author  Vladislav Doshev
 * @author  Alexi     Jordanov (alexi91@yahoo.com)
 * @author  Ivo       Karabashev
 * @version 1.00, 02/11/2000
 */

/* Frames */
typedef struct prof_frame_t {
	void * next;
  unsigned char marked;                /* been written to the output? */
  signed int lineno;                   /* line number */
  struct prof_method_t *method;        /* method being executed */
} prof_frame_t;

/* Stack traces */
typedef struct prof_trace_t {
	void * next;
  unsigned char marked;                /* been written to the output? */
  unsigned int serial_num;             /* serial number, starts from 1 */
  unsigned int num_hits;               /* number of hits */
  unsigned int thread_serial_num;      /* thread serial number */
  unsigned short n_frames;             /* number of frames */
  struct prof_frame_t *frames[1];      /* frames */
} prof_trace_t;

/* Auxilary data structure used for collecting trace info. */
typedef struct prof_trace_iterate_t {
  prof_trace_t **traces;
  int index;
  int total_count;
} prof_trace_iterate_t;

extern prof_hash_t prof_trace_table;
extern prof_hash_t prof_frame_table;

void prof_trace_table_init(void);
prof_trace_t * prof_alloc_tmp_trace(unsigned int n_frames, JNIEnv *env_id);
prof_trace_t * prof_intern_tmp_trace(prof_trace_t *trace_tmp);
prof_trace_t * prof_intern_jvmpi_trace(JVMPI_CallFrame *frames, unsigned int n_frames, JNIEnv *env_id);
prof_trace_t * prof_get_trace(JNIEnv *env_id, unsigned int depth);
void prof_frame_table_init(void);
prof_frame_t * prof_intern_jvmpi_frame(jmethodID method_id, int lineno);
void prof_output_unmarked_traces(void);
void prof_output_unmarked_traces_limited(void);
void prof_output_trace_cost(float cutoff);
void prof_clear_trace_cost(void);

#endif /* _PROF_TRACE_H */
