/*
 * prof_trigger.c
 *
 * Copyright (c) ProSyst Software
 * All rights reserved
 *
 * Written by Ivo Karabashev
 */

#include "prof.h"

/**
 * @author  Ivo Karabashev
 * @version 1.00, 02/22/2002
 */

#define PROF_TRG_ENTRY_TABLE_SIZE 101
#define PROF_TRG_EXIT_TABLE_SIZE 101

static prof_hash_t prof_trigger_entry_table;
static prof_hash_t prof_trigger_exit_table;

static unsigned int hash_trigger(void *_trigger)
{
  prof_trigger_t *trigger = _trigger;
  unsigned int hash = 0;
  int len = strlen(trigger->class_name->name);
  int i;

  for (i = 0; i < len; i++) {
    hash = 37 * hash + trigger->class_name->name[i];
  }

  return hash;
}

static unsigned int size_trigger(void *_trigger)
{
  return sizeof(prof_trigger_t);
}

static int compare_trigger(void *_trigger1, void *_trigger2)
{
  prof_trigger_t *trigger1 = _trigger1;
  prof_trigger_t *trigger2 = _trigger2;

  int result = trigger1->class_name - trigger2->class_name;
  if (result) return result;
  if (trigger2->method_name == NULL) return 0;
  return trigger1->method_name - trigger2->method_name;
}

void prof_trigger_table_init(void)
{
  prof_hash_init(&prof_trigger_entry_table, PROF_TRG_ENTRY_TABLE_SIZE,
    hash_trigger, size_trigger, compare_trigger);
  prof_hash_init(&prof_trigger_exit_table, PROF_TRG_EXIT_TABLE_SIZE,
    hash_trigger, size_trigger, compare_trigger);
}

static void prof_put_trigger(prof_hash_t *table, unsigned char action, char *cname, char *mname)
{
  prof_trigger_t trigger;

  trigger.action      = action;
  trigger.class_name  = prof_intern_name(cname);
  trigger.method_name = (mname == NULL ? NULL : prof_intern_name(mname));
  if (prof_hash_lookup(table, &trigger) == NULL) {
    prof_hash_put(table, &trigger);
  }
}

void prof_put_trigger_entry(unsigned char action, char *cname, char *mname)
{
  prof_put_trigger(&prof_trigger_entry_table, action, cname, mname);
}

void prof_put_trigger_exit(unsigned char action, char *cname, char *mname)
{
  prof_put_trigger(&prof_trigger_exit_table, action, cname, mname);
}

prof_trigger_t * prof_lookup_trigger(prof_hash_t *table, prof_method_t *method)
{
  prof_trigger_t trigger;

  if (method != NULL) {
    trigger.class_name  = method->class->name;
    trigger.method_name = method->method_name;
    return prof_hash_lookup(table, &trigger);
  }
  return NULL;
}

prof_trigger_t * prof_lookup_trigger_entry(prof_method_t *method)
{
  return prof_lookup_trigger(&prof_trigger_entry_table, method);
}

prof_trigger_t * prof_lookup_trigger_exit(prof_method_t *method)
{
  return prof_lookup_trigger(&prof_trigger_exit_table,  method);
}
