/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler;

import java.util.Vector;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.PrintWriter;

public class ProfilerIniConverter {
  
  private static final String INI_FILE = "profiler.ini";
  
  private static Vector sections      = new Vector();
  private static Vector section_names = new Vector();
  
  static {
    Vector v;
    
    v = new Vector();
    sections.addElement(v);
    section_names.addElement("[CLASS]");
    v.addElement(new Pair("class"));
    v.addElement(new Pair("workdir"));
    v.addElement(new Pair("javaparam"));
    v.addElement(new Pair("progparam"));
    v.addElement(new Pair("classpath"));
    v.addElement(new Pair("sourcepath"));
  
    v = new Vector();
    sections.addElement(v);
    section_names.addElement("[OPTIONS]");
    v.addElement(new Pair("mem"));
    v.addElement(new Pair("cpu"));
    v.addElement(new Pair("autocpu"));
    v.addElement(new Pair("profcmd"));
    v.addElement(new Pair("port"));
  
    v = new Vector();
    sections.addElement(v);
    section_names.addElement("[TOOLBAR]");
    v.addElement(new Pair("toolbar.orientation"));
    v.addElement(new Pair("toolbar.line"));
    v.addElement(new Pair("toolbar.position"));
    v.addElement(new Pair("toolbar.winx"));
    v.addElement(new Pair("toolbar.winy"));
    v.addElement(new Pair("toolbar.visible"));
    v.addElement(new Pair("toolbar.rolledup"));
  }
  
  public static void convert(String path) {
    String ini_name = INI_FILE;
    if (path != null) {
      ini_name = path + "/" + INI_FILE;
    }
    if (!new File(ini_name).exists()) {
      System.out.println("Usage: program [directory_of_ini_file]");
      return;
    }
    try {
      String line, key, value;
      Vector v;
      Pair   p;
      BufferedReader br = new BufferedReader(new FileReader(ini_name));
      nextline:
      while ((line = br.readLine()) != null) {
        int indx = line.indexOf('=');
        if (indx != -1) {
          key   = line.substring(0, indx).trim();
          value = line.substring(indx + 1).trim();
          StringBuffer sb = new StringBuffer(value);
          int l = 0;
          for (int i = 0; i < sb.length(); i++) {
            if (sb.charAt(i) == '\\') i++;
            if (i >= sb.length()) System.out.println("Error in value: " + value);
            sb.setCharAt(l, sb.charAt(i));
            l++;
          }
          sb.setLength(l);
          value = sb.toString();
          for (int i = 0; i < sections.size(); i++) {
            v = (Vector) sections.elementAt(i);
            for (int j = 0; j < v.size(); j++) {
              p = (Pair) v.elementAt(j);
              if (key.equals(p.key)) {
                p.value = value;
                continue nextline;
              }
            }
          }
        }
        System.out.println("Ignored: " + line);
      }
      br.close();
      PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(ini_name)));
      for (int i = 0; i < sections.size(); i++) {
        pw.println(section_names.elementAt(i));
        v = (Vector) sections.elementAt(i);
        for (int j = 0; j < v.size(); j++) {
          p = (Pair) v.elementAt(j);
          if (p.value != null) pw.println(p.key + " = " + p.value);
        }
        pw.println();
      }
      pw.close();
    } catch (java.io.IOException ex) {
      System.err.println(ex);
      return;
    }
    return;
  }
  
  public static void main(String args[]) {
    String path = null;
    if (args.length > 0) {
      path = args[0];
    }
    convert(path);
  }
}

class Pair {
  protected String key;
  protected String value;
  
  protected Pair(String k) {
    key   = k;
    value = null;
  }
}