/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler;

import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.CpuSampleInfo;
import com.prosyst.jpd.profiler.info.GraphInfo;
import com.prosyst.jpd.profiler.info.MonitorsDump;
import com.prosyst.jpd.profiler.storage.ProfilerNamespace;

/**
 * Listener interface for specific profiler events.
 *
 * @author Ivo Karabashev
 */
public interface ProfilerListener {
  
  /** Event meaning that profiler has started. */
  public void profilerStarted();
  
  /** Event with profiler support capability. */
  public void profilerSupport(boolean isSupported, boolean heapSupport);
  
  /** Event meaning that profiler has stopped. */
  public void profilerStopped();
  
  /** Event meaning that profiler error has occurred. */
  public void profilerError();
  
  /** Event meaning that profiler error has occurred. */
  public void profilerErrorMsg(Throwable ex);
  
  /**
   * Event meaning that alloc sites info has arrived.
   *
   * @param   allocSites  array of info containing allocated bytes and insts
   */
  public void eventAllocSites(AllocSiteInfo[] allocSites);
  
  /**
   * Event meaning that cpu samples info has arrived.
   *
   * @param   ticks                 the time in ms of the event
   * @param   totalNumberOfSamples  total number of samples after the preceding cpu samples event
   * @param   cpuSamples            array of info containing cost and trace
   */
  public void eventCpuSamples(int ticks, int totalNumberOfSamples, CpuSampleInfo[] cpuSamples);
  
  /**
   * Event meaning that heap dump info has arrived.
   *
   * @param   namespace            namespace with all heap classes and objects
   */
  public void eventHeapDump(ProfilerNamespace namespace);
  
  /**
   * Event meaning that graph info has arrived.
   *
   * @param   graphInfo  info containing live and alloc bytes and instances, thread, class and gc info
   */
  public void eventGraphInfo(GraphInfo graphInfo);
  
  /**
   * Event meaning that monitors dump has arrived.
   *
   * @param   monitorsDump  info containing stack traces of live threads and acquired monitors
   */
  public void eventMonitorsDump(MonitorsDump monitorsDump);
  
  /**
   * Common event notification.
   *
   * @param   event  object containing event information
   */
  public void eventCommon(Object event);
}