/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler;

import com.prosyst.jpd.profiler.gui.help.HelpListener;

public interface ProfilerMediatorInterface {
  
  public HelpListener getHelpListener();
  
  /** Event sent to plugin that the profiler has started. */
  public void profilerStarted();
  
  /** Event sent to plugin that the profiler is listening. */
  public void profilerListening();
  
  /** Event with profiler support capability. */
  public void profilerSupport(boolean isSupported, boolean heapSupport);
  
  /** Event sent to plugin that the profiler is stopping. */
  public void profilerStopping();
  
  /** Event sent to plugin that the profiler has stopped. */
  public void profilerStopped();
  
  /** Event sent to plugin that the profiler has not started. */
  public void profilerNotStarted();
  
  /** Event sent to plugin that the profiler has started remote monitoring. */
  public void profilerRemote();
  
  /** Event sent to plugin that the profiler has commited error. */
  public void profilerError(String errorTitle, String errorMsg, int errorCode);
  
  /** Event sent to plugin that the button state should be changed. */
  public void profilerStateEvent(int action_code, boolean state);
  
  /**
   * Gets the main class name from the plugin.
   *
   * @return     the main class name
   */
  public String getMainClass();
  
  /**
   * Gets the working directory from the plugin.
   *
   * @return     the working directory
   */
  public String getWorkDir();
  
  /**
   * Gets the class path from the plugin.
   *
   * @return     the class path
   */
  public String getClassPath();
  
  /**
   * Gets the source path from the plugin.
   *
   * @return     the source path
   */
  public String getSourcePath();
  
  public String getClassPathWithServerJar();
  
  /**
   * Gets the jdk path from the plugin.
   *
   * @return     the jdk path
   */
  public String getJDKPath();
  
  /**
   * Gets the java parameters and system properties from the plugin.
   *
   * @return     the java parameters
   */
  public String getJavaParams();
  
  /**
   * Gets the program command line arguments from the plugin.
   *
   * @return     the program arguments
   */
  public String getProgParams();
  
  /**
   * Shows source in IDE and positions on specific line.
   *
   * @param   file  the source file name
   * @param   line  the line number, to which should be positioned
   */
  public void showSource(java.io.File file, int line);
  
  /** Event sent to plugin that the profiler is loading. */
  public void stateLoading(boolean state);
  
  /** Event sent to plugin that the profiler is saving. */
  public void stateSaving(boolean state);
}