/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;

/**
 * ScriptParser parses mBS scripts and generates profiler mBS script.
 */
public class ScriptParser {
  
  /**
   * Parses script file and generates profiler script if it is allowed.
   *
   * @param      script         path to script
   * @param      allowOverride  true when overriding is permitted, false otherwise
   * @return     path to profiler script
   * @exception  Exception      is thrown when generating of profiler script has
   *                            failed due to some reason.
   */
  public static String parse(String script, boolean allowOverride, String mainClass, String profCmd) throws Exception {
    
    File scriptFile = new File(script);
    
    if (!scriptFile.exists()) new FileNotFoundException("No such script: " + script);
    
    String result;
    
    int idx = script.lastIndexOf(File.separator);
    idx = Math.max(idx, script.lastIndexOf('/'));
    idx = Math.max(idx, script.lastIndexOf('\\'));
    if (idx < 0) {
      result = "prof" + script;
    } else {
      result = script.substring(0, idx + 1) + "prof" + script.substring(idx + 1);
    }
    
    BufferedReader reader = null;
    PrintWriter writer = null;
    
    File resultFile = new File(result);
    
    if (allowOverride || !resultFile.exists()) {
      // processes and generates profiler script
      reader = new BufferedReader(new FileReader(script));
      writer = new PrintWriter(new BufferedWriter(new FileWriter(resultFile)));
      String line = null;
      while((line = reader.readLine()) != null) {
        writer.println(processLine(line, mainClass, profCmd));
      }
      writer.close();
      reader.close();
      
      if (!Parser.OS_NAME.startsWith("Windows")) {
        Runtime.getRuntime().exec("chmod a+x " + result);
      }
    }
    return result;
  }
  
  private static String processLine(String line, String mainClass, String profCmd) {
    if (line == null) return "";
    
    if (line.indexOf(mainClass) >= 0) {
      int idx = 0;
      
      while (Character.isWhitespace(line.charAt(idx))) {
        idx++;
      }
      
      while (line.charAt(idx) == '\"') {
        idx = line.indexOf('\"', idx + 1);
        idx++;
        if (line.length() == idx) return line;
      }
      
      idx = line.indexOf(' ', idx);
      
      if (idx >= 0) {
        if (line.length() == idx + 1) return line;
        line = line.substring(0, idx) + ' '
             + Parser.parseCommandWithClassic(profCmd, Parser.checkJavaVersion(line.substring(0, idx))) + ' '
             + line.substring(idx + 1);
      }
    }
    return line;
  }
}