/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.GraphInfo;
import com.prosyst.jpd.profiler.info.Misc;

/**
 * <PRE>
 * TAG           BODY      notes
 * -------------------------------------------------------------------------
 * PROF_ALLOC_SITES        a set of heap allocation sites, obtained after GC
 *
 *               u2         flags 0x0001: incremental vs. complete
 *                                0x0002: sorted by allocation vs. live
 *                                0x0004: whether to force a GC
 *               u4         cutoff ratio
 *               u4         total live bytes
 *               u4         total live instances
 *               u8         total bytes allocated
 *               u8         total instances allocated
 *               u4         number of sites that follow
 *              [u1         is_array: 0:  normal object
 *                                    2:  object array
 *                                    4:  boolean array
 *                                    5:  char array
 *                                    6:  float array
 *                                    7:  double array
 *                                    8:  byte array
 *                                    9:  short array
 *                                    10: int array
 *                                    11: long array
 *               u4         class serial number (may be zero during startup)
 *               u4         stack trace serial number
 *               u4         number of bytes alive
 *               u4         number of instances alive
 *               u4         number of bytes allocated
 *               u4]*       number of instance allocated
 */
public class BinBody_AllocSites extends BinBody {
  
  public BinRecordReader reader;
  public AllocSiteInfo   allocSite;
  public GraphInfo       graphInfo;
  
  public short flags;
  public int   cutoffRatio;
  public int   numberOfSitesThatFollow;
  
  public BinBody_AllocSites(BinRecord parent) {
    super(parent);
    reader = new BinRecordReader(in);
  }
  
  public void read() throws java.io.IOException {
    reader.read(parent.length);
    
    flags       = (short) reader.read_u2();
    cutoffRatio = reader.read_u4();
    
    graphInfo = new GraphInfo();
    
    graphInfo.totalLiveBytes          = reader.read_u4();
    graphInfo.totalLiveInstances      = reader.read_u4();
    graphInfo.totalBytesAllocated     = reader.read_u8();
    graphInfo.totalInstancesAllocated = reader.read_u8();
    
    numberOfSitesThatFollow = reader.read_u4();
    
    AllocSiteInfo[] allocSites = new AllocSiteInfo[numberOfSitesThatFollow];
    
    AllocSiteInfo site;
    for (int i = 0; i < numberOfSitesThatFollow; i++) {
      site = new AllocSiteInfo();
      
      byte type               = reader.read_u1();
      
      if (type == 0) {
        site.className        = namespace.lookupClass(reader.read_u4());
        if (site.className == null) site.className = Misc.UNKNOWN_CLASS;
      } else {
                                reader.skip(4);
        site.className        = Misc.toStringAllocSiteType(type);
      }
      
      site.stackTraceSerialNo = reader.read_u4();
      site.traceInfo          = namespace.lookupTrace(site.stackTraceSerialNo);
      
      site.bytesLive          = reader.read_u4();
      site.instsLive          = reader.read_u4();
      site.bytesAlloc         = reader.read_u4();
      site.instsAlloc         = reader.read_u4();
      
      allocSites[i] = site;
    }
    
    reader.clear();
    in.eventAllocSites(allocSites);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_ALLOC_SITES");
    out.println("  flags                       = " + flags);
    out.println("  cutoff ratio                = " + cutoffRatio);
    out.println("  total live bytes            = " + graphInfo.totalLiveBytes);
    out.println("  total live instances        = " + graphInfo.totalLiveInstances);
    out.println("  total bytes allocated       = " + graphInfo.totalBytesAllocated);
    out.println("  total instances allocated   = " + graphInfo.totalInstancesAllocated);
    out.println("  number of sites that follow = " + numberOfSitesThatFollow);
    // Print all allocation sites ...
    out.println();
  }
}