/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.info.CpuSampleInfo;

/**
 * <PRE>
 * TAG           BODY      notes
 * -----------------------------------------------------------------
 * PROF_CPU_SAMPLES        a set of sample traces of running threads
 *
 *                u4        total number of samples
 *                u4        # of traces
 *               [u4        # of samples (cost)
 *                u4]*      stack trace serial number
 */
public class BinBody_CpuSamples extends BinBody {
  
  public int totalNumberOfSamples;
  public int numberOfTraces;
  
  public BinBody_CpuSamples(BinRecord parent) {
    super(parent);
  }
  
  public void read() throws java.io.IOException {
    totalNumberOfSamples = in.read_u4();
    numberOfTraces       = in.read_u4();
    
    CpuSampleInfo[] cpuSamples = new CpuSampleInfo[numberOfTraces];
    
    CpuSampleInfo sample;
    for (int i = 0; i < numberOfTraces; i++) {
      sample = new CpuSampleInfo();
      
      sample.stackTraceCost     = in.read_u4();
      sample.stackTraceSerialNo = in.read_u4();
      sample.traceInfo          = namespace.lookupTrace(sample.stackTraceSerialNo);
      
      sample.resolve();
      cpuSamples[i] = sample;
    }
    
    in.eventCpuSamples(parent.ticks, totalNumberOfSamples, cpuSamples);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_CPU_SAMPLES");
    out.println("  total number of samples = " + totalNumberOfSamples);
    out.println("  number of traces        = " + numberOfTraces);
    // Print all cpu samples ...
    out.println();
  }
}
