/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.info.FrameInfo;

/**
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------
 * PROF_FRAME              a Java stack frame
 *
 *                id        stack frame ID
 *                id        method name ID
 *                id        method signature ID
 *                id        source file name ID
 *                u4        class serial number
 *                u4        line number >0: normal
 *                                      -1: unknown
 *                                      -2: compiled method
 *                                      -3: native method
 */
public class BinBody_Frame extends BinBody {
  
  public BinRecordReader reader;
  public FrameInfo       frameInfo;
  
  public BinBody_Frame(BinRecord parent) {
    super(parent);
    reader = new BinRecordReader(in);
  }
  
  public void read() throws java.io.IOException {
    reader.read(parent.length);
    
    frameInfo = new FrameInfo();
    
    frameInfo.stackFrameID      = reader.read_id();
    
    frameInfo.methodName        = namespace.lookupName(reader.read_id()); // methodNameID
    
                                  reader.skip(in.sizeofID); // not needed for now
    
    frameInfo.sourceFileName    = namespace.lookupName(reader.read_id()); // sourceFileNameID
    
    frameInfo.className         = namespace.lookupClass(reader.read_u4()); // classSerialNo
    
    frameInfo.lineNumber        = reader.read_u4();
    
    bind();
  }
  
  public void bind() {
    namespace.getFrameTable().bind(frameInfo.stackFrameID, frameInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_FRAME");
    out.println("  frame name = " + frameInfo.toString());
    out.println();
  }
}