/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.info.GraphInfo;

/**
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------
 * PROF_GRAPH_INFO         graph info
 *
 *               u4         event duration
 *               u4         total live bytes
 *               u4         total live instances
 *               u8         total bytes allocated
 *               u8         total instances allocated
 *               u4         number of total threads
 *               u4         number of active threads
 *               u4         number of classes
 *               u4         GC activity time
 */
public class BinBody_GraphInfo extends BinBody {
  
  public GraphInfo graphInfo;
  
  public BinBody_GraphInfo(BinRecord parent) {
    super(parent);
  }
  
  public void read() throws java.io.IOException {
    graphInfo = new GraphInfo();
    
    int eventDuration                 = in.read_u4();
    graphInfo.totalLiveBytes          = in.read_u4();
    graphInfo.totalLiveInstances      = in.read_u4();
    graphInfo.totalBytesAllocated     = in.read_u8();
    graphInfo.totalInstancesAllocated = in.read_u8();
    graphInfo.numOfThreads            = in.read_u4();
    graphInfo.numOfActiveThreads      = in.read_u4();
    graphInfo.numOfClasses            = in.read_u4();
    int gcActiveTime                  = in.read_u4();
    if (eventDuration != 0) graphInfo.GCActivity = (byte) (100 * gcActiveTime / eventDuration);
    
    in.eventGraphInfo(graphInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_GRAPH_INFO");
    out.println("  total live bytes          = " + graphInfo.totalLiveBytes);
    out.println("  total live instances      = " + graphInfo.totalLiveInstances);
    out.println("  total bytes allocated     = " + graphInfo.totalBytesAllocated);
    out.println("  total instances allocated = " + graphInfo.totalInstancesAllocated);
    out.println("  number of threads         = " + graphInfo.numOfThreads);
    out.println("  number of active threads  = " + graphInfo.numOfActiveThreads);
    out.println("  number of classes         = " + graphInfo.numOfClasses);
    out.println("  GC activity               = " + graphInfo.GCActivity * 10 + "ms");
    out.println();
  }
}