/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

/**
 * <PRE>
 * TAG           BODY      notes
 * --------------------------------------------
 * PROF_LOAD_CLASS         a newly loaded class
 *
 *                u4        class serial number (> 0)
 *                id        class object ID
 *                u4        stack trace serial number
 *                id        class name ID
 */
public class BinBody_LoadClass extends BinBody {
  
  public int    classSerialNo;
  public int    classObjectID;
  public String className;
  
  public BinBody_LoadClass(BinRecord parent) {
    super(parent);
  }
  
  public void read() throws java.io.IOException {
        classSerialNo = in.read_u4();
    
        classObjectID = in.read_id(); // needed for heap dump
    
                        in.read_u4(); // not needed for now
    
    int classNameID   = in.read_id();
        className     = namespace.lookupName(classNameID).replace('/', '.');
    
    if (className != null) bind();
  }
  
  public void bind() {
    namespace.getClassTable().bind(classSerialNo, className);
    namespace.getClassFromObjectIDTable().bind(classObjectID, className);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_CLASS_LOAD");
    out.println("  class name = " + className);
    out.println();
  }
}
