/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import java.io.IOException;
import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.MonitorInfo;
import com.prosyst.jpd.profiler.info.MonitorsDump;

/**
 * <PRE>
 * TAG           BODY      notes
 * -----------------------------------------------------------
 * PROF_MONITOR_DUMP       all threads status and monitor dump
 *
 *               u4         threads number
 *              [u4         stack trace serial number
 *               u4]*       thread status
 *
 *               u4         monitors number
 *              [u1                          monitor type
 *
 *               JVMPI_MONITOR_JAVA          java monitor
 *
 *                          u1               monitor class type
 *                          id               monitor name
 *                          id               monitor id
 *                          u4               owner thread
 *                          u4               entry count
 *                          u4               # of threads waiting to enter
 *                          [u4]             threads waiting to enter
 *                          u4               # of threads waiting to be notified
 *                          [u4]             threads waiting to be notified
 *
 *               JVMPI_MONITOR_RAW           raw monitor
 *
 *                          id               raw monitor name
 *                          id               raw monitor id
 *                          u4               owner thread
 *                          u4               entry count
 *                          u4               # of threads waiting to enter
 *                          [u4]             threads waiting to enter
 *                          u4               # of threads waiting to be notified
 *                          [u4]             threads waiting to be notified
 *                 ]*
 */
public class BinBody_MonitorDump extends BinBody {
  
  public int threadsNumber, monitorsNumber;
  
  public BinBody_MonitorDump(BinRecord parent) {
    super(parent);
  }
  
  public void read() throws IOException {
    MonitorsDump monitorsDump = new MonitorsDump();
    
    threadsNumber = in.read_u4();
    
    monitorsDump.traces   = new TraceInfo[threadsNumber];
    monitorsDump.statuses = new int[threadsNumber];
    
    for (int i = 0; i < threadsNumber; i++) {
      int stackTraceSerialNo   = in.read_u4();
      monitorsDump.traces[i]   = namespace.lookupTrace(stackTraceSerialNo);
      
      monitorsDump.statuses[i] = in.read_u4();
    }
    
    monitorsNumber = in.read_u4();
    
    monitorsDump.monitors = new MonitorInfo[monitorsNumber];
    
    MonitorInfo monitorInfo;
    for (int i = 0; i < monitorsNumber; i++) {
      monitorInfo = new MonitorInfo();
      
      monitorInfo.monitorType     = in.read_u1();
      
             if (monitorInfo.monitorType == BinRecord.JVMPI_MONITOR_JAVA) {
        monitorInfo.monitorObjectType = in.read_u1();
      } else if (monitorInfo.monitorType != BinRecord.JVMPI_MONITOR_RAW) {
        throw new IOException("Unknown monitor dump type = " + monitorInfo.monitorType);
      }
      
      int monNameID                 = in.read_id();
      String monitorName            = namespace.lookupName(monNameID);
      if (monitorName != null) monitorInfo.monitorName = monitorName.replace('/', '.');
      
      monitorInfo.monitorID         = in.read_id();
      
      if (monitorInfo.monitorType == BinRecord.JVMPI_MONITOR_JAVA) {
        monitorInfo.traceInfo = namespace.lookupTrace(in.read_id());
      }
      
      int         t_ownerSN         = in.read_u4();
      monitorInfo.t_ownerInfo       = namespace.lookupThread(t_ownerSN);
      
      monitorInfo.entryCount        = in.read_u4();
      
      int waitEnterCount            = in.read_u4();
      monitorInfo.t_enterInfo = new ThreadInfo[waitEnterCount];
      for (int j = 0; j < waitEnterCount; j++) {
        int         t_enterSN       = in.read_u4();
        monitorInfo.t_enterInfo[j]  = namespace.lookupThread(t_enterSN);
      }
      
      int waitNotifyCount           = in.read_u4();
      monitorInfo.t_notifyInfo = new ThreadInfo[waitNotifyCount];
      for (int j = 0; j < waitNotifyCount; j++) {
        int         t_notifySN      = in.read_u4();
        monitorInfo.t_notifyInfo[j] = namespace.lookupThread(t_notifySN);
      }
      
      monitorsDump.monitors[i] = monitorInfo;
    }
    
    in.eventMonitorsDump(monitorsDump);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_MONITOR_DUMP");
    out.println("  threads  number = " + threadsNumber);
    // Print all threads status ...
    out.println("  monitors number = " + monitorsNumber);
    // Print all monitors dump ...
    out.println();
  }
}