/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import java.io.IOException;
import com.prosyst.jpd.profiler.info.MonitorInfo;

/**
 * <PRE>
 * TAG           BODY      notes
 * ---------------------------------------------------------------------------
 * PROF_MONITOR_EVENT       monitor event (enter, entered, exit, wait, waited)
 *
 *               u1         monitor event type
 */
public class BinBody_MonitorEvent extends BinBody {
  
  int type;
  
  public BinBody_MonitorEvent(BinRecord parent, int type) {
    super(parent);
    this.type = type;
  }
  
  public void read() throws IOException {
    MonitorInfo monitorInfo = new MonitorInfo();
    
    monitorInfo.monitorType       = (byte) type;
    monitorInfo.monitorObjectType = in.read_u1();
    
    int monNameID                 = in.read_id();
    String monitorName            = namespace.lookupName(monNameID);
    if (monitorName != null) monitorInfo.monitorName = monitorName.replace('/', '.');
    
    monitorInfo.monitorID         = in.read_id();
    monitorInfo.traceInfo         = namespace.lookupTrace(in.read_id());
    
    monitorInfo.timeout           = in.read_u8();
    
    int t_ownerSN                 = in.read_u4();
    monitorInfo.t_ownerInfo       = namespace.lookupThread(t_ownerSN);
    
    monitorInfo.ticks             = parent.ticks;
    
    in.eventCommon(monitorInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_MONITOR_EVENT");
    switch (type) {
      case BinRecord.PROF_MONITOR_ENTER:   out.println("PROF_MONITOR_ENTER");   break;
      case BinRecord.PROF_MONITOR_ENTERED: out.println("PROF_MONITOR_ENTERED"); break;
      case BinRecord.PROF_MONITOR_EXIT:    out.println("PROF_MONITOR_EXIT");    break;
      case BinRecord.PROF_MONITOR_WAIT:    out.println("PROF_MONITOR_WAIT");    break;
      case BinRecord.PROF_MONITOR_WAITED:  out.println("PROF_MONITOR_WAITED");  break;
    }
    out.println();
  }
}