/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.info.ThreadInfo;

/**
 * <PRE>
 * TAG           BODY      notes
 * ----------------------------------------------
 * PROF_START_THREAD       a newly started thread
 *
 *               u4         thread serial number (> 0)
 *               id         thread object ID
 *               u4         stack trace serial number
 *               id         thread name ID
 *               id         thread group name ID
 *               id         thread group parent name ID
 */
public class BinBody_StartThread extends BinBody {
  
  public ThreadInfo threadInfo;
  
  public BinBody_StartThread(BinRecord parent) {
    super(parent);
  }
  
  public void read() throws java.io.IOException {
    threadInfo = new ThreadInfo();
    
    threadInfo.threadSerialNo          = in.read_u4();
    
                                         in.read_id(); // threadObjectID
    
                                         in.read_u4(); // stackTraceSerialNo
    
    int        threadNameID            = in.read_id();
    threadInfo.threadName              = namespace.lookupName(threadNameID);
    
                                         in.read_id(); // threadGroupNameID
    
                                         in.read_id(); // threadGroupParentNameID
    
    threadInfo.threadStartTime         = parent.ticks;
    
    bind();
    in.eventCommon(threadInfo);
  }
  
  public void bind() {
    namespace.getThreadTable().bind(threadInfo.threadSerialNo, threadInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_START_THREAD");
    out.println("  thread serial number = " + threadInfo.threadSerialNo);
    out.println("  thread name          = " + threadInfo.threadName);
    out.println();
  }
}
