/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import java.io.IOException;

import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;

/**
 * Trace represents a Java stack trace. Each trace consists of a
 * series of stack frames. Other records refer to traces to identify
 *  (1) where object allocations have taken place,
 *  (2) the frames in which GC roots were found, and
 *  (3) frequently executed methods.
 *
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------
 * PROF_TRACE              a Java stack trace
 *
 *               u4         stack trace serial number
 *               u4         thread serial number
 *               u4         number of frames
 *              [id]*       stack frame IDs
 */
public class BinBody_Trace extends BinBody {
  
  public BinRecordReader reader;
  public int             stackTraceSerialNo;
  public TraceInfo       traceInfo;
  
  public BinBody_Trace(BinRecord parent) {
    super(parent);
    reader = new BinRecordReader(in);
  }
  
  public void read() throws java.io.IOException {
    reader.read(parent.length);
    
              stackTraceSerialNo = reader.read_u4();
              
    traceInfo = namespace.lookupTrace(stackTraceSerialNo);
    if (traceInfo == null) traceInfo = new TraceInfo();
    
    traceInfo.threadInfo         = namespace.lookupThread(reader.read_u4()); // threadSerialNo
    
    traceInfo.numberOfFrames     = reader.read_u4();
    
    traceInfo.frameInfo          = new FrameInfo[traceInfo.numberOfFrames];
    if (in.sizeofID == 4) {
      for (int i = 0; i < traceInfo.numberOfFrames; i++) {
        traceInfo.frameInfo[i]   = namespace.lookupFrame(reader.read_u4());
      }
    } else if (in.sizeofID == 2) {
      for (int i = 0; i < traceInfo.numberOfFrames; i++) {
        traceInfo.frameInfo[i]   = namespace.lookupFrame(reader.read_u2());
      }
    } else {
      throw new IOException("Illegal identifier size: " + in.sizeofID);
    }
    
    bind();
  }
  
  public void bind() {
    namespace.getTraceTable().bind(stackTraceSerialNo, traceInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("PROF_TRACE");
    out.println("  stack trace info = " + traceInfo.toString());
    out.println();
  }
}
