/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.binbody;

import com.prosyst.jpd.profiler.BinInputImpl;
import java.io.IOException;

/**
 * Speeds the reading of some frequently used record types.
 */
public class BinRecordReader {
  
  protected BinInputImpl in;
  protected byte[]       buf;
  protected int          offset;
  
  public BinRecordReader(BinInputImpl in) {
    init(in);
  }
  
  public void init(BinInputImpl in) {
    this.in = in;
  }
  
  public void read(int length) throws IOException {
    buf = in.read_bytes(length);
    offset = 0;
  }
  
  public void clear() {
    buf = null;
  }
  
  public void skip(int length) {
    offset += length;
  }
  
  public byte read_u1() throws IOException {
    return buf[offset++];
  }
  
  public int read_u2() throws IOException {
    return ((buf[offset++] & 0xff) << 8)  + ((buf[offset++] & 0xff) << 0);
  }
  
  public int read_u4() throws IOException {
    return (buf[offset++] << 24) + ((buf[offset++] & 0xff) << 16)
         + ((buf[offset++] & 0xff) << 8)  + (buf[offset++] & 0xff);
  }
  
  public long read_u8() throws IOException {
    return ((long) (read_u4()) << 32) + (read_u4() & 0xFFFFFFFFL);
  }
  
  public int read_id() throws IOException {
    if (in.sizeofID == 4) {
      return read_u4();
    } else if (in.sizeofID == 2) {
      return read_u2();
    } else {
      throw new IOException("Illegal identifier size: " + in.sizeofID);
    }
  }
}