/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.io.File;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import com.prosyst.pgui.event.LookAndFeelListener;
import com.prosyst.pgui.event.LookAndFeelEvent;
import com.prosyst.pgui.event.ChangeListener;
import com.prosyst.pgui.event.ChangeEvent;
import com.prosyst.pgui.PGUIConstants;
import com.prosyst.pgui.UIManager;
import com.prosyst.pgui.PGUIUtilities;
import com.prosyst.pgui.PFrame;
import com.prosyst.pgui.PPanel;
import com.prosyst.pgui.PComponent;
import com.prosyst.pgui.POptionPane;
import com.prosyst.pgui.utils.filechooser.PFileChooser;

import com.prosyst.jpd.SecurityManager;
import com.prosyst.jpd.profiler.gui.dialogs.StartDialog;
import com.prosyst.jpd.profiler.gui.dialogs.ListenDialog;
import com.prosyst.jpd.profiler.gui.dialogs.PreferencesDialog;
import com.prosyst.jpd.profiler.gui.dialogs.OptionPanel;
import com.prosyst.jpd.profiler.gui.dialogs.FindDialog;
import com.prosyst.jpd.profiler.gui.dialogs.ExportDialog;
import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.CpuSampleInfo;
import com.prosyst.jpd.profiler.info.GraphInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;
import com.prosyst.jpd.profiler.info.Misc;
import com.prosyst.jpd.profiler.info.MonitorsDump;
import com.prosyst.jpd.profiler.info.MonitorInfo;
import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.TriggerEvent;
import com.prosyst.jpd.profiler.BinOutput;
import com.prosyst.jpd.profiler.ProfilerServer;
import com.prosyst.jpd.profiler.ProfilerListener;
import com.prosyst.jpd.profiler.Parser;
import com.prosyst.jpd.profiler.ScriptParser;
import com.prosyst.jpd.profiler.storage.ProfilerNamespace;
import com.prosyst.jpd.profiler.storage.ProfilerNamespaceDefaultImpl;
import com.prosyst.jpd.profiler.ProfilerMediatorInterface;

import com.prosyst.jpd.profiler.gui.help.HelpManager;
import com.prosyst.jpd.profiler.gui.help.HelpResource;

import com.prosyst.gui.CHelpMenu;                // from PUtil project
import com.prosyst.gui.CIOTextArea;              // from PUtil project
import com.prosyst.gui.CFindDialog;              // from PUtil project
import com.prosyst.util.log.Log;                 // from Util project
import com.prosyst.util.io.IniFile;              // from Util project

/**
 * ActionManager performs user activated tasks.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class ActionManager extends WindowAdapter implements ActionCodes, ActionListener,
                                                            ProfilerListener,
                                                            LookAndFeelListener, ChangeListener {
  /** launch mode */
  public static final int MODE_LAUNCH = 0;
  /** listen mode */
  public static final int MODE_LISTEN = 1;
  /** attach mode */
  public static final int MODE_ATTACH = 2;
  
  public static final int DEFAULT_DEPTH = 10;
  
  private static final int NOT_YET_STARTED = 0;
  private static final int CALCULATING     = 1;
  private static final int REMOTE          = 2;
  
  private static final String EMPTY_STRING = "";
  private static final String findTitle         = ResourceManager.getString("FindDialog.title");
  private static final String loadTitle         = ResourceManager.getString("Snapshot.title.load");
  private static final String saveTitle         = ResourceManager.getString("Snapshot.title.save");
  private static final String titleAddSrc       = ResourceManager.getString("InfoAddSrc.title");
  private static final String msgAddSrc         = ResourceManager.getString("InfoAddSrc.msg");
  private static final String titleExit         = ResourceManager.getString("InfoExit.title");
  private static final String msgExit           = ResourceManager.getString("InfoExit.msg");
  private static final String eclipseTitle      = ResourceManager.getString("InfoEclipseHome.title");
  private static final String addsrcTitle       = ResourceManager.getString("PathDialog.title.addfile");
  
  /* sampler intervals */
  private static final int MEM_SAMPLER_INTERVAL = 2000;
  private static final int WAIT_INTERVAL        = 1000;
  
  public  static final String PATH_SPRT       = System.getProperty("path.separator");
  public  static final char   PATH_SPRT_CHAR  = PATH_SPRT.charAt(0);
  
  private   boolean                   system_exit;
  private   VisualManager             visualMng;
  private   PFrame                    frame;
  private   StartDialog               startDialog;
  private   ListenDialog              listenDialog;
  private   PreferencesDialog         prefsDialog;
  protected ExportDialog              exportDialog;
  protected FindDialog                findDialog;
  protected CFindDialog               cFindDialog;
  
  protected AllocSamplerGuiImpl       allocSampler;
  protected CpuSamplerGuiImpl         cpuSampler;
  protected HeapDumpPanel             heapPanel;
  protected VMInfoPanel               vminfoPanel;
  protected ThreadsPanel              threadsPanel;
  public    TriggerPanel              triggerPanel;
  private   ProfilerServer            server;
  protected BinOutput                 output;
  private   ProfilerNamespace         namespace;
  private   Process                   process;
  
  private   ProfilerMediatorInterface plugin;
  private   OptionPanel               optPanel;
  private   boolean                   memDisabled     = false;
  private   boolean                   cpuDisabled     = false;
  private   boolean                   autoCpu         = false;
  private   boolean                   vmDisabled      = false;
  private   boolean                   threadsDisabled = false;
  private   int                       vminfoInterval  = 1998;
  private   int                       port            = ProfilerServer.DEFAULT_PORT;
  private   int                       mode;
  private   int                       ver;
  private   boolean                   isSupported     = true;
  private   boolean                   isNotRunning    = true;
  private   boolean                   isListening     = false;
  private   boolean                   isPaused        = false;
  private   boolean                   isProfiling     = false;
  private   boolean                   hasData         = false;
  private   boolean                   hasHeap         = false;
  protected boolean                   hasSource       = false;
  
  //  CONSTRUCTOR  --------------------------------------------------------------------------------
  
  /**
   * Creates new Profiler application. Used in profiler plugin.
   *
   * @param   plugin  mediator between the profiler and the plugin
   * @param   frame   parent frame
   */
  public ActionManager(ProfilerMediatorInterface mediator, PFrame parentFrame) {
    system_exit = false;
    plugin      = mediator;
    frame       = parentFrame;
    
    visualMng   = new VisualManager(this, false);
    
    UIManager.addLookAndFeelListener(this);
  }
  
  /**
   * Creates new Profiler application. Used in stand-alone mode.
   * See JavaDoc in Main for the syntax of command line arguments and examples.
   *
   * @param   args    the array of command line arguments
   */
  public ActionManager(String[] args) {
    system_exit  = true;
    
    HelpManager.setStandAloneHelp(false, false, VisualManager.getTitleShort());
    HelpResource.setResource("profiler-help");
    
    allocSampler = new AllocSamplerGuiImpl(this);
    cpuSampler   = new CpuSamplerGuiImpl(this);
    heapPanel    = new HeapDumpPanel(this);
    vminfoPanel  = new VMInfoPanel();
    threadsPanel = new ThreadsPanel(this);
    visualMng    = new VisualManager(this, true);
    frame        = visualMng.getFrame();
    frame.setDefaultCloseOperation(PGUIConstants.DO_NOTHING_ON_CLOSE);
    triggerPanel = new TriggerPanel(frame);
    
    UIManager.addLookAndFeelListener(this);
    
    visualMng.setVisible(true);
    
    CHelpMenu helpMenu = HelpManager.getHelpMenu();
    if (helpMenu != null) {
      helpMenu.getAboutDialog().setProductName(VisualManager.getTitle());
      int res = SecurityManager.getDays();
      helpMenu.setDomainInfo(res);
      if (res < 0) {
        PComponent.setCurrentEventFilterQueue(null);
        POptionPane.showMessageDialog(frame, "Version has expired !", "Warning", POptionPane.WARNING_MESSAGE);
        System.exit(1);
      }
    }
    
    if (args.length > 0) {
      parseParams(args);
    } else {
      int dlgIndex = PreferencesDialog.getDefaultDialog();
      if (dlgIndex == PreferencesDialog.LAUNCH) actionLaunch();
      if (dlgIndex == PreferencesDialog.LISTEN) actionListen();
      if (dlgIndex == PreferencesDialog.ATTACH) actionAttach();
    }
  }
  
  public void parseParams(String[] args) {
    if (startDialog == null) {
      startDialog = new StartDialog(this, frame);
      optPanel = startDialog.getOptionPanel();
    }
    if (args[0].equals("-ini")) {
      startDialog.loadFile(args[1]);
      setStatus();
      visualMng.setPlayEnabled(true);
      boolean ok = false;
      if (args.length > 3) ok = startDialog.getVMUsePanel().actionAdd(args[3]);
      if (args.length > 2 && "true".equalsIgnoreCase(args[2]) && ok) actionPlay();
      else actionLaunch();
    } if (args[0].equals("-script")) {
      boolean shouldOverwrite = args.length > 2 && "true".equalsIgnoreCase(args[2]);
      actionScript(args[1], shouldOverwrite);
    } if (args[0].equals("-attach")) {
      int port = (args.length > 2 ? Integer.parseInt(args[2]) : this.port);
      mode = MODE_ATTACH;
      setStatus();
      if (listenDialog == null) {
        listenDialog = new ListenDialog(frame);
        listenDialog.load();
      }
      memDisabled     = listenDialog.isMemDisabled();
      cpuDisabled     = listenDialog.isCpuDisabled();
      autoCpu         = listenDialog.isAutoCpu();
      vmDisabled      = listenDialog.isVMDisabled();
      threadsDisabled = listenDialog.isThreadsDisabled();
      actionAttach("localhost", port);
    }
  }
  
  private ServerJob serverJob;
  
  // Starts the server in a separate thread
  class ServerJob extends Thread {
    public void run() {
      isNotRunning = false;
      try {
        if (mode == MODE_LAUNCH) server.start(port, 5000);
        if (mode == MODE_LISTEN) {
          isListening = true;
          setStatus();
          server.start(port, 0);
        }
      } catch (Exception ex) {
        Log.printDebug("ACTION.ServerJob", ex);
        isNotRunning = true;
        process = null;
        if (plugin == null) {
          visualMng.setStartingEnabled(true);
          visualMng.setTabIndex(VisualManager.PANEL_CONSOLE);
          if (startDialog  != null) startDialog.startEnabled(true);
          if (listenDialog != null) listenDialog.startEnabled(true);
        } else {
          if (ex instanceof java.net.BindException) {
            plugin.profilerError(ResourceManager.getString("ProfilerPluginTitle.error"),
                                 ResourceManager.getString("ProfilerPluginMsg.errorbind") + port, -1);
          } else {
            plugin.profilerError("error", "errorconnect", 0);
          }
          if (listenDialog != null) listenDialog.startEnabled(true);
        }
      } finally {
        if (mode == MODE_LISTEN) {
          isListening = false;
          setStatus();
        }
      }
    }
  }
  
  // Waits the profiled process in a separate thread
  class RemoteServerJob extends Thread {
    RemoteServerJob() {
      setName("RemoteServerJob");
      setDaemon(true);
    }
    
    public void run() {
      isNotRunning = false;
      try {
        if (plugin == null) {
          visualMng.setStopEnabled(true);
          visualMng.setTabIndex(VisualManager.PANEL_CONSOLE);
        } else plugin.profilerRemote();
        process.waitFor();
      } catch (Exception ex) {
        Log.printDebug("ACTION.RemoteServerJob", ex);
      } finally {
        process = null;
        isNotRunning = true;
        initSamplers(NOT_YET_STARTED);
        if (plugin == null) {
          visualMng.setStartingEnabled(true);
          visualMng.setStopEnabled(false);
        } else {
          plugin.profilerNotStarted();
        }
      }
    }
  }
  
  private VMSampler vmSampler;
  
  // Sends commands to the server in a separate thread
  class VMSampler extends Thread {
    VMSampler() {
      setName("ProfilerVMSampler");
      setDaemon(true);
    }
    
    public void run() {
      try {
        sleep(WAIT_INTERVAL);
        if (!memDisabled) output.command_AllocSites(3, 0);
        while (!vmDisabled) {
          output.command_GraphInfo();
          sleep(vminfoInterval);
        }
      } catch (Throwable _) { }
    }
  }
  
  //
  //  PROFILER PLUGIN  ----------------------------------------------------------------------------
  //
  
  /**
   * Gets the main panel.
   *
   * @return     the main panel
   */
  public PPanel getMainPanel() {
    if (allocSampler == null) allocSampler = new AllocSamplerGuiImpl(this);
    if (cpuSampler   == null) cpuSampler   = new CpuSamplerGuiImpl(this);
    if (heapPanel    == null) heapPanel    = new HeapDumpPanel(this);
    if (vminfoPanel  == null) vminfoPanel  = new VMInfoPanel();
    if (threadsPanel == null) threadsPanel = new ThreadsPanel(this);
    return visualMng.getMainPanel();
  }
  
  /**
   * Gets the console text area.
   *
   * @return     the console text area
   */
  public PPanel getConsolePanel() {
    return visualMng.getConsolePanel();
  }
  
  /**
   * Sets the options panel.
   *
   * @param  optionPanel     the options panel
   */
  public void setOptionPanel(OptionPanel optionPanel) {
    optPanel = optionPanel;
  }
  
  /**
   * Sets the triggers panel.
   *
   * @param  triggersPanel     the triggers panel
   */
  public void setTriggerPanel(TriggerPanel trgrPanel) {
    triggerPanel = trgrPanel;
  }
  
  /**
   * Gets the tab index.
   */
  public int getTabIndex() {
    return visualMng.getTabIndex();
  }
  
  /**
   * Checks whether profiler is paused.
   *
   * @return     true if is paused
   */
  public boolean isPaused() {
    return isPaused;
  }
  
  /**
   * Checks whether find next should be enabled.
   *
   * @return     true if find next should be enabled
   */
  public boolean isFindNext() {
    if (findDialog == null) return false;
    return findDialog.hasNext() && visualMng.getTabIndex() <= VisualManager.PANEL_HEAP;
  }
  
  /** Event meaning that the profiler site in plugin has been hidden. */
  public void actionHideSite() {
    actionStop();
  }
  
  /** Event meaning that the profiler project in plugin has been closed. */
  public void actionCloseProject() {
    if (allocSampler != null || cpuSampler  != null
     || heapPanel    != null || vminfoPanel != null || threadsPanel != null) initSamplers(NOT_YET_STARTED);
    visualMng.clearConsole();
  }
  
  //
  //  ACTION LISTENER  ----------------------------------------------------------------------------
  //
  
  public void actionPerformed(ActionEvent ev) {
    try {
      perform(Integer.parseInt(ev.getActionCommand() + 100));
    } catch (NumberFormatException ex) {
      Log.printDebug("ACTION.actionPerformed", "Wrong action code.");
    }
  }
    
  public void perform(int action) {
    switch (action) {
      case ACTION_LAUNCH:
        actionLaunch();
        break;
      
      case ACTION_LAUNCHQ:
        actionLaunchQuick();
        break;
      
      case ACTION_LISTEN:
        actionListen();
        break;
      
      case ACTION_ATTACH:
        actionAttach();
        break;
      
      case ACTION_PLAY:
        actionPlay();
        break;
      
      case ACTION_PAUSE:
        actionPause();
        break;
      
      case ACTION_STOP:
        actionStop();
        break;
      
      case ACTION_FIND:
        actionFind();
        break;
      
      case ACTION_FIND_NEXT:
        actionFindNext();
        break;
      
      case ACTION_LOAD_DATA:
        actionLoadData();
        break;
      
      case ACTION_SAVE_DATA:
        actionSaveData();
        break;
      
      case ACTION_SAVE_TEXT:
        actionSaveText();
        break;
      
      case ACTION_CLEAR_DATA:
        actionClearData();
        break;
      
      case ACTION_START_MARK:
        actionStartMark(false);
        break;
      
      case ACTION_CONT_MARK:
        actionContinueMark(false);
        break;
      
      case ACTION_STOP_MARK:
        actionStopMark();
        break;
      
      case ACTION_GC:
        actionGC();
        break;
      
      case ACTION_SOURCE:
        actionSource();
        break;
      
      case ACTION_TREE:
        actionTree();
        break;
      
      case ACTION_HEAP_DUMP:
        actionHeapDump();
        break;
      
      case ACTION_MONITORS:
        actionMonitorDump();
        break;
      
      case ACTION_PREF:
        actionPrefs();
        break;
      
      case ACTION_ECLIPSE:
        actionEclipse();
        break;
      
      case ACTION_EXIT:
        actionExit();
        break;
      
      default:
        Log.printDebug("ACTION.perform", "Action not implemented yet.");
    }
  }
  
  /** Launches a program for profiling and sets the options. */
  public void actionLaunch() {
    if (startDialog == null) {
      startDialog = new StartDialog(this, frame);
      optPanel = startDialog.getOptionPanel();
    }
    int stat = startDialog.showDialog(isNotRunning);
    if (stat == StartDialog.RETURN_CANCEL) {
      if (isNotRunning) {
        setStatus();
      }
    }
    if (stat == StartDialog.RETURN_OK) {
      if (isNotRunning) {
        mode = MODE_LAUNCH;
        setStatus();
        visualMng.setPlayEnabled(true);
      }
    }
    if (stat == StartDialog.RETURN_START) {
      mode = MODE_LAUNCH;
      setStatus();
      actionPlay();
    }
  }
  
  /** Launches a program for profiling and sets the options. */
  public void actionLaunchQuick() {
    if (isNotRunning) {
      mode = MODE_LAUNCH;
      actionPlay();
    }
  }
  
  /** Listens on a port for remote profiling. */
  public void actionListen() {
    if (listenDialog == null) {
      listenDialog = new ListenDialog(frame);
    }
    int stat = listenDialog.showDialog(false, isNotRunning);
    if (stat == ListenDialog.RETURN_OK) {
      if (isNotRunning) {
        mode = MODE_LISTEN;
        setStatus();
        visualMng.setPlayEnabled(true);
      }
    }
    if (stat == ListenDialog.RETURN_START) {
      mode = MODE_LISTEN;
      setStatus();
      actionPlay();
    }
  }
  
  /** Attaches on a port for remote profiling. */
  public void actionAttach() {
    if (listenDialog == null) {
      listenDialog = new ListenDialog(frame);
    }
    int stat = listenDialog.showDialog(true, isNotRunning);
    if (stat == ListenDialog.RETURN_OK) {
      if (isNotRunning) {
        mode = MODE_ATTACH;
        setStatus();
        visualMng.setPlayEnabled(true);
      }
    }
    if (stat == ListenDialog.RETURN_START) {
      mode = MODE_ATTACH;
      setStatus();
      actionPlay();
    }
  }
  
  //-------------------------  PLAY, PAUSE, STOP  -------------------------------------------------
  
  /** Starts profiling of the program. */
  public void actionPlay() {
    try {
      if (isPaused) {
        if (output == null) return;
        isPaused = false;
        output.command_Resume();
        if (plugin == null) {
          visualMng.setPlayEnabled(false);
          visualMng.setPauseEnabled(true);
          visualMng.changePlayDesc(ResourceManager.getString("MenuBar.item.play"));
          if (!memDisabled) allocSampler.showAll = false;
          setStatus();
        }
        return;
      }
      isPaused        = false;
      if (mode == MODE_LAUNCH) {
        memDisabled     = optPanel.isMemDisabled();
        cpuDisabled     = optPanel.isCpuDisabled();
        autoCpu         = optPanel.isAutoCpu();
        vmDisabled      = optPanel.isVMDisabled();
        threadsDisabled = optPanel.isThreadsDisabled();
      } else {
        memDisabled     = listenDialog.isMemDisabled();
        cpuDisabled     = listenDialog.isCpuDisabled();
        autoCpu         = listenDialog.isAutoCpu();
        vmDisabled      = listenDialog.isVMDisabled();
        threadsDisabled = listenDialog.isThreadsDisabled();
      }
      
      if (mode == MODE_LAUNCH) {
        String host    = optPanel.getHost();
        port           = optPanel.getPort();
        String profCmd = optPanel.getCommand();
        
        String command;
        if (plugin == null) {
          command = createCommand(startDialog.getJDKPath(), getFullClassPath(), profCmd, host, port,
                                  startDialog.getJavaParam(), startDialog.getClassName(), startDialog.getProgParam());
        } else {
          String className = plugin.getMainClass();
          if (className == null) return;
          command = createCommand(plugin.getJDKPath(), plugin.getClassPath(), profCmd, host, port,
                                  plugin.getJavaParams(), className, plugin.getProgParams());
        }
        actionPlay(command);
      } else if (mode == MODE_LISTEN) {
        port = listenDialog.getPort();
        visualMng.clearConsole();
        visualMng.addToConsole(ResourceManager.getString("Console.msg.listening") + port + " ...\n");
        visualMng.clearSource();
        initSamplers(NOT_YET_STARTED);
        if (plugin == null) {
          visualMng.setStartingEnabled(false);
          visualMng.setStopEnabled(true);
        } else {
          plugin.profilerListening();
        }
        if (server == null) server = new ProfilerServer(this);
        serverJob = new ServerJob();
        serverJob.start();
      } else if (mode == MODE_ATTACH) {
        String host = listenDialog.getHost();
        port        = listenDialog.getPort();
        actionAttach(host, port);
      }
    } catch (Throwable ex) {
      Log.printDebug("ACTION.actionPlay", ex);
    }
  }
  
  public void actionAttach(String host, int port) {
    visualMng.clearConsole();
    visualMng.clearSource();
    try {
      isNotRunning = false;
      if (server == null) server = new ProfilerServer(this);
      server.start(host, port);
    } catch (Throwable _) {
      isNotRunning = true;
      visualMng.addToConsole(ResourceManager.getString("Console.msg.attach_error") + host + " on " + port + ".\n");
      if (plugin == null) visualMng.setTabIndex(VisualManager.PANEL_CONSOLE);
      else plugin.profilerNotStarted();
      return;
    }
    visualMng.addToConsole(ResourceManager.getString("Console.msg.attached") + host + " on " + port + ".\n");
    visualMng.setStartingEnabled(false);
    visualMng.setStopEnabled(true);
  }

  // Gets class path + work dir
  public String getFullClassPath() {
    String cp = startDialog.getClassPath();
    if (cp == null || cp.length() < 1) {
      return startDialog.getWorkDir();
    } else {
      return cp;
    }
  }
  
  /**
   * Starts profiling of the program.
   *
   * @param   command  command line string
   */
  private void actionPlay(String command) {
    try {
      isPaused = false;
      visualMng.setStartingEnabled(false);
      visualMng.clearConsole();
      visualMng.addToConsole(command + "\n");
      visualMng.clearSource();
      
      if (!optPanel.isRemoteMon()) {
        initSamplers(NOT_YET_STARTED);
        if (server == null) server = new ProfilerServer(this);
        serverJob = new ServerJob();
        serverJob.start();
      }
      String workDir = null;
      if (plugin == null) {
        workDir = startDialog.getWorkDir();
      } else {
        workDir = plugin.getWorkDir();
      }
      
      try {
        process = Parser.runProcess(System.getProperty("user.dir"), workDir, command);
        visualMng.setProcess(process);
        
        if (optPanel.isRemoteMon()) {
          initSamplers(REMOTE);
          visualMng.addToConsole(ResourceManager.getString("Sampler.root.remote_mon") + " ...\n");
          new RemoteServerJob().start();
        }
      } catch (IOException ex) {
        visualMng.addToConsole(ex.getMessage());
      }
    } catch (Throwable ex) {
      Log.printDebug("ACTION.actionPlay", ex);
    }
  }
  
  /** Creates command line string. */
  private String createCommand(String jdkPath, String classPath, String profCmd, String host,
                               int port, String javaParam, String className, String progParam) {
    ver = Parser.VER_UNKNOWN;
    
    if (classPath.endsWith("\\")) classPath = classPath.substring(0, classPath.length() - 1);
    if (jdkPath == null || jdkPath.equals(EMPTY_STRING)) {
      jdkPath = "java";
      ver = Parser.checkJavaVersion(jdkPath);
    } else {
      if (plugin == null) ver = startDialog.getJavaVersion();
      else ver = Parser.checkJavaVersion(jdkPath);
    }
    if (ver == Parser.VER_SUNW130) jdkPath += " -classic"; // classic fix !!
    if (className.endsWith(".jar")) jdkPath += " -jar";
    
    profCmd = Parser.parseCommand(profCmd, ver);
    
    String big_cmd = jdkPath + " " + profCmd + (host == null ? Integer.toString(port, 10) : host + ":" + port) + " " + javaParam;
    if (ver == Parser.VER_CVM) {
      big_cmd += " -Xbootclasspath:" + classPath + " ";
    } else {
      big_cmd += " -classpath \"" + classPath + "\" ";
    }
    big_cmd += className + " " + progParam;
    return big_cmd;
  }
  
  private void actionScript(String mbsDir, boolean shouldOverwrite) {
    String mbsScript = mbsDir + File.separatorChar + (Parser.OS_NAME.startsWith("Windows") ? "server.bat" : "server");
    try {
      mbsScript = ScriptParser.parse(mbsScript, shouldOverwrite, "com.prosyst.mbs.impl.framework.Start",
                  OptionPanel.DEF_CMD + ",net=localhost:" + OptionPanel.DEF_PORT);
      profileExternal(mbsDir, mbsScript, visualMng.getConsoleTextArea());
    } catch (Exception ex) {
      Log.printDebug("ACTION.actionScript", ex);
    }
  }
  
  /** Starts external profiling. */
  public Process profileExternal(String workDir, String className, CIOTextArea mBSConsole, String profCmd) {
    if (className == null) return null;
    port = optPanel.getPort();
    
    String command = createCommand(plugin.getJDKPath(), plugin.getClassPathWithServerJar(), profCmd, "localhost", port, // localhost ??
                                   plugin.getJavaParams(), className, plugin.getProgParams());
    return profileExternal(workDir, command, mBSConsole);
  }
  
  /** Starts external profiling. */
  public Process profileExternal(String workDir, String script, CIOTextArea mBSConsole) {
    if (script == null) return null;
    try {
      isPaused        = false;
      memDisabled     = optPanel.isMemDisabled();
      cpuDisabled     = optPanel.isCpuDisabled();
      autoCpu         = optPanel.isAutoCpu();
      vmDisabled      = optPanel.isVMDisabled();
      threadsDisabled = optPanel.isThreadsDisabled();
      port            = optPanel.getPort();
      
      visualMng.setStartingEnabled(false);
      visualMng.clearConsole();
      //visualMng.addToConsole(script + "\n");
      visualMng.clearSource();
      
      initSamplers(NOT_YET_STARTED);
      if (server == null) server = new ProfilerServer(this);
      serverJob = new ServerJob();
      serverJob.start();
      
      try {
        process = Parser.runProcess(System.getProperty("user.dir"), workDir, script);
        mBSConsole.setProcess(process);
        return process;
        
      } catch (IOException ex) {
        visualMng.addToConsole(ex.getMessage());
      }
    } catch (Throwable ex) {
      Log.printDebug("ACTION.profileExternal", ex);
    }
    return null;
  }
  
  /** Suspends / resumes profiling of the program. */
  public void actionPause() {
    if (output == null) return;
    try {
      if (!isPaused) {
        isPaused = true;
        output.command_Suspend();
        if (plugin == null) {
          visualMng.setPlayEnabled(true);
          visualMng.setPauseEnabled(false);
          visualMng.changePlayDesc(ResourceManager.getString("MenuBar.item.play.flip"));
          if (!memDisabled) allocSampler.showAllocSitesAll();
          setStatus();
        }
      }
    } catch (IOException ex) {
      output = null;
      Log.printDebug("ACTION.actionPause", ex);
    }
  }
  
  /** Stops profiling of the program. */
  public void actionStop() {
    if (vmSampler != null) {
      vmSampler.interrupt();
      vmSampler = null;
    }
    if (memSampler != null) {
      memSampler.interrupt();
      memSampler = null;
    }
    isProfiling = false;
    if (plugin == null) {
      visualMng.setGroupEnabled(false);
    } else {
      plugin.profilerStopping();
    }
    if (!memDisabled && output != null) output.command_DumpTraces();
    if (output == null || ver == Parser.VER_JEODE) {
      destroyProcess();
    } else {
      processDestroyer = new ProcessDestroyer();
      processDestroyer.start();
    }
    isNotRunning = true;
    if (mode == MODE_LISTEN || mode == MODE_ATTACH) {
      if (serverJob != null) serverJob.interrupt();
      if (server    != null) {
        if (!server.shutDown()) {
          visualMng.setStartingEnabled(true);
          visualMng.addToConsole(ResourceManager.getString("Console.msg.stopped") + "\n");
        }
      }
    }
  }
  
  private ProcessDestroyer processDestroyer;
  
  // Sends an exit command, waits for the process and then terminates it
  class ProcessDestroyer extends Thread {
    ProcessDestroyer() {
      setName("ProfilerProcessDestroyer");
      setDaemon(true);
    }
    
    public void run() {
      try {
        output.command_Exit();
      } catch (Throwable _) {
        destroyProcess();
        return;
      }
      try { sleep(10000); } catch (InterruptedException _) { }
      try {
        if (process != null) process.exitValue();
      } catch (IllegalThreadStateException _) {
        destroyProcess();
      }
    }
  }
    
  private void destroyProcess() {
    if (process != null) {
      if (dump_prof_events) Log.printDebug("Process destroyed"); // dump !!
      process.destroy();
    }
  }
  
  //-------------------------  START, CONTINUE, STOP MARK  ----------------------------------------
  
  boolean isMemStarted  = false;
  boolean hasMemStarted = false;
  boolean isCpuStarted  = false;
  boolean hasCpuStarted = false;
  
  /** Marks current state so that you can view the difference in mem and cpu. */
  public void actionStartMark(boolean shouldClear) {
    actionStartMemMark(shouldClear);
    actionStartCpuMark();
  }
  
  /** Marks current state so that you can view the difference in mem. */
  public void actionStartMemMark(boolean shouldClear) {
    isMemStarted  = true;
    hasMemStarted = true;
    if (!memDisabled) {
      if (shouldClear) try {
        output.command_ClearAllocSitesInfo();
      } catch (Exception _) { }
      allocSampler.startMark();
    }
  }
  
  /** Marks current state so that you can view the difference in cpu. */
  public void actionStartCpuMark() {
    isCpuStarted  = true;
    hasCpuStarted = true;
    if (!cpuDisabled) {
      try {
        output.command_ClearCpuSampling();
      } catch (Exception _) { }
      cpuSampler.init(ResourceManager.getString("Sampler.root.calculating.cpu"));
    }
  }
  
  /** Marks current state so that you can view the difference in mem and cpu. */
  public void actionContinueMark(boolean shouldClear) {
    actionContinueMemMark(shouldClear);
    actionContinueCpuMark();
  }
  
  /** Marks current state so that you can view the difference in mem. */
  public void actionContinueMemMark(boolean shouldClear) {
    if (!hasMemStarted) actionStartMemMark(shouldClear);
    if (isMemStarted) return;
    isMemStarted  = true;
    hasMemStarted = true;
    if (!memDisabled) {
      if (shouldClear) try {
        output.command_ClearAllocSitesInfo();
      } catch (Exception _) { }
      allocSampler.continueMark();
    }
  }
  
  /** Marks current state so that you can view the difference in cpu. */
  public void actionContinueCpuMark() {
    if (!hasCpuStarted) actionStartCpuMark();
    if (isCpuStarted) return;
    isCpuStarted  = true;
    hasCpuStarted = true;
    if (!cpuDisabled) {
      try {
        output.command_ClearCpuSampling();
      } catch (Exception _) { }
      cpuSampler.cont(ResourceManager.getString("Sampler.root.calculating.cpu"));
    }
  }
  
  /** Marks current state so that you can view the difference in mem and cpu. */
  public void actionStopMark() {
    actionStopMemMark();
    actionStopCpuMark();
  }
  
  /** Marks current state so that you can view the difference in mem. */
  public void actionStopMemMark() {
    if (!isMemStarted) return;
    if (!memDisabled && isMemStarted) {
      try {
        output.command_AllocSites(3, 0);
      } catch (Exception _) { }
      allocSampler.stopMark();
    }
    isMemStarted  = false;
  }
  
  /** Marks current state so that you can view the difference in cpu. */
  public void actionStopCpuMark() {
    if (!isCpuStarted && !autoCpu) return;
    isCpuStarted  = false;
    if (!cpuDisabled) {
      try {
        autoCpu = false;
        output.command_CpuSamples(0, (float) 0.0001);
      } catch (Exception _) { }
    }
  }
  
  //-------------------------  GC, SOURCE, TREE, HEAP, MONITORS  ----------------------------------
  
  /** Starts garbage collector. */
  public void actionGC() {
    if (output == null) return;
    try {
      output.command_GC();
    } catch (IOException ex) {
      output = null;
      Log.printDebug("ACTION.actionGC", ex);
    }
  }
  
  /** Shows source. */
  public void actionSource() {
    int index = visualMng.getTabIndex();
    if (index == VisualManager.PANEL_MEM) {
      allocSampler.showSource();
    }
    if (index == VisualManager.PANEL_CPU) {
      cpuSampler.showSource();
    }
    if (index == VisualManager.PANEL_HEAP) {
      heapPanel.showSource();
    }
  }
  
  protected void actionSource(FrameInfo frameInfo) {
    if (frameInfo == null || frameInfo.getSourceFileName() == null) return;
    showSource(getSourcePath(), frameInfo);
  }
  
  protected String getSourcePath() {
    String sourcePath = EMPTY_STRING;
    if (mode == MODE_LAUNCH) {
      if (plugin == null) {
        sourcePath = startDialog.getSourcePath();
      } else {
        sourcePath = plugin.getSourcePath();
      }
    }
    if (mode == MODE_LISTEN || mode == MODE_ATTACH) {
      if (plugin == null) {
        sourcePath = listenDialog.getSourcePath();
      } else {
        sourcePath = plugin.getSourcePath();
      }
    }
    return sourcePath;
  }
  
  protected void showSource(String sourcePath, FrameInfo frameInfo) {
    String srcName = frameInfo.getSourceFileName().replace('\\', '/');
    showSource(sourcePath, srcName, frameInfo.lineNumber);
  }
  
  protected void showSource(String sourcePath, String srcName, int lineNumber) {
    StringTokenizer st = new StringTokenizer(sourcePath, PATH_SPRT);
    File file = null;
    while (st.hasMoreTokens()) {
      String path = st.nextToken();
      if (isArchiveFile(path)) {
        if (plugin == null && readFromJar(path, srcName, lineNumber)) return;
      } else {
        file = new File(path + '/' + srcName);
        if (file.exists()) {
          hasSource = true;
          if (plugin == null) {
            visualMng.showSource(file, lineNumber);
          } else {
            plugin.showSource(file, lineNumber);
          }
          return;
        }
      }
    }
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.removeFileFilter(0);
    if (plugin == null) fileChooser.addFileFilter("Jar Files, Zip Files (*.jar, *.zip)", "jar,zip");
    fileChooser.addFileFilter("Java Files (*.java)", "java");
    fileChooser.setCheckExisting(false);
    fileChooser.setFileName(srcName);
    String[] theFile = fileChooser.showFileDialog(addsrcTitle + srcName.replace('/', '.'), PFileChooser.LAST_DIR, false, true);
    if (theFile != null) {
      theFile[0] = theFile[0].replace('\\', '/');
      if (isArchiveFile(theFile[0])) {
        if (plugin == null) {
          addToSourcePath(theFile[0]);
          if (readFromJar(theFile[0], srcName, lineNumber)) {
            hasSource = true;
          } else {
            hasSource = false;
            visualMng.showSource(null, 0);
          }
        }
      } else {
        file = new File(theFile[0]);
        hasSource = true;
        if (plugin == null) {
          if (theFile[0].endsWith(srcName)) {
            addToSourcePath(theFile[0].substring(0, theFile[0].length() - srcName.length() - 1));
          }
          visualMng.showSource(file, lineNumber);
        } else {
          plugin.showSource(file, lineNumber);
        }
      }
    }
    return;
  }
  
  private static boolean isArchiveFile(String archive) {
    String lc = archive.toLowerCase();
    if (lc.endsWith(".zip") || lc.endsWith(".jar")) return true;
    return false;
  }
  
  private boolean readFromJar(String jarFile, String srcFile, int lineNumber) {
    ZipFile zip = null;
    try {
      zip = new ZipFile(jarFile);
      ZipEntry zipEntry = zip.getEntry(srcFile);
      if (zipEntry == null) return false;
      InputStream is = zip.getInputStream(zipEntry);
      visualMng.showSource(jarFile + '#' + srcFile, is, lineNumber);
      return true;
    } catch (IOException ioex) {
    } finally {
      try { if (zip != null) zip.close(); } catch (IOException _) { }
    }
    return false;
  }
  
  public void addToSourcePath(String path) {
    if (plugin != null) return;
    if (POptionPane.showConfirmDialog(frame, msgAddSrc + path, titleAddSrc, POptionPane.YES_NO_OPTION) == POptionPane.NO_OPTION) return;
    String sourcePath = getSourcePath();
    if (sourcePath.length() > 0 && sourcePath.charAt(sourcePath.length() - 1) != PATH_SPRT_CHAR)
      sourcePath += PATH_SPRT;
    sourcePath += path;
    if (mode == MODE_LAUNCH) {
      startDialog.setSourcePathOut(sourcePath);
    }
    if (mode == MODE_LISTEN || mode == MODE_ATTACH) {
      listenDialog.setSourcePath(sourcePath);
    }
  }
  
  /** Refreshes tree. */
  public void actionTree() {
    allocSampler.refreshTree();
  }
  
  /** Gets heap dump info. */
  public void actionHeapDump() {
    if (output == null) return;
    try {
      if (plugin == null) visualMng.setEnabled(ActionCodes.ACTION_HEAP_DUMP, false);
      else plugin.profilerStateEvent(ActionCodes.ACTION_HEAP_DUMP, false);
      output.command_DumpHeap();
    } catch (IOException ex) {
      output = null;
      Log.printDebug("ACTION.actionHeapDump", ex);
    }
  }
  
  /** Gets monitors dump. */
  public void actionMonitorDump() {
    if (output == null) return;
    try {
      output.command_DumpMonitors();
    } catch (IOException ex) {
      output = null;
      Log.printDebug("ACTION.actionMonitorDump", ex);
    }
  }
  
  //-------------------------  LOAD, SAVE, CLEAR DATA; FIND, FIND NEXT  ---------------------------
  
  /** Loads previosly saved state from a file. */
  public void actionLoadData() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theFile = fileChooser.showFileDialog(loadTitle, PFileChooser.LAST_DIR, false, true);
    if (theFile != null) {
      if (allocSampler != null) allocSampler.init(ResourceManager.getString("Sampler.root.loading"));
      if (cpuSampler   != null) cpuSampler.init  (ResourceManager.getString("Sampler.root.loading"));
      visualMng.clearConsole();
      visualMng.clearSource();
      if (heapPanel    != null) heapPanel.clear();
      if (vminfoPanel  != null) vminfoPanel.clear();
      if (threadsPanel != null) threadsPanel.clear();
      new LoadThread(theFile[0]).start();
    }
  }
  
  // Loads data in a separate thread
  class LoadThread extends Thread {
    String file;
    
    LoadThread(String theFile) {
      setName("ProfilerLoader");
      setPriority(Thread.NORM_PRIORITY - 1);
      file = theFile;
    }
    
    public void run() {
      try {
        hasData = false;
        hasHeap = false;
        if (plugin == null) {
          setTabDependentMasks();
          visualMng.setDataLoadEnabled(false);
          visualMng.setDataSaveEnabled(false);
        } else plugin.stateLoading(true);
        DataInputStream reader = new DataInputStream(new BufferedInputStream(new FileInputStream(file)));
        namespace = new ProfilerNamespaceDefaultImpl();
        namespace.loadData(reader);
        allocSampler.loadData(reader, namespace);
        cpuSampler.loadData(reader, namespace);
        reader.close();
        allocSampler.showAllocSites();
        cpuSampler.showCpuSamples();
        hasData = true;
        if (plugin == null) {
          setTabDependentMasks();
          visualMng.setDataLoadEnabled(true);
          visualMng.setDataSaveEnabled(true);
        } else {
          plugin.profilerStopped();
          plugin.stateLoading(false);
        }
      } catch (Exception ex) {
        if (plugin == null) {
          visualMng.setEnabled(ActionCodes.ACTION_LOAD_DATA, true);
          POptionPane.showMessageDialog(frame,
              ResourceManager.getString("ProfilerPluginMsg.errorload") + file + '\n'
              + (ex.getMessage() != null ? ex.getMessage() : "unknown error"),
              "Error", POptionPane.ERROR_MESSAGE);
        } else {
          plugin.profilerNotStarted();
          plugin.stateLoading(false);
          plugin.profilerError(ResourceManager.getString("ProfilerPluginTitle.error"),
                             ResourceManager.getString("ProfilerPluginMsg.errorload") + file, -1);
        }
        if (allocSampler != null) allocSampler.init(ResourceManager.getString("Sampler.root.loaderror"));
        if (cpuSampler   != null) cpuSampler.init  (ResourceManager.getString("Sampler.root.loaderror"));
      }
    }
  }
  
  /** Saves current state to a file. */
  public void actionSaveData() {
    if (namespace == null) return;
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theFile = fileChooser.showFileDialog(saveTitle, PFileChooser.LAST_DIR, false);
    if (theFile != null) {
      new SaveThread(theFile[0]).start();
    }
  }
  
  // Saves data in a separate thread
  class SaveThread extends Thread {
    String file;
    
    SaveThread(String theFile) {
      setName("ProfilerSaver");
      setPriority(Thread.NORM_PRIORITY - 1);
      file = theFile;
    }
    
    public void run() {
      try {
        if (plugin == null) {
          visualMng.setDataLoadEnabled(false);
          visualMng.setDataSaveEnabled(false);
        } else plugin.stateSaving(true);
        DataOutputStream writer = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
        if (output != null) output.command_DumpTraces();
        namespace.saveData(writer);
        allocSampler.saveData(writer);
        cpuSampler.saveData(writer);
        writer.close();
        if (plugin == null) {
          visualMng.setDataLoadEnabled(true);
          visualMng.setDataSaveEnabled(true);
        } else {
          plugin.stateSaving(false);
        }
      } catch (Exception ex) {
        if (plugin == null) {
          visualMng.setDataLoadEnabled(true);
          visualMng.setDataSaveEnabled(true);
          POptionPane.showMessageDialog(frame,
              ResourceManager.getString("ProfilerPluginMsg.errorsave") + file + '\n'
              + (ex.getMessage() != null ? ex.getMessage() : "unknown error"),
              "Error", POptionPane.ERROR_MESSAGE);
        } else {
          plugin.stateSaving(false);
          plugin.profilerError(ResourceManager.getString("ProfilerPluginTitle.error"),
                               ResourceManager.getString("ProfilerPluginMsg.errorsave") + file, -2);
        }
      }
    }
  }
  
  /** Saves current state to a text file. */
  public void actionSaveText() {
    if (namespace == null) return;
    if (exportDialog == null) {
      exportDialog = new ExportDialog(frame);
    }
    int stat = exportDialog.showDialog();
    if (stat == ExportDialog.RETURN_OK) {
      try {
        String filePath = exportDialog.getFilePath();
        switch (exportDialog.getMemType()) {
          case 0:
            actionSaveMemData(filePath, allocSampler.getSortColumn(), false);
            break;
          case 1:
            actionSaveMemAllData(filePath, allocSampler.getSortColumn(), false);
            break;
          case 2:
            actionSaveMemDiffData(filePath, allocSampler.getSortColumn(), false);
            break;
        }
        if (exportDialog.shouldExportCpu()) {
          actionSaveCpuData(filePath, cpuSampler.getSortColumn(), exportDialog.getMemType() == 3 ? false : true);
        }
      } catch (IOException ioex) {
      }
    }
  }
  
  /** Saves all mem data. */
  public void actionSaveMemData(String file, int sortCol, boolean append) throws IOException {
    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file, append));
    allocSampler.sorter.showAllocSites(bos, sortCol, AllocSiteInfo.BOTH_INFO);
  }
  
  /** Saves alloc mem data. */
  public void actionSaveMemAllData(String file, int sortCol, boolean append) throws IOException {
    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file, append));
    allocSampler.sorter.showAllocSites(bos, sortCol, AllocSiteInfo.ALLOC_INFO);
  }
  
  /** Saves live mem data. */
  public void actionSaveMemDiffData(String file, int sortCol, boolean append) throws IOException {
    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file, append));
    allocSampler.sorter.showAllocSites(bos, sortCol, AllocSiteInfo.DIFF_INFO);
  }
  
  /** Saves all cpu data. */
  public int actionSaveCpuData(String file, int sortCol, boolean append) throws IOException {
    if (cpuSampler.totalNumberOfSamples == 0) return 0;
    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file, append));
    cpuSampler.sorter.showCpuSamples(bos, sortCol, cpuSampler.totalNumberOfSamples, (int) cpuSampler.endTicks);
    return cpuSampler.totalNumberOfSamples;
  }
  
  /** Clears all data. */
  public void actionClearData() {
    namespace = null;
    initSamplers(NOT_YET_STARTED);
    if (plugin != null) plugin.profilerNotStarted();
    visualMng.clearConsole();
    visualMng.clearSource();
    System.gc();
  }
  
  /** Finds text in tree. */
  public void actionFind() {
    int index = visualMng.getTabIndex();
    if (index == VisualManager.PANEL_SOURCE) {
      initCFindDialog();
      cFindDialog.show();
      return;
    }
    if (index == VisualManager.PANEL_CONSOLE) {
      visualMng.showConsoleFind();
      return;
    }
    initFindDialog();
    PPanel     parent = null;
    PComponent target = null;
    if (index == VisualManager.PANEL_MEM) {
      parent = allocSampler;
      target = allocSampler.getFindComponent();
    }
    if (index == VisualManager.PANEL_CPU) {
      parent = cpuSampler;
      target = cpuSampler.getFindComponent();
    }
    if (index == VisualManager.PANEL_HEAP) {
      parent = heapPanel;
      target = heapPanel.getFindComponent();
    }
    if (target != null) {
      findDialog.showDialog(findTitle, parent, target);
    }
  }
  
  /** Finds next text in tree. */
  public void actionFindNext() {
    int index = visualMng.getTabIndex();
    if (index != VisualManager.PANEL_SOURCE) {
      findDialog.findNext();
    }
  }
  
  private void initCFindDialog() {
    if (cFindDialog == null) {
      cFindDialog = CFindDialog.getInstance(frame, ResourceManager.getString("FindDialog.title"), false);
      cFindDialog.setGrabSelectedText(true);    
      cFindDialog.setSearchable(visualMng.getSourcePanel());
    }
  }
  
  private void initFindDialog() {
    if (findDialog == null) {
      findDialog = new FindDialog(this, frame, false);
    }
  }
  
  /** Sets find next button state. */
  public void setFindNext(boolean state) {
    setTabDependentMasks();
  }
  
  /** Shows preferences dialog. */
  public void actionPrefs() {
    if (prefsDialog == null) {
      prefsDialog = new PreferencesDialog(frame);
    }
    int stat = prefsDialog.showDialog();
    if (stat == PreferencesDialog.RETURN_OK) {
      setPreferences();
    }
  }
  
  public void setPreferences() {
    if (allocSampler != null) {
      allocSampler.maxLines = PreferencesDialog.getTopLines();
      allocSampler.setMemUnits(PreferencesDialog.getMemUnits());
      allocSampler.setShowZeroInsts(PreferencesDialog.isShowZeroInsts());
      allocSampler.setViewColumns(PreferencesDialog.getMemColumns());
      allocSampler.refreshTable();
    }
    
    if (cpuSampler != null) {
      cpuSampler.setTimePrecision(PreferencesDialog.getTimePrc());
      cpuSampler.setPrecision(PreferencesDialog.getSamplerPrecision());
      cpuSampler.setViewColumns(PreferencesDialog.getCpuColumns());
      cpuSampler.refreshTable();
    }
  }
  
  public void actionEclipse() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theDir = fileChooser.showDirDialog(eclipseTitle, PFileChooser.LAST_DIR, false, true);
    if (theDir != null) {
      try {
        File pDir = new File(theDir[0], "plugins");
        if (!(new File(theDir[0], ".eclipseproduct").exists()) || !pDir.exists() || !pDir.isDirectory()) {
          POptionPane.showMessageDialog(frame,
              ResourceManager.getString("ProfilerEclipseError.homedir"),
              "Error", POptionPane.ERROR_MESSAGE);
          return;
        }
        File pluginDir = new File(pDir, "com.prosyst.eclipse.profiler");
        String zipName = "plugins" + File.separatorChar + "testenv" + File.separatorChar + "jprofiler-eclipse.zip";
        extractZipFile(zipName, pluginDir);
        File optFile = new File(pluginDir, "options.ini");
        IniFile optIni = new IniFile();
        optIni.load(optFile);
        optIni.setValue("OPTIONS", "ProfilerInstallDir", System.getProperty("user.dir"));
        optIni.save(optFile);
      } catch (Exception ex) {
        Log.printDebug("ACTION.actionEclipseIntegrate", ex);
      }
    }
  }

  private static final int BUFFER = 2048;
  
  public void extractZipFile(String zipName, File destDir) throws IOException {
    byte data[] = new byte[BUFFER];
    destDir.mkdir();
    FileInputStream fis = new FileInputStream(zipName);
    ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
    ZipEntry entry;
    int count;
    while ((entry = zis.getNextEntry()) != null) {
      if (entry.isDirectory()) {
        new File(destDir, entry.getName()).mkdir();
      } else {
        FileOutputStream fos = new FileOutputStream(new File(destDir, entry.getName()));
        BufferedOutputStream dest = new BufferedOutputStream(fos, BUFFER);
        while ((count = zis.read(data, 0, BUFFER)) != -1) {
          dest.write(data, 0, count);
        }
        dest.flush();
        dest.close();
      }
    }
    zis.close();
  }
  
  /** Exits profiler. */
  public void actionExit() {
    if (plugin == null) if (startDialog != null && startDialog.shouldSave()) return;
    if (process != null || !isNotRunning) {
      PComponent.setSmokeEnabled(true);
      frame.setSmoked(true);
      if (POptionPane.showConfirmDialog(frame, msgExit, titleExit, POptionPane.YES_NO_OPTION) == POptionPane.NO_OPTION) {
        frame.setSmoked(false);
        frame.repaint();
        return;
      }
    }
    actionStop();
    if (system_exit) {
      HelpManager.closeHelp();
      visualMng.saveFramePos();
      System.exit(0);
    }
  }
  
  //
  //  PROFILER LISTENER  --------------------------------------------------------------------------
  //
  
  private static final boolean dump_prof_events = Boolean.getBoolean("dump.prof.events");
  
  /** Event meaning that the profiler has started. */
  public void profilerStarted() {
    if (dump_prof_events) Log.printDebug("Profiler started"); // dump !!
    setStatus();
    initSamplers(CALCULATING);
    if (memDisabled && !cpuDisabled && visualMng.getTabIndex() == VisualManager.PANEL_MEM)
      visualMng.setTabIndex(VisualManager.PANEL_CPU);
    isProfiling = true;
    hasData = true;
    if (plugin == null) {
      visualMng.setGroupEnabled(true);
      visualMng.setDataSaveEnabled(true);
      setTabDependentMasks();
    } else {
      plugin.profilerStarted();
    }
    output = server.getOutput();
    namespace = server.getNamespace();
    try {
      isMemStarted  = false;
      hasMemStarted = false;
      isCpuStarted  = autoCpu;
      hasCpuStarted = autoCpu;
      if (mode == MODE_LAUNCH) {
        output.command_SetMaxStackDepth(optPanel.getDepth());
      } else {
        output.command_SetMaxStackDepth(listenDialog.getDepth());
      }
      if (triggerPanel != null) {
        Vector triggers = triggerPanel.getTriggers();
        TriggerEvent te;
        for (int i = 0; i < triggers.size(); i++) {
          te = (TriggerEvent) triggers.elementAt(i);
          if (te.enabled) output.command_SetTrigger(te.type, te.action, te.className, te.methodName);
        }
      }
      if (cpuDisabled) output.command_SetCpuSamplingAll(false);
      if (autoCpu && isSupported) output.command_DumpOnExit();
      if (memDisabled && vmDisabled) output.command_SetAllocTraces(false);
      else {
        output.command_SetAllocTraces(true);
        vmSampler = new VMSampler();
        vmSampler.start();
      }
    } catch (Exception _) { }
  }
  
  /** Event with profiler support capability. */
  public void profilerSupport(boolean isSupported, boolean heapSupport) {
    this.isSupported = isSupported;
    if (!isSupported) {
      vmDisabled = true;
      threadsDisabled = true;
    } else {
//      if (ver == 2 && (OS_NAME.startsWith("SunOS") || OS_NAME.startsWith("Solaris"))) {
//        if (plugin == null) {
//          visualMng.setPauseEnabled(false);
//        } else {
//          plugin.profilerStateEvent(ActionCodes.ACTION_PAUSE, false);
//        }
//      }
      if (!threadsDisabled) try { output.command_MonitorEvents(true); } catch (Exception _) { }
    }
    vminfoPanel.setDisabled(vmDisabled);
    threadsPanel.setDisabled(threadsDisabled);
    if (plugin == null) {
      visualMng.setSupportedCommands(isSupported, heapSupport);
    } else {
      plugin.profilerSupport(isSupported, heapSupport);
    }
  }
  
  /** Event meaning that the profiler has stopped. */
  public void profilerStopped() {
    if (processDestroyer != null) {
      processDestroyer.interrupt();
      processDestroyer = null;
    }
    if (vmSampler != null) {
      vmSampler.interrupt();
      vmSampler = null;
    }
    if (memSampler != null) {
      memSampler.interrupt();
      memSampler = null;
    }
    isProfiling = false;
    if (plugin == null) {
      visualMng.setGroupEnabled(false);
    } else {
      plugin.profilerStopping();
    }
    isNotRunning = true;
    isPaused = false;
    output = null;
    server.clear();
    if (!memDisabled) allocSampler.showAllocSites();
    if (!cpuDisabled) {
      if (cpuSampler.getRootName().equals(ResourceManager.getString("Sampler.root.calculating.cpu"))) {
        cpuSampler.init(ResourceManager.getString("Sampler.root.no_data"));
      }
    }
    if (!threadsDisabled) threadsPanel.stop();
    if (plugin == null) {
      visualMng.setStartingEnabled(true);
      visualMng.setEnabled(ActionCodes.ACTION_CLEAR_DATA, true);
      visualMng.changePlayDesc(ResourceManager.getString("MenuBar.item.play"));
      if (startDialog  != null) startDialog.startEnabled(true);
      if (listenDialog != null) listenDialog.startEnabled(true);
    } else {
      plugin.profilerStopped();
      if (listenDialog != null) listenDialog.startEnabled(true);
    }
    if (mode == MODE_LISTEN || mode == MODE_ATTACH) {
      visualMng.addToConsole(ResourceManager.getString("Console.msg.stopped") + "\n");
    }
    setStatus();
    if (dump_prof_events) Log.printDebug("Profiler stopped"); // dump !!
    if (process != null && ver == Parser.VER_JEODE) try {
      process.exitValue();
    } catch (IllegalThreadStateException _) {
      if (dump_prof_events) Log.printDebug("Process destroyed"); // dump !!
      process.destroy();
    }
    process = null;
  }
  
  /** Event meaning that profiler error has occurred. */
  public void profilerError() {
    if (process != null) {
      process.destroy();
    }
    if (server  != null) {
      server.shutDown();
    }
  }
  
  public void profilerErrorMsg(Throwable ex) {
    errorMsgThread = new ErrorMsgThread(ex);
    errorMsgThread.start();
  }
  
  private ErrorMsgThread errorMsgThread;
  
  // Shows errors in separate thread
  class ErrorMsgThread extends Thread {
    ErrorMsgThread(Throwable ex) {
      setName("ProfilerErrorMessenger");
      setDaemon(true);
      this.ex = ex;
    }
    
    protected Throwable ex = null;
    
    public void run() {
      try {
        POptionPane.showMessageDialog(frame,
          (ex.getMessage() != null
          ? ResourceManager.getString("ProfilerInputError.msg") + ex.getMessage()
          : ResourceManager.getString("ProfilerInputErrorUnknown.msg")),
          ResourceManager.getString("ProfilerInputError.title"),
          POptionPane.ERROR_MESSAGE);
      } catch (Throwable _) { }
    }
  }
  
  private boolean allocSitesArrived = false;
  
  public void waitForAllocSites () {
    allocSitesArrived = false;
    try {
      for (int i = 0; i < 100 && !allocSitesArrived; i++) {
        Thread.sleep(50);
      }
    } catch (InterruptedException _) { }
  }
  
  /**
   * Event meaning that alloc sites info has arrived.
   *
   * @param   allocSites  array of info containing allocated bytes and insts
   */
  public void eventAllocSites(AllocSiteInfo[] allocSites) {
    if (dump_prof_events) Log.printDebug("Event alloc sites " + allocSites.length); // dump !!
    if (memDisabled) return;
    if (allocSampler == null) allocSampler = new AllocSamplerGuiImpl(this);
    allocSampler.eventAllocSites(allocSites);
    if (memSampler == null) {
      memSampler = new MemSampler();
      memSampler.start();
    } else {
      memSampler.loop = false;
    }
    allocSitesArrived = true;
  }
  
  private MemSampler memSampler;
  
  // Sends commands to the server in a separate thread
  class MemSampler extends Thread {
    MemSampler() {
      setName("ProfilerMemSampler");
      setDaemon(true);
    }
    
    protected boolean loop = false;
    
    public void run() {
      try {
        while (true) {
          sleep(MEM_SAMPLER_INTERVAL);
          output.command_AllocSites(3, 0);
          while (loop) sleep(50);
          loop = true;
        }
      } catch (Throwable _) { }
    }
  }
  
  private boolean cpuSamplesArrived = false;
  
  public void waitForCpuSamples() {
    cpuSamplesArrived = false;
    try {
      for (int i = 0; i < 100 && !cpuSamplesArrived; i++) {
        Thread.sleep(50);
      }
    } catch (InterruptedException _) { }
  }
  
  /**
   * Event meaning that cpu samples info has arrived.
   *
   * @param   ticks                 the time in ms of the event
   * @param   totalNumberOfSamples  total number of samples after the preceding cpu samples event
   * @param   cpuSamples            array of info containing cost and trace
   */
  public void eventCpuSamples(int ticks, int totalNumberOfSamples, CpuSampleInfo[] cpuSamples) {
    if (dump_prof_events) Log.printDebug("Event cpu samples " + cpuSamples.length); // dump !!
    if (cpuSampler == null) cpuSampler = new CpuSamplerGuiImpl(this);
    cpuSampler.eventCpuSamples(ticks, totalNumberOfSamples, cpuSamples);
    cpuSampler.showCpuSamples();
    cpuSamplesArrived = true;
  }
  
  /**
   * Event meaning that heap dump info has arrived.
   *
   * @param   namespace            namespace with all heap classes and objects
   */
  public void eventHeapDump(ProfilerNamespace namespace) {
    heapPanel.showInfo(namespace);
    if (visualMng.getTabIndex() == VisualManager.PANEL_MEM) heapPanel.setSelectedClass();
    visualMng.setTabIndex(VisualManager.PANEL_HEAP);
    hasHeap = true;
    if (plugin == null) {
      visualMng.setEnabled(ActionCodes.ACTION_HEAP_DUMP, true);
      setTabDependentMasks();
    } else plugin.profilerStateEvent(ActionCodes.ACTION_HEAP_DUMP, true);
  }
  
  /**
   * Event meaning that graph info has arrived.
   *
   * @param   graphInfo  info containing live and alloc bytes and instances, thread, class and gc info
   */
  public void eventGraphInfo(GraphInfo graphInfo) {
    long[] p = { graphInfo.totalLiveBytes,      graphInfo.totalLiveInstances,
                 graphInfo.totalBytesAllocated, graphInfo.totalInstancesAllocated,
                 graphInfo.numOfThreads,        graphInfo.numOfActiveThreads,
                 graphInfo.numOfClasses,        graphInfo.GCActivity };
    vminfoPanel.addPoint(new VMPoint(p));
  }
  
  /**
   * Event meaning that monitors dump has arrived.
   *
   * @param   monitorsDump  info containing stack traces of live threads and acquired monitors
   */
  public void eventMonitorsDump(MonitorsDump monitorsDump) {
    StringBuffer sb = new StringBuffer();
    Vector threads = new Vector();
    
    sb.append('\n');
    TraceInfo traceInfo;
    for (int i = 0; i < monitorsDump.traces.length; i++) {
      traceInfo = monitorsDump.traces[i];
      if (traceInfo != null) {
        sb.append(traceInfo.getThreadName());
        sb.append(", state: ");
        sb.append(Misc.toStringThreadStatus(monitorsDump.statuses[i]));
        sb.append('\n');
        for (int j = 0; j < traceInfo.numberOfFrames; j++) {
          sb.append("  at ");
          sb.append(traceInfo.frameInfo[j].toStringWithSource());
          sb.append('\n');
        }
        if (traceInfo.threadInfo != null) threads.addElement(traceInfo.threadInfo);
      }
    }
    sb.append('\n');
    
    for (int i = 0; i < monitorsDump.monitors.length; i++) {
      sb.append(monitorsDump.monitors[i].toString());
      sb.append('\n');
    }
    sb.append('\n');
    
    // workaround for 1.4, because no threads statuses are sent
    if (threads.size() == 0) {
      Object[] threadsArray = namespace.getThreadTable().elements();
      for (int i = threadsArray.length - 1; i >= 0; i--) {
        if (threadsArray[i] != null) threads.addElement(threadsArray[i]);
      }
    }
    
    for (;;) {
      boolean deadlock = true;
      for (int i = 0; i < threads.size(); i++) {
        boolean isWaiting = false;
        for (int j = 0; j < monitorsDump.monitors.length; j++) {
          if (threads.contains(monitorsDump.monitors[j].t_ownerInfo)) {
            for (int k = 0; k < monitorsDump.monitors[j].t_enterInfo.length; k++) {
              if (monitorsDump.monitors[j].t_enterInfo[k] == threads.elementAt(i)) {
                isWaiting = true;
              }
            }
          }
        }
        if (!isWaiting) {
          threads.removeElementAt(i);
          deadlock = false;
        }
      }
      if (deadlock) break;
    }
    if (threads.size() != 0) {
      sb.append(ResourceManager.getString("Console.msg.deadlock"));
      for (int i = 0; i < threads.size(); i++) {
        sb.append("  ");
        sb.append(threads.elementAt(i).toString());
        sb.append('\n');
        for (int j = 0; j < monitorsDump.monitors.length; j++) {
          if (threads.contains(monitorsDump.monitors[j].t_ownerInfo)) {
            for (int k = 0; k < monitorsDump.monitors[j].t_enterInfo.length; k++) {
              if (monitorsDump.monitors[j].t_enterInfo[k] == threads.elementAt(i)) {
                sb.append("    is waiting for  ");
                sb.append(monitorsDump.monitors[j].toStringMonitorName());
                sb.append("  owned by  ");
                sb.append(monitorsDump.monitors[j].t_ownerInfo.toString());
                sb.append('\n');
              }
            }
          }
        }
      }
      sb.append('\n');
    }
    //sb.append(ResourceManager.getString("Console.msg.no_deadlock"));
    sb.append('\n');
    
    visualMng.addToConsole(sb.toString());
    visualMng.setTabIndex(VisualManager.PANEL_CONSOLE);
  }
  
  /**
   * Common event notification.
   *
   * @param   event  object containing event information
   */
  public void eventCommon(Object event) {
    if (event instanceof TriggerEvent) {
      if (triggerPanel == null) return;
      TriggerEvent ev = (TriggerEvent) event;
      if (dump_prof_events) Log.printDebug(ev.toString()); // dump !!
      Vector triggers = triggerPanel.getTriggers();
      TriggerEvent te;
      for (int i = 0; i < triggers.size(); i++) {
        te = (TriggerEvent) triggers.elementAt(i);
        if (te.enabled && te.equals(ev)) switch (te.action) {
          case TriggerEvent.PROF_TRGACT_BEGIN:
            actionStartMark(false);
            break;
          case TriggerEvent.PROF_TRGACT_CONT:
            actionContinueMark(false);
            break;
          case TriggerEvent.PROF_TRGACT_END:
            actionStopMark();
            break;
          case TriggerEvent.PROF_TRGACT_MEM_BEGIN:
            actionStartMemMark(false);
            break;
          case TriggerEvent.PROF_TRGACT_MEM_CONT:
            actionContinueMemMark(false);
            break;
          case TriggerEvent.PROF_TRGACT_MEM_END:
            actionStopMemMark();
            break;
          case TriggerEvent.PROF_TRGACT_CPU_BEGIN:
            actionStartCpuMark();
            break;
          case TriggerEvent.PROF_TRGACT_CPU_CONT:
            actionContinueCpuMark();
            break;
          case TriggerEvent.PROF_TRGACT_CPU_END:
            actionStopCpuMark();
            break;
          case TriggerEvent.PROF_TRGACT_EXPORT:
            break;
          case TriggerEvent.PROF_TRGACT_PAUSE:
            actionPause();
            break;
          case TriggerEvent.PROF_TRGACT_STOP:
            actionStop();
            break;
        }
      }
    }
    if (event instanceof ThreadInfo) {
      if (!threadsDisabled) threadsPanel.addThread((ThreadInfo) event);
    }
    if (event instanceof MonitorInfo) {
      if (!threadsDisabled) threadsPanel.addEvent((MonitorInfo) event);
    }
  }
  
  private void setStatus() {
    if (plugin != null) return;
    String status = (mode == MODE_LAUNCH ? " - [" + startDialog.getProfileName() + "]" : "");
    if (!isNotRunning) {
      if (isListening) status = " LISTENING";
      else {
        if (isPaused) status += " PAUSED";
        else          status += " RUNNING";
      }
    }
    visualMng.setTitle(status);
  }
  
  private void initSamplers(int state) {
    if (allocSampler == null) allocSampler = new AllocSamplerGuiImpl(this);
    if (cpuSampler   == null) cpuSampler   = new CpuSamplerGuiImpl(this);
    if (heapPanel    == null) heapPanel    = new HeapDumpPanel(this);
    if (vminfoPanel  == null) vminfoPanel  = new VMInfoPanel();
    if (threadsPanel == null) threadsPanel = new ThreadsPanel(this);
    
    if (state == REMOTE)  allocSampler.init(ResourceManager.getString("Sampler.root.remote_mon"));
    else if (memDisabled) allocSampler.init(ResourceManager.getString("Sampler.root.disabled"));
    else {
      if (state == NOT_YET_STARTED) allocSampler.init(ResourceManager.getString("Sampler.root.not_yet_started"));
      if (state == CALCULATING)     allocSampler.init(ResourceManager.getString("Sampler.root.calculating.alloc"));
    }
    if (state == REMOTE)  cpuSampler.init(ResourceManager.getString("Sampler.root.remote_mon"));
    else if (cpuDisabled) cpuSampler.init(ResourceManager.getString("Sampler.root.disabled"));
    else if (!autoCpu)    cpuSampler.init(ResourceManager.getString("Sampler.root.not_yet_started"));
    else {
      if (state == NOT_YET_STARTED) cpuSampler.init(ResourceManager.getString("Sampler.root.not_yet_started"));
      if (state == CALCULATING)     cpuSampler.init(ResourceManager.getString("Sampler.root.calculating.cpu"));
    }
    hasData = false;
    hasHeap = false;
    setTabDependentMasks();
    if (plugin == null) {
      visualMng.setEnabled(ActionCodes.ACTION_CLEAR_DATA, false);
      visualMng.setDataSaveEnabled(false);
    }
    heapPanel.clear();
    vminfoPanel.clear();
    vminfoPanel.setDisabled(vmDisabled);
    threadsPanel.clear();
    threadsPanel.setDisabled(threadsDisabled);
  }
  
  /**
   * Gets the parent frame of the application.
   *
   * @return     the parent frame
   */
  public PFrame getFrame() {
    return frame;
  }
  
  //  ChangeListener  -----------------------------------------------------------------------------
  
  public void stateChanged(ChangeEvent e) {
    setTabDependentMasks();
  }
  
  public void setTabDependentMasks() {
    int index = visualMng.getTabIndex();
    boolean findState = getFindState();
    if (plugin == null) {
      visualMng.setEnabled(ActionCodes.ACTION_FIND,      findState);
      visualMng.setEnabled(ActionCodes.ACTION_FIND_NEXT, findState && isFindNext());
      visualMng.setEnabled(ActionCodes.ACTION_SOURCE,    findState && index <= VisualManager.PANEL_HEAP);
      visualMng.setEnabled(ActionCodes.ACTION_TREE,      isProfiling && index == VisualManager.PANEL_MEM);
    } else {
      plugin.profilerStateEvent(ActionCodes.ACTION_FIND,      findState);
      plugin.profilerStateEvent(ActionCodes.ACTION_FIND_NEXT, findState && isFindNext());
      plugin.profilerStateEvent(ActionCodes.ACTION_SOURCE,    findState && index <= VisualManager.PANEL_HEAP);
      plugin.profilerStateEvent(ActionCodes.ACTION_TREE,      isProfiling && index == VisualManager.PANEL_MEM);
    }
  }
  
  public boolean getFindState() {
    int index = visualMng.getTabIndex();
    switch (index) {
      case VisualManager.PANEL_MEM:        return !memDisabled && hasData;
      case VisualManager.PANEL_CPU:        return !cpuDisabled && hasData;
      case VisualManager.PANEL_HEAP:       return hasHeap;
      case VisualManager.PANEL_SOURCE:     return hasSource;
      case VisualManager.PANEL_CONSOLE:    return hasData;
      default:                             return false;
    }
  }
  
  //  WindowListener  -----------------------------------------------------------------------------
  
  /**
   * Invoked when a window is closing.
   * The close operation can be intercepted here.
   */
  public void windowClosing(WindowEvent e) {
    actionExit();
  }
  
  //  LookAndFeelListener  ------------------------------------------------------------------------
  
  /**
   * Updates the visual components after the L&F has been changed.
   */
  public void lookAndFeelChanged(LookAndFeelEvent lfe) {
    if (lfe.getLookAndFeel() != null) {
      if (frame        != null) PGUIUtilities.updateComponentTreeUI(frame);
      if (startDialog  != null) PGUIUtilities.updateComponentTreeUI(startDialog);
      if (listenDialog != null) PGUIUtilities.updateComponentTreeUI(listenDialog);
      if (prefsDialog  != null) PGUIUtilities.updateComponentTreeUI(prefsDialog);
      if (findDialog   != null) PGUIUtilities.updateComponentTreeUI(findDialog);
      if (exportDialog != null) PGUIUtilities.updateComponentTreeUI(findDialog);
    }
  }
}