/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;
import java.util.Vector;
import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.table.PTableHeader;
import com.prosyst.pgui.table.TableColumn;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.tree.TreeNode;
import com.prosyst.pgui.tree.TreeNodeImpl;
import com.prosyst.pgui.tree.TreeModelImpl;
import com.prosyst.pgui.event.TreeExpansionListener;
import com.prosyst.pgui.event.TreeShouldExpandListener;
import com.prosyst.pgui.event.TreeExpansionEvent;

import com.prosyst.jpd.profiler.storage.ProfilerNamespace;
import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.AllocSiteGroup;
import com.prosyst.jpd.profiler.info.AllocSiteSorter;
import com.prosyst.jpd.profiler.info.AllocSiteData;
import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;
import com.prosyst.jpd.profiler.info.Misc;

import com.prosyst.jpd.profiler.gui.dialogs.PreferencesDialog;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

public class AllocSamplerGuiImpl extends PPanel implements ActionListener, KeyListener,
                                                           MouseListener, FocusListener,
                                                           TreeExpansionListener, TreeShouldExpandListener {
  
  private static final String[] columnNames = {
          ResourceManager.getString("Sampler.memcol.classname"),
          ResourceManager.getString("Sampler.memcol.bytes"),
          ResourceManager.getString("Sampler.memcol.insts"),
          ResourceManager.getString("Sampler.memcol.diff_bytes"),
          ResourceManager.getString("Sampler.memcol.diff_insts"),
          ResourceManager.getString("Sampler.memcol.thread"),
          ResourceManager.getString("Sampler.memcol.method") };
  private static final int[]    columnSizes = { 180, 70, 70, 70, 70, 100, 450 };
  
  private static final String INI_SMP_GROUPS = "sampler.show_groups";
  
  protected ActionManager          actionMng;
  protected AllocSiteSorter        sorter     = new AllocSiteSorter();
  protected PSplitPane             splitpane  = new PSplitPane(PSplitPane.VERTICAL_SPLIT);
  
  protected TreeNodeImpl           root       = new TreeNodeImpl(ResourceManager.getString(
                                                "Sampler.root.not_yet_started"));
  protected TreeModelImpl          treeModel  = new TreeModelImpl(root);
  protected PTree                  tree       = new PTree(treeModel);
  protected SamplerTreeRenderer    treeRndr   = new SamplerTreeRenderer();
  
  protected SamplerTableModel      tableModel = new SamplerTableModel(columnNames);
  protected PTable                 table      = new PTable(tableModel);
  protected SamplerTableRenderer   tableRndr  = new SamplerTableRenderer();
  
  protected PScrollPane            treeScrollpane  = new PScrollPane(tree);
  protected PScrollPane            tableScrollpane = new PScrollPane(table);
  protected PTextField             ctextFilter, mtextFilter;
  protected PCheckBox              grpCheckBox, gcCheckBox;
  
  private boolean  showGroups = true;
  private boolean  isFinal    = false;
  private int      rowHeight;
  private String   cfilters, mfilters;
  
  private int       columnCount;
  private boolean[] columnViews;
  private int[]     columnIndices = { 0, 1, 2, 3, 4, 5, 6 };
  private int       lastRow = -1;
  
  public  int       memUnits;
  public  int       maxLines = 40;
  public  boolean   showAll  = false;
  public  boolean   isDownStack = false;
  
  public AllocSamplerGuiImpl(ActionManager actionMng) {
    super(new BorderLayout());
    this.actionMng = actionMng;
    
    tree.setCellRenderer(treeRndr);
    tree.setEditable(false);
    tree.setAutoExpand(false);
    tree.addMouseListener(this);
    tree.addTreeExpansionListener(this);
    tree.addTreeShouldExpandListener(this); // dynamic !!
    tree.addKeyListener(this);
    treeScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    table.setResizeMode(PTable.RESIZE_OFF);
    table.setRowSelectionAllowed(true);
    table.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    table.addMouseListener(this);
    table.addKeyListener(this);
    tableScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    maxLines = PreferencesDialog.getTopLines();
    setMemUnits(PreferencesDialog.getMemUnits());
    setShowZeroInsts(PreferencesDialog.isShowZeroInsts());
    setViewColumns(PreferencesDialog.getMemColumns());
    
    PTableHeader th = table.getHeader();
    th.addMouseListener(this);
    rowHeight = table.getRowHeight();
    
    ctextFilter = new PTextField(12);
    ctextFilter.addKeyListener(this);
    ctextFilter.addFocusListener(this);
    
    mtextFilter = new PTextField(20);
    mtextFilter.addKeyListener(this);
    mtextFilter.addFocusListener(this);
    
    gcCheckBox = new PCheckBox(ResourceManager.getString("Sampler.check.disable_gc"));
    gcCheckBox.setSelected(sorter.isGCDisabled());
    gcCheckBox.addActionListener(this);
    
    grpCheckBox = new PCheckBox(ResourceManager.getString("Sampler.check.showgroups"));
    grpCheckBox.setSelected(ResourceManager.getPersistentBoolean(PreferencesDialog.PROF_SECTION, INI_SMP_GROUPS, false));
    grpCheckBox.addActionListener(this);
    setShowGroups(grpCheckBox.isSelected());
    
    PPanel clsPnl = new PPanel(new BorderLayout());
    clsPnl.add(new PLabel(ResourceManager.getString("Sampler.label.classfilter")), BorderLayout.WEST);
    clsPnl.add(ctextFilter,                                                        BorderLayout.CENTER);
    
    PPanel mtdPnl = new PPanel(new BorderLayout());
    mtdPnl.add(new PLabel(" " + ResourceManager.getString("Sampler.label.methodfilter")), BorderLayout.WEST);
    mtdPnl.add(mtextFilter,                                                               BorderLayout.CENTER);
    
    PPanel fltPnl = new PPanel(new BorderLayout());
    fltPnl.add(clsPnl, BorderLayout.WEST);
    fltPnl.add(mtdPnl, BorderLayout.CENTER);
    
    PPanel chkPnl = new PPanel(new BorderLayout());
    chkPnl.add(grpCheckBox, BorderLayout.WEST);
    chkPnl.add(gcCheckBox,  BorderLayout.EAST);
    
    PPanel btmPnl = new PPanel(new BorderLayout());
    btmPnl.add(fltPnl, BorderLayout.CENTER);
    btmPnl.add(chkPnl, BorderLayout.EAST);
    
    PPanel panel  = new PPanel(new BorderLayout());
    panel.add(tableScrollpane, BorderLayout.CENTER);
    panel.add(btmPnl,          BorderLayout.SOUTH);
    
    splitpane.setTop   (treeScrollpane);
    splitpane.setBottom(panel);
    splitpane.setDividerPosition(200);
    this.add(splitpane);
    ToolTipManager.sharedInstance().registerComponent(table);
  }
  
  public void init(String rootname) {
    isFinal = false;
    showAll = false;
    sorter.clear();
    root = new TreeNodeImpl(rootname);
    treeModel.setRoot(root);
    refreshTable();
  }
  
  public void eventAllocSites(AllocSiteInfo[] allocSites) {
    if (allocSites.length == 0) return;
    sorter.addAllocSites(allocSites);
    refreshTableSelect();
  }
  
  public void showAllocSites() {
    isFinal = true;
    root.setNodeData(ResourceManager.getString("Sampler.root.final"));
    refreshTree();
    refreshTableSelect();
    root.setNodeData(ResourceManager.getString("Sampler.root.system"));
    treeModel.nodeChanged(root);
  }
  
  public void showAllocSitesAll() {
    showAll = true;
    refreshTree();
    refreshTableSelect();
    treeModel.nodeChanged(root);
  }
  
  public void startMark() {
    sorter.startMark();
    refreshTable();
  }
  
  public void continueMark() {
    sorter.continueMark();
  }
  
  public void stopMark() {
    sorter.stopMark();
  }
  
  public void setMemUnits(int memUnits) {
    if (treeRndr.memUnits != memUnits) {
      this.memUnits      = memUnits;
      treeRndr.memUnits  = memUnits;
      tableRndr.memUnits = memUnits;
      treeModel.reload();
    }
  }
  
  public void setShowZeroInsts(boolean state) {
    sorter.setShowZeroInsts(state);
  }
  
  public PComponent getFindComponent() {
    if (table.hasFocus()) return table;
    return tree;
  }
  
  public AllocSiteSorter getSorter() {
    return sorter;
  }
  
  public boolean isShowGroups() {
    return showGroups;
  }
  
  public boolean isThreadVisible() {
    return columnViews[AllocSiteSorter.THREAD_COLUMN];
  }
  
  public boolean isFrameVisible() {
    return columnViews[AllocSiteSorter.FRAME_COLUMN];
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent ae) {
    Object src = ae.getSource();
    if (src == gcCheckBox) {
      sorter.setGCDisabled(gcCheckBox.isSelected());
      refreshTable();
    }
    if (src == grpCheckBox) {
      setShowGroups(grpCheckBox.isSelected());
      Object site = null;
      int row = table.getSelectedRow();
      if (row != -1) site = table.getCellValue(row, 0);
      refreshTable();
      if (showGroups && site != null && site instanceof AllocSiteInfo) {
        AllocSiteGroup asg = sorter.getAllocGroup((AllocSiteInfo) site);
        for (int i = 0; i < tableModel.getRowCount(); i++) {
          if (tableModel.getCellValue(i, 0) == asg) {
            openTableNode(asg, i);
            break;
          }
        }
      }
      selectRow(site);
    }
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == tree || src == treeScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_ENTER) moveInTable();
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.MemTree");
    }
    if (src == table || src == tableScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_ENTER) tableAction(0);
      if (keyCode == KeyEvent.VK_BACK_SPACE) tableUpDown();
      if (keyCode == KeyEvent.VK_UP      || keyCode == KeyEvent.VK_DOWN
       || keyCode == KeyEvent.VK_PAGE_UP || keyCode == KeyEvent.VK_PAGE_DOWN) lastRow = -1;
      if ((keyCode == KeyEvent.VK_HOME   || keyCode == KeyEvent.VK_END) && ke.isControlDown()) lastRow = -1;
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.MemTable");
    }
    if (src == ctextFilter) {
      if (keyCode == KeyEvent.VK_ENTER) {
        cfilters = ctextFilter.getText();
        StringTokenizer st = new StringTokenizer(cfilters, ",");
        String[] filtersIA = new String[st.countTokens()];
        String[] filtersEA = new String[st.countTokens()];
        int fi = 0, fe = 0;
        while (st.hasMoreTokens()) {
          String filter = st.nextToken().trim();
          if (filter.length() == 0) continue;
          if (filter.charAt(0) == '!') filtersEA[fe++] = filter.substring(1);
          else                         filtersIA[fi++] = filter;
        }
        String[] filtersI = new String[fi]; System.arraycopy(filtersIA, 0, filtersI, 0, fi);
        String[] filtersE = new String[fe]; System.arraycopy(filtersEA, 0, filtersE, 0, fe);
        sorter.setClassFilters((filtersI.length == 0 ? null : filtersI), (filtersE.length == 0 ? null : filtersE));
        /* if (isFinal) */ refreshTable();
      }
      if (keyCode == KeyEvent.VK_ESCAPE) {
        ctextFilter.setText(cfilters);
      }
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.MemFilters");
    }
    if (src == mtextFilter) {
      if (keyCode == KeyEvent.VK_ENTER) {
        mfilters = mtextFilter.getText();
        StringTokenizer st = new StringTokenizer(mfilters, ",");
        String[] filtersIA = new String[st.countTokens()];
        String[] filtersEA = new String[st.countTokens()];
        int fi = 0, fe = 0;
        while (st.hasMoreTokens()) {
          String filter = st.nextToken().trim();
          if (filter.length() == 0) continue;
          if (filter.charAt(0) == '!') filtersEA[fe++] = filter.substring(1);
          else                         filtersIA[fi++] = filter;
        }
        String[] filtersI = new String[fi]; System.arraycopy(filtersIA, 0, filtersI, 0, fi);
        String[] filtersE = new String[fe]; System.arraycopy(filtersEA, 0, filtersE, 0, fe);
        sorter.setMethodFilters((filtersI.length == 0 ? null : filtersI), (filtersE.length == 0 ? null : filtersE));
        /* if (isFinal) */ refreshTable();
      }
      if (keyCode == KeyEvent.VK_ESCAPE) {
        mtextFilter.setText(mfilters);
      }
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.MemFilters");
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
  
  //  FocusListener
  
  public void focusGained(FocusEvent fe) { }
  
  public void focusLost(FocusEvent fe) {
    if (fe.getComponent() == ctextFilter) ctextFilter.setText(cfilters);
    if (fe.getComponent() == mtextFilter) mtextFilter.setText(mfilters);
  }
  
  //  MouseListener
  
  public void mouseClicked(MouseEvent me) {
    if (me.getModifiers() != MouseEvent.BUTTON1_MASK) return;
    Object src = me.getSource();
    if (src instanceof PTree) {
      if (me.getClickCount() == 2) {
        TreeNode selNode = tree.getSelectedNode();
        if (selNode == null || selNode.getLevel() < 2 || selNode.getNodeData() == null) return;
        Rectangle rect = tree.getNodeRect(selNode);
        if (rect != null && rect.contains(new Point(me.getX(), me.getY()))) showSource();
      } else moveInTable();
      return;
    }
    
    int col = table.getColumnAtX(me.getX());
    
    if (src instanceof PTableHeader) {
      if (me.getClickCount() == 1 && col != -1) sortColumn(col);
    }
    
    if (src instanceof PTable) {
      if (me.getClickCount() == 1) tableAction(me.getX());
      if (me.getClickCount() == 2) {
        if (col == 0) tableAction(0);
        if (viewToColumn(col) == AllocSiteSorter.FRAME_COLUMN) showSource();
      }
    }
  }
  
  public void mousePressed(MouseEvent me) {}
  public void mouseReleased(MouseEvent me) {}
  public void mouseEntered(MouseEvent me) {}
  public void mouseExited(MouseEvent me) {}
  
  // TreeExpansionListener
  
  public void treeExpanded(TreeExpansionEvent event) {
    expandSingle(event.getNode());
  }
  
  private void expandSingle(TreeNode node) {
    tree.expandNode(node);
    if (node.getChildCount() == 1) {
      expandSingle(node.getChildAt(0));
    }
  }
  
  public void treeCollapsed(TreeExpansionEvent event) {}
  
  // Move / filter
  
  private void tableAction(int x) {
    lastRow = -1;
    int row = table.getSelectedRow();
    if (row == -1) return;
    Object obj = table.getCellValue(row, 0);
    if (obj instanceof AllocSiteInfo) {
      sasi = (AllocSiteInfo) obj;
      snode = root;
      if (searchNode(0)) {
        tree.selectNode(snode);
        int treerow = tree.getRowForNode(snode);
        if (treerow != -1) tree.scrollRowToVisible(treerow);
      }
      sasi  = null;
      snode = null;
    }
    if (obj instanceof AllocSiteGroup && x < 16) {
      AllocSiteGroup asg = (AllocSiteGroup) obj;
      if (asg.isOpened) closeTableNode(asg, row);
      else openTableNode(asg, row);
    }
  }
  
  private void tableUpDown() {
    synchronized (lockTable) {
      int row = table.getSelectedRow();
      if (row == -1) return;
      Object obj = table.getCellValue(row, 0);
      if (obj instanceof AllocSiteInfo) {
        lastRow = row;
        for (int i = row - 1; i >= 0; i--) {
          if (tableModel.getCellValue(i, 0) instanceof AllocSiteGroup) {
            table.selectRows(i, i);
            table.scrollRowToVisible(i);
            return;
          }
        }
      } else {
        if (lastRow != -1) {
          table.selectRows(lastRow, lastRow);
          table.scrollRowToVisible(lastRow);
        }
      }
    }
  }
  
  public void openTableNode(AllocSiteGroup asg, int row) {
    if (asg.isOpened) return;
    synchronized (lockTable) {
      asg.isOpened = true;
      AllocSiteInfo[] data = sorter.getFilteredSites(asg);
      for (int i = 0; i < data.length; i++) {
        tableModel.insertRowAt(data[i], row + i + 1);
      }
      table.updateTable();
    }
  }
  
  private void closeTableNode(AllocSiteGroup asg, int row) {
    if (!asg.isOpened) return;
    synchronized (lockTable) {
      asg.isOpened = false;
      for (int i = row + 1; i < tableModel.getRowCount();) {
        if (tableModel.getCellValue(i, 0) instanceof AllocSiteGroup) break;
        tableModel.removeRowAt(i);
      }
      table.updateTable();
    }
  }
  
  private void moveInTable() {
    if (isDownStack) refreshTable();
    TreeNode selectedNode = tree.getSelectedNode();
    if (selectedNode == null || selectedNode.getParent() == null) return;
    AllocSiteNode asn = (AllocSiteNode) selectedNode;
    if (asn.allocSite == null) return;
//    Vector allNodes = sorter.getSitesFromNode(traceFromNode(selectedNode), false);
    if (showGroups) {
      AllocSiteInfo  asi = asn.allocSite;
      AllocSiteGroup asg = sorter.getAllocGroup(asi);
      for (int row = 0; row < tableModel.getRowCount(); row++) {
        if (tableModel.getCellValue(row, 0) == asg) {
          openTableNode(asg, row);
          for (int i = row + 1; i < tableModel.getRowCount(); i++) {
            if (tableModel.getCellValue(i, 0) == asi) {
              table.selectRows(i, i);
              PScrollBar scrollbar = tableScrollpane.getScrollBar(PGUIConstants.VERTICAL);
              if (scrollbar.getVisibleAmount() < (i - row + 1) * rowHeight) {
                scrollbar.setValue((i + 1) * rowHeight - scrollbar.getVisibleAmount());
              } else {
                scrollbar.setValue(row * rowHeight);
              }
              return;
            }
          }
        }
      }
    } else {
//      int row = -1;
//      table.clearSelection();
//      for (int n = 0; n < allNodes.size(); n++) {
//        row = sorter.getAllocSiteRow((AllocSiteInfo) allNodes.elementAt(n));
//        if (row != -1) table.addRowSelectionInterval(row, row);
//      }
//      if (row != -1) table.scrollRowToVisible(row);
      int row = sorter.getAllocSiteRow(asn.allocSite);
      if (row == -1) return;
      table.selectRows(row, row);
      table.scrollRowToVisible(row);
    }
  }
  
  // Source
  
  public void showSource() {
    if (tree.hasFocus()) {
      TreeNode selectedNode = tree.getSelectedNode();
      if (selectedNode == null || selectedNode.getLevel() < 2 || selectedNode.getNodeData() == null) return;
      actionMng.actionSource((FrameInfo) selectedNode.getNodeData());
    } else {
      int row = table.getSelectedRow();
      if (row != -1 && table.getCellValue(row, 0) instanceof AllocSiteInfo) {
        TraceInfo trace = ((AllocSiteInfo) table.getCellValue(row, 0)).traceInfo;
        if (trace != null && trace.numberOfFrames > 0) {
          FrameInfo frame = trace.frameInfo[0];
          if (frame != null) {
            actionMng.actionSource(frame);
          }
        }
      }
    }
  }
  
  // Selected class name
  
  public String getSelectedClassName() {
    int row = table.getSelectedRow();
    if (row == -1) return null;
    Object obj = table.getCellValue(row, 0);
    if (obj instanceof AllocSiteInfo)  return ((AllocSiteInfo)  obj).className;
    if (obj instanceof AllocSiteGroup) return ((AllocSiteGroup) obj).className;
    return null;
  }
  
  // Tree
  
  private Object lockTree = new Object();
  
  public void refreshTree() {
    synchronized (lockTree) {
      root.removeAllChildren();
      treeModel.nodeChanged(root);
      
      Vector allNodes = sorter.getAllSitesFromRoot(); // dynamic !!
      AllocSiteData data;
      AllocSiteNode asn;
      for (int i = 0; i < allNodes.size(); i++) {
        data = (AllocSiteData) allNodes.elementAt(i);
        asn = new AllocSiteNode(data.obj, data.bytes, data.insts, null);
        asn.add(new TreeNodeImpl(Misc.EMPTY_STRING));
        root.add(asn);
      }
      
      if (actionMng.output != null) actionMng.output.command_DumpTraces();
      treeModel.reload();
      tree.expandRow(0);
    }
  }
  
  public void refreshWholeTree() {
    synchronized (lockTree) {
      root.removeAllChildren();
      treeModel.nodeChanged(root);
      
      AllocSiteInfo[] allSites = sorter.getAllSites(); // static !!
      int lines = allSites.length;
      for (int i = 0; i < lines; i++) {
        uasi  = allSites[i];
        unode = root;
        ab = uasi.bytesAlloc;
        ai = uasi.instsAlloc;
        updateNode(0);
      }
      uasi  = null;
      unode = null;
      
      treeModel.reload();
      tree.expandRow(0);
    }
  }
  
  private AllocSiteInfo uasi, sasi;
  private TreeNodeImpl unode, snode;
  private int ab;
  private int ai;
  
  private boolean updateNode(int depth) {
    Object compobj = null;
    
    if (depth == 0) {
      compobj = uasi.getThread();
    } else {
      TraceInfo trace = uasi.traceInfo;
      if (trace == null) return false;
      if (depth > 1 && trace.numberOfFrames < depth) return true;
      FrameInfo[] frames = trace.frameInfo;
      if (frames != null && trace.numberOfFrames > 0) {
        compobj = frames[trace.numberOfFrames - depth];
      }
    }
    
    int pos = -2;
    int count = unode.getChildCount();
    TreeNodeImpl parent = unode;
    for (int i = 0; i < count; i++) {
      AllocSiteNode asn = (AllocSiteNode) unode.getChildAt(i);
      if (asn.getNodeData() == compobj) {
        unode = asn;
        if (updateNode(depth + 1)) {
          if (asn.allocSite != null) {
            if (asn.allocSite.className == uasi.className) {
              ab -= asn.allocSite.bytesAlloc;
              ai -= asn.allocSite.instsAlloc;
            }
          }
          asn.allocSite = uasi;
        }
        asn.bytesAlloc += ab;
        asn.instsAlloc += ai;
        unode = parent;
        for (pos = i - 1; pos >= 0; pos--) {
          if (((AllocSiteNode) unode.getChildAt(pos)).bytesAlloc >= asn.bytesAlloc) break;
        }
        if (pos != -2 && pos != i - 1) unode.insert(asn, pos + 1);
        return false;
      }
    }
    
    AllocSiteNode asn = new AllocSiteNode(compobj, ab, ai, null);
    for (pos = unode.getChildCount() - 1; pos >= 0; pos--) {
      if (((AllocSiteNode) unode.getChildAt(pos)).bytesAlloc >= asn.bytesAlloc) break;
    }
    unode.insert(asn, pos + 1);
    unode = asn;
    if (updateNode(depth + 1)) {
      asn.allocSite = uasi;
    }
    return false;
  }
  
  private boolean searchNode(int depth) {
    Object compobj = null;
    
    if (depth == 0) {
      compobj = sasi.getThread();
    } else {
      TraceInfo trace = sasi.traceInfo;
      if (trace == null) return false;
      if (depth > 1 && trace.numberOfFrames < depth) return true;
      FrameInfo[] frames = trace.frameInfo;
      if (frames != null && trace.numberOfFrames > 0) {
        compobj = frames[trace.numberOfFrames - depth];
      }
    }
    
    expandFake(snode); // dynamic !!
    int count = snode.getChildCount();
    for (int i = 0; i < count; i++) {
      AllocSiteNode asn = (AllocSiteNode) snode.getChildAt(i);
      if (asn.getNodeData() == compobj) {
        snode = asn;
        return searchNode(depth + 1);
      }
    }
    return false;
  }
  
  // TreeShouldExpansionListener
  
  private boolean block = false;
  
  public boolean treeShouldExpand(TreeExpansionEvent event) {
    if (block) return false;
    block = true;
    expandFake(event.getNode());
    block = false;
    return true;
  }
  
  public void expandFake(TreeNode node) {
    if (!node.isLeaf() && node.getLevel() > 0 && !(node.getChildAt(0) instanceof AllocSiteNode)) {
      node.removeAllChildren();
      Vector allNodes = sorter.getAllSitesByTrace(traceFromNode(node));
      for (int i = 0; i < allNodes.size(); i++) {
        AllocSiteData data = (AllocSiteData) allNodes.elementAt(i);
        AllocSiteNode asn = new AllocSiteNode(data.obj, data.bytes, data.insts, data.allocSite);
        if (data.hasChildren) asn.add(new TreeNodeImpl(Misc.EMPTY_STRING));
        node.add(asn);
      }
    }
  }
  
  private TraceInfo traceFromNode(TreeNode node) {
    TraceInfo trace = new TraceInfo();
    trace.numberOfFrames = node.getLevel() - 1;
    FrameInfo[] frames = new FrameInfo[trace.numberOfFrames];
    TreeNode parent = node;
    for (int i = 0; i < trace.numberOfFrames; i++) {
      frames[i] = (FrameInfo) parent.getNodeData();
      parent = parent.getParent();
    }
    trace.frameInfo  = frames;
    trace.threadInfo = (ThreadInfo) parent.getNodeData();
    return trace;
  }
  
  public boolean treeShouldCollapse(TreeExpansionEvent event) {
    return true;
  }
  
  // Table
  
  private Object lockTable = new Object();
  
  public void setShowGroups(boolean state) {
    synchronized (lockTable) {
      showGroups = state;
      tableRndr.showGroups = state;
      ResourceManager.setPersistentBoolean(PreferencesDialog.PROF_SECTION, INI_SMP_GROUPS, state);
      try {
        ResourceManager.save();
      } catch (Exception _) { }
    }
  }
  
  public void setViewColumns(boolean[] cv) {
    synchronized (lockTable) {
      if (columnViews != null) {
        boolean shouldUpdate = false;
        for (int i = 0; i < cv.length; i++) {
          if (columnViews[i] != cv[i]) { shouldUpdate = true; break; }
        }
        if (!shouldUpdate) return;
      }
      
      columnCount = 0;
      columnViews = cv;
      tableModel = new SamplerTableModel(columnNames);
      table.setModel(tableModel);
      TableColumn column;
      for (int i = columnNames.length - 1; i >= 0; i--) {
        column = table.getColumn(i);
        if (columnViews[i]) {
          columnIndices[i] = columnToView(i);
          column.setPreferredWidth(columnSizes[i]);
          column.setCellRenderer(tableRndr);
          column.setHeaderRenderer(new SamplerTableRenderer());
          columnCount++;
        } else {
          columnIndices[i] = -1;
          table.removeColumn(column);
        }
      }
      tableRndr.columnIndices = columnIndices;
      setColumnHeader();
    }
  }
  
  private int columnToView(int col) {
    int index = 0;
    for (int i = 0; i < col && i < columnViews.length; i++) {
      if (columnViews[i]) index++;
    }
    return index;
  }
  
  private int viewToColumn(int view) {
    int index = -1, i;
    for (i = 0; index != view && i < columnViews.length; i++) {
      if (columnViews[i]) index++;
    }
    return i - 1;
  }
  
  public int getSortColumn() {
    return sorter.getColumn();
  }
  
  private void sortColumn(int col) {
    synchronized (lockTable) {
      removeColumnHeader();
      Object site = null;
      int row = table.getSelectedRow();
      if (row != -1) site = table.getCellValue(row, 0);
      sorter.setSort(viewToColumn(col));
      setColumnHeader();
      refreshTable();
      selectRow(site);
    }
  }
  
  private void removeColumnHeader() {
    int index = sorter.getColumn();
    if (columnViews[index]) {
      index = columnToView(index);
      TableColumn column = table.getColumn(index);
      SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
      if (SamplerTableRenderer.noIcon != null) renderer.setIcon(SamplerTableRenderer.noIcon);
    }
  }
  
  private void setColumnHeader() {
    int index = sorter.getColumn();
    if (columnViews[index]) index = columnToView(index);
    else { index = 0; sorter.setSort(0); }
    TableColumn column = table.getColumn(index);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (sorter.isAscending()) {
      if (SamplerTableRenderer.downIcon != null) renderer.setIcon(SamplerTableRenderer.downIcon);
    } else {
      if (SamplerTableRenderer.upIcon   != null) renderer.setIcon(SamplerTableRenderer.upIcon);
    }
    table.getHeader().repaint();
  }
  
  private void selectRow(Object site) {
    if (site == null) return;
    for (int i = 0; i < tableModel.getRowCount(); i++) {
      if (tableModel.getCellValue(i, 0) == site) {
        table.selectRows(i, i);
        table.scrollRowToVisible(i);
        return;
      }
    }
  }
  
  public void refreshTableSelect() {
    Object site = null;
    int row = table.getSelectedRow();
    if (row != -1) site = table.getCellValue(row, 0);
    refreshTable();
    if (site == null) return;
    for (int i = 0; i < tableModel.getRowCount(); i++) {
      if (tableModel.getCellValue(i, 0) == site) {
        table.selectRows(i, i);
        return;
      }
    }
  }
  
  public void refreshTable() {
    synchronized (lockTable) {
      tableModel.setData(getData());
      table.updateTable();
    }
  }
  
  private Vector getData() {
    Vector data;
    tableRndr.isGCDisabled = sorter.isGCDisabled();
    if (showGroups) {
      AllocSiteGroup[] allGroups = sorter.getFilteredGroups((isFinal || showAll ? -1 : maxLines));
      int lines = allGroups.length;
      if (!isFinal && !showAll && lines > maxLines) lines = maxLines;
      data = new Vector(lines);
      AllocSiteGroup asg;
      for (int i = 0; i < lines; i++) {
        asg = allGroups[i];
        data.addElement(asg);
        if (asg.isOpened) {
          AllocSiteInfo[] grpdata = sorter.getFilteredSites(asg);
          for (int k = 0; k < grpdata.length; k++) {
            data.addElement(grpdata[k]);
          }
        }
      }
    } else {
      TreeNode selectedNode = tree.getSelectedNode();
      if (!isDownStack || selectedNode == null || selectedNode.getParent() == null) {
        AllocSiteInfo[] allSites = sorter.getFilteredSites((isFinal || showAll ? -1 : maxLines));
        int lines = allSites.length;
        if (!isFinal && !showAll && lines > maxLines) lines = maxLines;
        data = new Vector(lines);
        for (int i = 0; i < lines; i++) {
          data.addElement(allSites[i]);
        }
      } else {
        data = sorter.getSitesFromNode(traceFromNode(selectedNode), true);
        if (!isFinal && !showAll && data.size() > maxLines) data.setSize(maxLines);
      }
    }
    return data;
  }
  
  public void loadData(DataInputStream input, ProfilerNamespace namespace) throws Exception {
    int count = input.readInt();
    AllocSiteInfo[] allocSites = new AllocSiteInfo[count];
    AllocSiteInfo site;
    for (int i = 0; i < count; i++) {
      site = new AllocSiteInfo();
      site.stackTraceSerialNo = input.readInt();
      site.bytesLive          = input.readInt();
      site.instsLive          = input.readInt();
      site.bytesAlloc         = input.readInt();
      site.instsAlloc         = input.readInt();
      site.bytesLiveMark      = input.readInt();
      site.instsLiveMark      = input.readInt();
      site.bytesAllocMark     = input.readInt();
      site.instsAllocMark     = input.readInt();
      site.className          = input.readUTF();
      site.traceInfo = namespace.lookupTrace(site.stackTraceSerialNo);
      allocSites[i] = site;
    }
    root = new TreeNodeImpl(ResourceManager.getString("Sampler.root.system"));
    treeModel.setRoot(root);
    sorter.clear();
    sorter.addAllocSites(allocSites);
  }
  
  public void saveData(DataOutputStream output) throws Exception {
    AllocSiteInfo[] allocSites = sorter.getAllSites();
    output.writeInt(allocSites.length);
    AllocSiteInfo site;
    for (int i = 0; i < allocSites.length; i++) {
      site = allocSites[i];
      output.writeInt(site.stackTraceSerialNo);
      output.writeInt(site.bytesLive);
      output.writeInt(site.instsLive);
      output.writeInt(site.bytesAlloc);
      output.writeInt(site.instsAlloc);
      output.writeInt(site.bytesLiveMark);
      output.writeInt(site.instsLiveMark);
      output.writeInt(site.bytesAllocMark);
      output.writeInt(site.instsAllocMark);
      output.writeUTF(site.className);
    }
  }
}