/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.util.EventObject;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import com.prosyst.pgui.PComboBox;
import com.prosyst.pgui.PTable;
import com.prosyst.pgui.PTextField;
import com.prosyst.pgui.UIManager;
import com.prosyst.pgui.event.ChangeEvent;
import com.prosyst.pgui.event.CellEditorListener;
import com.prosyst.pgui.event.EventListenerList;
import com.prosyst.pgui.border.Border;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.table.TableCellEditor;

public class ComboBoxEditor extends PComboBox implements TableCellEditor,
                                                         FocusListener,
                                                         ActionListener {
  
  protected EventListenerList listenerList = new EventListenerList();
  protected transient ChangeEvent changeEvent = new ChangeEvent(this);
  protected Color selectedForeground;
  protected Color selectedBackground;
  protected Border border = new EtchedBorder(EtchedBorder.RAISED, Color.white, Color.black);
  protected String[] posValues;
  
  public ComboBoxEditor(String[] posValues) { 
    super();
    this.posValues = posValues;
    setEditable(false);
    setListData(posValues);
    setOpaque(true);
    setForeground(selectedForeground);
    setBackground(selectedBackground);
    ((PTextField) getComboBoxEditor()).addActionListener(this);
  }
  
  public void updateUI () {
    super.updateUI(); 
    selectedForeground = UIManager.getColor(UIManager.Table_selectionForeground);
    selectedBackground = UIManager.getColor(UIManager.Table_selectionBackground);
    setForeground(selectedForeground);
    setBackground(selectedBackground);
  }  

  public void focusGained(FocusEvent e) { }
  
  public void focusLost(FocusEvent e) {
    fireEditingStopped();
  }
  
  public void actionPerformed(ActionEvent ae) {
    String cmd = ae.getActionCommand();
    if (cmd.equals(PTextField.ACTION_ESCAPE)) {
      cancelCellEditing();
    } else if (cmd.equals(PTextField.ACTION_ENTER)) {
      stopCellEditing();
    }
  }
  
  public boolean isMultilineEnabled() {
    return false;
  }
  
  public void setMultilineEnabled(boolean enabled) { }
  
  public void startCellEditing() {
    requestFocus();
  }
  
  public boolean stopCellEditing() {
    fireEditingStopped();
    return true;
  }
  
  public void cancelCellEditing() {
    fireEditingCanceled();
  }
  
  public Component getEditor() {
    return this;
  }
  
  public Object getValue() {
    return getEditorValue();       
  }
  
  public void setValue(Object value) {
    setEditorValue(value);
  }
  
  public boolean isCellEditable(EventObject event) {
    return true;
  }
  
  public boolean shouldSelectCell(EventObject event) { 
    return true; 
  }
  
  public void addCellEditorListener(CellEditorListener l) {
    listenerList.add(CellEditorListener.class, l);
  }
  
  public void removeCellEditorListener(CellEditorListener l) {
    listenerList.remove(CellEditorListener.class, l);
  }
  
  protected void fireEditingStopped() {
    Object[] listeners = listenerList.getListenerList();
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == CellEditorListener.class)
        ((CellEditorListener) listeners[i + 1]).editingStopped(changeEvent);
    }
  }
  
  protected void fireEditingCanceled() {
    Object[] listeners = listenerList.getListenerList();
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i]==CellEditorListener.class)
        ((CellEditorListener) listeners[i + 1]).editingCanceled(changeEvent);
    }
  }
  
  public Component getTableCellEditorComponent(PTable table, Object value,
                                               boolean isSelected, int row, int column) {
    setValue(value);
    setSelectedItemIndex(getModel().getElements().indexOf((String) value));
    setBorder(border);
    return this;
  }
}