/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;
import java.util.Vector;
import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.table.PTableHeader;
import com.prosyst.pgui.table.TableColumn;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.tree.TreeNode;
import com.prosyst.pgui.tree.TreeNodeImpl;
import com.prosyst.pgui.tree.TreeModelImpl;
import com.prosyst.pgui.event.TreeExpansionListener;
import com.prosyst.pgui.event.TreeExpansionEvent;

import com.prosyst.jpd.profiler.storage.ProfilerNamespace;
import com.prosyst.jpd.profiler.info.CpuSampleInfo;
import com.prosyst.jpd.profiler.info.CpuSampleSorter;
import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;

import com.prosyst.jpd.profiler.gui.dialogs.PreferencesDialog;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

public class CpuSamplerGuiImpl extends PPanel implements KeyListener, MouseListener,
                                                         FocusListener, TreeExpansionListener {
  
  private static final String[] columnNames = {
          ResourceManager.getString("Sampler.cpucol.thread"),
          ResourceManager.getString("Sampler.cpucol.method"),
          ResourceManager.getString("Sampler.cpucol.timeprc"),
          ResourceManager.getString("Sampler.cpucol.timems") };
  private static final int[]    columnSizes = { 100, 450, 70, 70 };
  
  protected ActionManager          actionMng;
  protected CpuSampleSorter        sorter     = new CpuSampleSorter();
  protected PSplitPane             splitpane  = new PSplitPane(PSplitPane.VERTICAL_SPLIT);
  
  protected TreeNodeImpl           root       = new TreeNodeImpl(ResourceManager.getString(
                                                "Sampler.root.not_yet_started"));
  protected TreeModelImpl          treeModel  = new TreeModelImpl(root);
  protected PTree                  tree       = new PTree(treeModel);
  protected SamplerTreeRenderer    treeRndr   = new SamplerTreeRenderer();
  
  protected SamplerTableModel      tableModel = new SamplerTableModel(columnNames);
  protected PTable                 table      = new PTable(tableModel);
  protected SamplerTableRenderer   tableRndr  = new SamplerTableRenderer();
  
  protected PScrollPane            treeScrollpane  = new PScrollPane(tree);
  protected PScrollPane            tableScrollpane = new PScrollPane(table);
  protected PTextField             textFilter;
  
  protected long                   beginTicks;
  protected long                   endTicks;
  protected int                    totalNumberOfSamples;
  
  private String   rootName = ResourceManager.getString("Sampler.root.not_yet_started");
  private String   filters;
  
  private int       columnCount;
  private boolean[] columnViews;
  
  public  boolean   methodPrc;
  public  int       timePercentPrecision;
  
  public CpuSamplerGuiImpl(ActionManager actionMng) {
    super(new BorderLayout());
    this.actionMng = actionMng;
    
    tree.setCellRenderer(treeRndr);
    tree.setEditable(false);
    tree.setAutoExpand(false);
    tree.addMouseListener(this);
    tree.addTreeExpansionListener(this);
    tree.addKeyListener(this);
    treeScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    table.setResizeMode(PTable.RESIZE_OFF);
    table.setRowSelectionAllowed(true);
    table.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    table.addMouseListener(this);
    table.addKeyListener(this);
    tableScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    setTimePrecision(PreferencesDialog.getTimePrc());
    setPrecision(PreferencesDialog.getSamplerPrecision());
    setViewColumns(PreferencesDialog.getCpuColumns());
    
    PTableHeader th = table.getHeader();
    th.addMouseListener(this);
    
    textFilter = new PTextField();
    textFilter.addKeyListener(this);
    textFilter.addFocusListener(this);
    
    PPanel fpanel = new PPanel(new BorderLayout());
    fpanel.add(new PLabel(ResourceManager.getString("Sampler.label.methodfilter")), BorderLayout.WEST);
    fpanel.add(textFilter,                                                          BorderLayout.CENTER);
    
    PPanel panel  = new PPanel(new BorderLayout());
    panel.add(tableScrollpane, BorderLayout.CENTER);
    panel.add(fpanel,          BorderLayout.SOUTH);
    
    splitpane.setTop   (treeScrollpane);
    splitpane.setBottom(panel);
    splitpane.setDividerPosition(200);
    this.add(splitpane);
    ToolTipManager.sharedInstance().registerComponent(table);
  }
  
  public void init(String rootName) {
    this.rootName = rootName;
    totalNumberOfSamples = 0;
    sorter.clear();
    root = new TreeNodeImpl(rootName);
    treeModel.setRoot(root);
    refreshTable();
    beginTicks = System.currentTimeMillis();
    endTicks = 0;
  }
  
  public void cont(String rootName) {
    this.rootName = rootName;
    root.setNodeData(rootName);
    treeModel.nodeChanged(root);
    beginTicks = System.currentTimeMillis();
    //endTicks = 0;
  }
  
  public String getRootName() {
    return rootName;
  }
  
  public void eventCpuSamples(int profticks, int numberOfSamples, CpuSampleInfo[] cpuSamples) {
    endTicks += System.currentTimeMillis() - beginTicks;
    totalNumberOfSamples += numberOfSamples;
    sorter.addCpuSamples(cpuSamples);
  }
  
  public void showCpuSamples() {
    rootName = ResourceManager.getString("Sampler.root.final");
    refreshTree();
    refreshTable();
    rootName = ResourceManager.getString("Sampler.root.system") + " (" + endTicks + " ms)";
    root.setNodeData(rootName);
    treeModel.nodeChanged(root);
  }
  
  public void setTimePrecision(int timePercentPrecision) {
    if (treeRndr.timePercentPrecision != timePercentPrecision) {
      this.timePercentPrecision      = timePercentPrecision;
      treeRndr.timePercentPrecision  = timePercentPrecision;
      tableRndr.timePercentPrecision = timePercentPrecision;
      treeModel.reload();
    }
  }
  
  public void setPrecision(boolean state) {
    if (methodPrc != state) {
      methodPrc = state;
      treeRndr.methodPrc = state;
      refreshTree();
    }
  }
  
  public PComponent getFindComponent() {
    if (table.hasFocus()) return table;
    return tree;
  }
  
  public boolean isThreadVisible() {
    return columnViews[CpuSampleSorter.THREAD_COLUMN];
  }
  
  public boolean isFrameVisible() {
    return columnViews[CpuSampleSorter.FRAME_COLUMN];
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == tree || src == treeScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_ENTER) moveInTable();
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.CpuTree");
    }
    if (src == table || src == tableScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_ENTER) moveInTree();
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.CpuTable");
    }
    if (src instanceof PTextField) {
      if (keyCode == KeyEvent.VK_ENTER) {
        filters = textFilter.getText();
        StringTokenizer st = new StringTokenizer(filters, ",");
        String[] filtersIA = new String[st.countTokens()];
        String[] filtersEA = new String[st.countTokens()];
        int fi = 0, fe = 0;
        while (st.hasMoreTokens()) {
          String filter = st.nextToken().trim();
          if (filter.length() == 0) continue;
          if (filter.charAt(0) == '!') filtersEA[fe++] = filter.substring(1);
          else                         filtersIA[fi++] = filter;
        }
        String[] filtersI = new String[fi]; System.arraycopy(filtersIA, 0, filtersI, 0, fi);
        String[] filtersE = new String[fe]; System.arraycopy(filtersEA, 0, filtersE, 0, fe);
        sorter.setFilters((filtersI.length == 0 ? null : filtersI), (filtersE.length == 0 ? null : filtersE));
        refreshTable();
      }
      if (keyCode == KeyEvent.VK_ESCAPE) {
        textFilter.setText(filters);
      }
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.samplers.CpuFilters");
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
  
  //  FocusListener
  
  public void focusGained(FocusEvent fe) { }
  
  public void focusLost(FocusEvent fe) {
    if (fe.getComponent() == textFilter) textFilter.setText(filters);
  }
  
  //  MouseListener
  
  public void mouseClicked(MouseEvent me) {
    if (me.getModifiers() != MouseEvent.BUTTON1_MASK) return;
    Object src = me.getSource();
    if (src instanceof PTree) {
      if (me.getClickCount() == 2) {
        TreeNode selNode = tree.getSelectedNode();
        if (selNode == null || selNode.getLevel() < 2 || selNode.getNodeData() == null) return;
        Rectangle rect = tree.getNodeRect(selNode);
        if (rect != null && rect.contains(new Point(me.getX(), me.getY()))) showSource();
      } else moveInTable();
      return;
    }
    
    int col = table.getColumnAtX(me.getX());
    
    if (src instanceof PTableHeader) {
      if (me.getClickCount() == 1 && col != -1) sortColumn(col);
    }
    
    if (src instanceof PTable) {
      if (me.getClickCount() == 1) moveInTree();
      if (me.getClickCount() == 2 && viewToColumn(col) == CpuSampleSorter.FRAME_COLUMN) showSource();
    }
  }
  
  public void mousePressed(MouseEvent me) {}
  public void mouseReleased(MouseEvent me) {}
  public void mouseEntered(MouseEvent me) {}
  public void mouseExited(MouseEvent me) {}
  
  // TreeExpansionListener
  
  public void treeExpanded(TreeExpansionEvent event) {
    expand(event.getNode());
  }
  
  private void expand(TreeNode node) {
    tree.expandNode(node);
    if (node.getChildCount() == 1) {
      expand(node.getChildAt(0));
    }
  }
  
  public void treeCollapsed(TreeExpansionEvent event) {}
  
  // Move / filter
  
  private void moveInTree() {
    int row = table.getSelectedRow();
    if (row != -1) {
      scsi = sorter.getCpuSample(row);
      snode = root;
      if (searchNode(0)) {
        tree.selectNode(snode);
        int treerow = tree.getRowForNode(snode);
        if (treerow != -1) tree.scrollRowToVisible(treerow);
      }
      scsi  = null;
      snode = null;
    }
  }
  
  private void moveInTable() {
    TreeNode selectedNode = tree.getSelectedNode();
    if (selectedNode == null || selectedNode.getParent() == null) return;
    CpuSampleNode csn = (CpuSampleNode) selectedNode;
    if (csn.cpuSample == null) return;
    int row = sorter.getCpuSampleRow(csn.cpuSample);
    if (row != -1) {
      table.selectRows(row, row);
      table.scrollRowToVisible(row);
    }
  }
  
  // Source
  
  public void showSource() {
    if (tree.hasFocus()) {
      TreeNode selectedNode = tree.getSelectedNode();
      if (selectedNode == null || selectedNode.getLevel() < 2 || selectedNode.getNodeData() == null) return;
      actionMng.actionSource((FrameInfo) selectedNode.getNodeData());
    } else {
      int row = table.getSelectedRow();
      if (row == -1) return;
      TraceInfo trace = sorter.getCpuSample(row).traceInfo;
      if (trace != null && trace.numberOfFrames > 0) {
        FrameInfo frame = trace.frameInfo[0];
        if (frame != null) {
          actionMng.actionSource(frame);
        }
      }
    }
  }
  
  // Tree
  
  private Object lockTree = new Object();
  
  public void refreshTree() {
    synchronized (lockTree) {
      root = new TreeNodeImpl(rootName);
      treeModel.nodeChanged(root);
      CpuSampleInfo[] allSamples = sorter.getAllSamples();
      int lines = allSamples.length;
      for (int i = 0; i < lines; i++) {
        ucsi  = allSamples[i];
        unode = root;
        cost    = ucsi.stackTraceCost;
        time    =    endTicks * cost / totalNumberOfSamples;
        percent = (float) 100 * cost / totalNumberOfSamples;
        updateNode(0);
      }
      ucsi  = null;
      unode = null;
      treeModel.setRoot(root);
      tree.expandRow(0);
    }
  }
  
  private CpuSampleInfo ucsi, scsi;
  private TreeNodeImpl unode, snode;
  private int   cost;
  private long  time;
  private float percent;
  
  private boolean updateNode(int depth) {
    Object compobj = null;
    
    if (depth == 0) {
      compobj = ucsi.getThread();
    } else {
      TraceInfo trace = ucsi.traceInfo;
      if (trace == null) return false;
      if (depth > 1 && trace.numberOfFrames < depth) return true;
      FrameInfo[] frames = trace.frameInfo;
      if (frames != null && trace.numberOfFrames > 0) {
        compobj = frames[trace.numberOfFrames - depth];
      }
    }
    
    int pos = -2;
    int count = unode.getChildCount();
    TreeNodeImpl parent = unode;
    for (int i = 0; i < count; i++) {
      CpuSampleNode csn = (CpuSampleNode) unode.getChildAt(i);
      if ((methodPrc && compobj instanceof FrameInfo
          ? ((FrameInfo) csn.getNodeData()).compareMethodFrame((FrameInfo) compobj)
          : csn.getNodeData() == compobj)) {
        unode = csn;
        if (updateNode(depth + 1)) {
          if (csn.cpuSample != null) {
            if (csn.cpuSample.traceInfo == ucsi.traceInfo) {
              time    -= csn.time;
              percent -= csn.percent;
            }
          }
          csn.cpuSample = ucsi;
        }
        csn.time    += time;
        csn.percent += percent;
        unode = parent;
        for (pos = i - 1; pos >= 0; pos--) {
          if (((CpuSampleNode) unode.getChildAt(pos)).time >= csn.time) break;
        }
        if (pos != -2 && pos != i - 1) unode.insert(csn, pos + 1);
        return false;
      }
    }
    
    CpuSampleNode csn = new CpuSampleNode(compobj, time, percent, null);
    for (pos = unode.getChildCount() - 1; pos >= 0; pos--) {
      if (((CpuSampleNode) unode.getChildAt(pos)).time >= csn.time) break;
    }
    unode.insert(csn, pos + 1);
    unode = csn;
    if (updateNode(depth + 1)) {
      csn.cpuSample = ucsi;
    }
    return false;
  }
  
  private boolean searchNode(int depth) {
    Object compobj = null;
    
    if (depth == 0) {
      compobj = scsi.getThread();
    } else {
      TraceInfo trace = scsi.traceInfo;
      if (trace == null) return false;
      if (depth > 1 && trace.numberOfFrames < depth) return true;
      FrameInfo[] frames = trace.frameInfo;
      if (frames != null && trace.numberOfFrames > 0) {
        compobj = frames[trace.numberOfFrames - depth];
      }
    }
    
    int count = snode.getChildCount();
    for (int i = 0; i < count; i++) {
      CpuSampleNode csn = (CpuSampleNode) snode.getChildAt(i);
      if ((methodPrc && compobj instanceof FrameInfo
          ? ((FrameInfo) csn.getNodeData()).compareMethodFrame((FrameInfo) compobj)
          : csn.getNodeData() == compobj)) {
        snode = csn;
        return searchNode(depth + 1);
      }
    }
    return false;
  }
  
  // Table
  
  private Object lockTable = new Object();
  
  public void setViewColumns(boolean[] cv) {
    synchronized (lockTable) {
      if (columnViews != null) {
        boolean shouldUpdate = false;
        for (int i = 0; i < cv.length; i++) {
          if (columnViews[i] != cv[i]) { shouldUpdate = true; break; }
        }
        if (!shouldUpdate) return;
      }
      
      columnCount = 0;
      columnViews = cv;
      tableModel = new SamplerTableModel(columnNames);
      table.setModel(tableModel);
      TableColumn column;
      for (int i = columnNames.length - 1; i >= 0; i--) {
        column = table.getColumn(i);
        if (columnViews[i]) {
          column.setPreferredWidth(columnSizes[i]);
          column.setCellRenderer(tableRndr);
          column.setHeaderRenderer(new SamplerTableRenderer());
          columnCount++;
        } else {
          table.removeColumn(column);
        }
      }
      setColumnHeader();
    }
  }
  
  private int columnToView(int col) {
    int index = 0;
    for (int i = 0; i < col && i < columnViews.length; i++) {
      if (columnViews[i]) index++;
    }
    return index;
  }
  
  private int viewToColumn(int view) {
    int index = -1, i;
    for (i = 0; index != view && i < columnViews.length; i++) {
      if (columnViews[i]) index++;
    }
    return i - 1;
  }
  
  public int getSortColumn() {
    return sorter.getColumn();
  }
  
  private void sortColumn(int col) {
    synchronized (lockTable) {
      removeColumnHeader();
      CpuSampleInfo sample = null;
      int row = table.getSelectedRow();
      if (row != -1) sample = sorter.getCpuSample(row);
      sorter.setSort(viewToColumn(col));
      setColumnHeader();
      refreshTable();
      if (sample != null) {
        row = sorter.getCpuSampleRow(sample);
        if (row != -1) {
          table.selectRows(row, row);
          table.scrollRowToVisible(row);
        }
      }
    }
  }
  
  private void removeColumnHeader() {
    int index = sorter.getColumn();
    if (columnViews[index]) {
      index = columnToView(index);
      TableColumn column = table.getColumn(index);
      SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
      if (SamplerTableRenderer.noIcon != null) renderer.setIcon(SamplerTableRenderer.noIcon);
    }
  }
  
  private void setColumnHeader() {
    int index = sorter.getColumn();
    if (columnViews[index]) index = columnToView(index);
    else { index = 0; if (sorter.getColumn() != viewToColumn(0)) sorter.setSort(viewToColumn(0)); }
    TableColumn column = table.getColumn(index);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (sorter.isAscending()) {
      if (SamplerTableRenderer.downIcon != null) renderer.setIcon(SamplerTableRenderer.downIcon);
    } else {
      if (SamplerTableRenderer.upIcon   != null) renderer.setIcon(SamplerTableRenderer.upIcon);
    }
    table.getHeader().repaint();
  }
  
  public void refreshTable() {
    synchronized (lockTable) {
      tableModel.setData(getData());
      table.updateTable();
    }
  }
  
  private Vector getData() {
    Vector data;
    Object[] rowdata;
    CpuSampleInfo[] allSamples = sorter.getFilteredSamples();
    int lines = allSamples.length;
    data = new Vector(lines);
    CpuSampleInfo csi;
    long cost;
    int j;
    for (int i = 0; i < lines; i++) {
      csi = allSamples[i];
      cost = csi.stackTraceCost;
      j = 0;
      rowdata = new Object[columnCount];
      if (columnViews[0]) { rowdata[j++] = csi.threadName;                                   }
      if (columnViews[1]) { rowdata[j++] = csi.frameName;                                    }
      if (columnViews[2]) { rowdata[j++] = new Float(   100F * cost / totalNumberOfSamples); }
      if (columnViews[3]) { rowdata[j++] = new Long(endTicks * cost / totalNumberOfSamples); }
      data.addElement(rowdata);
    }
    return data;
  }
  
  public void loadData(DataInputStream input, ProfilerNamespace namespace) throws Exception {
    totalNumberOfSamples = input.readInt();
    endTicks = input.readLong();
    int count = input.readInt();
    CpuSampleInfo[] cpuSamples = new CpuSampleInfo[count];
    for (int i = 0; i < count; i++) {
      cpuSamples[i] = new CpuSampleInfo();
      cpuSamples[i].stackTraceCost     = input.readInt();
      cpuSamples[i].stackTraceSerialNo = input.readInt();
      cpuSamples[i].traceInfo = namespace.lookupTrace(cpuSamples[i].stackTraceSerialNo);
      cpuSamples[i].resolve();
    }
    sorter.clear();
    sorter.addCpuSamples(cpuSamples);
  }
  
  public void saveData(DataOutputStream output) throws Exception {
    CpuSampleInfo[] cpuSamples = sorter.getAllSamples();
    output.writeInt(totalNumberOfSamples);
    output.writeLong(endTicks);
    output.writeInt(cpuSamples.length);
    for (int i = 0; i < cpuSamples.length; i++) {
      output.writeInt(cpuSamples[i].stackTraceCost);
      output.writeInt(cpuSamples[i].stackTraceSerialNo);
    }
  }
}
