/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.util.StringTokenizer;
import java.util.Vector;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.event.ListSelectionListener;
import com.prosyst.pgui.event.ListSelectionEvent;
import com.prosyst.pgui.event.TableSelectionListener;
import com.prosyst.pgui.event.TableSelectionEvent;
import com.prosyst.pgui.list.ListModel;
import com.prosyst.pgui.list.ListSelectionModel;
import com.prosyst.pgui.table.PTableHeader;
import com.prosyst.pgui.table.TableColumn;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.tree.TreeNode;
import com.prosyst.pgui.tree.TreeNodeImpl;
import com.prosyst.pgui.tree.TreeModelImpl;
import com.prosyst.pgui.event.TreeShouldExpandListener;
import com.prosyst.pgui.event.TreeExpansionEvent;

import com.prosyst.jpd.profiler.storage.ProfilerNamespace;
import com.prosyst.jpd.profiler.info.ClassDumpInfo;
import com.prosyst.jpd.profiler.info.InstDumpInfo;
import com.prosyst.jpd.profiler.info.ObjectDumpInfo;
import com.prosyst.jpd.profiler.info.ObjArrayDumpInfo;
import com.prosyst.jpd.profiler.info.PrimArrayDumpInfo;
import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.JavaFrameRoot;
import com.prosyst.jpd.profiler.info.NativeStackRoot;
import com.prosyst.jpd.profiler.info.Misc;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

public class HeapDumpPanel extends PPanel implements ActionListener, ListSelectionListener, TableSelectionListener,
                                                     KeyListener, MouseListener, FocusListener, TreeShouldExpandListener {
  
  protected static final String[] columnNames = {
          ResourceManager.getString("HeapPanel.column.class_loader"),
          ResourceManager.getString("HeapPanel.column.classes"),
          ResourceManager.getString("HeapPanel.column.insts") };
  private   static final int      LOADER      = 0;
  private   static final int      CLASS       = 1;
  
  protected ActionManager     actionMng;
  
  protected PTextField        clsFilter           = new PTextField();
  
  protected SamplerTableModel clsTableModel       = new SamplerTableModel(columnNames);
  protected PTable            clsTable            = new PTable(clsTableModel);
  protected PScrollPane       clsTableScrollpane  = new PScrollPane(clsTable);
  
  protected PTabbedPane       clsTabpane          = new PTabbedPane();
  
  protected PList             instList            = new PList();
  protected PScrollPane       instListScrollpane  = new PScrollPane(instList);
  
  protected PList             statList            = new PList();
  protected PScrollPane       statListScrollpane  = new PScrollPane(statList);
  
  protected TreeNodeImpl      superRoot           = new TreeNodeImpl();
  protected TreeModelImpl     superTreeModel      = new TreeModelImpl(superRoot);
  protected PTree             superTree           = new PTree(superTreeModel);
  protected PScrollPane       superTreeScrollpane = new PScrollPane(superTree);
  
  protected PRadioButton      outrefRB, inrefRB;
  
  protected TreeNodeImpl      objRoot             = new TreeNodeImpl();
  protected TreeModelImpl     objTreeModel        = new TreeModelImpl(objRoot);
  protected PTree             objTree             = new PTree(objTreeModel);
  protected PScrollPane       objTreeScrollpane   = new PScrollPane(objTree);
  
  protected PList             allocList           = new PList();
  protected PScrollPane       allocListScrollpane = new PScrollPane(allocList);
  
  protected PSplitPane        mainSplit, topSplit, bottomSplit;
  
  private   ProfilerNamespace namespace;
  private   int               lastClassRow = -2;
  private   String            cfilters;
  private   String[]          cfiltersI = null;
  private   String[]          cfiltersE = null;
  private   boolean           isOutRef  = false;
  private   int               sortedBy  = CLASS;
  private   boolean           ascending = true;
  
  //  Constructor
  
  public HeapDumpPanel(ActionManager actionMng) {
    super(new BorderLayout());
    this.actionMng = actionMng;
    
    PPanel clsFltPnl = new PPanel(new BorderLayout());
    clsFltPnl.add(new PLabel(ResourceManager.getString("HeapPanel.label.classfilter")), BorderLayout.WEST);
    clsFltPnl.add(clsFilter,                                                            BorderLayout.CENTER);
    
    clsFilter.addKeyListener(this);
    clsFilter.addFocusListener(this);
    
    PPanel clsPnl = new PPanel(new BorderLayout());
    clsPnl.add(clsFltPnl,          BorderLayout.NORTH);
    clsPnl.add(clsTableScrollpane, BorderLayout.CENTER);
    
    clsTable.setRowSelectionAllowed(true);
    clsTable.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    clsTable.getSelectionModel().addTableSelectionListener(this);
    clsTable.addKeyListener(this);
    clsTableScrollpane.getViewport().getGlassPane().addKeyListener(this);
    clsTable.addMouseListener(this);
    clsTable.getColumn(0).setPreferredWidth(250);
    clsTable.getColumn(0).setHeaderRenderer(new SamplerTableRenderer());
    clsTable.getColumn(1).setPreferredWidth(250);
    clsTable.getColumn(1).setHeaderRenderer(new SamplerTableRenderer());
    clsTable.getColumn(2).setHeaderRenderer(new SamplerTableRenderer());
    clsTable.getColumn(2).setCellRenderer(new SamplerTableRenderer());
    setColumnHeader();
    //clsTableScrollpane.getViewport().setDraggingEnabled(false); // F1 fix !!
    
    PTableHeader th = clsTable.getHeader();
    th.addMouseListener(this);
    
    clsTabpane.addTab(ResourceManager.getString("HeapPanel.tabInsts"), instListScrollpane);
    clsTabpane.addTab(ResourceManager.getString("HeapPanel.tabStats"), statListScrollpane);
    clsTabpane.addTab(ResourceManager.getString("HeapPanel.tabSuper"), superTreeScrollpane);
    clsTabpane.setMinimumSize(new Dimension(150, 50));
    
    instList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    instList.addListSelectionListener(this);
    instList.addKeyListener(this);
    instListScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    statList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    statList.addListSelectionListener(this);
    statList.addKeyListener(this);
    statListScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    superTree.setEditable(false);
    superTree.setAutoExpand(false);
    superTree.setRootVisible(false);
    superTree.setCellRenderer(new SamplerTreeRenderer());
    superTree.addMouseListener(this);
    superTree.addKeyListener(this);
    superTreeScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    topSplit = new PSplitPane(PSplitPane.HORIZONTAL_SPLIT);
    topSplit.setLeft (clsPnl);
    topSplit.setRight(clsTabpane);
    topSplit.setDividerPosition(350);
    
    PPanel refPnl = new PPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
    refPnl.add(outrefRB = createRadioButton("outref", false));
    refPnl.add(inrefRB  = createRadioButton("inref",  true));
    ButtonGroup refrg = new ButtonGroup();
    refrg.add(outrefRB);
    refrg.add(inrefRB);
    
    objTree.setEditable(false);
    objTree.setAutoExpand(false);
    objTree.setRootVisible(false);
    objTree.setCellRenderer(new SamplerTreeRenderer());
    objTree.addTreeShouldExpandListener(this);
    objTree.addMouseListener(this);
    objTree.addKeyListener(this);
    objTreeScrollpane.getViewport().getGlassPane().addKeyListener(this); // F1 fix !!
    
    PPanel objPnl = new PPanel(new BorderLayout());
    objPnl.add(refPnl,            BorderLayout.NORTH);
    objPnl.add(objTreeScrollpane, BorderLayout.CENTER);
    
    allocList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    allocList.addMouseListener(this);
    allocList.addKeyListener(this);
    allocListScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    PPanel allocPnl = new PPanel(new BorderLayout());
    allocPnl.add(new PLabel(ResourceManager.getString("HeapPanel.label.alloc")), BorderLayout.NORTH);
    allocPnl.add(allocListScrollpane,                                            BorderLayout.CENTER);
    
    bottomSplit = new PSplitPane(PSplitPane.HORIZONTAL_SPLIT);
    bottomSplit.setLeft (objPnl);
    bottomSplit.setRight(allocPnl);
    bottomSplit.setDividerPosition(400);
    
    mainSplit = new PSplitPane(PSplitPane.VERTICAL_SPLIT);
    mainSplit.setTop   (topSplit);
    mainSplit.setBottom(bottomSplit);
    mainSplit.setDividerPosition(200);
    this.add(mainSplit);
  }
  
  private PRadioButton createRadioButton(String name, boolean flag) {
    PRadioButton rb = new PRadioButton(ResourceManager.getString("HeapPanel.radio." + name));
    //String mnem = ResourceManager.getString("HeapPanel.radio.mnem." + name);
    //if (mnem != null) rb.setMnemonic(mnem.charAt(0));
    rb.setSelected(flag);
    rb.addActionListener(this);
    return rb;
  }
  
  public void clear() {
    namespace = null;
    clearClassTable();
    clearClassInfo();
    clearObjectInfo();
    clearAllocStack();
  }
  
  public void clearClassTable() {
    lastClassRow = -2;
    clsTableModel.removeAllRows();
    clsTable.getSelectionModel().clearSelection();
    clsTable.updateTable();
  }
  
  public void clearClassInfo() {
    instList.getModel().removeAllElements();
    statList.getModel().removeAllElements();
    superRoot.removeAllChildren();
    superTreeModel.reload();
  }
  
  public void clearObjectInfo() {
    objRoot.removeAllChildren();
    objTreeModel.reload();
  }
  
  public void clearAllocStack() {
    allocList.getModel().removeAllElements();
  }
  
  public void showInfo(ProfilerNamespace namespace) {
    this.namespace = namespace;
    setClassTable();
  }
  
  public PComponent getFindComponent() {
    if (clsTable.hasFocus())  return clsTable;
    if (instList.hasFocus())  return instList;
    if (statList.hasFocus())  return statList;
    if (superTree.hasFocus()) return superTree;
    if (objTree.hasFocus())   return objTree;
    if (allocList.hasFocus()) return allocList;
    return objTree;
  }
  
  private void setClassTable() {
    if (namespace == null) return;
    lastClassRow = -2;
    clsTable.getSelectionModel().clearSelection();
    clearClassInfo();
    
    Vector v = new Vector();
    Object[] classes = namespace.getClsdumpTable().elements();
    for (int i = 0; i < classes.length; i++) {
      ClassDumpInfo clsDumpInfo = (ClassDumpInfo) classes[i];
      if (clsDumpInfo != null) {
        if (clsDumpInfo.numberOfInstances != 0 || hasStaticReferences(clsDumpInfo)) {
          if (matchesClassFilters(clsDumpInfo)) v.addElement(clsDumpInfo);
        }
      }
    }
    for (byte i = 0; i < namespace.heap_primarr.length; i++) {
      if (namespace.heap_primarr[i] != 0) {
        ClassDumpInfo clsDumpInfo = new ClassDumpInfo();
        clsDumpInfo.type = i;
        clsDumpInfo.className = InstDumpInfo.toStringType(i) + " []";
        clsDumpInfo.numberOfInstances = namespace.heap_primarr[i];
        clsDumpInfo.staticFields = new ClassDumpInfo.StaticField[0];
        clsDumpInfo.instanceFields = new ClassDumpInfo.InstanceField[0];
        if (matchesClassFilters(clsDumpInfo)) v.addElement(clsDumpInfo);
      }
    }
    int clsCount = v.size();
    sort(v, 0, clsCount - 1);
    clsTableModel.setData(v);
    clsTable.updateTable();
  }
  
  public void setSelectedClass() {
    String selClassName = actionMng.allocSampler.getSelectedClassName();
    int row = -1;
    Vector v = clsTableModel.getData();
    for (int i = 0; i < v.size(); i++) {
      if (((ClassDumpInfo) v.elementAt(i)).getClassName().equals(selClassName)) { row = i; break; }
    }
    if (row != -1) {
      clsTable.selectRows(row, row);
      clsTable.scrollRowToVisible(row);
    }
  }
  
  public void gotoClassLoader() {
    int row = clsTable.getSelectedRow();
    if (row == -1) return;
    Object clsldrInst = clsTable.getCellValue(row, LOADER);
    if (!(clsldrInst instanceof ObjectDumpInfo)) return;
    ObjectDumpInfo classLoader = (ObjectDumpInfo) clsldrInst;
    String loaderClassName = classLoader.classDumpInfo.className;
    Vector v = clsTableModel.getData();
    for (int i = 0; i < v.size(); i++) {
      if (((ClassDumpInfo) v.elementAt(i)).getClassName().equals(loaderClassName)) { row = i; break; }
    }
    if (row != -1) {
      clsTable.selectRows(row, row);
      clsTable.scrollRowToVisible(row);
    }
    ListModel listModel =  instList.getModel();
    for (int i = 0; i < v.size(); i++) {
      if (((InstDumpInfo) listModel.elementAt(i)) == classLoader) { row = i; break; }
    }
    if (row != -1) {
      instList.setSelectionInterval(row, row);
      instList.scrollRowToVisible(row);
    }
  }
  
  private boolean hasStaticReferences(ClassDumpInfo clsDumpInfo) {
    for (int i = 0; i < clsDumpInfo.staticFields.length; i++) {
      int id = clsDumpInfo.staticFields[i].value;
      if (id != 0 && namespace.lookupObjectDump(id) != null) return true;
    }
    return false;
  }
  
  private boolean matchesClassFilters(ClassDumpInfo clsDumpInfo) {
    if (cfiltersI == null) {
      if (cfiltersE != null) {
        for (int i = 0; i < cfiltersE.length; i++) {
          if (clsDumpInfo.className.indexOf(cfiltersE[i]) != -1) return false;
        }
      }
      return true;
    } else {
      if (cfiltersE != null) {
        for (int i = 0; i < cfiltersE.length; i++) {
          if (clsDumpInfo.className.indexOf(cfiltersE[i]) != -1) return false;
        }
      }
      for (int i = 0; i < cfiltersI.length; i++) {
        if (clsDumpInfo.className.indexOf(cfiltersI[i]) != -1) return true;
      }
      return false;
    }
  }
  
  // Init tabpane
  
  private void setDataForCurrentClass() {
    int row = clsTable.getSelectedRow();
    if (row == -1 || row == lastClassRow) return;
    lastClassRow = row;
    ClassDumpInfo classDumpInfo = (ClassDumpInfo) clsTable.getCellValue(row, CLASS);
    setInstsForClass(classDumpInfo);
    setStatsForClass(classDumpInfo);
    setSuperForClass(classDumpInfo);
  }
  
  private void setInstsForClass(ClassDumpInfo clsDumpInfo) {
    Vector v = new Vector();
    if (clsDumpInfo.numberOfInstances > 0) {
      Object[] objects = namespace.getObjdumpTable().elements();
      for (int i = 0; i < objects.length; i++) {
        if (objects[i] != null) {
          InstDumpInfo objdump = (InstDumpInfo) objects[i];
          if (clsDumpInfo.type == 0) {
            if ((objdump.getType() == InstDumpInfo.TYPE_OBJECT
                && ((ObjectDumpInfo) objdump).classDumpInfo == clsDumpInfo)
             || (objdump.getType() == InstDumpInfo.TYPE_ARRAY
                && ((ObjArrayDumpInfo) objdump).classDumpInfo == clsDumpInfo)) {
              v.addElement(objdump);
            }
          } else {
            if ((objdump instanceof PrimArrayDumpInfo) && objdump.getType() == clsDumpInfo.type) {
              v.addElement(objdump);
            }
          }
        }
      }
      int instCount = v.size();
      Misc.sort(v, 0, instCount - 1);
    }
    instList.getModel().setListData(v);
    if (clsTabpane.getSelectedIndex() == 0 && v.size() > 0) instList.setSelectedIndex(0);
  }
  
  private void setStatsForClass(ClassDumpInfo clsDumpInfo) {
    Vector v = new Vector();
    for (int i = 0; i < clsDumpInfo.staticFields.length; i++) {
      int id = clsDumpInfo.staticFields[i].value;
      if (id != 0) {
        InstDumpInfo obj = namespace.lookupObjectDump(id);
        if (obj != null) v.addElement(new FieldNode(clsDumpInfo.staticFields[i].name, 0, obj));
      }
    }
    statList.getModel().setListData(v);
    if (clsTabpane.getSelectedIndex() == 1 && v.size() > 0) statList.setSelectedIndex(0);
  }
  
  private void setSuperForClass(ClassDumpInfo clsDumpInfo) {
    superRoot.removeAllChildren();
    TreeNodeImpl node, subnode;
    TreeNodeImpl cnode = new TreeNodeImpl(clsDumpInfo);
    node = cnode;
    ClassDumpInfo superClass = clsDumpInfo.superClassInfo;
    if (superClass != null) {
      while (superClass != null) {
        subnode = node;
        node = new TreeNodeImpl(superClass);
        node.add(subnode);
        superClass = superClass.superClassInfo;
      }
      superRoot.add(node);
    } else {
      superRoot.add(new TreeNodeImpl(ResourceManager.getString("HeapPanel.label.no_super_class")));
    }
    superTreeModel.reload();
    superTree.selectNode(cnode);
    int treerow = superTree.getRowForNode(cnode);
    if (treerow != -1) superTree.scrollRowToVisible(treerow);
  }
  
  // Reference panel
  
  private void setRefsForObject(InstDumpInfo instDumpInfo) {
    clearObjectInfo();
    clearAllocStack();
    if (instDumpInfo == null) return;
    setStackTrace(instDumpInfo.traceInfo);
    TreeNodeImpl onode = new TreeNodeImpl(instDumpInfo);
    objRoot.add(onode);
    objTree.selectNode(onode);
    if (isOutRef) expandOutRefFieldNode(onode);
    else expandInRefFieldNode(onode);
    objTree.expandNode(onode);
    objTree.updateTree();
  }
  
  // Outgoing references
  
  private void expandOutRefFieldNode(TreeNode node) {
    InstDumpInfo instDumpInfo = (InstDumpInfo) node.getNodeData();
    if (instDumpInfo.getType() == InstDumpInfo.TYPE_OBJECT) {
      ObjectDumpInfo objDumpInfo = (ObjectDumpInfo) instDumpInfo;
      ClassDumpInfo currClass = objDumpInfo.classDumpInfo;
      int fieldNo = 0;
      int[] ids = objDumpInfo.values;
      for (int i = 0; i < ids.length; i++) {
        while (fieldNo >= currClass.instanceFields.length) {
          currClass = currClass.superClassInfo;
          fieldNo = 0;
        }
        createOutRefsNode(node, ids[i], currClass.instanceFields[fieldNo++].name);
      }
    } else if (instDumpInfo.getType() == InstDumpInfo.TYPE_ARRAY) {
      int[] ids = ((ObjArrayDumpInfo) instDumpInfo).values;
      for (int i = 0; i < ids.length; i++) {
        createOutRefsNode(node, ids[i], "element[" + i + "]");
      }
    }
  }
  
  private void createOutRefsNode(TreeNode node, int id, String fieldName) {
    if (id != 0) {
      InstDumpInfo obj = namespace.lookupObjectDump(id);
      if (obj != null) {
        int state = getOutRefStateFor(node, obj);
        FieldNode child = new FieldNode(fieldName, state, obj);
        node.add(child);
        if (state == FieldNode.NODE_MORE) child.add(new TreeNodeImpl(Misc.EMPTY_STRING));
      }
    }
  }
  
  private static int getOutRefStateFor(TreeNode node, InstDumpInfo instDumpInfo) {
    int type = instDumpInfo.getType();
    if (type == InstDumpInfo.TYPE_OBJECT || type == InstDumpInfo.TYPE_ARRAY) {
      TreeNode parent = node;
      while (parent != null) {
        if (parent.getNodeData() == instDumpInfo) return FieldNode.NODE_LOOP;
        parent = parent.getParent();
      }
      int[] ids;
      if (type == InstDumpInfo.TYPE_OBJECT) {
        ids = ((ObjectDumpInfo) instDumpInfo).values;
      } else {
        ids = ((ObjArrayDumpInfo) instDumpInfo).values;
      }
      for (int i = 0; i < ids.length; i++) {
        if (ids[i] != 0) return FieldNode.NODE_MORE;
      }
      return FieldNode.NODE_END;
    }
    return FieldNode.NODE_END;
  }
  
  // Incoming references
  
  private void expandInRefFieldNode(TreeNode node) {
    int objectID = ((InstDumpInfo) node.getNodeData()).objectID;
    Object[] objects = namespace.getObjdumpTable().elements();
    for (int i = 0; i < objects.length; i++) {
      if (objects[i] != null) {
        InstDumpInfo objdump = (InstDumpInfo) objects[i];
               if (objdump.getType() == InstDumpInfo.TYPE_OBJECT) {
          ObjectDumpInfo objDumpInfo = (ObjectDumpInfo) objdump;
          ClassDumpInfo currClass = objDumpInfo.classDumpInfo;
          int fieldNo = 0;
          int[] ids = objDumpInfo.values;
          for (int j = 0; j < ids.length; j++) {
            while (fieldNo >= currClass.instanceFields.length) {
              currClass = currClass.superClassInfo;
              fieldNo = 0;
            }
            if (objectID == ids[j]) createInRefsNode(node, objdump, currClass.instanceFields[fieldNo].name);
            fieldNo++;
          }
        } else if (objdump.getType() == InstDumpInfo.TYPE_ARRAY) {
          int[] ids = ((ObjArrayDumpInfo) objdump).values;
          for (int j = 0; j < ids.length; j++) {
            if (objectID == ids[j]) createInRefsNode(node, objdump, "element[" + j + "]");
          }
        }
      }
    }
    Object[] classes = namespace.getClsdumpTable().elements();
    for (int i = 0; i < classes.length; i++) {
      if (classes[i] != null) {
        ClassDumpInfo clsDumpInfo = (ClassDumpInfo) classes[i];
        for (int j = 0; j < clsDumpInfo.staticFields.length; j++) {
          if (objectID == clsDumpInfo.staticFields[j].value) {
            node.add(new FieldNode(clsDumpInfo.staticFields[j].name, FieldNode.NODE_END, clsDumpInfo, true));
          }
        }
      }
    }
    Vector javaFrames = namespace.getJavaFrameRootTable();
    JavaFrameRoot javaFrame;
    for (int i = 0; i < javaFrames.size(); i++) {
      javaFrame = (JavaFrameRoot) javaFrames.elementAt(i);
      if (objectID == javaFrame.objectID) {
        ThreadInfo threadInfo = namespace.lookupThread(javaFrame.threadSerialNo);
        if (threadInfo != null) {
          node.add(new FieldNode("Java stack", FieldNode.NODE_END, threadInfo, true));
        }
      }
    }
    Vector nativeStacks = namespace.getNativeStackRootTable();
    NativeStackRoot nativeStack;
    for (int i = 0; i < nativeStacks.size(); i++) {
      nativeStack = (NativeStackRoot) nativeStacks.elementAt(i);
      if (objectID == nativeStack.objectID) {
        ThreadInfo threadInfo = namespace.lookupThread(nativeStack.threadSerialNo);
        if (threadInfo != null) {
          node.add(new FieldNode("Native stack", FieldNode.NODE_END, threadInfo, true));
        }
      }
    }
  }
  
  private void createInRefsNode(TreeNode node, InstDumpInfo obj, String fieldName) {
    int state = getInRefStateFor(node, obj);
    FieldNode child = new FieldNode(fieldName, state, obj, true);
    node.add(child);
    if (state == FieldNode.NODE_MORE) child.add(new TreeNodeImpl(Misc.EMPTY_STRING));
  }
  
  private int getInRefStateFor(TreeNode node, InstDumpInfo instDumpInfo) {
    TreeNode parent = node;
    while (parent != null) {
      if (parent.getNodeData() == instDumpInfo) return FieldNode.NODE_LOOP;
      parent = parent.getParent();
    }
    int objectID = instDumpInfo.objectID;
    Object[] objects = namespace.getObjdumpTable().elements();
    for (int i = 0; i < objects.length; i++) {
      if (objects[i] != null) {
        InstDumpInfo objdump = (InstDumpInfo) objects[i];
               if (objdump.getType() == InstDumpInfo.TYPE_OBJECT) {
          int[] ids = ((ObjectDumpInfo) objdump).values;
          for (int j = 0; j < ids.length; j++) {
            if (objectID == ids[j]) return FieldNode.NODE_MORE;
          }
        } else if (objdump.getType() == InstDumpInfo.TYPE_ARRAY) {
          int[] ids = ((ObjArrayDumpInfo) objdump).values;
          for (int j = 0; j < ids.length; j++) {
            if (objectID == ids[j]) return FieldNode.NODE_MORE;
          }
        }
      }
    }
    Object[] classes = namespace.getClsdumpTable().elements();
    for (int i = 0; i < classes.length; i++) {
      if (classes[i] != null) {
        ClassDumpInfo clsDumpInfo = (ClassDumpInfo) classes[i];
        for (int j = 0; j < clsDumpInfo.staticFields.length; j++) {
          if (objectID == clsDumpInfo.staticFields[j].value) return FieldNode.NODE_MORE;
        }
      }
    }
    return FieldNode.NODE_END;
  }
  
  //  Allocation stack trace
  
  private void setStackTrace(TraceInfo traceInfo) {
    Vector v = new Vector();
    if (traceInfo == null || traceInfo.numberOfFrames == 0) {
      v.addElement(TraceInfo.UNKNOWN_TRACE);
    } else {
      FrameInfo[] frameInfo = traceInfo.frameInfo;
      for (int i = 0; i < frameInfo.length; i++) {
        v.addElement(frameInfo[i]);
      }
    }
    allocList.getModel().setListData(v);
  }
  
  //  ListSelectionListener
  
  public void valueChanged(ListSelectionEvent ev) {
    InstDumpInfo instDumpInfo = null;
    if (ev.getSource() == instList) {
      instDumpInfo = (InstDumpInfo) instList.getSelectedValue();
      if (instDumpInfo != null) statList.getSelectionModel().clearSelection();
    }
    if (ev.getSource() == statList) {
      FieldNode node = (FieldNode) statList.getSelectedValue();
      if (node != null) {
        instDumpInfo = (InstDumpInfo) node.getNodeData();
        if (instDumpInfo != null) instList.getSelectionModel().clearSelection();
      }
    }
    setRefsForObject(instDumpInfo);
  }
  
  //  TableSelectionListener
  
  public void valueChanged(TableSelectionEvent ev) {
    setDataForCurrentClass();
  }
  
  // Source
  
  public void showSource() {
    if (allocList.hasFocus()) {
      Object frame = allocList.getSelectedValue();
      if (frame != null && frame instanceof FrameInfo) actionMng.actionSource((FrameInfo) frame);
    }
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == outrefRB) {
      isOutRef = true;
      if (objRoot.getChildCount() > 0) {
        setRefsForObject((InstDumpInfo) objRoot.getChildAt(0).getNodeData());
      }
    } else if (src == inrefRB) {
      isOutRef = false;
      if (objRoot.getChildCount() > 0) {
        setRefsForObject((InstDumpInfo) objRoot.getChildAt(0).getNodeData());
      }
    }
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == clsTable || src == clsTableScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.ClassTable");
    }
    if (src == instList || src == instListScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.InstList");
    }
    if (src == statList || src == statListScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.StatList");
    }
    if (src == superTree || src == superTreeScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.SuperTree");
    }
    if (src == objTree || src == objTreeScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.ObjectTree");
    }
    if (src == allocList || src == allocListScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.AllocTrace");
    }
    if (src == clsFilter) {
      if (keyCode == KeyEvent.VK_ENTER) {
        cfilters = clsFilter.getText();
        StringTokenizer st = new StringTokenizer(cfilters, ",");
        String[] filtersIA = new String[st.countTokens()];
        String[] filtersEA = new String[st.countTokens()];
        int fi = 0, fe = 0;
        while (st.hasMoreTokens()) {
          String filter = st.nextToken().trim();
          if (filter.length() == 0) continue;
          if (filter.charAt(0) == '!') filtersEA[fe++] = filter.substring(1);
          else                         filtersIA[fi++] = filter;
        }
        cfiltersI = null;
        cfiltersE = null;
        if (fi != 0) { cfiltersI = new String[fi]; System.arraycopy(filtersIA, 0, cfiltersI, 0, fi); }
        if (fe != 0) { cfiltersE = new String[fe]; System.arraycopy(filtersEA, 0, cfiltersE, 0, fe); }
        setClassTable();
      }
      if (keyCode == KeyEvent.VK_ESCAPE) {
        clsFilter.setText(cfilters);
      }
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.heap.ClassFilters");
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
  
  //  FocusListener
  
  public void focusGained(FocusEvent fe) { }
  
  public void focusLost(FocusEvent fe) {
    if (fe.getComponent() == clsFilter) clsFilter.setText(cfilters);
  }
  
  //  MouseListener
  
  public void mouseClicked(MouseEvent me) {
    if (me.getModifiers() != MouseEvent.BUTTON1_MASK) return;
    Object src = me.getSource();
    if (src instanceof PTableHeader) {
      int col = clsTable.getColumnAtX(me.getX());
      if (me.getClickCount() == 1 && col != -1) sortColumn(col);
    }
    if (me.getClickCount() != 2) return;
    if (src == clsTable) {
      int col = clsTable.getColumnAtX(me.getX());
      if (col == LOADER) gotoClassLoader();
    }
    if (src == superTree) {
      TreeNode selectedNode = superTree.getSelectedNode();
      if (selectedNode == null || !(selectedNode.getNodeData() instanceof ClassDumpInfo)) return;
      ClassDumpInfo classDumpInfo = (ClassDumpInfo) selectedNode.getNodeData();
      Vector v = clsTableModel.getData();
      int row = v.indexOf(classDumpInfo);
      clsTable.selectRows(row, row);
      clsTable.scrollRowToVisible(row);
    }
    if (src == objTree) {
      TreeNode selectedNode = objTree.getSelectedNode();
      if (selectedNode == null) return;
      Object o = selectedNode.getNodeData();
      if (o instanceof InstDumpInfo) {
        InstDumpInfo obj = (InstDumpInfo) o;
        setStackTrace(obj.traceInfo);
      }
    }
    if (src == allocList) {
      Object frame = allocList.getSelectedValue();
      if (frame != null && frame instanceof FrameInfo) actionMng.actionSource((FrameInfo) frame);
    }
  }
  
  public void mousePressed(MouseEvent me) { }
  public void mouseReleased(MouseEvent me) { }
  public void mouseEntered(MouseEvent me) { }
  public void mouseExited(MouseEvent me) { }
  
  // TreeShouldExpansionListener
  
  private boolean block = false;
  
  public boolean treeShouldExpand(TreeExpansionEvent event) {
    if (block) return false;
    block = true;
    expand(event.getNode());
    block = false;
    return true;
  }
  
  public void expand(TreeNode node) {
    if (!(node instanceof FieldNode)) return;
    FieldNode fnode = (FieldNode) node;
    if (fnode.state == FieldNode.NODE_MORE) {
      node.removeAllChildren();
      if (isOutRef) expandOutRefFieldNode(node);
      else expandInRefFieldNode(node);
      fnode.state = FieldNode.NODE_EXPN;
    }
  }
  
  public boolean treeShouldCollapse(TreeExpansionEvent event) {
    return true;
  }
  
  // Table
  
  private void sortColumn(int column) {
    removeColumnHeader();
    if (sortedBy == column) {
      ascending = !ascending;
    } else {
      sortedBy  = column;
      ascending = true;
    }
    setColumnHeader();
    Vector v = clsTableModel.getData();
    ClassDumpInfo classDump = null;
    int row = clsTable.getSelectedRow();
    if (row != -1) classDump = (ClassDumpInfo) v.elementAt(row);
    sort(v, 0, v.size() - 1);
    clsTableModel.setData(v);
    clsTable.updateTable();
    if (classDump != null) {
      for (int i = 0; i < v.size(); i++) {
        if (v.elementAt(i) == classDump) {
          clsTable.selectRows(i, i);
          clsTable.scrollRowToVisible(i);
          return;
        }
      }
    }
  }
  
  private void removeColumnHeader() {
    TableColumn column = clsTable.getColumn(sortedBy);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (SamplerTableRenderer.noIcon != null) renderer.setIcon(SamplerTableRenderer.noIcon);
  }
  
  private void setColumnHeader() {
    TableColumn column = clsTable.getColumn(sortedBy);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (ascending) {
      if (SamplerTableRenderer.downIcon != null) renderer.setIcon(SamplerTableRenderer.downIcon);
    } else {
      if (SamplerTableRenderer.upIcon   != null) renderer.setIcon(SamplerTableRenderer.upIcon);
    }
    clsTable.getHeader().repaint();
  }
  
  //  Sort
  
  public void sort(Vector v, int lo0, int hi0) {
    if (lo0 >= hi0) return;
    int lo = lo0;
    int hi = hi0;
    Object mid = v.elementAt((lo + hi) / 2);
    while (lo <= hi) {
      while ((lo < hi0) && (compare(v.elementAt(lo), mid) < 0)) lo++;
      while ((lo0 < hi) && (compare(v.elementAt(hi), mid) > 0)) hi--;
      if (lo <= hi) {
        Object temp = v.elementAt(lo);
        v.setElementAt(v.elementAt(hi), lo);
        v.setElementAt(temp, hi);
        lo++;
        hi--;
      }
    }
    if (lo0 < hi) sort(v, lo0, hi);
    if (lo < hi0) sort(v, lo, hi0);
  }
  
  private int compare(Object info1, Object info2) {
    if (sortedBy == CLASS) {
      if (ascending) return Misc.compareToIgnoreCase(info1.toString(), info2.toString());
      else           return Misc.compareToIgnoreCase(info2.toString(), info1.toString());
    } else if (sortedBy == LOADER) {
      InstDumpInfo cl1 = ((ClassDumpInfo) info1).classLoaderInfo;
      InstDumpInfo cl2 = ((ClassDumpInfo) info2).classLoaderInfo;
      if (ascending) return Misc.compareToIgnoreCase((cl1 == null ? Misc.SYSTEM_CLASS_LOADER : cl1.toString()),
                                                     (cl2 == null ? Misc.SYSTEM_CLASS_LOADER : cl2.toString()));
      else           return Misc.compareToIgnoreCase((cl2 == null ? Misc.SYSTEM_CLASS_LOADER : cl2.toString()),
                                                     (cl1 == null ? Misc.SYSTEM_CLASS_LOADER : cl1.toString()));
    } else {
      if (ascending) return ((ClassDumpInfo) info2).numberOfInstances - ((ClassDumpInfo) info1).numberOfInstances;
      else           return ((ClassDumpInfo) info1).numberOfInstances - ((ClassDumpInfo) info2).numberOfInstances;
    }
  }
}