/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.util.Vector;

import com.prosyst.pgui.table.TableModel;
import com.prosyst.pgui.event.EventListenerList;
import com.prosyst.pgui.event.TableModelEvent;
import com.prosyst.pgui.event.TableModelListener;

import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.AllocSiteGroup;
import com.prosyst.jpd.profiler.info.ClassDumpInfo;
import com.prosyst.jpd.profiler.info.Misc;

/**
 * Default table model for AllocSiteTable and CpuSampleTable.
 *
 * @author Ivo Karabashev
 */
public class SamplerTableModel implements TableModel {
  
  private EventListenerList listenerList = new EventListenerList();
  private String[] columnNames;
  protected Vector data = new Vector();
  
  public SamplerTableModel(String[] columnNames) {
    this.columnNames = columnNames;
  }
  
  public void setData(Vector data) {
    fireTableChanged(new TableModelEvent(this, 0, getRowCount() - 1,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
    this.data = data;
    fireTableChanged(new TableModelEvent(this, 0, getRowCount() - 1,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
  }
  
  public Vector getData() {
    return data;
  }
  
  public int getRowCount() {
    return data.size();
  }
  
  public int getColumnCount() {
    return columnNames.length;
  }
  
  public String getNameForColumn(int col) {
    return columnNames[col];
  }
  
  public Class getClassForColumn(int col) {
    return getCellValue(0, col).getClass();
  }
  
  public boolean isCellEditable(int row, int col) {
    return false;
  }
  
  public Object getCellValue(int row, int col) {
    Object obj = data.elementAt(row);
    if (obj instanceof AllocSiteInfo || obj instanceof AllocSiteGroup) {
      return obj;
    } else if (obj instanceof ClassDumpInfo) {
      if (col == 0) {
        Object cl = ((ClassDumpInfo) obj).classLoaderInfo;
        if (cl == null) return Misc.SYSTEM_CLASS_LOADER;
        return cl;
      } else if (col == 1) {
        return obj;
      } else {
        return new Integer(((ClassDumpInfo) obj).numberOfInstances);
      }
    } else {
      return ((Object[]) obj)[col];
    }
  }
  
  public void removeAllRows() {
    int rows = getRowCount();
    data.removeAllElements();
    fireTableChanged(new TableModelEvent(this, 0, rows - 1,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
  }
  
  // Unimplemented from TableModel interface
  
  public void setCellValue(Object value, int row, int col) { }
  
  public void addTableModelListener(TableModelListener l) {
    listenerList.add(TableModelListener.class, l);
  }
  
  public void removeTableModelListener(TableModelListener l) {
    listenerList.remove(TableModelListener.class, l);
  }
  
  public void setRows(int newSize) { }
  
  public void removeRow(int row) { }
    
  public void fireTableChanged(TableModelEvent e) {
    Object[] listeners = listenerList.getListenerList();
    for (int i = listeners.length - 2; i >= 0; i -= 2)
      if (listeners[i] == TableModelListener.class)
        ((TableModelListener) listeners[i + 1]).tableChanged(e);
  }
  
  // Insert / remove
  
  public void updateRowAt(int rowindex) {
    fireTableChanged(new TableModelEvent(this, rowindex, rowindex,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE));
  }
  
  public void insertRowAt(Object rowdata, int rowindex) {
    data.insertElementAt(rowdata, rowindex);
    fireTableChanged(new TableModelEvent(this, rowindex, rowindex,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
  }
  
  public void removeRowAt(int rowindex) {
    data.removeElementAt(rowindex);
    fireTableChanged(new TableModelEvent(this, rowindex, rowindex,
                         TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
  }
}