/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.awt.Component;
import java.awt.Color;

import com.prosyst.pgui.Icon;
import com.prosyst.pgui.PLabel;
import com.prosyst.pgui.PTable;
import com.prosyst.pgui.PCheckBox;
import com.prosyst.pgui.border.Border;
import com.prosyst.pgui.border.BevelBorder;
import com.prosyst.pgui.table.TableCellRendererImpl;
import com.prosyst.pgui.table.PTableHeader;
import com.prosyst.gui.images.ImageHolder;

import com.prosyst.jpd.profiler.info.AllocSiteInfo;
import com.prosyst.jpd.profiler.info.AllocSiteGroup;
import com.prosyst.jpd.profiler.info.Misc;
import com.prosyst.jpd.profiler.info.TriggerEvent;

/**
 * SamplerTableRenderer manages headers and cell view in the memory and cpu table.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class SamplerTableRenderer extends TableCellRendererImpl {
  
  protected static final Icon downIcon    = ImageHolder.sortDownIcon;
  protected static final Icon upIcon      = ImageHolder.sortUpIcon;
  protected static final Icon noIcon      = VisualManager.getIcon("SamplerTableRenderer.icon.no");
  
  private static final Icon collapsedIcon = VisualManager.getIcon("SamplerTableRenderer.icon.collapsed");
  private static final Icon expandedIcon  = VisualManager.getIcon("SamplerTableRenderer.icon.expanded");
  
  private static final Border bevelBorder = new BevelBorder(BevelBorder.RAISED);
  private static final String EMPTY_STRING = "";
  
  public boolean isGCDisabled = true;
  public boolean showGroups   = true;
  public int[]   columnIndices;
  public int     timePercentPrecision = 3; // default precision is 0.00, max is 5
  public int     memUnits = 1;
  
  private static final int COLOR_FACTOR = 12;
  private Color defColor, grpColor;
  
  public Component getTableCellRendererComponent(PTable table, Object value,
                   boolean isSelected, boolean hasFocus, int row, int column) {
    if (row == -1) {
      if (table != null) {
        PTableHeader header = table.getHeader();
        if (header != null) {
          setForeground(header.getForeground());
          setBackground(header.getBackground());
          setFont(header.getFont());
        }
      }
      setHorizontalAlignment(CENTER);
      setHorizontalTextPosition(LEADING);
      setBorder(bevelBorder);
      setText((value == null) ? EMPTY_STRING : value.toString());
      return this;
    } else {
      PLabel cell = (PLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
      cell.setOpaque(true);
      if (value instanceof Integer || value instanceof Long) {
        cell.setHorizontalAlignment(RIGHT);
      } else if (value instanceof Float) {
        cell.setHorizontalAlignment(RIGHT);
        cell.setText(Misc.formatPercent(((Float) value).floatValue(), timePercentPrecision));
        return cell;
      } else if (value instanceof AllocSiteInfo) {
        AllocSiteInfo asi = (AllocSiteInfo) value;
        cell.setIcon(null);
        if (column == 0) {
          if (showGroups) {
            cell.setText(EMPTY_STRING);
          } else {
            cell.setHorizontalAlignment(LEFT);
            cell.setText(asi.className);
          }
        } else {
          cell.setHorizontalAlignment(RIGHT);
          if (isGCDisabled) {
                 if (column == columnIndices[1]) cell.setText(Misc.formatMemUnit(asi.bytesAlloc, memUnits));
            else if (column == columnIndices[2]) cell.setText(Integer.toString(asi.instsAlloc, 10));
            else if (column == columnIndices[3]) cell.setText(Misc.formatMemUnit(asi.bytesAllocMark, memUnits));
            else if (column == columnIndices[4]) cell.setText(Integer.toString(asi.instsAllocMark, 10));
            else {
              cell.setHorizontalAlignment(LEFT);
                   if (column == columnIndices[5]) cell.setText(asi.getThreadName());
              else if (column == columnIndices[6]) cell.setText(asi.getFrameName());
            }
          } else {
                 if (column == columnIndices[1]) cell.setText(Misc.formatMemUnit(asi.bytesLive, memUnits));
            else if (column == columnIndices[2]) cell.setText(Integer.toString(asi.instsLive, 10));
            else if (column == columnIndices[3]) cell.setText(Misc.formatMemUnit(asi.bytesLiveMark, memUnits));
            else if (column == columnIndices[4]) cell.setText(Integer.toString(asi.instsLiveMark, 10));
            else {
              cell.setHorizontalAlignment(LEFT);
                   if (column == columnIndices[5]) cell.setText(asi.getThreadName());
              else if (column == columnIndices[6]) cell.setText(asi.getFrameName());
            }
          }
        }
        return cell;
      } else if (value instanceof AllocSiteGroup) {
        AllocSiteGroup asg = (AllocSiteGroup) value;
        if (!isSelected) {
          if (!cell.getBackground().equals(defColor)) {
            defColor = cell.getBackground();
            grpColor = new Color(Math.max(defColor.getRed()   - COLOR_FACTOR, 0),
                                 Math.max(defColor.getGreen() - COLOR_FACTOR, 0),
                                 Math.max(defColor.getBlue()  - COLOR_FACTOR, 0));
          }
          cell.setBackground(grpColor);
        }
        if (column == 0) {
          cell.setHorizontalAlignment(LEFT);
          cell.setHorizontalTextPosition(TRAILING);
          cell.setText(asg.className);
          if (asg.isOpened) cell.setIcon(expandedIcon);
          else              cell.setIcon(collapsedIcon);
        } else {
          cell.setHorizontalAlignment(RIGHT);
          cell.setIcon(null);
          if (isGCDisabled) {
                 if (column == columnIndices[1]) cell.setText(Misc.formatMemUnit(asg.bytesAlloc, memUnits));
            else if (column == columnIndices[2]) cell.setText(Integer.toString(asg.instsAlloc, 10));
            else if (column == columnIndices[3]) cell.setText(Misc.formatMemUnit(asg.bytesAllocMark, memUnits));
            else if (column == columnIndices[4]) cell.setText(Integer.toString(asg.instsAllocMark, 10));
            else { cell.setHorizontalAlignment(RIGHT); cell.setText(EMPTY_STRING); }
          } else {
                 if (column == columnIndices[1]) cell.setText(Misc.formatMemUnit(asg.bytesLive, memUnits));
            else if (column == columnIndices[2]) cell.setText(Integer.toString(asg.instsLive, 10));
            else if (column == columnIndices[3]) cell.setText(Misc.formatMemUnit(asg.bytesLiveMark, memUnits));
            else if (column == columnIndices[4]) cell.setText(Integer.toString(asg.instsLiveMark, 10));
            else { cell.setHorizontalAlignment(RIGHT); cell.setText(EMPTY_STRING); }
          }
        }
        return cell;
      } else if (value instanceof TriggerEvent) {
        TriggerEvent trg = (TriggerEvent) value;
        if (column == 0) {
          PCheckBox check = new PCheckBox();
          check.setOpaque(true);
          check.setSelected(trg.enabled);
          if (isSelected) {
            check.setForeground(table.getSelectionForeground());
            check.setBackground(table.getSelectionBackground());
          } else {
            check.setForeground(table.getForeground());
            check.setBackground(table.getBackground());
          }
          return check;
        }
        return cell;
      } else {
        cell.setHorizontalAlignment(LEFT);
      }
      cell.setText((value == null) ? EMPTY_STRING : value.toString());
      return cell;
    }
  }
}