/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.awt.Component;
import com.prosyst.pgui.Icon;
import com.prosyst.pgui.PTree;
import com.prosyst.pgui.tree.TreeCellRendererImpl;
import com.prosyst.pgui.tree.TreeNodeImpl;

import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.ClassDumpInfo;
import com.prosyst.jpd.profiler.info.InstDumpInfo;

/**
 * SamplerTreeRenderer manages icons in front of threads and methods
 * in the memory and cpu tree.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class SamplerTreeRenderer extends TreeCellRendererImpl {
  
  private static final Icon rootIcon    = VisualManager.getIcon("SamplerTree.icon.root");
  private static final Icon threadIcon  = VisualManager.getIcon("SamplerTree.icon.thread");
  private static final Icon normalIcon  = VisualManager.getIcon("SamplerTree.icon.normal");
  private static final Icon allocIcon   = VisualManager.getIcon("SamplerTree.icon.alloc");
  private static final Icon consumeIcon = VisualManager.getIcon("SamplerTree.icon.consume");
  
  private static final Icon classIcon   = VisualManager.getIcon("HeapTree.icon.class");
  private static final Icon objectIcon  = VisualManager.getIcon("HeapTree.icon.object");
  private static final Icon fieldeIcon  = VisualManager.getIcon("HeapTree.icon.field_e");
  private static final Icon fieldlIcon  = VisualManager.getIcon("HeapTree.icon.field_l");
  private static final Icon fieldmIcon  = VisualManager.getIcon("HeapTree.icon.field_m");
  
  public boolean methodPrc = false;
  public int     timePercentPrecision = 3; // default precision is 0.00, max is 5
  public int     memUnits = 1;
  
  public Component getTreeCellRendererComponent(PTree tree, Object value,
                   boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
    super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
    chooseIcon(value);
    return this;
  }
  
  private void chooseIcon(Object value) {
    TreeNodeImpl node = (TreeNodeImpl) value;
    if (node.isRoot()) {
      setIcon(rootIcon);
      return;
    }
    Object obj = node.getNodeData();
    if (node instanceof AllocSiteNode) {
      setText(((AllocSiteNode) node).toString(memUnits));
      if (obj instanceof ThreadInfo) {
        setIcon(threadIcon);
      } else {
        if (((AllocSiteNode) node).allocSite == null) setIcon(normalIcon);
        else                       setIcon(allocIcon);
      }
      return;
    }
    if (node instanceof CpuSampleNode) {
      CpuSampleNode csn = (CpuSampleNode) node;
      if (obj instanceof ThreadInfo) {
        setText(csn.toString(false, timePercentPrecision));
        setIcon(threadIcon);
      } else {
        if (methodPrc) {
          setText(csn.toString(true, timePercentPrecision));
        } else {
          setText(csn.toString(false, timePercentPrecision));
        }
        if (csn.cpuSample == null) setIcon(normalIcon);
        else                       setIcon(consumeIcon);
      }
      return;
    }
    if (obj instanceof ClassDumpInfo) {
      setIcon(classIcon);
      return;
    }
    if (node instanceof FieldNode) {
      switch (((FieldNode) node).state) {
        case FieldNode.NODE_END:  { setIcon(fieldeIcon); return; }
        case FieldNode.NODE_LOOP: { setIcon(fieldlIcon); return; }
      }
      setIcon(fieldmIcon);
      return;
    }
    if (obj instanceof InstDumpInfo) {
      setIcon(objectIcon);
      return;
    }
  }
  
  public void setIcon(Icon icon) {
    if (icon != null) super.setIcon(icon);
  }
}