/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.awt.BorderLayout;

import com.prosyst.pgui.PPanel;
import com.prosyst.pgui.PLabel;

import com.prosyst.jpd.profiler.info.FrameInfo;
import com.prosyst.gui.editor.JavaTokenMarker;
import com.prosyst.gui.editor.ProTextEdit;
import com.prosyst.gui.find.Searchable;
import com.prosyst.util.log.Log; // from Util project

public class SourcePanel extends PPanel implements Searchable {
  
  private static final String NOT_EXIST = ResourceManager.getString("Source.not_found");
  
  private PLabel      titleLbl = new PLabel(" ", PLabel.CENTER);
  private ProTextEdit sourceArea;
  
  //  Constructor
  
  public SourcePanel() {
    super(new BorderLayout());
  }
  
  public void init() {
    if (sourceArea != null) return;
    sourceArea = new ProTextEdit();
    sourceArea.setEditable(false);
    sourceArea.getDocument().setTokenMarker(new JavaTokenMarker());
    add(titleLbl,   BorderLayout.NORTH);
    add(sourceArea, BorderLayout.CENTER);
  }
  
  public void clear() {
    titleLbl.setText(" ");
    if (sourceArea != null) sourceArea.setText("");
  }
  
  /**
   * Shows the source file and positions to the given line.
   *
   * @param   file  the source file
   * @param   line  the source line
   */
  public void setSource(File file, int line) {
    init();
    if (file == null) {
      titleLbl.setText(NOT_EXIST);
      sourceArea.setText("");
      return;
    }
    try {
      setSource(file.toString(), new FileInputStream(file), line);
    } catch (Exception ex) {
      Log.printDebug("SOURCE.setSource", ex);
    }
  }
  
  public void setSource(String srcName, InputStream is, int line) {
    init();
    titleLbl.setText(srcName + ":" + FrameInfo.toStringLineNumber(line));
    StringBuffer buffer = new StringBuffer();
    try {
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String tempLine;
      while ((tempLine = br.readLine()) != null) {
        buffer.append(tempLine + "\n");
      }
      br.close();
    } catch (java.io.IOException ioex) {
      Log.printDebug("SOURCE.setSource", ioex);
    }
    sourceArea.setText(buffer.toString());
    sourceArea.setCaretPosition((line > 0 && line <= sourceArea.getNumberOfLines()
                               ? sourceArea.getBeginOfLine(line - 1) : 0));
    sourceArea.requestFocus();
  }
  
  //
  // Searchable implementation
  //
  
  public void search(String search, boolean direction, boolean caseSensitive, boolean wholeWord) {
    if (sourceArea == null) return;
    if (direction) {
      if (caseSensitive) {
        sourceArea.findAndSeekUp(search, wholeWord);
      } else {
        sourceArea.findAndSeekLowerCaseUp(search, wholeWord);
      }
    } else {
      if (caseSensitive) {
        sourceArea.findAndSeek(search, wholeWord);
      } else {
        sourceArea.findAndSeekLowerCase(search, wholeWord);
      }
    }
  }

  public void replace(String search, String replace, boolean direction, boolean caseSensitive, boolean wholeWord) { }

  public void replaceAll(String search, String replace, boolean caseSensitive, boolean wholeWord) { }
  
  public String getSelectedText() {
    if (sourceArea == null) return null;
    return sourceArea.getSelectedText();
  }
  
  public void setSelectedSearch() { }
}