/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.util.Vector;

import com.prosyst.pgui.PComponent;
import com.prosyst.pgui.PScrollBar;

import com.prosyst.jpd.profiler.binbody.BinRecord;
import com.prosyst.jpd.profiler.info.Misc;
import com.prosyst.jpd.profiler.info.MonitorInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;

import com.prosyst.util.log.Log;                 // from Util project

public class ThreadsCanvas extends PComponent implements AdjustmentListener {
  
  /** Shows ten seconds from the graph in the component. */
  public static final int SCALE_10SEC = 10;
  /** Shows one minute from the graph in the component. */
  public static final int SCALE_1MIN = 60;
  /** Shows two minutes from the graph in the component. */
  public static final int SCALE_2MIN = 120;
  /** Shows five minutes from the graph in the component. */
  public static final int SCALE_5MIN = 300;
  
  public static Color BKG_COLOR          = new Color(ResourceManager.getInt("ColorBkgrnd", 0xF7F7FB));
  public static Color BKGHL_COLOR        = new Color(ResourceManager.getInt("ColorBkgrhl", 0xE7EFF3));
  public static Color SCALE_COLOR        = new Color(ResourceManager.getInt("ColorScale",  0xD0E0E8));
  public static Color SCALE_COLOR_DARK   = new Color(ResourceManager.getInt("ColorScaleD", 0x7080A0));
  public static Color SCALE_TIME_COLOR   = new Color(ResourceManager.getInt("ColorScaleT", 0x000000));
  public static Color THREAD_RUN_COLOR   = new Color(ResourceManager.getInt("ColorTRun",   0x00D000));
  public static Color THREAD_BLOCK_COLOR = new Color(ResourceManager.getInt("ColorTBlock", 0xFFD000));
  public static Color THREAD_WAIT_COLOR  = new Color(ResourceManager.getInt("ColorTWait",  0xFF0000));
  public static Color THREAD_SLEEP_COLOR = new Color(ResourceManager.getInt("ColorTSleep", 0x0000FF));

  private static final int LIST_ITEM_HEIGHT = 18;
  
  private PScrollBar scrollBar;
  private Vector     threads = new Vector();
  private Vector     events  = new Vector();
  
  // size dependant variables
  private int        scale_mode = SCALE_10SEC;
  private int        scale_step_w;
  private int        scale_step_time;
  private int        width;
  private int        height;
  private int        extent = 9;
  
  // graph points coherent variables
  private int        cur_time;
  private int        time_index;
  private int        vert_sp;
  
  // Constructor
  
  public ThreadsCanvas(PScrollBar sb) {
    scrollBar = sb;
    _clear();
  }
  
  // Properties
  
  /* Clears variables and repaints. */
  public void clear() {
    _clear();
    repaintGraph();
  }
  
  /* Only clears variables. */
  public void _clear() {
    threads.removeAllElements();
    events.removeAllElements();
    cur_time = 0;
    time_index = 0;
    vert_sp = 0;
    scrollBar.setValues(0, 1, 0, 1);
    scrollBar.setEnabled(false);
  }
  
  /* Changes the display mode between SCALE_10SEC, SCALE_1MIN, SCALE_2MIN, SCALE_5MIN. */
  public void setScale(int sm) {
    switch (sm) {
      case 0:
        scale_mode = SCALE_10SEC; break;
      case 1:
        scale_mode = SCALE_1MIN; break;
      case 2:
        scale_mode = SCALE_2MIN; break;
      case 3:
        scale_mode = SCALE_5MIN; break;
    }
    time_index = 0;
    scrollBar.setValues(0, 1, 0, 1);
    adjustHScale();
    adjustScroll();
    repaintGraph();
  }
  
  private TUpdateThread t_updateThread;
  
  class TUpdateThread extends Thread {
    TUpdateThread(int fe_time) {
      setName("ProfilerThreadsSampler");
      setDaemon(true);
      this.startTime = System.currentTimeMillis();
      this.firstEventTime = fe_time;
    }
    
    protected long startTime;
    protected int firstEventTime;
    
    public void run() {
      try {
        while (true) {
          sleep(1000);
          update();
        }
      } catch (Throwable _) { }
    }
    
    protected void update() {
      setEndTime(firstEventTime + (int) (System.currentTimeMillis() - startTime));
    }
  }
  
  /* Adds a new thread to the list. */
  public void addThread(ThreadInfo thread) {
    if (threads.size() == 0) {
      t_updateThread = new TUpdateThread(thread.threadStartTime);
      t_updateThread.start();
    }
    threads.addElement(thread);
    if ("Finalizer".equals(thread.threadName) || "Reference Handler".equals(thread.threadName)) {
       addFakeEvent(thread);
    }
  }
  
  /* Adds a new point to the graph. */
  public void addEvent(MonitorInfo event) {
    int insertIndex = -1;
    if (event.monitorType == BinRecord.PROF_MONITOR_WAITED) {
      int index = getPreviousEvent(events.size(), event.t_ownerInfo);
      if (getEventType(index) != BinRecord.PROF_MONITOR_WAIT) {
        index = getPreviousEvent(index, event.t_ownerInfo);
        int indexB = getPreviousEvent(index, event.t_ownerInfo);
        if (getEventType(indexB) == BinRecord.PROF_MONITOR_WAIT) insertIndex = index;
      }
    }
    if (insertIndex != -1) events.insertElementAt(event, insertIndex);
    else events.addElement(event);
  }
  
  private int getPreviousEvent(int index, ThreadInfo owner) {
    for (int i = index - 1; i >= 0; i--) {
      if (owner == ((MonitorInfo) events.elementAt(i)).t_ownerInfo) return i;
    }
    return -1;
  }
  
  private byte getEventType(int index) {
    if (index == -1) return -1;
    return ((MonitorInfo) events.elementAt(index)).monitorType;
  }
  
  private void addFakeEvent(ThreadInfo thread) {
    MonitorInfo fakeEvent = new MonitorInfo();
    fakeEvent.monitorType       = BinRecord.PROF_MONITOR_WAIT;
    fakeEvent.monitorObjectType = Misc.JVMPI_NORMAL_OBJECT;
    fakeEvent.monitorName       = Misc.EMPTY_STRING;
    fakeEvent.monitorID         = 0;
    fakeEvent.traceInfo         = null;
    fakeEvent.t_ownerInfo       = thread;
    fakeEvent.ticks             = thread.threadStartTime;
    events.addElement(fakeEvent);
  }
  
  public Vector getEventsFor(ThreadInfo thread) {
    Vector ev = new Vector();
    MonitorInfo monInfo;
    for (int i = 0; i < events.size(); i++) {
      monInfo = (MonitorInfo) events.elementAt(i);
      if (thread == monInfo.t_ownerInfo) ev.addElement(monInfo);
    }
    return ev;
  }
  
  public void stop() {
    if (t_updateThread != null) {
      t_updateThread.update();
      t_updateThread.interrupt();
      t_updateThread = null;
    }
    analyzeEvents();
  }
  
  private void analyzeEvents() {
    for (int j = 0; j < threads.size(); j++) {
      ThreadInfo thread = (ThreadInfo) threads.elementAt(j);
      int mode = -1;
      MonitorInfo lastEvent;
      for (int i = 0; i < events.size(); i++) {
        lastEvent = (MonitorInfo) events.elementAt(i);
        if (thread == lastEvent.t_ownerInfo) {
          if (lastEvent.monitorType == BinRecord.PROF_MONITOR_WAITED)
            if (mode != BinRecord.PROF_MONITOR_WAIT) {
              Log.printDebug("PROBLEM: expected WAIT  got " + Misc.toStringMonitorEvent(mode));
              printEventHistory(lastEvent, i);
            }
          if (lastEvent.monitorType == BinRecord.PROF_MONITOR_ENTERED)
            if (mode != BinRecord.PROF_MONITOR_ENTER) {
              Log.printDebug("PROBLEM: expected ENTER got " + Misc.toStringMonitorEvent(mode));
              printEventHistory(lastEvent, i);
            }
          mode = lastEvent.monitorType;
        }
      }
    }
  }
  
  private void printEventHistory(MonitorInfo lastEvent, int index) {
    Log.printDebug(lastEvent.toStringMonitorEventShort());
    for (int k = 0; k < 6; k++) {
      index = getPreviousEvent(index, lastEvent.t_ownerInfo);
      if (index != -1) Log.printDebug(((MonitorInfo) events.elementAt(index)).toStringMonitorEventShort());
    }
  }
  
  /* Sets the last visible point of the graph. */
  public void setEndTime(int time) {
    if (time <= cur_time) return;
    cur_time = time;
    scrollBar.setEnabled(true);
    adjustScroll();
  }
  
  /* Gets the last visible point of the graph. */
  public int getEndTime() {
    return cur_time;
  }
  
  /* Sets the first visible point of the graph. */
  public void setIndexPoint(int ip) {
    time_index = ip;
    repaintGraph();
  }
  
  /* Sets vertical scroll value. */
  public void setVertical(int vs) {
    vert_sp = vs;
    repaintGraph();
  }
  
  /* Intercepts changes in width and height and adjusts according to them. */
  public void setBounds(int x, int y, int w, int h) {
    if (height != h) {
      height = h;
    }
    if (width != w) {
      width = w;
      adjustHScale();
      adjustScroll();
    }
    super.setBounds(x, y, w, h);
  }
  
  private void adjustHScale() {
    scale_step_w = width / scale_mode;
    if (scale_mode == SCALE_5MIN) scale_step_time = 60;
    else                          scale_step_time = scale_mode / 6;
    if (scale_step_w < 1) scale_step_w = 1;
    extent = (width - 2) / scale_step_w;
  }
  
  private void adjustScroll() {
    int cur_time_sec = (cur_time + 999) / 1000;
    int max = scrollBar.getMaximum();
    if (cur_time_sec > extent) {
      if (scrollBar.getValue() >= max - extent) {
        scrollBar.setValues(cur_time_sec - extent, extent, 0, cur_time_sec);
      } else {
        scrollBar.setMaximum(cur_time_sec);
      }
    } else {
      time_index = 0;
      scrollBar.setValues(0, 1, 0, 1);
      repaintGraph();
    }
  }
  
  // Paint
  
  public void repaintGraph() {
    if (!isShowing()) return;
    repaint(0, 0, width, height);
  }
  
  public void paint(Graphics g) {
    Rectangle r = g.getClipBounds();
    
    // clear background
    g.setColor(BKG_COLOR);
    g.fillRect(r.x, r.y, r.width, r.height);
    
    // draw scale
    drawScale(g);
    
    // draw graph
    MonitorInfo event, lastEvent;
    ThreadInfo thread;
    int rangeBeginTime  = time_index * 1000;
    int rangeEndTime    = Math.min(rangeBeginTime + width * 1000 / scale_step_w, cur_time);
    int eventBeginIndex = -1;
    int eventEndIndex   = events.size();
    boolean hasEvents;
    int tc;
    for (int i = 0; i < events.size(); i++) {
      event = (MonitorInfo) events.elementAt(i);
      if (eventBeginIndex == -1 && event.ticks >= rangeBeginTime) eventBeginIndex = i;
      if (event.ticks > rangeEndTime) { eventEndIndex = i; break; }
    }
    if (eventBeginIndex == -1) eventBeginIndex = 0;
    for (int i = Math.max(0, vert_sp / LIST_ITEM_HEIGHT); i < Math.min(threads.size(), (height + vert_sp - 12) / LIST_ITEM_HEIGHT); i++) {
      thread = (ThreadInfo) threads.elementAt(i);
      int threadStart = Math.max(thread.threadStartTime * scale_step_w / 1000 - time_index * scale_step_w, 0);
      int threadEnd = (thread.threadEndTime == 0 ? cur_time : thread.threadEndTime) * scale_step_w / 1000;
      threadEnd = Math.min(threadEnd - time_index * scale_step_w, width);
      hasEvents = false;
      lastEvent = null;
      for (int j = eventBeginIndex; j < eventEndIndex; j++) {
        event = (MonitorInfo) events.elementAt(j);
        if (event.t_ownerInfo == thread) {
          lastEvent = event;
          switch (event.monitorType) {
            case BinRecord.PROF_MONITOR_ENTER:   g.setColor(THREAD_RUN_COLOR);   break;
            case BinRecord.PROF_MONITOR_ENTERED: g.setColor(THREAD_BLOCK_COLOR); break;
            case BinRecord.PROF_MONITOR_WAIT:    g.setColor(THREAD_RUN_COLOR);   break;
            case BinRecord.PROF_MONITOR_WAITED:
              if (event.isSleep())               g.setColor(THREAD_SLEEP_COLOR);
              else                               g.setColor(THREAD_WAIT_COLOR);
              break;
          }
          tc = event.ticks * scale_step_w / 1000 - time_index * scale_step_w;
          g.fillRect(threadStart, 3 + i * LIST_ITEM_HEIGHT + 7 - vert_sp, tc - threadStart + 1, LIST_ITEM_HEIGHT - 14);
          threadStart = tc;
          hasEvents = true;
        }
      }
      if (!hasEvents && eventBeginIndex != 0) for (int j = eventBeginIndex - 1; j >= 0; j--) {
        event = (MonitorInfo) events.elementAt(j);
        if (event.t_ownerInfo == thread) {
          if (event.monitorType == BinRecord.PROF_MONITOR_ENTER || event.monitorType == BinRecord.PROF_MONITOR_WAIT)
            lastEvent = event;
            break;
        }
      }
      if (lastEvent == null) g.setColor(THREAD_RUN_COLOR);
      else {
        switch (lastEvent.monitorType) {
          case BinRecord.PROF_MONITOR_ENTER:   g.setColor(THREAD_BLOCK_COLOR); break;
          case BinRecord.PROF_MONITOR_WAIT:
            if (lastEvent.isSleep())           g.setColor(THREAD_SLEEP_COLOR);
            else                               g.setColor(THREAD_WAIT_COLOR);
            break;
          case BinRecord.PROF_MONITOR_ENTERED:
          case BinRecord.PROF_MONITOR_WAITED:  g.setColor(THREAD_RUN_COLOR);   break;
        }
      }
      if (threadEnd > threadStart)
        g.fillRect(threadStart, 3 + i * LIST_ITEM_HEIGHT + 7 - vert_sp, threadEnd - threadStart + 1, LIST_ITEM_HEIGHT - 14);
    }
  }
  
  private void drawScale(Graphics g) {
    g.setColor(BKGHL_COLOR);
    for (int i = 1; i < threads.size(); i += 2) {
      g.fillRect(0, 3 + i * LIST_ITEM_HEIGHT - vert_sp, width, LIST_ITEM_HEIGHT);
    }
    g.setColor(SCALE_COLOR);
    for (int i = 0; i < width; i += scale_step_w) {
      g.drawLine(i, 0, i, height);
    }
    g.setColor(SCALE_COLOR_DARK);
    for (int i = (10 - time_index % 10) * scale_step_w; i < width; i += scale_step_w * 10) {
      g.drawLine(i, 0, i, height - 18);
    }
    g.setColor(SCALE_TIME_COLOR);
    int bs = scale_step_time - time_index % scale_step_time;
    for (int i = bs * scale_step_w; i < width;
         i += scale_step_w * scale_step_time, bs += scale_step_time) {
      g.drawLine(i, height - 15, i, height - 12);
      g.drawString(convertTime(time_index + bs), i, height);
    }
  }
  
  private String convertTime(int time) {
    StringBuffer sb = new StringBuffer();
    sb.append(Integer.toString(time / 3600, 10));
    sb.append(":");
    time %= 3600;
    int t = time / 60;
    if (t < 10) sb.append('0');
    sb.append(Integer.toString(t, 10));
    sb.append(":");
    time %= 60;
    if (time < 10) sb.append('0');
    sb.append(Integer.toString(time, 10));
    return sb.toString();
  }
  
  // AdjustmentListener
  
  public void adjustmentValueChanged(AdjustmentEvent ev) {
    setIndexPoint(ev.getValue());
  }
}