/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.awt.BorderLayout;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.util.Vector;

import com.prosyst.pgui.PGUIConstants;
import com.prosyst.pgui.PPanel;
import com.prosyst.pgui.PLabel;
import com.prosyst.pgui.PComboBox;
import com.prosyst.pgui.PList;
import com.prosyst.pgui.PScrollBar;
import com.prosyst.pgui.PScrollPane;
import com.prosyst.pgui.PSplitPane;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.list.ListModel;
import com.prosyst.pgui.event.ChangeListener;
import com.prosyst.pgui.event.ChangeEvent;

import com.prosyst.jpd.profiler.binbody.BinRecord;
import com.prosyst.jpd.profiler.gui.dialogs.ThreadInfoDialog;
import com.prosyst.jpd.profiler.gui.help.HelpManager;
import com.prosyst.jpd.profiler.info.MonitorInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;

import com.prosyst.util.log.Log;                 // from Util project

public class ThreadsPanel extends PPanel implements ItemListener, KeyListener, MouseListener, ChangeListener {
  
  protected ActionManager actionMng;
  protected ThreadInfoDialog tid;
  
  private PLabel        infoLabel = new PLabel();
  private PComboBox     scaleCombo;
  private PList         threadList = new PList();
  private PScrollPane   threadListScrollpane = new PScrollPane(threadList);
  private ThreadsCanvas threadCanvas;
  private PScrollBar    vScrollBar;
  private PScrollBar    hScrollBar = new PScrollBar(com.prosyst.pgui.PGUIConstants.HORIZONTAL);
  private boolean       isFinished;
  
  //  Constructor
  
  public ThreadsPanel(ActionManager actionMng) {
    super(new BorderLayout());
    this.actionMng = actionMng;
    
    scaleCombo = new PComboBox();
    scaleCombo.addElement(ResourceManager.getString("ThreadsPanel.scalecombo.10sec"));
    scaleCombo.addElement(ResourceManager.getString("ThreadsPanel.scalecombo.1min"));
    scaleCombo.addElement(ResourceManager.getString("ThreadsPanel.scalecombo.2min"));
    scaleCombo.addElement(ResourceManager.getString("ThreadsPanel.scalecombo.5min"));
    scaleCombo.setEditable(false);
    scaleCombo.setSelectedItemIndex(0);
    scaleCombo.addItemListener(this);
    scaleCombo.getComboBoxEditor().getEditor().addKeyListener(this);
    
    PPanel upPanel = new PPanel(new BorderLayout());
    upPanel.add(new com.prosyst.pgui.PComponent() {
      private final int SPACE  = 70;
      private final int SPUP   =  4;
      private final int RECT_W = 20;
      private final int RECT_H = 10;
      
      public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(5 * SPACE, 0);
      }
      
      public void paint(java.awt.Graphics g) {
        g.setColor(java.awt.Color.black);
        g.drawString(ResourceManager.getString("ThreadsPanel.legend.run"),               RECT_W + 10, SPUP + 10);
        g.drawString(ResourceManager.getString("ThreadsPanel.legend.block"),     SPACE + RECT_W + 10, SPUP + 10);
        g.drawString(ResourceManager.getString("ThreadsPanel.legend.wait"),  2 * SPACE + RECT_W + 10, SPUP + 10);
        g.drawString(ResourceManager.getString("ThreadsPanel.legend.sleep"), 3 * SPACE + RECT_W + 10, SPUP + 10);
        
        g.setColor(ThreadsCanvas.THREAD_RUN_COLOR);
        g.fillRect(5, SPUP, RECT_W, RECT_H);
        
        g.setColor(ThreadsCanvas.THREAD_BLOCK_COLOR);
        g.fillRect(SPACE + 5, SPUP, RECT_W, RECT_H);
        
        g.setColor(ThreadsCanvas.THREAD_WAIT_COLOR);
        g.fillRect(2 * SPACE + 5, SPUP, RECT_W, RECT_H);
        
        g.setColor(ThreadsCanvas.THREAD_SLEEP_COLOR);
        g.fillRect(3 * SPACE + 5, SPUP, RECT_W, RECT_H);
      }
    }, BorderLayout.WEST);
    upPanel.add(infoLabel,  BorderLayout.CENTER);
    upPanel.add(scaleCombo, BorderLayout.EAST);
    
    PPanel listPanel = new PPanel(new BorderLayout());
    PLabel listLabel = new PLabel(ResourceManager.getString("ThreadsPanel.list.label"));
    listLabel.setBorder(new EtchedBorder());
    listPanel.add(listLabel, BorderLayout.NORTH);
    listPanel.add(threadListScrollpane, BorderLayout.CENTER);
    vScrollBar = threadListScrollpane.getScrollBar(PGUIConstants.VERTICAL);
    threadListScrollpane.getViewport().addChangeListener(this);
    threadList.addMouseListener(this);
    threadList.addKeyListener(this);
    threadListScrollpane.getViewport().getGlassPane().addKeyListener(this);
    
    threadCanvas = new ThreadsCanvas(hScrollBar);
    threadCanvas.addKeyListener(this);
    
    hScrollBar.setUnitIncrement(1);
    hScrollBar.addAdjustmentListener(threadCanvas);
    
    PPanel canvasPanel = new PPanel(new BorderLayout());
    canvasPanel.add(upPanel, BorderLayout.NORTH);
    canvasPanel.add(threadCanvas, BorderLayout.CENTER);
    canvasPanel.add(hScrollBar, BorderLayout.SOUTH);
    
    PSplitPane split = new PSplitPane(PSplitPane.HORIZONTAL_SPLIT);
    split.setLeft (listPanel);
    split.setRight(canvasPanel);
    split.setDividerPosition(160);
    
    add(split);
  }
  
  /* Clears variables and repaints. */
  public void clear() {
    isFinished = false;
    infoLabel.setText("");
    threadList.getModel().removeAllElements();
    threadCanvas.clear();
  }
  
  /* Sets enabled / disabled state. */
  public void setDisabled(boolean state) {
    if (state) infoLabel.setText(ResourceManager.getString("ThreadsPanel.label.disabled"));
  }
  
  /* Adds a new thread to the list. */
  public void addThread(ThreadInfo thread) {
    if (thread.threadEndTime == 0) {
      if (!"Prof listener".equals(thread.threadName) && !"PROF CPU profiler".equals(thread.threadName)) {
        ListModel lm = threadList.getModel();
        lm.addElement(thread);
        threadCanvas.addThread(thread);
      }
    } else {
      threadCanvas.setEndTime(thread.threadEndTime);
    }
  }
  
  /* Adds a new point to the graph. */
  public void addEvent(MonitorInfo event) {
    threadCanvas.addEvent(event);
  }
  
  public void stop() {
    isFinished = true;
    threadCanvas.stop();
  }
  
  private void analyzeThread(ThreadInfo thread) {
    if (thread == null) return;
    Vector ev = threadCanvas.getEventsFor(thread);
    int runTime = thread.threadEndTime;
    if (runTime == 0) runTime = threadCanvas.getEndTime();
    int blockTime = 0;
    int waitTime = 0;
    int sleepTime = 0;
    int lastEvent = 0;
    MonitorInfo monInfo = null;
    for (int i = 0; i < ev.size(); i++) {
      monInfo = (MonitorInfo) ev.elementAt(i);
      switch (monInfo.monitorType) {
        case BinRecord.PROF_MONITOR_ENTER:
        case BinRecord.PROF_MONITOR_WAIT:
          if (lastEvent != 0) Log.printDebug("PROBLEM: expected ENTERED or WAITED got ENTER or WAIT");
          lastEvent = monInfo.ticks;
          break;
        case BinRecord.PROF_MONITOR_ENTERED: 
        case BinRecord.PROF_MONITOR_WAITED:
          if (lastEvent == 0) Log.printDebug("PROBLEM: expected ENTER or WAIT got ENTERED or WAITED");
          int eventTime = monInfo.ticks - lastEvent;
          if (monInfo.monitorType == BinRecord.PROF_MONITOR_ENTERED) blockTime += eventTime;
          else if (monInfo.isSleep()) sleepTime += eventTime;
          else waitTime += eventTime;
          lastEvent = 0;
          break;
      }
    }
    if (lastEvent != 0) {
      int eventTime = runTime - lastEvent;
      if (monInfo.monitorType == BinRecord.PROF_MONITOR_ENTERED) blockTime += eventTime;
      else if (monInfo.isSleep()) sleepTime += eventTime;
      else waitTime += eventTime;
    }
    runTime -= blockTime + waitTime + sleepTime + thread.threadStartTime;
    
    if (tid == null) {
      tid = new ThreadInfoDialog(actionMng.getFrame());
    }
    tid.clear();
    tid.addEvent(thread.threadStartTime, ResourceManager.getString("ThreadInfoDialog.label.threadstart"));
    for (int i = 0; i < ev.size(); i++) {
      monInfo = (MonitorInfo) ev.elementAt(i);
      tid.addEvent(monInfo.ticks, monInfo.toStringMonitorEventDescr());
    }
    if (isFinished) {
      tid.addEvent(thread.threadEndTime == 0 ? threadCanvas.getEndTime() : thread.threadEndTime,
                   ResourceManager.getString("ThreadInfoDialog.label.threadend"));
    }
    tid.showDialog(thread.threadName, runTime, blockTime, waitTime, sleepTime);
  }
  
  //  ItemListener
  
  /* Item listener for scale combo box, changes the time scale in Thread Debugger. */
  public void itemStateChanged(ItemEvent ie) {
    Object src = ie.getSource();
    if (src == scaleCombo) {
      threadCanvas.setScale(scaleCombo.getSelectedItemIndex());
    }
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == threadList || src == threadListScrollpane.getViewport().getGlassPane()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.threaddbg.ThreadDbg");
      if (keyCode == KeyEvent.VK_ENTER) analyzeThread((ThreadInfo) threadList.getSelectedValue());
    }
    if (src == threadCanvas) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.threaddbg.ThreadDbg");
    }
    if (src == scaleCombo.getComboBoxEditor().getEditor()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.threaddbg.ThreadDbg");
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
  
  //  MouseListener
  
  public void mouseClicked(MouseEvent me) {
    Object src = me.getSource();
    if (src == threadList && me.getClickCount() == 2) analyzeThread((ThreadInfo) threadList.getSelectedValue());
  }
  
  public void mousePressed(MouseEvent me) { }
  public void mouseReleased(MouseEvent me) { }
  public void mouseEntered(MouseEvent me) { }
  public void mouseExited(MouseEvent me) { }
  
  //  ChangeListener
  
  public void stateChanged(ChangeEvent ev) {
    threadCanvas.setVertical(vScrollBar.getValue());
  }
}