/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.util.Vector;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.table.PTableHeader;
import com.prosyst.pgui.table.TableColumn;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.utils.filechooser.PFileChooser;
import com.prosyst.util.io.IniFile;              // from Util project

import com.prosyst.jpd.profiler.info.Misc;
import com.prosyst.jpd.profiler.info.TriggerEvent;
import com.prosyst.jpd.profiler.gui.help.HelpManager;
import com.prosyst.jpd.profiler.gui.dialogs.HelpClassLoader;

public class TriggerPanel extends PPanel implements ActionListener, KeyListener, MouseListener {
  
  protected static final String[] columnNames = {
          ResourceManager.getString("TriggerPanel.column.enabled"),
          ResourceManager.getString("TriggerPanel.column.classname"),
          ResourceManager.getString("TriggerPanel.column.method"),
          ResourceManager.getString("TriggerPanel.column.event"),
          ResourceManager.getString("TriggerPanel.column.action") };
  public static final int[]    columnSizes  = { 20, 240, 120, 40, 135 };
  public static final int      ENABLED      = 0;
  public static final int      CLASS        = 1;
  public static final int      METHOD       = 2;
  public static final int      EVENT        = 3;
  public static final int      ACTION       = 4;
  
  private static final String clsTitle      = ResourceManager.getString("TriggerPanel.title.class");
  
  private static final String TRIGGER_SECTION  = "TRIGGERS";
  private static final String ALL_STRING       = "*";
  private static final String TYPE_ENTRY       = "Entry";
  private static final String TYPE_EXIT        = "Exit";
  private static final String ACTION_BEGIN     = "Begin mark";
  private static final String ACTION_CONT      = "Continue mark";
  private static final String ACTION_END       = "End mark";
  private static final String ACTION_MEM_BEGIN = "Begin mem mark";
  private static final String ACTION_MEM_CONT  = "Continue mem mark";
  private static final String ACTION_MEM_END   = "End mem mark";
  private static final String ACTION_CPU_BEGIN = "Begin cpu mark";
  private static final String ACTION_CPU_CONT  = "Continue cpu mark";
  private static final String ACTION_CPU_END   = "End cpu mark";
  private static final String ACTION_EXPORT    = "Export data";
  private static final String ACTION_PAUSE     = "Pause profiling";
  private static final String ACTION_STOP      = "Stop profiling";
  
  private static final String[] typeNames   = { TYPE_ENTRY, TYPE_EXIT };
  private static final String[] actionNames = { ACTION_BEGIN,     ACTION_CONT,     ACTION_END,
                                                ACTION_MEM_BEGIN, ACTION_MEM_CONT, ACTION_MEM_END,
                                                ACTION_CPU_BEGIN, ACTION_CPU_CONT, ACTION_CPU_END,
                                                /* ACTION_EXPORT, */    ACTION_PAUSE,    ACTION_STOP };
  
  private   PFrame                frame;
  protected SamplerTableRenderer  tableRndr     = new SamplerTableRenderer();
  protected SamplerTableModel     trgTableModel = new TriggerTableModel(columnNames);
  protected PTable                trgTable      = new PTable(trgTableModel);
  protected PScrollPane           trgScrollpane = new PScrollPane(trgTable);
  
  protected PTextField            trgClassName  = new PTextField();
  protected PButton               browseBtn, addBtn, removeBtn;
  
  private   int                   sortedBy  = CLASS;
  private   boolean               ascending = true;
  private   Vector                trgB;
  
  //  Constructor
  
  public TriggerPanel(PFrame frame) {
    super(new BorderLayout());
    this.frame = frame;
    
    trgTable.setRowSelectionAllowed(true);
    trgTable.setResizeMode(PTable.RESIZE_OFF);
    trgTable.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    trgTable.addKeyListener(this);
    trgTable.addMouseListener(this);
    TableColumn column;
    for (int i = columnNames.length - 1; i >= 0; i--) {
      column = trgTable.getColumn(i);
      column.setPreferredWidth(columnSizes[i]);
      column.setHeaderRenderer(new SamplerTableRenderer());
      if (i == ENABLED) column.setCellRenderer(tableRndr);
      if (i == EVENT)   column.setCellEditor(createComboBoxEditor(typeNames));
      if (i == ACTION)  column.setCellEditor(createComboBoxEditor(actionNames));
    }
    setColumnHeader();
    
    PTableHeader th = trgTable.getHeader();
    th.addMouseListener(this);
    
    trgClassName.addKeyListener(this);
    
    PPanel btnRPnl = new PPanel(new FlowLayout(FlowLayout.RIGHT, 2, 1));
    btnRPnl.add(browseBtn = createButton("browsebtn"));
    btnRPnl.add(addBtn    = createButton("addbtn"));
    btnRPnl.add(removeBtn = createButton("removebtn"));
    
    PPanel btmPnl = new PPanel(new BorderLayout());
    btmPnl.add(new PLabel(ResourceManager.getString("TriggerPanel.label.classname")), BorderLayout.WEST);
    btmPnl.add(trgClassName, BorderLayout.CENTER);
    btmPnl.add(btnRPnl, BorderLayout.EAST);
    btmPnl.setBorder(new EtchedBorder());
    
    add(trgScrollpane, BorderLayout.CENTER);
    add(btmPnl, BorderLayout.SOUTH);
  }
  
  private PButton createButton(String name) {
    PButton btn = new PButton();
    btn.setToolTipText(ResourceManager.getString("TriggerPanel.label." + name));
    Icon icon = VisualManager.getIcon("TriggerPanel.icon." + name);
    if (icon != null) btn.setIcon(icon);
    Dimension btnSize = btn.getPreferredSize();
    btnSize.height = trgClassName.getPreferredSize().height;
    btn.setPreferredSize(btnSize);
    btn.addActionListener(this);
    return btn;
  }
  
  private ComboBoxEditor createComboBoxEditor(String[] items) {
    ComboBoxEditor cbe = new ComboBoxEditor(items);
    cbe.addActionListener(this);
    return cbe;
  }
  
  //  Properties
  
  public Vector getTriggers() {
    return trgTableModel.getData();
  }
  
  public void setTriggers(Vector triggers) {
    if (triggers == null) triggers = new Vector();
    trgTableModel.setData(triggers);
  }
  
  public void backup() {
    trgB = new Vector();
    for (int i = 0; i < trgTableModel.getRowCount(); i++) {
      trgB.addElement(((TriggerEvent) trgTableModel.getData().elementAt(i)).copy());
    }
  }
  
  public void reload() {
    Vector rows = new Vector();
    for (int i = 0; i < trgB.size(); i++) {
      rows.addElement(((TriggerEvent) trgB.elementAt(i)).copy());
    }
    trgTableModel.setData(rows);
  }
  
  public void load(IniFile iniFile) {
    TriggerEvent trgEvent;
    int num = 0;
    try {
      num = ResourceManager.parseInt(iniFile.getValue(TRIGGER_SECTION, "number"));
    } catch (NumberFormatException _) { }
    
    Vector rows = new Vector();
    for (int i = 0; i < num; i++) {
      try {
        trgEvent = new TriggerEvent();
        trgEvent.enabled    = ResourceManager.convertBoolean(iniFile.getValue(TRIGGER_SECTION, "enabled." + i));
        trgEvent.type       = Byte.parseByte                (iniFile.getValue(TRIGGER_SECTION, "type."    + i));
        trgEvent.className  =                                iniFile.getValue(TRIGGER_SECTION, "class."   + i);
        trgEvent.methodName =                                iniFile.getValue(TRIGGER_SECTION, "method."  + i);
        trgEvent.action     = ResourceManager.parseInt      (iniFile.getValue(TRIGGER_SECTION, "action."  + i));
        rows.addElement(trgEvent);
      } catch (NumberFormatException _) { }
    }
    trgTableModel.setData(rows);
  }
  
  public void save(IniFile iniFile) {
    TriggerEvent trgEvent;
    int num = trgTableModel.getRowCount();
    iniFile.setValue(TRIGGER_SECTION, "number", Integer.toString(num, 10));
    
    for (int i = 0; i < num; i++) {
      trgEvent = (TriggerEvent) trgTableModel.getData().elementAt(i);
      iniFile.setValue(TRIGGER_SECTION, "enabled." + i, String.valueOf(trgEvent.enabled));
      iniFile.setValue(TRIGGER_SECTION, "type."    + i, Integer.toString(trgEvent.type, 10));
      iniFile.setValue(TRIGGER_SECTION, "class."   + i, trgEvent.className);
      iniFile.setValue(TRIGGER_SECTION, "method."  + i, trgEvent.methodName);
      iniFile.setValue(TRIGGER_SECTION, "action."  + i, Integer.toString(trgEvent.action, 10));
    }
  }
  
  public boolean isChanged(IniFile iniFile) {
    TriggerEvent trgEvent;
    int num = 0;
    try {
      num = ResourceManager.parseInt(iniFile.getValue(TRIGGER_SECTION, "number"));
    } catch (NumberFormatException _) { }
    
    if (num != trgTableModel.getRowCount()) return true;
    for (int i = 0; i < num; i++) {
      try {
        trgEvent = new TriggerEvent();
        trgEvent.enabled    = ResourceManager.convertBoolean(iniFile.getValue(TRIGGER_SECTION, "enabled." + i));
        trgEvent.type       = Byte.parseByte                (iniFile.getValue(TRIGGER_SECTION, "type."    + i));
        trgEvent.className  =                                iniFile.getValue(TRIGGER_SECTION, "class."   + i);
        trgEvent.methodName =                                iniFile.getValue(TRIGGER_SECTION, "method."  + i);
        trgEvent.action     = ResourceManager.parseInt      (iniFile.getValue(TRIGGER_SECTION, "action."  + i));
        if (!trgEvent.equalsFull((TriggerEvent) trgTableModel.getData().elementAt(i))) return true;
      } catch (NumberFormatException _) { }
    }
    return false;
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == browseBtn) {
      actionBrowseClass();
    } else if (src == addBtn) {
      addTrigger();
    } else if (src == removeBtn) {
      int row = trgTable.getSelectedRow();
      if (row != -1) trgTableModel.removeRowAt(row);
      if (row >= trgTableModel.getRowCount()) row = trgTableModel.getRowCount() - 1;
      if (row != -1) trgTable.selectRows(row, row);
    } else if (src instanceof ComboBoxEditor) {
      trgTable.stopEditing();
    }
  }
  
  private void actionBrowseClass() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.removeFileFilter(0);
    fileChooser.addFileFilter("Class Files (*.class)", "class");
    String[] theFile = fileChooser.showFileDialog(clsTitle, PFileChooser.LAST_DIR, false, true);
    if (theFile != null && theFile[0].endsWith(".class")) {
      HelpClassLoader loader = new HelpClassLoader();
      String clsName = loader.resolveClassName(theFile[0]);
      if (clsName != null) {
        trgClassName.setText(clsName);
      }
    }
  }
  
  private void addTrigger() {
    String className = trgClassName.getText();
    if (className == null || className.length() < 1) return;
    TriggerEvent trgEvent = new TriggerEvent();
    trgEvent.enabled    = true;
    trgEvent.type       = TriggerEvent.PROF_TRIGGER_ENTRY;
    trgEvent.className  = className;
    trgEvent.methodName = null;
    trgEvent.action     = TriggerEvent.PROF_TRGACT_BEGIN;
    trgTableModel.insertRowAt(trgEvent, trgTableModel.getRowCount());
  }
  
  /* private Method[] getMethods(String className) {
    try {
      DynamicClassLoader cl = new DynamicClassLoader(actionMng.getFullClassPath());
      Class c = cl.loadClass(className, false);
      return c.getDeclaredMethods();
    } catch (Throwable t) {
      Log.printDebug("ACTION.loadClass", t);
      return null;
    }
  } */
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == trgTable) {
      if (keyCode == KeyEvent.VK_F1) {
        ke.consume();
        HelpManager.showHelp("profiler.gui.trigger.Table");
      }
    }
    if (src == trgClassName) {
      if (keyCode == KeyEvent.VK_ENTER)  addTrigger();
      if (keyCode == KeyEvent.VK_ESCAPE) trgClassName.setText("");
      if (keyCode == KeyEvent.VK_F1) {
        ke.consume();
        HelpManager.showHelp("profiler.gui.trigger.ClassName");
      }
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
  
  //  MouseListener
  
  public void mouseClicked(MouseEvent me) {
    if (me.getModifiers() != MouseEvent.BUTTON1_MASK || me.getClickCount() == 2) return;
    Object src = me.getSource();
    if (src instanceof PTableHeader) {
      int col = trgTable.getColumnAtX(me.getX());
      if (col != -1) sortColumn(col);
    }
    if (src == trgTable) {
      int col = trgTable.getColumnAtX(me.getX());
      if (col == ENABLED) {
        int row = trgTable.getSelectedRow();
        if (row == -1) return;
        TriggerEvent te = (TriggerEvent) trgTableModel.getData().elementAt(row);
        te.enabled = !te.enabled;
        trgTableModel.updateRowAt(row);
      }
    }
  }
  
  public void mousePressed(MouseEvent me) { }
  public void mouseReleased(MouseEvent me) { }
  public void mouseEntered(MouseEvent me) { }
  public void mouseExited(MouseEvent me) { }
  
  // Table
  
  private void sortColumn(int column) {
    removeColumnHeader();
    if (sortedBy == column) {
      ascending = !ascending;
    } else {
      sortedBy  = column;
      ascending = true;
    }
    setColumnHeader();
    Vector v = trgTableModel.getData();
    Object obj = null;
    int row = trgTable.getSelectedRow();
    if (row != -1) obj = v.elementAt(row);
    sort(v, 0, v.size() - 1);
    trgTableModel.setData(v);
    trgTable.updateTable();
    if (obj != null) {
      for (int i = 0; i < v.size(); i++) {
        if (v.elementAt(i) == obj) {
          trgTable.selectRows(i, i);
          trgTable.scrollRowToVisible(i);
          return;
        }
      }
    }
  }
  
  private void removeColumnHeader() {
    TableColumn column = trgTable.getColumn(sortedBy);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (SamplerTableRenderer.noIcon != null) renderer.setIcon(SamplerTableRenderer.noIcon);
  }
  
  private void setColumnHeader() {
    TableColumn column = trgTable.getColumn(sortedBy);
    SamplerTableRenderer renderer = (SamplerTableRenderer) column.getHeaderRenderer();
    if (ascending) {
      if (SamplerTableRenderer.downIcon != null) renderer.setIcon(SamplerTableRenderer.downIcon);
    } else {
      if (SamplerTableRenderer.upIcon   != null) renderer.setIcon(SamplerTableRenderer.upIcon);
    }
    trgTable.getHeader().repaint();
  }
  
  //  Sort
  
  public void sort(Vector v, int lo0, int hi0) {
    if (lo0 >= hi0) return;
    int lo = lo0;
    int hi = hi0;
    Object mid = v.elementAt((lo + hi) / 2);
    while (lo <= hi) {
      while ((lo < hi0) && (compare(v.elementAt(lo), mid) < 0)) lo++;
      while ((lo0 < hi) && (compare(v.elementAt(hi), mid) > 0)) hi--;
      if (lo <= hi) {
        Object temp = v.elementAt(lo);
        v.setElementAt(v.elementAt(hi), lo);
        v.setElementAt(temp, hi);
        lo++;
        hi--;
      }
    }
    if (lo0 < hi) sort(v, lo0, hi);
    if (lo < hi0) sort(v, lo, hi0);
  }
  
  private int compare(Object info1, Object info2) {
    if (info1 instanceof TriggerEvent && info2 instanceof TriggerEvent) {
      TriggerEvent te1 = (TriggerEvent) info1;
      TriggerEvent te2 = (TriggerEvent) info2;
      if (sortedBy == METHOD) {
        if (te1.methodName != null || te2.methodName != null) {
          if (te1.methodName == null) return (ascending ? -1 :  1);
          if (te2.methodName == null) return (ascending ?  1 : -1);
          if (ascending) return Misc.compareToIgnoreCase(te1.methodName, te2.methodName);
          else           return Misc.compareToIgnoreCase(te2.methodName, te1.methodName);
        }
      } else if (sortedBy == ACTION) {
        int i = te1.action - te2.action;
        if (i != 0) return (ascending ? i : -i);
      } else if (sortedBy == EVENT) {
        int i = te1.type - te2.type;
        if (i != 0) return (ascending ? i : -i);
      } else if (sortedBy == ENABLED) {
        int i = (te1.enabled ? 1 : 0) - (te2.enabled ? 1 : 0);
        if (i != 0) return (ascending ? i : -i);
      }
      if (ascending) return Misc.compareToIgnoreCase(te1.className, te2.className);
      else           return Misc.compareToIgnoreCase(te2.className, te1.className);
    }
    return 0;
  }

public class TriggerTableModel extends SamplerTableModel {
  
  public TriggerTableModel(String[] columnNames) {
    super(columnNames);
  }
  
  public boolean isCellEditable(int row, int col) {
    if (col == TriggerPanel.ENABLED) return false;
    return true;
  }
  
  public Object getCellValue(int row, int col) {
    TriggerEvent te = (TriggerEvent) data.elementAt(row);
    switch (col) {
      case TriggerPanel.CLASS:
        return te.className;
      case TriggerPanel.METHOD:
        if (te.methodName == null) return ALL_STRING;
        return te.methodName;
      case TriggerPanel.EVENT:
        if (te.type == TriggerEvent.PROF_TRIGGER_ENTRY) return TYPE_ENTRY;
        return TYPE_EXIT;
      case TriggerPanel.ACTION:
        switch (te.action) {
          case TriggerEvent.PROF_TRGACT_BEGIN:     return ACTION_BEGIN;
          case TriggerEvent.PROF_TRGACT_CONT:      return ACTION_CONT;
          case TriggerEvent.PROF_TRGACT_END:       return ACTION_END;
          case TriggerEvent.PROF_TRGACT_MEM_BEGIN: return ACTION_MEM_BEGIN;
          case TriggerEvent.PROF_TRGACT_MEM_CONT:  return ACTION_MEM_CONT;
          case TriggerEvent.PROF_TRGACT_MEM_END:   return ACTION_MEM_END;
          case TriggerEvent.PROF_TRGACT_CPU_BEGIN: return ACTION_CPU_BEGIN;
          case TriggerEvent.PROF_TRGACT_CPU_CONT:  return ACTION_CPU_CONT;
          case TriggerEvent.PROF_TRGACT_CPU_END:   return ACTION_CPU_END;
          case TriggerEvent.PROF_TRGACT_EXPORT:    return ACTION_EXPORT;
          case TriggerEvent.PROF_TRGACT_PAUSE:     return ACTION_PAUSE;
          case TriggerEvent.PROF_TRGACT_STOP:      return ACTION_STOP;
        }
    }
    return data.elementAt(row);
  }
  
  public void setCellValue(Object value, int row, int col) {
    TriggerEvent te = (TriggerEvent) data.elementAt(row);
    switch (col) {
      case TriggerPanel.CLASS:
        te.className = (String) value;
        break;
      case TriggerPanel.METHOD:
        if (ALL_STRING.equals(value)) te.methodName = null;
        else te.methodName = (String) value;
        break;
      case TriggerPanel.EVENT:
        if (TYPE_ENTRY.equals(value)) te.type = TriggerEvent.PROF_TRIGGER_ENTRY;
        else te.type = TriggerEvent.PROF_TRIGGER_EXIT;
        break;
      case TriggerPanel.ACTION:
             if (ACTION_BEGIN.    equals(value)) te.action = TriggerEvent.PROF_TRGACT_BEGIN;
        else if (ACTION_CONT.     equals(value)) te.action = TriggerEvent.PROF_TRGACT_CONT;
        else if (ACTION_END.      equals(value)) te.action = TriggerEvent.PROF_TRGACT_END;
        else if (ACTION_MEM_BEGIN.equals(value)) te.action = TriggerEvent.PROF_TRGACT_MEM_BEGIN;
        else if (ACTION_MEM_CONT. equals(value)) te.action = TriggerEvent.PROF_TRGACT_MEM_CONT;
        else if (ACTION_MEM_END.  equals(value)) te.action = TriggerEvent.PROF_TRGACT_MEM_END;
        else if (ACTION_CPU_BEGIN.equals(value)) te.action = TriggerEvent.PROF_TRGACT_CPU_BEGIN;
        else if (ACTION_CPU_CONT. equals(value)) te.action = TriggerEvent.PROF_TRGACT_CPU_CONT;
        else if (ACTION_CPU_END.  equals(value)) te.action = TriggerEvent.PROF_TRGACT_CPU_END;
        else if (ACTION_EXPORT.   equals(value)) te.action = TriggerEvent.PROF_TRGACT_EXPORT;
        else if (ACTION_PAUSE.    equals(value)) te.action = TriggerEvent.PROF_TRGACT_PAUSE;
        else if (ACTION_STOP.     equals(value)) te.action = TriggerEvent.PROF_TRGACT_STOP;
        break;
    }
  }
}
}