/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.awt.BorderLayout;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.PPanel;
import com.prosyst.pgui.PLabel;
import com.prosyst.pgui.PComboBox;
import com.prosyst.pgui.PScrollBar;

import com.prosyst.jpd.profiler.gui.help.HelpManager;

public class VMInfoPanel extends PPanel implements ItemListener, KeyListener {
  
  private PLabel          viewLabel = new PLabel();
  private PComboBox       viewCombo;
  private PComboBox       scaleCombo;
  private VMInfoComponent vmInfo;
  private PScrollBar      scrollBar;
  private boolean         isDisabled;
  
  //  Constructor
  
  public VMInfoPanel() {
    super(new BorderLayout());
    
    viewCombo = new PComboBox();
    viewCombo.addElement(ResourceManager.getString("VMInfoPanel.viewcombo.bytes"));
    viewCombo.addElement(ResourceManager.getString("VMInfoPanel.viewcombo.insts"));
    viewCombo.addElement(ResourceManager.getString("VMInfoPanel.viewcombo.threads"));
    viewCombo.addElement(ResourceManager.getString("VMInfoPanel.viewcombo.classes"));
    viewCombo.addElement(ResourceManager.getString("VMInfoPanel.viewcombo.gc"));
    viewCombo.setEditable(false);
    viewCombo.setSelectedItemIndex(0);
    viewCombo.addItemListener(this);
    viewCombo.getComboBoxEditor().getEditor().addKeyListener(this);
    
    scaleCombo = new PComboBox();
    //scaleCombo.addElement(ResourceManager.getString("VMInfoPanel.scalecombo.all")); // lots of bugs !!
    scaleCombo.addElement(ResourceManager.getString("VMInfoPanel.scalecombo.1min"));
    scaleCombo.addElement(ResourceManager.getString("VMInfoPanel.scalecombo.2min"));
    scaleCombo.addElement(ResourceManager.getString("VMInfoPanel.scalecombo.5min"));
    scaleCombo.setEditable(false);
    scaleCombo.setSelectedItemIndex(0); // without all scale !!
    scaleCombo.addItemListener(this);
    scaleCombo.getComboBoxEditor().getEditor().addKeyListener(this);
    
    PPanel viewPanel = new PPanel(new BorderLayout());
    viewPanel.add(viewLabel, BorderLayout.WEST);
    viewPanel.add(viewCombo, BorderLayout.EAST);
    
    PPanel cmdPanel = new PPanel(new BorderLayout());
    cmdPanel.add(viewPanel,  BorderLayout.CENTER);
    cmdPanel.add(scaleCombo, BorderLayout.EAST);
    
    scrollBar = new PScrollBar(com.prosyst.pgui.PGUIConstants.HORIZONTAL);
    scrollBar.setUnitIncrement(1);
    vmInfo = new VMInfoComponent(scrollBar);
    scrollBar.addAdjustmentListener(vmInfo);
    vmInfo.addKeyListener(this);
    
    add(cmdPanel,  BorderLayout.NORTH);
    add(vmInfo,    BorderLayout.CENTER);
    add(scrollBar, BorderLayout.SOUTH);
  }
  
  /* Adds a new point to the graph. */
  public void addPoint(VMPoint vmp) {
    vmInfo.addPoint(vmp);
    viewLabel.setText(vmInfo.getLabel());
  }
  
  /* Clears variables and repaints. */
  public void clear() {
    viewLabel.setText("");
    vmInfo.clear();
  }
  
  /* Sets enabled / disabled state. */
  public void setDisabled(boolean state) {
    isDisabled = state;
    if (state) viewLabel.setText(ResourceManager.getString("VMInfoPanel.label.disabled"));
  }
  
  //  ItemListener
  
  /* Item listener for view combo box, changes the view in VM Info. */
  public void itemStateChanged(ItemEvent ie) {
    Object src = ie.getSource();
    if (src == viewCombo) {
      vmInfo.setView(viewCombo.getSelectedItemIndex());
      if (!isDisabled) viewLabel.setText(vmInfo.getLabel());
    }
    if (src == scaleCombo) {
      vmInfo.setScale(scaleCombo.getSelectedItemIndex() + 1); // without all scale !!
    }
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    Object src = ke.getSource();
    int keyCode = ke.getKeyCode();
    if (src == vmInfo) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.vminfo.VMInfo");
    }
    if (src == viewCombo.getComboBoxEditor().getEditor()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.vminfo.VMInfo");
    }
    if (src == scaleCombo.getComboBoxEditor().getEditor()) {
      if (keyCode == KeyEvent.VK_F1) HelpManager.showHelp("profiler.gui.vminfo.VMInfo");
    }
  }
  
  public void keyReleased(KeyEvent ke) { }
  public void keyTyped(KeyEvent ke) { }
}