/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui;

import java.io.File;
import java.util.Hashtable;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;

import com.prosyst.pgui.*;

import com.prosyst.jpd.profiler.gui.dialogs.StartDialog;
import com.prosyst.jpd.profiler.gui.dialogs.PreferencesDialog;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

import com.prosyst.gui.application.PApplication; // from PUtil project
import com.prosyst.gui.CIOTextArea;              // from PUtil project
import com.prosyst.gui.images.ImageHolder;       // from PUtil project
import com.prosyst.gui.iotextarea.LineListener;  // from PUtil project
import com.prosyst.util.log.Log;                 // from Util project

/**
 * VisualManager displays the result from profiler.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class VisualManager implements ActionCodes, LineListener {
  
  public static final int PANEL_MEM     = 0;
  public static final int PANEL_CPU     = 1;
  public static final int PANEL_HEAP    = 2;
  public static final int PANEL_VMINFO  = 3;
  //public static final int PANEL_THREADS = 4;
  public static final int PANEL_SOURCE  = 4;
  public static final int PANEL_CONSOLE = 5;
  
  private boolean       isStandAlone;
  private ActionManager actionMng;
  private PApplication  application;
  private PPanel        panel;
  
  private SourcePanel   sourcePanel;
  private PPanel        consolePanel;
  private CIOTextArea   consoleTextArea;
  private PTabbedPane   tabPane;
  
  private PMenuItem     playMenu;
  private PButton       playButton;
  
  //  Constructor
  
  public VisualManager(ActionManager actionMng, boolean isStandAlone) {
    this.isStandAlone = isStandAlone;
    this.actionMng = actionMng;
    
    consolePanel = createConsolePanel();
    
    if (isStandAlone) {
      tabPane = new PTabbedPane();
      tabPane.addTab(ResourceManager.getString("MainPanel.tabMemory"),  actionMng.allocSampler);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabCPU"),     actionMng.cpuSampler);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabHeap"),    actionMng.heapPanel);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabVMInfo"),  actionMng.vminfoPanel);
      //tabPane.addTab(ResourceManager.getString("MainPanel.tabThreads"), actionMng.threadsPanel);
    
      application = new PApplication(PApplication.AS_FRAME, getTitleShort());
      application.setIconBase(ResourceManager.class);
      try {
        application.setMainIcon("images/" + ResourceManager.getString("Profiler.icon"));
      } catch (IllegalArgumentException ex) {
        Log.printDebug("VISUAL.setMainIcon", "Error in Profiler.icon");
      }
      loadFramePos();
      application.setWindowListener(actionMng);
      application.addActionListener(actionMng);
      
      panel = application.getPanel();
      initMenuBar();
      initToolBar();
      application.setEnabled(false, mask_begin);
      setGroupEnabled(false);
      
      sourcePanel = new SourcePanel();
      
      tabPane.addTab(ResourceManager.getString("MainPanel.tabSource"),  sourcePanel);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabConsole"), consolePanel);
      int tabIndex = PreferencesDialog.getDefaultTab();
      if (tabIndex != -1) setTabIndex(tabIndex);
      panel.add(tabPane, BorderLayout.CENTER);
      tabPane.addChangeListener(actionMng);
    }
  }
  
  private void initMenuBar() {
    addMenu(0, "program");
    addItem(0, "launch",    ACTION_LAUNCH);
    addItem(0, "listen",    ACTION_LISTEN);
    addItem(0, "attach",    ACTION_ATTACH);
    application.addMenuSeparator(0);
    playMenu = addItem(0, "play",      ACTION_PLAY);
    addItem(0, "pause",     ACTION_PAUSE);
    addItem(0, "stop",      ACTION_STOP);
    application.addMenuSeparator(0);
    addItem(0, "pref",      ACTION_PREF);
//    application.addMenuSeparator(0);
//    addItem(0, "eclipse",   ACTION_ECLIPSE);
    application.addMenuSeparator(0);
    addItem(0, "exit",      ACTION_EXIT);
    
    addMenu(1, "data");
    addItem(1, "find",      ACTION_FIND);
    addItem(1, "findnext",  ACTION_FIND_NEXT);
    application.addMenuSeparator(1);
    addItem(1, "loaddata",  ACTION_LOAD_DATA);
    addItem(1, "savedata",  ACTION_SAVE_DATA);
    //addItem(1, "savetext",  ACTION_SAVE_TEXT);
    addItem(1, "cleardata", ACTION_CLEAR_DATA);
    
    addMenu(2, "tools");
    addItem(2, "startmark", ACTION_START_MARK);
    addItem(2, "contmark",  ACTION_CONT_MARK);
    addItem(2, "stopmark",  ACTION_STOP_MARK);
    application.addMenuSeparator(2);
    addItem(2, "gc",        ACTION_GC);
    addItem(2, "source",    ACTION_SOURCE);
    addItem(2, "tree",      ACTION_TREE);
    addItem(2, "heapdump",  ACTION_HEAP_DUMP);
    addItem(2, "monitors",  ACTION_MONITORS);
    
    PMenu helpMenu = HelpManager.getHelpMenu();
    if (helpMenu != null) {
      application.getMenuBar().add(helpMenu, 3);
    }
  }
  
  private void initToolBar() {
    playButton = addButton("play",      ACTION_PLAY);
    addButton("pause",     ACTION_PAUSE);
    addButton("stop",      ACTION_STOP);
    application.addToolSeparator();
    addButton("startmark", ACTION_START_MARK);
    addButton("contmark",  ACTION_CONT_MARK);
    addButton("stopmark",  ACTION_STOP_MARK);
    application.addToolSeparator();
    addButton("gc",        ACTION_GC);
    addButton("tree",      ACTION_TREE);
    addButton("heapdump",  ACTION_HEAP_DUMP);
    addButton("monitors",  ACTION_MONITORS);
  }
  
  private PPanel createConsolePanel() {
    PPanel consolePanel = new PPanel(new BorderLayout());
    consoleTextArea = new CIOTextArea();
    if (isStandAlone) consoleTextArea.addLineListener(this);
    PScrollPane consoleScrollPane = new PScrollPane(consoleTextArea);
    consolePanel.add(consoleScrollPane, BorderLayout.CENTER);
    return consolePanel;
  }
  
  private PMenu addMenu(int pos, String s) {
    PMenu menu = application.addMenu(pos, ResourceManager.getString("MenuBar.menu." + s));
    try {
      String vname = ResourceManager.getString("MenuBar.menu.acckey." + s);
      java.lang.reflect.Field field = java.awt.event.KeyEvent.class.getField(vname);
      menu.setMnemonic(field.getInt(null));
    } catch (Exception _) { }
    return menu;
  }
  
  private PMenuItem addItem(int pos, String s, int command) {
    return application.addMenuItem(pos, ResourceManager.getString("MenuBar.item." + s),
                               command, getIcon                  ("MenuBar.icon." + s),
                                        ResourceManager.getString("MenuBar.acckey." + s),
                                        ResourceManager.getString("MenuBar.accmod." + s),
                                        ResourceManager.getString("MenuBar.mnem." + s));
  }
  
  private PButton addButton(String s, int command) {
    return application.addToolButton(command, getIcon                  ("ToolBar.icon." + s),
                                              ResourceManager.getString("ToolBar.tooltip." + s));
  }
  
  //  Properties
  
  /**
   * Gets the string displayed for the application title.
   * It is a concatenation of the application name and version.
   * The actual strings are taken from the properties file.
   *
   * @return     the title
   */
  public static String getTitle() {
    return ResourceManager.getString("AboutDialog.title");
  }
  
  public static String getTitleShort() {
    String title = getTitle();
    int i = title.indexOf("Build");
    if (i != -1) return title.substring(0, i - 1);
    return title;
  }
  
  /**
   * Sets the string displayed for the application title plus status msg.
   *
   * @param   status  the title
   */
  public void setTitle(String status) {
    if (!isStandAlone) return;
    if (status == null || status.length() < 1) application.setTitle(getTitleShort());
    else                                       application.setTitle(getTitleShort() + status);
  }
  
  /**
   * Sets visible state of the application frame.
   *
   * @param   state   if true show the frame, if false hides it
   */
  public void setVisible(boolean state) {
    application.setVisible(state);
  }
  
  /**
   * Gets the parent frame of the application.
   *
   * @return     the parent frame
   */
  public PFrame getFrame() {
    return application.getFrame();
  }
  
  /**
   * Saves the frame size and location of the application.
   */
  public void saveFramePos() {
    if (!ResourceManager.getPersistentBoolean(PreferencesDialog.PROF_SECTION, "savepos", true)) return;
    Dimension fsize = application.getFrame().getSize();
    Point     floc  = application.getFrame().getLocationOnScreen();
    if (floc.x > -1000 && floc.y > -1000) {
      ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "frame.x", floc.x);
      ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "frame.y", floc.y);
    }
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "frame.w", fsize.width);
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "frame.h", fsize.height);
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "split.alloc",
      actionMng.allocSampler.splitpane.getDividerPosition());
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "split.cpu",
      actionMng.cpuSampler.splitpane.getDividerPosition());
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "split.heap.main",
      actionMng.heapPanel.mainSplit.getDividerPosition());
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "split.heap.top",
      actionMng.heapPanel.topSplit.getDividerPosition());
    ResourceManager.setPersistentInt(StartDialog.PROF_SECTION, "split.heap.bottom",
      actionMng.heapPanel.bottomSplit.getDividerPosition());
    try {
      ResourceManager.save();
    } catch (Exception _) { }
  }
  
  /**
   * Loads the frame size and location of the application.
   */
  public void loadFramePos() {
    boolean savepos = ResourceManager.getPersistentBoolean(PreferencesDialog.PROF_SECTION, "savepos", true);
    Dimension fsize = new Dimension();
    Point     floc  = new Point();
    if (savepos) {
      floc.x       = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "frame.x", Integer.MIN_VALUE);
      floc.y       = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "frame.y", Integer.MIN_VALUE);
      fsize.width  = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "frame.w");
      fsize.height = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "frame.h");
    }
    if (!savepos || fsize.width < 1 || fsize.height < 1) {
      fsize = Toolkit.getDefaultToolkit().getScreenSize();
      if (fsize.width  > 640) {
        fsize.width  = (fsize.width  * 3) / 4;
      }
      if (fsize.height > 480) {
        fsize.height = (fsize.height * 2) / 3;
      }
    }
    application.setSize(fsize);
    if (!savepos || floc.x < -1000 || floc.y < -1000) {
      application.setCenterLocation();
    } else {
      application.setLocation(floc);
    }
    if (savepos) {
      int pos;
      pos = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "split.alloc",       200);
      actionMng.allocSampler.splitpane.setDividerPosition(pos);
      pos = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "split.cpu",         200);
      actionMng.cpuSampler.splitpane.setDividerPosition(pos);
      pos = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "split.heap.main",   200);
      actionMng.heapPanel.mainSplit.setDividerPosition(pos);
      pos = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "split.heap.top",    350);
      actionMng.heapPanel.topSplit.setDividerPosition(pos);
      pos = ResourceManager.getPersistentInt(StartDialog.PROF_SECTION, "split.heap.bottom", 400);
      actionMng.heapPanel.bottomSplit.setDividerPosition(pos);
    } else {
      actionMng.heapPanel.topSplit.setDividerPosition(fsize.width / 2);
      actionMng.heapPanel.bottomSplit.setDividerPosition(fsize.width / 32 * 22);
    }
  }
  
  /**
   * Gets the main panel.
   *
   * @return     the main panel
   */
  public PPanel getMainPanel() {
    if (panel == null) {
      panel = new PPanel(new BorderLayout());
      tabPane = new PTabbedPane();
      tabPane.addTab(ResourceManager.getString("MainPanel.tabMemory"),  actionMng.allocSampler);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabCPU"),     actionMng.cpuSampler);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabHeap"),    actionMng.heapPanel);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabVMInfo"),  actionMng.vminfoPanel);
      tabPane.addTab(ResourceManager.getString("MainPanel.tabThreads"), actionMng.threadsPanel);
      setTabIndex(PANEL_MEM);
      panel.add(tabPane, BorderLayout.CENTER);
      tabPane.addChangeListener(actionMng);
    }
    return panel;
  }
  
  /**
   * Gets the console panel.
   *
   * @return     the console panel
   */
  public PPanel getConsolePanel() {
    return consolePanel;
  }
  
  /**
   * Gets the console text area.
   *
   * @return     the console text area
   */
  public CIOTextArea getConsoleTextArea() {
    return consoleTextArea;
  }
  
  /**
   * Gets panel index from the tabbed pane.
   * See PANE_*** fields.
   *
   * @return     panel index
   */
  public int getTabIndex() {
    if (tabPane == null) return -1;
    return tabPane.getSelectedIndex();
  }
  
  /**
   * Sets the appropriate panel in the tabbed pane.
   * See PANE_*** fields.
   *
   * @param   index  panel index
   */
  public void setTabIndex(int index) {
    tabPane.setSelectedIndex(index);
  }
  
  /**
   * Clears the console area.
   */
  public void clearConsole() {
    consoleTextArea.clearConsole();
  }
  
  /**
   * Adds the specified string to the console area.
   *
   * @param   msg  the msg to be appended
   */
  public void addToConsole(String msg) {
    consoleTextArea.append(msg);
    consoleTextArea.setCaretAtEndOfText();
  }
  
  /**
   * Sets process input and output to the console area.
   *
   * @param   process  the profiled process
   */
  public void setProcess(Process process) {
    consoleTextArea.setProcess(process);
  }
  
  /**
   * Shows find dialog of the console area.
   */
  public void showConsoleFind() {
    consoleTextArea.getPopup().performFind();
  }
  
  /**
   * Gets the source panel.
   */
  public SourcePanel getSourcePanel() {
    return sourcePanel;
  }
  
  /**
   * Clears the source panel.
   */
  public void clearSource() {
    actionMng.hasSource = false;
    if (isStandAlone) {
      actionMng.setTabDependentMasks();
      sourcePanel.clear();
    }
  }
  
  /**
   * Shows the source file and positions to the given line.
   *
   * @param   file  the source file
   * @param   line  the source line
   */
  public void showSource(File file, int line) {
    actionMng.setTabDependentMasks();
    setTabIndex(PANEL_SOURCE);
    sourcePanel.setSource(file, line);
  }
  
  public void showSource(String srcName, java.io.InputStream is, int line) {
    actionMng.setTabDependentMasks();
    setTabIndex(PANEL_SOURCE);
    sourcePanel.setSource(srcName, is, line);
  }
  
  /**
   * Changes the description of pause menu item and button.
   *
   * @param   desc  nes description
   */
  public void changePlayDesc(String desc) {
    if (isStandAlone) {
      playMenu.setText(desc);
      playButton.setToolTipText(desc);
    }
  }
  
  private static final int[] mask_begin    = { ACTION_PLAY,       ACTION_FIND,       ACTION_FIND_NEXT,
                                               ACTION_SOURCE,     ACTION_CLEAR_DATA, ACTION_SAVE_DATA,  ACTION_SAVE_TEXT };
  private static final int[] mask_support  = { ACTION_PAUSE,      ACTION_GC,         ACTION_MONITORS };
  private static final int[] mask_starting = { ACTION_PLAY,       ACTION_LOAD_DATA };
  private static final int[] mask_data_ld  = { ACTION_LOAD_DATA,  ACTION_CLEAR_DATA };
  private static final int[] mask_data_sv  = { ACTION_SAVE_DATA,  ACTION_SAVE_TEXT };
  private static final int[] mask_clear    = { ACTION_CLEAR_DATA };
  private static final int[] mask_heapdump = { ACTION_HEAP_DUMP };
  private static final int[] mask_group    = { ACTION_STOP,       ACTION_TREE,
                                               ACTION_START_MARK, ACTION_CONT_MARK,  ACTION_STOP_MARK };
  
  /**
   * Enables or disables buttons.
   *
   * @param   status  if true enables the buttons
   */
  public void setEnabled(int action_code, boolean status) {
    if (isStandAlone) application.setEnabled(status, new int[] { action_code });
  }
  
  /**
   * Enables or disables the stop, tree and mark buttons.
   *
   * @param   status  if true enables the buttons
   */
  public void setGroupEnabled(boolean status) {
    if (isStandAlone) {
      application.setEnabled(status, mask_group);
      if (!status) {
        application.setEnabled(false, mask_support);
        application.setEnabled(false, mask_heapdump);
      }
    }
  }
  
  /**
   * Enables or disables the extended commands.
   *
   * @param   status  if true enables them
   */
  public void setSupportedCommands(boolean status, boolean heapStatus) {
    application.setEnabled(status, mask_support);
    application.setEnabled(heapStatus, mask_heapdump);
  }
  
  /**
   * Enables or disables the play button.
   *
   * @param   status  if true enables the play button
   */
  public void setPlayEnabled(boolean status) {
    setEnabled(ACTION_PLAY, status);
  }
  
  /**
   * Enables or disables the pause button.
   *
   * @param   status  if true enables the pause button
   */
  public void setPauseEnabled(boolean status) {
    setEnabled(ACTION_PAUSE, status);
  }
  
  /**
   * Enables or disables the stop button.
   *
   * @param   status  if true enables the stop button
   */
  public void setStopEnabled(boolean status) {
    setEnabled(ACTION_STOP, status);
  }
  
  /**
   * Enables or disables the playing buttons.
   *
   * @param   status  if true enables the playing buttons
   */
  public void setStartingEnabled(boolean status) {
    if (isStandAlone) {
      application.setEnabled(status, mask_starting);
      if (!status) application.setEnabled(false, mask_clear);
    }
  }
  
  /**
   * Enables or disables load and clear snapshot data buttons.
   *
   * @param   status  if true enables the snapshot data buttons
   */
  public void setDataLoadEnabled(boolean status) {
    if (isStandAlone) application.setEnabled(status, mask_data_ld);
  }
  
  /**
   * Enables or disables save snapshot data buttons.
   *
   * @param   status  if true enables the snapshot data buttons
   */
  public void setDataSaveEnabled(boolean status) {
    if (isStandAlone) application.setEnabled(status, mask_data_sv);
  }
  
  //  LineListener
  
  public void setCaretLineInFile(int lineNumber, String fileName) { }
  
  public void setCaretLineInClass(int lineNumber, String className) {
    className = className.replace('.', '/') + ".java";
    actionMng.showSource(actionMng.getSourcePath(), className, lineNumber);
  }
  
  //  Icon
  
  private static Hashtable iconStorage = new Hashtable();
  
  /**
   * Searches for the icon with the specified key in the property file.
   * The method returns null if the property is not found
   * or there is an error creating the image.
   *
   * @param   key        The property key
   * @return             The icon in the property file with the specified key
   */
  public static Icon getIcon(String key) {
    return getIcon(key, null);
  }
  
  /**
   * Searches for the icon with the specified key in the property file.
   * The method returns the default value argument if the property is not found
   * or there is an error creating the image.
   *
   * @param   key        The property key
   * @param   defValue   A default value
   * @return             The icon in the property file with the specified key
   */
  public static Icon getIcon(String key, Icon defValue) {
    String iconName = ResourceManager.getString(key);
    if ("browse.gif".equals(iconName))   return ImageHolder.browseIcon;
    if ("modify.gif".equals(iconName))   return ImageHolder.editIcon;
    if ("add.gif".equals(iconName))      return ImageHolder.addIcon;
    if ("remove.gif".equals(iconName))   return ImageHolder.removeIcon;
    if ("moveup.gif".equals(iconName))   return ImageHolder.upArrowIcon;
    if ("movedown.gif".equals(iconName)) return ImageHolder.downArrowIcon;
    if ("exit.gif".equals(iconName))     return ImageHolder.exitIcon;
    if ("find.gif".equals(iconName))     return ImageHolder.findIcon;
    if ("search.gif".equals(iconName))   return ImageHolder.searchIcon;
    if ("new.gif".equals(iconName))      return ImageHolder.newItemIcon;
    if ("reset.gif".equals(iconName))    return ImageHolder.defaultIcon;
    String iconPath = "images/" + iconName;
    try {
      Icon icon = (Icon) iconStorage.get(iconPath);
      if (icon != null) return icon;
      java.io.InputStream is = ResourceManager.class.getResourceAsStream(iconPath.trim());
      if (is != null) { icon = new ImageIcon(is); iconStorage.put(iconPath, icon); return icon; }
    } catch (Exception _) { }
    Log.printDebug("VISUAL.getIcon", "Error in " + key);
    return defValue;
  }
}