/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.TextListener;
import java.awt.event.TextEvent;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.pgui.utils.filechooser.PFileChooser;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

/**
 * ExportDialog saves profiling information in the specified file in text form.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class ExportDialog extends CButtonsDialog implements ActionListener, TextListener {
  
  /** user has pressed ok button */
  public static final int RETURN_OK     = 0;
  /** user has pressed cancel */
  public static final int RETURN_CANCEL = 1;
  
  private static final String resTitle    = ResourceManager.getString("ExportDialog.title");
  private static final String browseTitle = ResourceManager.getString("ExportDialog.title.browse");
  
  private PFrame       frame;
  private PTextField   filepathTxt;
  private PRadioButton memBothRB, memAllRB, memDiffRB, memNoneRB;
  private PCheckBox    cpuChk;
  private PButton      okBtn, cancelBtn, browseBtn;
  
  private int          returned;
  
  //  Constructor
  
  public ExportDialog(PFrame frame) {
    super(frame, resTitle, true);
    this.frame = frame;
    setBasePanel(createMainPanel());
    createButtons();
    addHelpButton();
    getRootPane().setDefaultButton(okBtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  private PPanel createMainPanel() {
    PPanel mainPnl = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    mainPnl.setBorder(new EtchedBorder());
    
    PPanel filePnl = new PPanel(new BorderLayout(5, 5));
    filePnl.add(filepathTxt = createTextField(30), BorderLayout.CENTER);
    filePnl.add(browseBtn = createButton("browsebtn"), BorderLayout.EAST);
    filepathTxt.addTextListener(this);
    Dimension btnSize = browseBtn.getPreferredSize();
    btnSize.height = filepathTxt.getPreferredSize().height;
    browseBtn.setPreferredSize(btnSize);
    mainPnl.add(filePnl);
    
    mainPnl.add(memBothRB = createRadioButton("memboth", true));
    mainPnl.add(memAllRB  = createRadioButton("memall",  false));
    mainPnl.add(memDiffRB = createRadioButton("memdiff", false));
    mainPnl.add(memNoneRB = createRadioButton("memnone", false));
    ButtonGroup mem = new ButtonGroup();
    mem.add(memBothRB);
    mem.add(memAllRB);
    mem.add(memDiffRB);
    mem.add(memNoneRB);
    
    mainPnl.add(cpuChk = createCheckBox("cpuall", true));
    
    return mainPnl;
  }
  
  private void createButtons() {
    addButton(okBtn     = createButton("okbtn"));
    addButton(cancelBtn = createButton("cancelbtn"));
  }
  
  private PButton createButton(String name) {
    PButton btn;
         if (name.equals("okbtn"))     btn = createOKButton();
    else if (name.equals("cancelbtn")) btn = createCancelButton();
    else {
      btn = new PButton(ResourceManager.getString("ExportDialog.label." + name));
      Icon icon = VisualManager.getIcon("ExportDialog.icon." + name);
      if (icon != null) btn.setIcon(icon);
    }
    btn.addActionListener(this);
    return btn;
  }
  
  private PTextField createTextField(int col) {
    PTextField txt = new PTextField(col);
    txt.addActionListener(this);
    return txt;
  }
  
  private static PRadioButton createRadioButton(String name, boolean flag) {
    PRadioButton rb = new PRadioButton(ResourceManager.getString("ExportDialog.radio." + name));
    String mnem = ResourceManager.getString("ExportDialog.radio.mnem." + name);
    if (mnem != null) rb.setMnemonic(mnem.charAt(0));
    rb.setSelected(flag);
    return rb;
  }
  
  private static PCheckBox createCheckBox(String name, boolean flag) {
    PCheckBox chk = new PCheckBox(ResourceManager.getString("ExportDialog.check." + name));
    String mnem = ResourceManager.getString("ExportDialog.check.mnem." + name);
    if (mnem != null) chk.setMnemonic(mnem.charAt(0));
    chk.setSelected(flag);
    return chk;
  }
  
  //  Properties
  
  /**
   * Shows export dialog.
   */
  public int showDialog() {
    setVisible(true);
    return returned;
  }
  
  public void setVisible(boolean state) {
    if (state) {
      returned = RETURN_CANCEL;
      actionEnabled();
    }
    super.setVisible(state);
  }
  
  public String getFilePath() {
    return filepathTxt.getText();
  }
  
  public int getMemType() {
    if (memBothRB.isSelected()) return 0;
    if (memAllRB.isSelected())  return 1;
    if (memDiffRB.isSelected()) return 2;
    if (memNoneRB.isSelected()) return 3;
    return 0;
  }
  
  /** Checks whether cpu information should be exported. */
  public boolean shouldExportCpu() {
    return cpuChk.isSelected();
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == okBtn) {
      okButton_actionPerformed();
    } else if (src == cancelBtn) {
      cancelButton_actionPerformed();
    } else if (src == browseBtn) {
      actionBrowseFileName();
    } else {
      super.actionPerformed(e);
    }
  }
  
  private void actionBrowseFileName() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.addFileFilter("Text Files (*.txt)", "txt");
    String[] theFile = fileChooser.showFileDialog(browseTitle, PFileChooser.LAST_DIR, false, false);
    if (theFile != null) {
      filepathTxt.setText(theFile[0]);
      actionEnabled();
    }
  }
  
  protected void performEnterAction(KeyEvent e) {
    okButton_actionPerformed();
  }
  
  protected void okButton_actionPerformed() {
    returned = RETURN_OK;
    setVisible(false);
  }
  
  protected void cancelButton_actionPerformed() {
    returned = RETURN_CANCEL;
    dispose();
  }
  
  public void doHelp() {
    HelpManager.showHelp("profiler.gui.dialogs.ExportDialog");
  }
  
  //  TextListener
  
  public void textValueChanged(TextEvent te) {
    actionEnabled();
  }
  
  private void actionEnabled() {
    if (filepathTxt.getText().length() == 0) {
      okBtn.setEnabled(false);
    } else {
      okBtn.setEnabled(true);
    }
  }
}