/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import com.prosyst.pgui.*;
import com.prosyst.pgui.list.ListModel;
import com.prosyst.pgui.table.TableModel;
import com.prosyst.pgui.tree.TreeNode;
import com.prosyst.pgui.layout.GridBagConstraints2;
import com.prosyst.gui.images.ImageHolder;

import com.prosyst.jpd.profiler.gui.ActionManager;
import com.prosyst.jpd.profiler.gui.AllocSamplerGuiImpl;
import com.prosyst.jpd.profiler.gui.CpuSamplerGuiImpl;
import com.prosyst.jpd.profiler.gui.AllocSiteNode;
import com.prosyst.jpd.profiler.gui.CpuSampleNode;
import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.HeapDumpPanel;
import com.prosyst.jpd.profiler.info.AllocSiteGroup;
import com.prosyst.jpd.profiler.info.AllocSiteInfo;

/**
 * FindDialog searches for given text.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class FindDialog extends PDialog implements ActionListener, KeyListener, ItemListener {
  
  private static final String msgMore = ResourceManager.getString("FindDialog.msg.more");
  
  PLabel         label         = new PLabel(ResourceManager.getString("FindDialog.label.find"));
  PComboBox      findTextCombo = new PComboBox();
  PCheckBox      wholeChk      = new PCheckBox(ResourceManager.getString("FindDialog.chk.whole"));
  PCheckBox      caseChk       = new PCheckBox(ResourceManager.getString("FindDialog.chk.case"));
  PButton        findNextBtn   = new PButton(ResourceManager.getString("FindDialog.btn.next"));
  PButton        cancelBtn     = new PButton(ResourceManager.getString("FindDialog.btn.cancel"));
  boolean        caseMode      = false;
  boolean        wholeMode     = false;
  int            maxRows       = 20;
  int            count         = 0;
  
  ActionManager  actionMng;
  PFrame         frame;
  PPanel         parent;
  PComponent     target;
  String         title;
  String         findText;
  
  int            memUnits;
  boolean        methodPrc;
  int            timePercentPrecision;
  
  //
  //  Constructor
  //
  
  public FindDialog(ActionManager actionMng, PFrame frame, boolean modal) {
    super(frame, "Find", modal);
    this.actionMng = actionMng;
    this.frame = frame;
    getContentPane().add(createMainPanel());
    pack();
    setLocationRelativeTo(frame);
  }
  
  PPanel createMainPanel() {
    createFindPanel();
    createMatchPanel();
    createBtnPanel();
    PPanel panel = new PPanel(new GridBagLayout());
    panel.add(label, new GridBagConstraints2(0, 0, 1, 1, 0.0, 0.0,
              GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    panel.add(findTextCombo, new GridBagConstraints2(1, 0, 2, 1, 1.0, 0.0,
              GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    panel.add(caseChk, new GridBagConstraints2(0, 1, 2, 1, 0.0, 0.0,
              GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    panel.add(wholeChk, new GridBagConstraints2(0, 2, 2, 1, 0.0, 0.0,
              GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    panel.add(cancelBtn, new GridBagConstraints2(3, 1, 1, 1, 0.0, 0.0,
              GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    panel.add(findNextBtn, new GridBagConstraints2(3, 0, 1, 1, 0.0, 0.0,
              GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    return panel;
  }
  
  void createFindPanel() {
    findTextCombo.setEditable(true);
    findTextCombo.setVisibleRows(maxRows);
    findTextCombo.setPreferredSize(new Dimension(120, 18));
    findTextCombo.getComboBoxEditor().getEditor().addKeyListener(this);
    findTextCombo.addItemListener(this);
  }
  
  void createMatchPanel() {
    wholeChk.addKeyListener(this);
    wholeChk.addActionListener(this);
    String mnem = ResourceManager.getString("FindDialog.chk.whole.mnem");
    if (mnem != null) wholeChk.setMnemonic(mnem.charAt(0));
    wholeChk.setSelected(false);
    
    caseChk.addKeyListener(this);
    caseChk.addActionListener(this);
    mnem = ResourceManager.getString("FindDialog.chk.case.mnem");
    if (mnem != null) caseChk.setMnemonic(mnem.charAt(0));
    caseChk.setSelected(false);
  }
  
  void createBtnPanel() {
    findNextBtn.setEnabled(false);
    findNextBtn.addKeyListener(this);
    findNextBtn.addActionListener(this);
    String mnem = ResourceManager.getString("FindDialog.btn.next.mnem");
    if (mnem != null) findNextBtn.setMnemonic(mnem.charAt(0));
    findNextBtn.setPreferredSize(new Dimension(100, 25));
    getRootPane().setDefaultButton(findNextBtn);
    
    Icon icon = ImageHolder.cancelIcon;
    if (icon != null) cancelBtn.setIcon(icon);
    cancelBtn.addKeyListener(this);
    cancelBtn.addActionListener(this);
    cancelBtn.setPreferredSize(new Dimension(100, 25));
  }
  
  //
  //  Properties
  //
  
  /**
   * Shows Find dialog.
   *
   * @param   title       find title
   * @param   panel       the parent panel
   */
  public void showDialog(String title, PPanel panel, PComponent comp) {
    setTitle(title);
    this.title = title;
    parent = panel;
    target = comp;
    findTextCombo.requestFocus();
    actionEnabled();
    super.setVisible(true);
  }
  
  public void setVisible(boolean state) {
    if (!state) actionClose();
    super.setVisible(state);
  }
  
  /**
   * Checks whether find next should be enabled.
   */
  public boolean hasNext() {
    return findTextCombo.getModel().size() > 0;
  }
  
  /**
   * Searches for previous input from the find index.
   */
  public void findNext() {
    if (!isVisible()) { // because search is a long operation
      findTextCombo.requestFocus();
      super.setVisible(true);
    }
    findThread = new FindThread();
    findThread.start();
  }
  
  private FindThread findThread;
  private boolean    findThreadLoop = false;
  
  // Searches for text in separate thread
  class FindThread extends Thread {
    FindThread() {
      setName("FindThread");
      setDaemon(true);
      setPriority(Thread.NORM_PRIORITY - 1);
    }
    
    public void run() {
      try {
        findThreadLoop = true;
        findNextBtn.setEnabled(false);
        setTitle(ResourceManager.getString("FindDialog.title.searching"));
        findNextInThread();
      } catch (Throwable _) {
      }
      setTitle(title);
      findNextBtn.setEnabled(true);
      findThreadLoop = false;
    }
  }
  
  /**
   * Searches for previous input from the find index.
   */
  private void findNextInThread() {
    if (parent == null || target == null) return;
    if (target instanceof PTree) {
      PTree tree = (PTree) target;
      TreeNode node = tree.getSelectedNode();
      if (node == null) node = tree.getRoot();
      if (parent instanceof AllocSamplerGuiImpl) {
        memUnits = ((AllocSamplerGuiImpl) parent).memUnits;
      }
      if (parent instanceof CpuSamplerGuiImpl) {
        methodPrc            = ((CpuSamplerGuiImpl) parent).methodPrc;
        timePercentPrecision = ((CpuSamplerGuiImpl) parent).timePercentPrecision;
      }
      
      TreeNode fnode = null;
      int index = 0;
      for (;;) {
        fnode = (TreeNode) findNext(node, index);
        if (fnode != null) break;
        fnode = (TreeNode) node.getParent();
        if (fnode == null) break;
        index = fnode.getIndex(node) + 1;
        node = fnode;
      }
      if (fnode == null) {
        if (findThreadLoop) POptionPane.showMessageDialog(this, msgMore, title, POptionPane.INFORMATION_MESSAGE);
      } else {
        tree.selectNode(fnode);
        int treerow = tree.getRowForNode(fnode);
        tree.scrollRowToVisible(treerow);
      }
    } else if (target instanceof PList) {
      if (!findNext((PList) target)) {
        if (findThreadLoop) POptionPane.showMessageDialog(this, msgMore, title, POptionPane.INFORMATION_MESSAGE);
      }
    } else if (target instanceof PTable) {
      if (!findNext((PTable) target)) {
        if (findThreadLoop) POptionPane.showMessageDialog(this, msgMore, title, POptionPane.INFORMATION_MESSAGE);
      }
    }
  }
  
  private boolean findNext(PTable table) {
    TableModel model = table.getModel();
    if (parent instanceof AllocSamplerGuiImpl) {
      AllocSamplerGuiImpl sampler = (AllocSamplerGuiImpl) parent;
      int index = table.getSelectedRow();
      int i = index;
      if (i == -1 || table.getCellValue(i, 0) instanceof AllocSiteInfo) i++;
      for (; i < model.getRowCount(); i++) {
        if (!findThreadLoop) return false;
        Object obj = table.getCellValue(i, 0);
        if (obj instanceof AllocSiteGroup) {
          int si = checkAllocGroup(sampler, (AllocSiteGroup) obj, i, (i == index));
          if (si != -1) {
            table.selectRows(si, si);
            table.scrollRowToVisible(si);
            return true;
          }
        } else {
          if (checkAllocInfo(sampler, (AllocSiteInfo) obj)) {
            table.selectRows(i, i);
            table.scrollRowToVisible(i);
            return true;
          }
        }
      }
    } else {
      for (int i = table.getSelectedRow() + 1; i < model.getRowCount(); i++) {
        if (!findThreadLoop) return false;
        for (int j = 0; j < model.getColumnCount(); j++) {
          Object obj = table.getCellValue(i, j);
          if (compareText(obj.toString())) {
            table.selectRows(i, i);
            table.scrollRowToVisible(i);
            return true;
          }
        }
      }
    }
    return false;
  }
  
  private int checkAllocGroup(AllocSamplerGuiImpl sampler, AllocSiteGroup asg, int index, boolean isFirst) {
    if (!isFirst && compareText(asg.getClassName())) return index;
    if (!asg.isOpened) {
      AllocSiteInfo[] groupInfo = sampler.getSorter().getFilteredSites(asg);
      for (int i = 0; i < groupInfo.length; i++) {
        if (checkAllocInfo(sampler, groupInfo[i])) {
          sampler.openTableNode(asg, index);
          return index + i + 1;
        }
      }
    }
    return -1;
  }
  
  private boolean checkAllocInfo(AllocSamplerGuiImpl sampler, AllocSiteInfo asi) {
    if (!sampler.isShowGroups()) {
      if (compareText(asi.getClassName())) return true;
    }
    if (sampler.isThreadVisible()) {
      if (compareText(asi.getThreadName())) return true;
    }
    if (sampler.isFrameVisible()) {
      return compareText(asi.getFrameName());
    }
    return false;
  }
  
  private boolean findNext(PList list) {
    ListModel model = list.getModel();
    for (int i = list.getSelectedIndex() + 1; i < model.size(); i++) {
      if (!findThreadLoop) return false;
      if (compareText(model.elementAt(i).toString())) {
        list.setSelectedIndex(i);
        list.scrollRowToVisible(i);
        return true;
      }
    }
    return false;
  }
  
  private TreeNode findNext(TreeNode node, int index) {
    if (!findThreadLoop) return null;
    TreeNode nextNode;
    if (parent instanceof HeapDumpPanel)       ((HeapDumpPanel)       parent).expand(node);
    if (parent instanceof AllocSamplerGuiImpl) ((AllocSamplerGuiImpl) parent).expandFake(node);
    for (int i = index; i < node.getChildCount(); i++) {
      index = 0;
      nextNode = node.getChildAt(i);
      String cmpText;
      if (nextNode instanceof AllocSiteNode) {
        cmpText = ((AllocSiteNode) nextNode).toString(memUnits);
      } else if (nextNode instanceof CpuSampleNode) {
        cmpText = ((CpuSampleNode) nextNode).toString(methodPrc, timePercentPrecision);
      } else {
        cmpText = nextNode.toString();
      }
      if (compareText(cmpText)) {
        return nextNode;
      } else {
        nextNode = findNext(nextNode, index);
        if (nextNode != null) return nextNode;
      }
    }
    return null;
  }
  
  private boolean compareText(String ctext) {
    if (!caseMode) ctext = ctext.toLowerCase();
    return (wholeMode ? compareWhole(ctext) : (ctext.indexOf(findText) != -1));
  }
  
  private boolean compareWhole(String text) {
    int bidx = text.indexOf(findText);
    if (bidx == - 1) return false;
    
    if (text.length() == findText.length()) return true;
    
    int eidx = bidx + findText.length();
    if ((bidx == 0 || isSeparator(text.charAt(bidx - 1)))
      && (eidx >= text.length() || isSeparator(text.charAt(eidx)))) return true;
    return false;
  }
  
  private static boolean isSeparator(char c) {
    if (c == ':' || c == '.' || c == ' ' || c == '>' || c == '<' || c == '@') return true;
    return false;
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == wholeChk) {
      wholeMode = !wholeMode;
      return;
    }
    if (e.getSource() == caseChk) {
      caseMode = !caseMode;
      return;
    }
    if (e.getSource() == findNextBtn) {
      actionFind();
      return;
    }
    if (e.getSource() == cancelBtn) {
      actionClose();
      return;
    }
  }
  
  private void actionFind() {
    findText = findTextCombo.getComboBoxEditor().getValue().toString();
    if (findText.length() < 1) return;
    addToCombo(findText);
    if (!caseMode) findText = findText.toLowerCase();
    actionMng.setFindNext(true);
    findNext();
  }
  
  private void addToCombo(String findText) {
    String trimText = findText; // no trim !
    int i = findTextCombo.getModel().getElements().indexOf(trimText);
    if (i != -1) {
      findTextCombo.removeElementAt(i);
      findTextCombo.insertElementAt(trimText, 0);
      return;
    }
    count++;
    if (count > maxRows) findTextCombo.removeElementAt(maxRows - 1);
    findTextCombo.insertElementAt(trimText, 0);
    findTextCombo.setSelectedItemIndex(findTextCombo.getModel().getElements().indexOf(trimText));
  }
  
  private void actionClose() {
    if (findThreadLoop) {
      findThreadLoop = false;
    } else {
      super.setVisible(false);
    }
  }
  
  //  KeyListener
  
  public void keyPressed(KeyEvent ke) {
    if (ke.getKeyCode() == KeyEvent.VK_ENTER) {
      actionFind();
    }
    if (ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
      ke.consume();
      actionClose();
    }
  }
  
  public void keyReleased(KeyEvent ke) {
    if (ke.getKeyCode() != KeyEvent.VK_F3) actionEnabled();
  }
  
  public void keyTyped(KeyEvent ke) { }
  
  private void actionEnabled() {
    if (findTextCombo.getComboBoxEditor().getValue().toString().length() == 0) {
      findNextBtn.setEnabled(false);
    } else {
      findNextBtn.setEnabled(true);
    }
  }
  
  //  ItemListener
  
  public void itemStateChanged(ItemEvent lse) {
    actionEnabled();
  }
}