/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.io.File;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 * Resolves class name from path name.
 *
 * @author  Ivo Karabashev
 */
public class HelpClassLoader extends ClassLoader {
  private static final String WRONG_FORMAT = "Wrong class file format";
  private String clsName;
  private String dirPath;
  
  public String resolveClassName(String name) {
    Class res = null;
    try {
      byte[] classData = readFromFile(new File(name));
      if (classData != null) {
        try {
          res = defineClass(null, classData, 0, classData.length);
          clsName = res.getName();
        } catch (Throwable _) {
          int[] cp = init_cp(classData);
          int this_indx = cp[byte2int(classData, cp[0] + 2)]; // constant pool entry CONSTANT_Class_info
          if (classData[this_indx] != 7) throw new IllegalArgumentException(WRONG_FORMAT);
          int name_indx = cp[byte2int(classData, this_indx + 1)];
          int len = byte2int(classData, name_indx + 1);
          clsName = new String(classData, name_indx + 3, len).replace('/', '.');
        }
        dirPath = name.replace('\\', '/');
        int index = dirPath.lastIndexOf(clsName.replace('.', '/'));
        if (index != -1) {
          dirPath = dirPath.substring(0, index - 1);
        }
        return clsName;
      }
    } catch (Throwable _) { }
    return null;
  }
  
  private static final int[] init_cp(byte[] data) {
    int   cp_count = byte2int(data, 8);
    int[] cp = new int[cp_count];
    int   i = 1; // indices in constant pool begin from 1
    int   data_indx = 10; // beginning of constant pool
    while (i < cp_count && data_indx < data.length) {
      cp[i] = data_indx;
      int tag = data[data_indx];
      if (tag < 0 || tag == 2 || tag > 12) throw new IllegalArgumentException(WRONG_FORMAT);
      else if (tag == 1)                   data_indx += byte2int(data, data_indx + 1) + 3; // utf8
      else if (tag == 5 || tag == 6)     { data_indx += offset[tag - 3]; i++; } // next index is invalid
      else                                 data_indx += offset[tag - 3]; // 0, 1, 2 are handled
      i++;
    }
    cp[0] = data_indx; // end of constant pool
    return cp;
  }
  
  private static final int byte2int(byte[] a, int i) {
    int j = a[i];     if (j < 0) j += 256;
    int k = a[i + 1]; if (k < 0) k += 256;
    return (j << 8) + k;
  }
  
  private static final int[] offset = { 5, 5, 9, 9, 3, 3, 5, 5, 5, 5 };
  
  public String getClassName() {
    return clsName;
  }
  
  public String getDirPath() {
    return dirPath;
  }
  
  private byte[] readFromFile(File classFile) {
    InputStream ins = null;
    try {
      if (classFile.isFile()) {
        ins = new FileInputStream(classFile);
        ByteArrayOutputStream buf = new ByteArrayOutputStream();
        int c;
        while ((c = ins.read()) != -1) {
          buf.write(c);
        }
        buf.close();
        ins.close();
        return buf.toByteArray();
      }
    } catch (IOException ex) {
      try {
        if (ins != null) ins.close();
      } catch (IOException _) { }
    }
    return null;
  }
  
  protected Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
    return Class.forName(name);
  }
}