/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

/**
 * ListenDialog for entering the listener port.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class ListenDialog extends CButtonsDialog implements ActionListener {
  
  /** user has pressed start button */
  public static final int RETURN_START  = 0;
  /** user has pressed ok button */
  public static final int RETURN_OK     = 1;
  /** user has pressed cancel */
  public static final int RETURN_CANCEL = 2;
  
  private static final int    DEF_PORT     = com.prosyst.jpd.profiler.ProfilerServer.DEFAULT_PORT;
  private static final int    DEF_DEPTH    = com.prosyst.jpd.profiler.gui.ActionManager.DEFAULT_DEPTH;
  private static final String PROF_SECTION = "OPTIONS";
  private static final String titleListen  = ResourceManager.getString("ListenDialog.title");
  private static final String titleAttach  = ResourceManager.getString("ListenDialog.title.attach");
  private static final String srcpathTitle = ResourceManager.getString("ListenDialog.title.srcpath");
  
  private PFrame     frame;
  private PathDialog pathDialog;
  
  private PLabel     hostLbl    = new PLabel(ResourceManager.getString("ListenDialog.label.host"));
  private PLabel     portLbl    = new PLabel(ResourceManager.getString("ListenDialog.label.port"));
  private PLabel     depthLbl   = new PLabel(ResourceManager.getString("ListenDialog.label.depth"));
  private PTextField hostTxt;
  private PScrollNumberField portTxt, depthTxt;
  private PLabel     srcpathLbl = new PLabel(ResourceManager.getString("ListenDialog.label.srcpath"));
  private PTextField srcpathTxt;
  private PButton    srcpathBtn;
  private PCheckBox  memChk, cpuChk, autoCpuChk, vmChk, threadChk;
  
  private PButton    startBtn, okBtn, cancelBtn;
  
  private int        returned;
  private boolean    isAttach;

  //  Constructor
  
  public ListenDialog(PFrame frame) {
    super(frame, titleListen, true);
    this.frame = frame;
    setBasePanel(createMainPanel());
    createButtons();
    addHelpButton();
    getRootPane().setDefaultButton(startBtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  private PPanel createMainPanel() {
    PPanel mainPanel = new PPanel(new BorderLayout());
    
    PPanel upPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    upPanel.setBorder(new EtchedBorder());
    
    if (isAttach) {
      upPanel.add(hostLbl);
      upPanel.add(hostTxt = createTextField(50));
    }
    
    upPanel.add(portLbl);
    PPanel portPanel = new PPanel(new FlowLayout(0, 0, FlowLayout.LEFT));
    portPanel.add(portTxt = new PScrollNumberField(5));
    portTxt.setMinimum(1);
    portTxt.setMaximum(65535);
    portTxt.setValue(DEF_PORT);
    upPanel.add(portPanel);
    
    upPanel.add(srcpathLbl);
    PPanel srcpathPnl = new PPanel(new BorderLayout(5, 5));
    srcpathPnl.add(srcpathTxt = createTextField(50), BorderLayout.CENTER);
    srcpathPnl.add(srcpathBtn = createButton("srcpathbtn"), BorderLayout.EAST);
    Dimension btnSize = srcpathBtn.getPreferredSize();
    btnSize.height = srcpathTxt.getPreferredSize().height;
    srcpathBtn.setPreferredSize(btnSize);
    upPanel.add(srcpathPnl);
    
    mainPanel.add(upPanel, BorderLayout.NORTH);
    mainPanel.add(createEnablePanel(), BorderLayout.CENTER);
    
    return mainPanel;
  }
  
  private PPanel createEnablePanel() {
    PPanel enablePanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    enablePanel.setBorder(new EtchedBorder());
    
    enablePanel.add(memChk     = createCheckBox("memchk",     false));
    enablePanel.add(cpuChk     = createCheckBox("cpuchk",     false));
    enablePanel.add(autoCpuChk = createCheckBox("autocpuchk", false));
    enablePanel.add(vmChk      = createCheckBox("vmchk",      false));
    threadChk  = createCheckBox("threadchk",  false);
    
    PPanel depthPanel = new PPanel(new FlowLayout(0, 0, FlowLayout.LEFT));
    depthPanel.add(depthLbl);
    depthTxt = new PScrollNumberField(3);
    depthTxt.setMinimum(1);
    depthTxt.setMaximum(256);
    depthTxt.setValue(DEF_DEPTH);
    depthPanel.add(depthTxt);
    enablePanel.add(depthPanel);
        
    return enablePanel;
  }
  
  private void createButtons() {
    addButton(startBtn  = createButton("listenbtn"));
    addButton(okBtn     = createButton("okbtn"));
    addButton(cancelBtn = createButton("cancelbtn"));
  }
  
  private PButton createButton(String name) {
    PButton btn;
         if (name.equals("okbtn"))     btn = createOKButton();
    else if (name.equals("cancelbtn")) btn = createCancelButton();
    else {
      btn = new PButton(ResourceManager.getString("ListenDialog.label." + name));
      Icon icon = VisualManager.getIcon("ListenDialog.icon." + name);
      if (icon != null) btn.setIcon(icon);
    }
    btn.addActionListener(this);
    return btn;
  }
  
  private PCheckBox createCheckBox(String name, boolean flag) {
    PCheckBox chk = new PCheckBox(ResourceManager.getString("ListenDialog.label." + name));
    String mnem = ResourceManager.getString("ListenDialog.mnem." + name);
    if (mnem != null) chk.setMnemonic(mnem.charAt(0));
    chk.setSelected(flag);
    return chk;
  }
  
  private PTextField createTextField(int col) {
    PTextField txt = new PTextField(col);
    txt.addActionListener(this);
    return txt;
  }
  
  //  Properties
  
  /**
   * Shows Listen dialog.
   *
   * @return     return code from dialog: RETURN_OK, RETURN_CANCEL
   */
  public int showDialog(boolean isAttach, boolean state) {
    this.isAttach = isAttach;
    setTitle((isAttach ? titleAttach : titleListen));
    setBasePanel(createMainPanel());
    String text;
    Icon icon;
    if (isAttach) {
      text = ResourceManager.getString("ListenDialog.label.attachbtn");
      icon = VisualManager.getIcon("ListenDialog.icon.attachbtn");
    } else {
      text = ResourceManager.getString("ListenDialog.label.listenbtn");
      icon = VisualManager.getIcon("ListenDialog.icon.listenbtn");
    }
    startBtn.setText(text);
    if (icon != null) startBtn.setIcon(icon);
    startEnabled(state);
    pack();
    setVisible(true);
    return returned;
  }
  
  public void startEnabled(boolean state) {
    startBtn.setEnabled(state);
  }
  
  public void setVisible(boolean state) {
    if (state) {
      returned = RETURN_CANCEL;
      load();
    }
    super.setVisible(state);
  }
  
  /** Gets host name. */
  public String getHost() {
    if (hostTxt == null) return "";
    return hostTxt.getText();
  }
  
  /** Gets port number. */
  public int getPort() {
    return portTxt.getValue();
  }
  
  /** Gets stack depth. */
  public int getDepth() {
    return depthTxt.getValue();
  }
  
  /** Gets source path. */
  public String getSourcePath() {
    return srcpathTxt.getText();
  }
  
  /** Checks whether mem profiling is disabled. */
  public static boolean isMemDisabled() {
    return ResourceManager.getPersistentBoolean(PROF_SECTION, "mem", false);
  }
  
  /** Checks whether cpu profiling is disabled. */
  public static boolean isCpuDisabled() {
    return ResourceManager.getPersistentBoolean(PROF_SECTION, "cpu", false);
  }
  
  /** Checks whether auto start cpu profiling is enabled. */
  public static boolean isAutoCpu() {
    return ResourceManager.getPersistentBoolean(PROF_SECTION, "autocpu", false);
  }
  
  /** Checks whether vm info is disabled. */
  public static boolean isVMDisabled() {
    return ResourceManager.getPersistentBoolean(PROF_SECTION, "vminfo", false);
  }
  
  /** Checks whether threads info is disabled. */
  public boolean isThreadsDisabled() {
    return threadChk.isSelected();
  }
  
  /** Sets host name. */
  public void setHost(String host) {
    if (hostTxt != null) hostTxt.setText(host);
  }
  
  /** Sets port number. */
  public void setPort(int port) {
    portTxt.setText(Integer.toString(port));
  }
  
  /** Sets stack depth. */
  public void setDepth(int depth) {
    depthTxt.setText(Integer.toString(depth));
  }
  
  /** Sets source path. */
  public void setSourcePath(String srcpath) {
    srcpathTxt.setText(srcpath);
  }
  
  /** Sets mem profiling disabled. */
  public void setMemDisabled(boolean state) {
    memChk.setSelected(state);
  }
  
  /** Sets cpu profiling disabled. */
  public void setCpuDisabled(boolean state) {
    cpuChk.setSelected(state);
  }
  
  /** Sets auto start cpu profiling. */
  public void setAutoCpu(boolean state) {
    autoCpuChk.setSelected(state);
  }
  
  /** Sets vm info disabled. */
  public void setVMDisabled(boolean state) {
    vmChk.setSelected(state);
  }
  
  /** Sets threads info disabled. */
  public void setThreadsDisabled(boolean state) {
    threadChk.setSelected(state);
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == startBtn) {
      actionStart();
    } else if (src == okBtn) {
      okButton_actionPerformed();
    } else if (src == cancelBtn) {
      cancelButton_actionPerformed();
    } else if (src == srcpathBtn) {
      actionSrcPath();
    } else {
      super.actionPerformed(e);
    }
  }
  
  private void actionStart() {
    returned = RETURN_START;
    save();
    setVisible(false);
  }
  
  protected void performEnterAction(KeyEvent e) {
    actionStart();
  }
  
  protected void okButton_actionPerformed() {
    returned = RETURN_OK;
    save();
    setVisible(false);
  }
  
  protected void cancelButton_actionPerformed() {
    returned = RETURN_CANCEL;
    load();
    dispose();
  }
  
  private void actionSrcPath() {
//    PFileChooser fileChooser = PFileChooser.getInstance(frame);
//    fileChooser.reinit();
//    String theDir[] = fileChooser.showDirDialog(srcpathTitle, PFileChooser.LAST_DIR, false);
//    if (theDir != null) {
//      String sp = srcpathTxt.getText();
//      if (sp.length() > 0) sp += File.pathSeparator + theDir[0];
//      else sp = theDir[0];
//      srcpathTxt.setText(sp);
//    }
    if (pathDialog == null) {
      pathDialog = new PathDialog(frame);
    }
    int stat = pathDialog.showDialog(srcpathTitle, srcpathTxt.getText(), null, null);
    if (stat == PathDialog.RETURN_OK) {
      srcpathTxt.setText(pathDialog.getPath());
    }
  }
  
  public void doHelp() {
    HelpManager.showHelp((isAttach ? "profiler.gui.dialogs.AttachDialog" : "profiler.gui.dialogs.ListenDialog"));
  }
  
  //  Load / Save
  
  public void load() {
    setHost       (ResourceManager.getPersistentString (PROF_SECTION, "listen.host"));
    setPort       (ResourceManager.getPersistentInt    (PROF_SECTION, "listen.port", DEF_PORT));
    setDepth      (ResourceManager.getPersistentInt    (PROF_SECTION, "listen.depth", DEF_DEPTH));
    setSourcePath (ResourceManager.getPersistentString (PROF_SECTION, "listen.sourcepath"));
    setMemDisabled(ResourceManager.getPersistentBoolean(PROF_SECTION, "mem",     false));
    setCpuDisabled(ResourceManager.getPersistentBoolean(PROF_SECTION, "cpu",     false));
    setAutoCpu    (ResourceManager.getPersistentBoolean(PROF_SECTION, "autocpu", false));
    setVMDisabled (ResourceManager.getPersistentBoolean(PROF_SECTION, "vminfo",  false));
    setThreadsDisabled(ResourceManager.getPersistentBoolean(PROF_SECTION, "threadinfo", true));
  }
  
  public void save() {
    if (isAttach) ResourceManager.setPersistentString(PROF_SECTION, "listen.host", getHost());
    ResourceManager.setPersistentInt    (PROF_SECTION, "listen.port",       getPort());
    ResourceManager.setPersistentInt    (PROF_SECTION, "listen.depth",      getDepth());
    ResourceManager.setPersistentString (PROF_SECTION, "listen.sourcepath", getSourcePath());
    ResourceManager.setPersistentBoolean(PROF_SECTION, "mem",               memChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, "cpu",               cpuChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, "autocpu",           autoCpuChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, "vminfo",            vmChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, "threadinfo",        threadChk.isSelected());
    try {
      ResourceManager.save();
    } catch (Exception _) { }
  }
}