/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.util.io.IniFile;              // from Util project

/**
 * OptionPanel for the common between the standalone and plugin version options.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class OptionPanel extends PPanel implements ActionListener, ItemListener {
  
  public static final int    DEF_DEPTH = com.prosyst.jpd.profiler.gui.ActionManager.DEFAULT_DEPTH;
  public static final int    DEF_PORT  = com.prosyst.jpd.profiler.ProfilerServer.DEFAULT_PORT;
  public static final String DEF_HOST  = "localhost";
  public static final String DEF_CMD   =
    "-Djava.compiler=NONE -Xrunprof:format=b,heap=sites,cpu=samples,cutoff=0.0001,lineno=y,thread=y,doe=n";
  public static final String PROF_SECTION = "OPTIONS";
  public static final String PROF_LAUNCH  = "launch";
  public static final String PROF_LISTEN  = "listen";
  public static final String PROF_ATTACH  = "attach";
  
  private PLabel       portLbl  = new PLabel(ResourceManager.getString("StartDialog.label.port"));
  private PLabel       depthLbl = new PLabel(ResourceManager.getString("StartDialog.label.depth"));
  private PLabel       cmdLbl   = new PLabel(ResourceManager.getString("StartDialog.label.profcmd"));
  
  private PCheckBox    memChk,   cpuChk,   autoCpuChk, vmChk, threadChk;
  private PRadioButton launchRB, listenRB, attachRB;
  private PScrollNumberField portTxt, depthTxt;
  private PTextField   hostTxt,  cmdTxt;
  private PButton      defBtn;
  
  private boolean memB, cpuB, autoCpuB, vmB, threadB;
  private String  remoteB;
  private String  hostB;
  private int     portB;
  private String  cmdB;
  
  //  Constructor
  
  public OptionPanel() {
    setLayout(new BorderLayout(5, 5));
    
    add(createEnablePanel(), BorderLayout.NORTH);
    add(createCmdPanel(),    BorderLayout.CENTER);
  }
  
  private PPanel createEnablePanel() {
    PPanel enablePanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    enablePanel.setBorder(new EtchedBorder());
    
    enablePanel.add(memChk     = createCheckBox("memchk",     false));
    enablePanel.add(cpuChk     = createCheckBox("cpuchk",     false));
    enablePanel.add(autoCpuChk = createCheckBox("autocpuchk", false));
    enablePanel.add(vmChk      = createCheckBox("vmchk",      false));
    threadChk  = createCheckBox("threadchk",  false);
    
    PPanel depthPanel = new PPanel(new FlowLayout(0, 0, FlowLayout.LEFT));
    depthPanel.add(depthLbl);
    depthTxt = new PScrollNumberField(3);
    depthTxt.setMinimum(1);
    depthTxt.setMaximum(256);
    depthTxt.setValue(DEF_DEPTH);
    depthPanel.add(depthTxt);
    enablePanel.add(depthPanel);
        
    return enablePanel;
  }
  
  private PPanel createCmdPanel() {
    PPanel cmdPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    cmdPanel.setBorder(new EtchedBorder());
    
    cmdPanel.add(cmdLbl);
    PPanel pPanel = new PPanel(new BorderLayout(5, 5));
    pPanel.add(cmdTxt = createTextField(50, DEF_CMD), BorderLayout.CENTER);
    pPanel.add(defBtn = createButton("resetbtn"),     BorderLayout.EAST);
    cmdPanel.add(pPanel);
    
    PPanel mPanel = new PPanel(new FlowLayout(FlowLayout.LEFT));
    mPanel.add(launchRB = createRadioButton("launch", true));
    launchRB.addItemListener(this);
    mPanel.add(attachRB = createRadioButton("attach", false));
    attachRB.addItemListener(this);
    mPanel.add(listenRB = createRadioButton("listen", false));
    listenRB.addItemListener(this);
    mPanel.add(hostTxt = createTextField(20, DEF_HOST));
    hostTxt.setEditable(false);
    cmdPanel.add(mPanel);
    ButtonGroup mrg = new ButtonGroup();
    mrg.add(launchRB);
    mrg.add(attachRB);
    mrg.add(listenRB);
    
    PPanel rPanel = new PPanel(new FlowLayout(FlowLayout.LEFT));
    rPanel.add(portLbl);
    rPanel.add(portTxt = new PScrollNumberField(5));
    portTxt.setMinimum(1);
    portTxt.setMaximum(65535);
    portTxt.setValue(DEF_PORT);
    cmdPanel.add(rPanel);
    
    return cmdPanel;
  }
  
  private PButton createButton(String name) {
    PButton btn = new PButton(ResourceManager.getString("StartDialog.label." + name));
    Icon icon = VisualManager.getIcon("StartDialog.icon." + name);
    if (icon != null) btn.setIcon(icon);
    String mnem = ResourceManager.getString("StartDialog.mnem." + name);
    if (mnem != null) btn.setMnemonic(mnem.charAt(0));
    Dimension btnSize = btn.getPreferredSize();
    btnSize.height = cmdTxt.getPreferredSize().height;
    btn.setPreferredSize(btnSize);
    btn.addActionListener(this);
    return btn;
  }
  
  private PCheckBox createCheckBox(String name, boolean flag) {
    PCheckBox chk = new PCheckBox(ResourceManager.getString("StartDialog.label." + name));
    String mnem = ResourceManager.getString("StartDialog.mnem." + name);
    if (mnem != null) chk.setMnemonic(mnem.charAt(0));
    chk.setSelected(flag);
    return chk;
  }
  
  private PRadioButton createRadioButton(String name, boolean flag) {
    PRadioButton rb = new PRadioButton(ResourceManager.getString("StartDialog.radio." + name));
    String mnem = ResourceManager.getString("StartDialog.radio.mnem." + name);
    if (mnem != null) rb.setMnemonic(mnem.charAt(0));
    rb.setSelected(flag);
    return rb;
  }
  
  private PTextField createTextField(int col, String text) {
    PTextField txt = createTextField(col);
    txt.setText(text);
    return txt;
  }
  
  private PTextField createTextField(int col) {
    PTextField txt = new PTextField(col);
    return txt;
  }
  
  //  Properties
  
  /** Checks whether mem profiling is disabled. */
  public boolean isMemDisabled() {
    return memChk.isSelected();
  }
  
  /** Checks whether cpu profiling is disabled. */
  public boolean isCpuDisabled() {
    return cpuChk.isSelected();
  }
  
  /** Checks whether auto start cpu profiling is enabled. */
  public boolean isAutoCpu() {
    return autoCpuChk.isSelected();
  }
  
  /** Checks whether vm info is disabled. */
  public boolean isVMDisabled() {
    return vmChk.isSelected();
  }
  
  /** Checks whether threads info is disabled. */
  public boolean isThreadsDisabled() {
    return threadChk.isSelected();
  }
  
  /** Checks whether remote monitoring is enabled. */
  public boolean isRemoteMon() {
    return listenRB.isSelected() || attachRB.isSelected();
  }
  
  /** Checks whether remote monitoring is listen. */
  public boolean isListen() {
    return listenRB.isSelected();
  }
  
  /** Checks whether remote monitoring is attach. */
  public boolean isAttach() {
    return attachRB.isSelected();
  }
  
  /** Gets remote monitoring state. */
  public String getRemoteMon() {
    if (listenRB.isSelected()) return PROF_LISTEN;
    if (attachRB.isSelected()) return PROF_ATTACH;
    return PROF_LAUNCH;
  }
  
  /** Gets command string. */
  public String getCommand() {
    return cmdTxt.getText() + (isAttach() ? ",attach=" : ",net=");
  }
  
  /** Gets command string. */
  public String getNormalCommand() {
    return cmdTxt.getText() + ",net=";
  }
  
  /** Gets host name. */
  public String getHost() {
    if (isListen()) return hostTxt.getText();
    if (isAttach()) return null;
    return DEF_HOST;
  }
  
  /** Gets port number. */
  public int getPort() {
    return portTxt.getValue();
  }
  
  /** Gets stack depth. */
  public int getDepth() {
    return depthTxt.getValue();
  }
  
  public String getOptions() {
    String opt = memChk.isSelected() + " " + cpuChk.isSelected() + " " + autoCpuChk.isSelected() + " "
               + vmChk.isSelected()  + " " + isThreadsDisabled() + " " + getRemoteMon()          + " "
               + getHost()           + " " + getPort()           + " " + cmdTxt.getText();
    return opt.trim();
  }
  
  /** Sets mem profiling disabled. */
  public void setMemDisabled(boolean state) {
    memChk.setSelected(state);
  }
  
  /** Sets cpu profiling disabled. */
  public void setCpuDisabled(boolean state) {
    cpuChk.setSelected(state);
  }
  
  /** Sets auto start cpu profiling. */
  public void setAutoCpu(boolean state) {
    autoCpuChk.setSelected(state);
  }
  
  /** Sets vm info disabled. */
  public void setVMDisabled(boolean state) {
    vmChk.setSelected(state);
  }
  
  /** Sets threads info disabled. */
  public void setThreadsDisabled(boolean state) {
    threadChk.setSelected(state);
  }
  
  /** Sets remote monitoring. */
  public void setRemoteMon(String state) {
    if (PROF_LISTEN.equals(state)) { listenRB.setSelected(true); return; }
    if (PROF_ATTACH.equals(state)) { attachRB.setSelected(true); return; }
    launchRB.setSelected(true);
  }
  
  /** Sets command string. */
  public void setCommand(String cmd) {
    cmdTxt.setText(cmd);
  }
  
  /** Sets host name. */
  public void setHost(String host) {
    if (host == null) {
      hostTxt.setText(DEF_HOST);
    } else {
      hostTxt.setText(host);
    }
  }
  
  /** Sets port number. */
  public void setPort(int port) {
    portTxt.setText(Integer.toString(port));
  }
  
  /** Sets stack depth. */
  public void setDepth(int depth) {
    depthTxt.setText(Integer.toString(depth));
  }
  
  /** Sets enabled / disabled state of profiler command. */
  public void setEnabledFields(boolean state) {
    cmdTxt.setEnabled(state);
  }
  
  //  Load / Save
  
  protected void backup() {
    memB     = memChk.isSelected();
    cpuB     = cpuChk.isSelected();
    autoCpuB = autoCpuChk.isSelected();
    vmB      = vmChk.isSelected();
    threadB  = threadChk.isSelected();
    remoteB  = getRemoteMon();
    hostB    = getHost();
    portB    = getPort();
    cmdB     = cmdTxt.getText();
  }
  
  protected void reload() {
    setMemDisabled    (memB);
    setCpuDisabled    (cpuB);
    setAutoCpu        (autoCpuB);
    setVMDisabled     (vmB);
    setThreadsDisabled(threadB);
    setRemoteMon      (remoteB);
    if (isListen())
      setHost         (hostB);
    setPort           (portB);
    setCommand        (cmdB);
  }
  
  public void load(IniFile iniFile) {
    setMemDisabled    (ResourceManager.convertBoolean(iniFile.getValue(PROF_SECTION, "mem")));
    setCpuDisabled    (ResourceManager.convertBoolean(iniFile.getValue(PROF_SECTION, "cpu")));
    setAutoCpu        (ResourceManager.convertBoolean(iniFile.getValue(PROF_SECTION, "autocpu")));
    setVMDisabled     (ResourceManager.convertBoolean(iniFile.getValue(PROF_SECTION, "vminfo")));
    setThreadsDisabled(ResourceManager.convertBoolean(iniFile.getValue(PROF_SECTION, "threadinfo"), true));
    setRemoteMon                                     (iniFile.getValue(PROF_SECTION, "remotemon"));
    if (isListen()) setHost                          (iniFile.getValue(PROF_SECTION, "host"));
    int port;
    try {
      port = ResourceManager.parseInt(iniFile.getValue(PROF_SECTION, "port"));
    } catch (NumberFormatException ex) {
      port = DEF_PORT;
    }
    setPort(port);
    setCommand(parseCmd(iniFile.getValue(PROF_SECTION, "profcmd")));
    int depth;
    try {
      depth = ResourceManager.parseInt(iniFile.getValue(PROF_SECTION, "depth"));
    } catch (NumberFormatException ex) {
      depth = DEF_DEPTH;
    }
    setDepth(depth);
  }
  
  public static String parseCmd(String profCmd) {
    if (profCmd == null) return DEF_CMD;
    int indx = profCmd.indexOf("net=");
    if (indx != -1) {
      if (profCmd.charAt(indx - 1) == ',') indx--;
      int indx2 = profCmd.indexOf(':', indx);
      if (indx2 != -1) profCmd = profCmd.substring(0, indx) + profCmd.substring(indx2 + 1);
      else             profCmd = profCmd.substring(0, indx) + profCmd.substring(indx + 5);
    }
    return profCmd;
  }
  
  public void save(IniFile iniFile) {
    iniFile.setValue(PROF_SECTION, "mem",        String.valueOf(memChk.isSelected()));
    iniFile.setValue(PROF_SECTION, "cpu",        String.valueOf(cpuChk.isSelected()));
    iniFile.setValue(PROF_SECTION, "autocpu",    String.valueOf(autoCpuChk.isSelected()));
    iniFile.setValue(PROF_SECTION, "vminfo",     String.valueOf(vmChk.isSelected()));
    iniFile.setValue(PROF_SECTION, "threadinfo", String.valueOf(threadChk.isSelected()));
    iniFile.setValue(PROF_SECTION, "remotemon",  getRemoteMon());
    if (isListen()) iniFile.setValue(PROF_SECTION, "host", getHost());
    iniFile.setValue(PROF_SECTION, "port",       Integer.toString(getPort(), 10));
    iniFile.setValue(PROF_SECTION, "profcmd",    cmdTxt.getText());
    iniFile.setValue(PROF_SECTION, "depth",      Integer.toString(getDepth(), 10));
  }
  
  public boolean isChanged(IniFile iniFile) {
    if (!String.valueOf(memChk.isSelected()).    equals(iniFile.getValue(PROF_SECTION, "mem")))        return true;
    if (!String.valueOf(cpuChk.isSelected()).    equals(iniFile.getValue(PROF_SECTION, "cpu")))        return true;
    if (!String.valueOf(autoCpuChk.isSelected()).equals(iniFile.getValue(PROF_SECTION, "autocpu")))    return true;
    if (!String.valueOf(vmChk.isSelected()).     equals(iniFile.getValue(PROF_SECTION, "vminfo")))     return true;
    if (iniFile.getValue(PROF_SECTION, "threadinfo") != null || !threadChk.isSelected())
     if (!String.valueOf(threadChk.isSelected()).equals(iniFile.getValue(PROF_SECTION, "threadinfo"))) return true;
    if (!getRemoteMon().                         equals(iniFile.getValue(PROF_SECTION, "remotemon")))  return true;
    if (isListen())               if (!getHost().equals(iniFile.getValue(PROF_SECTION, "host")))       return true;
    if (!Integer.toString(getPort()).            equals(iniFile.getValue(PROF_SECTION, "port")))       return true;
    if (!cmdTxt.getText().                       equals(iniFile.getValue(PROF_SECTION, "profcmd")))    return true;
    String depth = iniFile.getValue(PROF_SECTION, "depth");
    if (depth == null && getDepth() != DEF_DEPTH) return true;
    if (depth != null && !Integer.toString(getDepth()).equals(depth)) return true;
    return false;
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent ae) {
    if (ae.getSource() == defBtn) setCommand(DEF_CMD);
  }
  
  //  ItemListener
  
  public void itemStateChanged(ItemEvent ie) {
    if (isRemoteMon()) {
      hostTxt.setEditable(isListen());
      memChk.setEnabled(false);
      cpuChk.setEnabled(false);
      autoCpuChk.setEnabled(false);
      vmChk.setEnabled(false);
      threadChk.setEnabled(false);
    } else {
      hostTxt.setText(DEF_HOST);
      hostTxt.setEditable(false);
      memChk.setEnabled(true);
      cpuChk.setEnabled(true);
      autoCpuChk.setEnabled(true);
      vmChk.setEnabled(true);
      threadChk.setEnabled(true);
    }
  }
}