/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.StringTokenizer;
import java.util.Vector;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.pgui.table.TableModelImpl;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.event.TableSelectionListener;
import com.prosyst.pgui.event.TableSelectionEvent;
import com.prosyst.pgui.utils.filechooser.PFileChooser;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

/**
 * PathDialog for easy setting of paths.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class PathDialog extends CButtonsDialog implements ActionListener, TableSelectionListener {
  
  /** user has pressed ok button */
  public static final int RETURN_OK     = 0;
  /** user has pressed cancel */
  public static final int RETURN_CANCEL = 1;
  
  private static final String   addTitle = ResourceManager.getString("PathDialog.title.addpath");
  private static final String[] colName  = { ResourceManager.getString("PathDialog.title.column") };
  
  private PFrame         frame;
  
  private PTable         pathTable;
  private TableModelImpl tableModel = new TableModelImpl(colName, 0);
  private PButton        addBtn, moveupBtn, movedownBtn, deleteBtn;
  private PButton        okBtn,  cancelBtn;
  
  private int            returned;
  private String         filter, ext;
  
  //  Constructor
  
  public PathDialog(PFrame frame) {
    super(frame, "", true);
    this.frame = frame;
    setBasePanel(createMainPanel());
    createButtons();
    addHelpButton();
    getRootPane().setDefaultButton(okBtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  private PPanel createMainPanel() {
    PPanel mainPanel = new PPanel(new BorderLayout(5, 5));
    
    pathTable = new PTable(tableModel);
    pathTable.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    pathTable.getSelectionModel().addTableSelectionListener(this);
    ToolTipManager.sharedInstance().registerComponent(pathTable);
    PScrollPane scrollPane = new PScrollPane(pathTable);
    scrollPane.setPreferredSize(new Dimension(300, 200));
    
    mainPanel.add(scrollPane,       BorderLayout.CENTER);
    mainPanel.add(createCmdPanel(), BorderLayout.EAST);
    
    return mainPanel;
  }
  
  private PPanel createCmdPanel() {
    PPanel cmdPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    cmdPanel.setBorder(new EtchedBorder());
    
    cmdPanel.add(addBtn      = createButton("addbtn"));
    cmdPanel.add(moveupBtn   = createButton("moveupbtn"));
    cmdPanel.add(movedownBtn = createButton("movedownbtn"));
    cmdPanel.add(deleteBtn   = createButton("deletebtn"));
    
    return cmdPanel;
  }
  
  private void createButtons() {
    addButton(okBtn     = createButton("okbtn"));
    addButton(cancelBtn = createButton("cancelbtn"));
  }
  
  private PButton createButton(String name) {
    PButton btn;
         if (name.equals("okbtn"))     btn = createOKButton();
    else if (name.equals("cancelbtn")) btn = createCancelButton();
    else {
      btn = new PButton(ResourceManager.getString("PathDialog.label." + name));
      btn.setHorizontalAlignment(PButton.LEFT);
      Icon icon = VisualManager.getIcon("PathDialog.icon." + name);
      if (icon != null) btn.setIcon(icon);
      String mnem = ResourceManager.getString("PathDialog.mnem." + name);
      if (mnem != null) btn.setMnemonic(mnem.charAt(0));
    }
    btn.addActionListener(this);
    return btn;
  }
  
  //  Properties
  
  /**
   * Shows Preferences dialog.
   */
  public int showDialog(String title, String path, String fltr, String ex) {
    setTitle(title);
    setPath(path);
    filter = fltr;
    ext = ex;
    setVisible(true);
    return returned;
  }
  
  public void setVisible(boolean state) {
    if (state) returned = RETURN_CANCEL;
    super.setVisible(state);
  }
  
  /** Gets the path string. */
  public String getPath() {
    StringBuffer sb = new StringBuffer();
    String ps = System.getProperty("path.separator");
    int rows = tableModel.getRowCount();
    if (rows > 0) sb.append((String) tableModel.getCellValue(0, 0));
    for (int i = 1; i < rows; i++) {
      sb.append(ps);
      sb.append(tableModel.getCellValue(i, 0));
    }
    return sb.toString();
  }
  
  /** Sets the path string. */
  public void setPath(String path) {
    StringTokenizer sTok = new StringTokenizer(path, System.getProperty("path.separator"));
    tableModel.removeAllRows();
    while (sTok.hasMoreTokens()) {
      Vector data = new Vector();
      data.addElement(sTok.nextToken());
      tableModel.addRow(data);
    }
    if (tableModel.getRowCount() == 0) {
      moveupBtn.setEnabled(false);
      movedownBtn.setEnabled(false);
      deleteBtn.setEnabled(false);
    } else {
      moveupBtn.setEnabled(true);
      movedownBtn.setEnabled(true);
      deleteBtn.setEnabled(true);
      pathTable.selectRows(0, 0);
    }
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == addBtn) {
      actionAdd();
    } else if (src == moveupBtn) {
      actionMoveUp();
    } else if (src == movedownBtn) {
      actionMoveDown();
    } else if (src == deleteBtn) {
      actionDelete();
    } else if (src == okBtn) {
      okButton_actionPerformed();
    } else if (src == cancelBtn) {
      cancelButton_actionPerformed();
    } else {
      super.actionPerformed(e);
    }
  }
  
  public void actionAdd() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theFile;
    if (filter == null) {
      fileChooser.removeFileFilter(0);
      theFile = fileChooser.showDirDialog(addTitle, PFileChooser.LAST_DIR, false);
    } else {
      if (filter.length() != 0) fileChooser.addFileFilter(filter, ext);
      theFile = fileChooser.showDialog(addTitle, PFileChooser.LAST_DIR, false, null, PFileChooser.BOTH);
    }
    if (theFile != null) {
      String path = theFile[0];
      if (path.length() < 1) return;
      int i = 0;
      for (;i < tableModel.getRowCount(); i++) {
        if (tableModel.getCellValue(i, 0).equals(path)) {
          pathTable.selectRows(i, i);
          return;
        }
      }
      Vector data = new Vector();
      data.addElement(path);
      tableModel.addRow(data);
      pathTable.selectRows(i, i);
      if (tableModel.getRowCount() == 1) {
        moveupBtn.setEnabled(true);
        movedownBtn.setEnabled(true);
        deleteBtn.setEnabled(true);
      }
    }
  }
 
  public void actionMoveUp() {
    int row = pathTable.getSelectedRow();
    if (row > 0) {
      tableModel.moveRow(row, row, row - 1);
      pathTable.selectRows(row - 1, row - 1);
    }
  }
   
  public void actionMoveDown() {
    int row = pathTable.getSelectedRow();
    if ((row >= 0) && (row < tableModel.getRowCount() - 1)) {
      tableModel.moveRow(row, row, row + 1);
      pathTable.selectRows(row + 1, row + 1);
    }
  }
  
  public void actionDelete() {
    int row = pathTable.getSelectedRow();
    if (row != -1) tableModel.removeRow(row);
    if (row >= tableModel.getRowCount()) row = tableModel.getRowCount() - 1;
    if (row != -1) pathTable.selectRows(row, row);
    else {
      moveupBtn.setEnabled(false);
      movedownBtn.setEnabled(false);
      deleteBtn.setEnabled(false);
    }
  }
  
  protected void performEnterAction(KeyEvent e) {
    okButton_actionPerformed();
  }
  
  protected void okButton_actionPerformed() {
    returned = RETURN_OK;
    setVisible(false);
  }
  
  protected void cancelButton_actionPerformed() {
    returned = RETURN_CANCEL;
    dispose();
  }
  
  public void doHelp() {
    HelpManager.showHelp("profiler.gui.dialogs.PathDialog");
  }
  
  //  TableSelectionListener
  
  public void valueChanged(TableSelectionEvent ev) {
    int row = pathTable.getSelectedRow();
    moveupBtn.setEnabled(row != -1);
    movedownBtn.setEnabled(row != -1);
    deleteBtn.setEnabled(row != -1);
  }
}