/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

/**
 * PreferencesDialog for the profiler behavior options.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class PreferencesDialog extends CButtonsDialog implements ActionListener {
  
  /** user has pressed ok button */
  public static final int RETURN_OK     = 0;
  /** user has pressed cancel */
  public static final int RETURN_CANCEL = 1;
  
  public static final int LAUNCH = 0;
  public static final int LISTEN = 1;
  public static final int ATTACH = 2;
  public static final int NONE   = 3;
  
  public  static final String PROF_SECTION    = "PREFS";
  private static final String resTitle        = ResourceManager.getString("Preferences.title");
  
  private static final String INI_TAB         = "startup.tab";
  private static final String INI_TAB_MEM     = "mem";
  private static final String INI_TAB_CPU     = "cpu";
  private static final String INI_TAB_HEAP    = "heap";
  private static final String INI_TAB_VM      = "vm";
  private static final String INI_TAB_THREAD  = "thread";
  private static final String INI_TAB_CON     = "con";
  
  private static final String INI_DIALOG      = "startup.dialog";
  private static final String INI_DLG_LAUNCH  = "launch";
  private static final String INI_DLG_LISTEN  = "listen";
  private static final String INI_DLG_ATTACH  = "attach";
  private static final String INI_DLG_NONE    = "none";
  
  private static final String INI_SAVEPOS     = "savepos";
  
  private static final String INI_MCV_BYTES   = "m_bytes";
  private static final String INI_MCV_INSTS   = "m_insts";
  private static final String INI_MCV_DFBYTES = "m_diff_bytes";
  private static final String INI_MCV_DFINSTS = "m_diff_insts";
  private static final String INI_MCV_THREAD  = "m_thread";
  private static final String INI_MCV_METHOD  = "m_method";
  
  private static final String INI_CCV_THREAD  = "c_thread";
  private static final String INI_CCV_METHOD  = "c_method";
  private static final String INI_CCV_TIMEPRC = "c_time_prc";
  private static final String INI_CCV_TIMEMS  = "c_time_ms";

  private static final String INI_SMP_PRECSN  = "sampler.precision";
  private static final String INI_SMP_MTHDPRC = "method";
  private static final String INI_SMP_LINEPRC = "line";
  private static final String INI_SMP_MEMUNIT = "sampler.mem_units";
  private static final String INI_SMP_MUB     = "B";
  private static final String INI_SMP_MUK     = "KB";
  private static final String INI_SMP_MUM     = "MB";
  private static final String INI_SMP_TIMEPRC = "sampler.time_precision";
  private static final String INI_SMP_TP1     = "0.0";
  private static final String INI_SMP_TP2     = "0.00";
  private static final String INI_SMP_TP3     = "0.000";
  private static final String INI_SMP_TP4     = "0.0000";
  private static final String INI_SMP_NILINST = "sampler.show_zeroinsts";
  private static final String INI_SMP_TOPLINE = "sampler.top_lines";
  
  private static final PLabel tabLbl = new PLabel(ResourceManager.getString("Preferences.label.tab"));
  private static final PLabel dlgLbl = new PLabel(ResourceManager.getString("Preferences.label.dialog"));
  private static final PLabel spLbl  = new PLabel(ResourceManager.getString("Preferences.label.samplerprc"));
  private static final PLabel muLbl  = new PLabel(ResourceManager.getString("Preferences.label.memunits"));
  private static final PLabel tpLbl  = new PLabel(ResourceManager.getString("Preferences.label.timeprc"));
  private static final PLabel tlnLbl = new PLabel(ResourceManager.getString("Preferences.label.toplines"));
  
  private PRadioButton memRB,       cpuRB,       heapRB,       vmRB,         conRB,       threadRB;
  private PRadioButton launchRB,    listenRB,    attachRB,     noneRB;
  private PCheckBox    saveposChk;
  private PCheckBox    m_bytesChk,  m_instsChk,  m_dfbytesChk, m_dfinstsChk, m_threadChk, m_methodChk;
  private PCheckBox    c_threadChk, c_methodChk, c_timeprcChk, c_timemsChk;
  private PRadioButton mpRB,        lpRB;
  private PCheckBox    zeroChk;
  private PScrollNumberField toplnTxt;
  private PRadioButton mubRB,       mukRB,       mumRB;
  private PRadioButton tp1RB,       tp2RB,       tp3RB,        tp4RB;
  private PButton      okBtn,       cancelBtn;
  
  private PPanel       prefsPanel;
  
  private int          returned;
  private boolean      isDialog;
  
  //  Constructor
  
  public PreferencesDialog() {
    super(null);
    isDialog = false;
    prefsPanel = new PPanel(new BorderLayout(5, 5));
    prefsPanel.add(createSamplerPanel(),   BorderLayout.NORTH);
    prefsPanel.add(createMemColumnPanel(), BorderLayout.WEST);
    prefsPanel.add(createCpuColumnPanel(), BorderLayout.CENTER);
  }
  
  public PreferencesDialog(PFrame frame) {
    super(frame, resTitle, true);
    isDialog = true;
    setBasePanel(createMainPanel());
    createButtons();
    addHelpButton();
    getRootPane().setDefaultButton(okBtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  public PPanel getMainPanel() {
    return prefsPanel;
  }
  
  private PPanel createMainPanel() {
    PPanel mainPnl = new PPanel(new BorderLayout(5, 5));
    PTabbedPane tabPane = new PTabbedPane(isDialog ? PGUIConstants.TOP : PGUIConstants.BOTTOM);
    if (isDialog) tabPane.addTab(ResourceManager.getString("Preferences.tabStartup"),   createStartUpPanel());
    tabPane.addTab(ResourceManager.getString("Preferences.tabSampler"),   createSamplerPanel());
    tabPane.addTab(ResourceManager.getString("Preferences.tabMemColumn"), createMemColumnPanel());
    tabPane.addTab(ResourceManager.getString("Preferences.tabCpuColumn"), createCpuColumnPanel());
    mainPnl.add(tabPane, BorderLayout.CENTER);
    return mainPnl;
  }
  
  private PPanel createStartUpPanel() {
    PPanel startPnl = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    startPnl.setBorder(new EtchedBorder());
    
    PPanel tabPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    tabPnl.add(memRB    = createRadioButton("mem",    true));
    tabPnl.add(cpuRB    = createRadioButton("cpu",    false));
    tabPnl.add(heapRB   = createRadioButton("heap",   false));
    tabPnl.add(vmRB     = createRadioButton("vm",     false));
    tabPnl.add(threadRB = createRadioButton("thread", false));
    tabPnl.add(conRB    = createRadioButton("con",    false));
    ButtonGroup drg = new ButtonGroup();
    drg.add(memRB);
    drg.add(cpuRB);
    drg.add(heapRB);
    drg.add(vmRB);
    drg.add(threadRB);
    drg.add(conRB);
    startPnl.add(tabLbl);
    startPnl.add(tabPnl);
    
    PPanel dlgPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    dlgPnl.add(launchRB = createRadioButton("launch", true));
    dlgPnl.add(listenRB = createRadioButton("listen", false));
    dlgPnl.add(attachRB = createRadioButton("attach", false));
    dlgPnl.add(noneRB   = createRadioButton("none",   false));
    ButtonGroup srg = new ButtonGroup();
    srg.add(launchRB);
    srg.add(listenRB);
    srg.add(attachRB);
    srg.add(noneRB);
    startPnl.add(dlgLbl);
    startPnl.add(dlgPnl);
    
    startPnl.add(saveposChk = createCheckBox("savepos", true));
    
    return startPnl;
  }
  
  private PPanel createSamplerPanel() {
    PPanel sampPnl = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    sampPnl.setBorder(new EtchedBorder());
    
    PPanel spPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    spPnl.add(spLbl);
    spPnl.add(mpRB = createRadioButton("methodprc", false));
    spPnl.add(lpRB = createRadioButton("lineprc",   true));
    ButtonGroup spg = new ButtonGroup();
    spg.add(mpRB);
    spg.add(lpRB);
    sampPnl.add(spPnl);
    
    PPanel muPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    muPnl.add(muLbl);
    muPnl.add(mubRB = new PRadioButton(INI_SMP_MUB));
    muPnl.add(mukRB = new PRadioButton(INI_SMP_MUK));
    muPnl.add(mumRB = new PRadioButton(INI_SMP_MUM));
    ButtonGroup mug = new ButtonGroup();
    mug.add(mubRB);
    mug.add(mukRB);
    mug.add(mumRB);
    mubRB.setSelected(true);
    sampPnl.add(muPnl);
    
    PPanel tpPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    tpPnl.add(tpLbl);
    tpPnl.add(tp1RB = new PRadioButton(INI_SMP_TP1));
    tpPnl.add(tp2RB = new PRadioButton(INI_SMP_TP2));
    tpPnl.add(tp3RB = new PRadioButton(INI_SMP_TP3));
    tpPnl.add(tp4RB = new PRadioButton(INI_SMP_TP4));
    ButtonGroup tpg = new ButtonGroup();
    tpg.add(tp1RB);
    tpg.add(tp2RB);
    tpg.add(tp3RB);
    tpg.add(tp4RB);
    tp2RB.setSelected(true);
    sampPnl.add(tpPnl);
    
    sampPnl.add(zeroChk = createCheckBox("showzeroinsts", false));
    
    PPanel tlPnl = new PPanel(new FlowLayout(FlowLayout.LEFT));
    tlPnl.add(tlnLbl);
    tlPnl.add(toplnTxt = new PScrollNumberField(3));
    toplnTxt.setMinimum(1);
    toplnTxt.setMaximum(999);
    toplnTxt.setValue(40);
    sampPnl.add(tlPnl);
    
    return sampPnl;
  }
  
  private PPanel createMemColumnPanel() {
    PPanel mcPnl = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    mcPnl.setBorder(new EtchedBorder());
    
    mcPnl.add(m_bytesChk   = createCheckBox("mbytes",   true));
    mcPnl.add(m_instsChk   = createCheckBox("minsts",   true));
    mcPnl.add(m_dfbytesChk = createCheckBox("mdfbytes", true));
    mcPnl.add(m_dfinstsChk = createCheckBox("mdfinsts", true));
    mcPnl.add(m_threadChk  = createCheckBox("mthread",  true));
    mcPnl.add(m_methodChk  = createCheckBox("mmethod",  true));
    
    return mcPnl;
  }
  
  private PPanel createCpuColumnPanel() {
    PPanel ccPnl = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    ccPnl.setBorder(new EtchedBorder());
    
    ccPnl.add(c_threadChk  = createCheckBox("cthread",  true));
    ccPnl.add(c_methodChk  = createCheckBox("cmethod",  true));
    ccPnl.add(c_timeprcChk = createCheckBox("ctimeprc", true));
    ccPnl.add(c_timemsChk  = createCheckBox("ctimems",  true));
    
    return ccPnl;
  }
  
  private static PRadioButton createRadioButton(String name, boolean flag) {
    PRadioButton rb = new PRadioButton(ResourceManager.getString("Preferences.radio." + name));
    //String mnem = ResourceManager.getString("Preferences.radio.mnem." + name);
    //if (mnem != null) rb.setMnemonic(mnem.charAt(0));
    rb.setSelected(flag);
    return rb;
  }
  
  private static PCheckBox createCheckBox(String name, boolean flag) {
    PCheckBox chk = new PCheckBox(ResourceManager.getString("Preferences.check." + name));
    //String mnem = ResourceManager.getString("Preferences.check.mnem." + name);
    //if (mnem != null) chk.setMnemonic(mnem.charAt(0));
    chk.setSelected(flag);
    return chk;
  }
  
  private void createButtons() {
    addButton(okBtn     = createButton("okbtn"));
    addButton(cancelBtn = createButton("cancelbtn"));
  }
  
  private PButton createButton(String name) {
    PButton btn;
         if (name.equals("okbtn"))     btn = createOKButton();
    else if (name.equals("cancelbtn")) btn = createCancelButton();
    else {
      btn = new PButton(ResourceManager.getString("Preferences.label." + name));
    }
    btn.addActionListener(this);
    return btn;
  }
  
  //  Properties
  
  /**
   * Shows Preferences dialog.
   */
  public int showDialog() {
    setVisible(true);
    return returned;
  }
  
  public void setVisible(boolean state) {
    if (state) {
      returned = RETURN_CANCEL;
      load();
    }
    super.setVisible(state);
  }
  
  /** Gets the name of the default displayed tab at startup. */
  public String getDefaultTabName() {
    if (memRB.   isSelected()) return INI_TAB_MEM;
    if (cpuRB.   isSelected()) return INI_TAB_CPU;
    if (heapRB.  isSelected()) return INI_TAB_HEAP;
    if (vmRB.    isSelected()) return INI_TAB_VM;
    if (threadRB.isSelected()) return INI_TAB_THREAD;
    if (conRB.   isSelected()) return INI_TAB_CON;
    return INI_TAB_MEM;
  }
  
  /** Gets the index of the default displayed tab at startup. */
  public static int getDefaultTab() {
    return getDefaultTabFromName(ResourceManager.getPersistentString(PROF_SECTION, INI_TAB));
  }
  
  /** Gets the index of the default displayed tab at startup from its name. */
  public static int getDefaultTabFromName(String name) {
    if (INI_TAB_MEM.   equals(name)) return VisualManager.PANEL_MEM;
    if (INI_TAB_CPU.   equals(name)) return VisualManager.PANEL_CPU;
    if (INI_TAB_HEAP.  equals(name)) return VisualManager.PANEL_HEAP;
    if (INI_TAB_VM.    equals(name)) return VisualManager.PANEL_VMINFO;
    if (INI_TAB_THREAD.equals(name)) return VisualManager.PANEL_THREADS;
    if (INI_TAB_CON.   equals(name)) return VisualManager.PANEL_CONSOLE;
    return VisualManager.PANEL_MEM;
  }
  
  /** Sets state of the radio buttons of the default displayed tab at startup. */
  public void setDefaultTab(String name) {
    if (INI_TAB_MEM.   equals(name)) memRB.   setSelected(true);
    if (INI_TAB_CPU.   equals(name)) cpuRB.   setSelected(true);
    if (INI_TAB_HEAP.  equals(name)) heapRB.  setSelected(true);
    if (INI_TAB_VM.    equals(name)) vmRB.    setSelected(true);
    if (INI_TAB_THREAD.equals(name)) threadRB.setSelected(true);
    if (INI_TAB_CON.   equals(name)) conRB.   setSelected(true);
  }
  
  /** Gets the name of the default displayed dialog at startup. */
  public String getDefaultDialogName() {
    if (launchRB.isSelected()) return INI_DLG_LAUNCH;
    if (listenRB.isSelected()) return INI_DLG_LISTEN;
    if (attachRB.isSelected()) return INI_DLG_ATTACH;
    if (noneRB.  isSelected()) return INI_DLG_NONE;
    return INI_DLG_LAUNCH;
  }
  
  /** Gets the index of the default displayed dialog at startup. */
  public static int getDefaultDialog() {
    return getDefaultDialogFromName(ResourceManager.getPersistentString(PROF_SECTION, INI_DIALOG));
  }
  
  /** Gets the index of the default displayed dialog at startup from its name. */
  public static int getDefaultDialogFromName(String name) {
    if (INI_DLG_LAUNCH.equals(name)) return LAUNCH;
    if (INI_DLG_LISTEN.equals(name)) return LISTEN;
    if (INI_DLG_ATTACH.equals(name)) return ATTACH;
    if (INI_DLG_NONE.  equals(name)) return NONE;
    return LAUNCH;
  }
  
  /** Sets state of the radio buttons of the default displayed dialog at startup. */
  public void setDefaultDialog(String name) {
    if (INI_DLG_LAUNCH.equals(name)) launchRB.setSelected(true);
    if (INI_DLG_LISTEN.equals(name)) listenRB.setSelected(true);
    if (INI_DLG_ATTACH.equals(name)) attachRB.setSelected(true);
    if (INI_DLG_NONE.  equals(name)) noneRB.  setSelected(true);
  }
  
  /** Gets the memory columns visibility. */
  public static boolean[] getMemColumns() {
    boolean[] columnsView = new boolean[7];
    columnsView[0] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_BYTES,   true)) columnsView[1] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_INSTS,   true)) columnsView[2] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_DFBYTES, true)) columnsView[3] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_DFINSTS, true)) columnsView[4] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_THREAD,  true)) columnsView[5] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_MCV_METHOD,  true)) columnsView[6] = true;
    return columnsView;
  }
  
  /** Gets the cpu columns visibility. */
  public static boolean[] getCpuColumns() {
    boolean[] columnsView = new boolean[4];
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_CCV_THREAD,  true)) columnsView[0] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_CCV_METHOD,  true)) columnsView[1] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_CCV_TIMEPRC, true)) columnsView[2] = true;
    if (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_CCV_TIMEMS,  true)) columnsView[3] = true;
    if (!columnsView[0] && !columnsView[1] && !columnsView[2] && !columnsView[3])  columnsView[1] = true;
    return columnsView;
  }
  
  /** Sets the memory columns visibility. */
  private void setMemColumns() {
    boolean[] columnsView = getMemColumns();
    m_bytesChk.  setSelected(columnsView[1]);
    m_instsChk.  setSelected(columnsView[2]);
    m_dfbytesChk.setSelected(columnsView[3]);
    m_dfinstsChk.setSelected(columnsView[4]);
    m_threadChk. setSelected(columnsView[5]);
    m_methodChk. setSelected(columnsView[6]);
  }
  
  /** Sets the cpu columns visibility. */
  public void setCpuColumns() {
    boolean[] columnsView = getCpuColumns();
    c_threadChk. setSelected(columnsView[0]);
    c_methodChk. setSelected(columnsView[1]);
    c_timeprcChk.setSelected(columnsView[2]);
    c_timemsChk. setSelected(columnsView[3]);
  }
  
  /** Gets the default sampler precision name. */
  public String getSamplerPrecisionName() {
    if (mpRB.isSelected()) return INI_SMP_MTHDPRC;
    if (lpRB.isSelected()) return INI_SMP_LINEPRC;
    return INI_SMP_LINEPRC;
  }
  
  /** Gets the default sampler precision. */
  public static boolean getSamplerPrecision() {
    return getSamplerPrecisionFromName(ResourceManager.getPersistentString(PROF_SECTION, INI_SMP_PRECSN));
  }
  
  /** Gets the default sampler precision from their name. */
  public static boolean getSamplerPrecisionFromName(String name) {
    if (INI_SMP_MTHDPRC.equals(name)) return true;
    return false;
  }
  
  /** Sets state of the radio buttons of the default displayed sampler precision. */
  public void setSamplerPrecision(String name) {
    if (INI_SMP_MTHDPRC.equals(name)) mpRB.setSelected(true);
    if (INI_SMP_LINEPRC.equals(name)) lpRB.setSelected(true);
  }
  
  /** Checks whether 0 insts should be displayed. */
  public static boolean isShowZeroInsts() {
    return ResourceManager.getPersistentBoolean(PROF_SECTION, INI_SMP_NILINST, false);
  }
  
  /** Sets whether 0 insts should be displayed. */
  public void setShowZeroInsts(boolean state) {
    zeroChk.setSelected(state);
  }
  
  /** Get the number of top lines that should be displayed. */
  public static int getTopLines() {
    return ResourceManager.getPersistentInt(PROF_SECTION, INI_SMP_TOPLINE, 40);
  }
  
  /** Sets whether 0 insts should be displayed. */
  public void setTopLines(int lines) {
    toplnTxt.setText(Integer.toString(lines, 10));
  }
  
  /** Gets the default mem units name. */
  public String getMemUnitsName() {
    if (mubRB.isSelected()) return INI_SMP_MUB;
    if (mukRB.isSelected()) return INI_SMP_MUK;
    if (mumRB.isSelected()) return INI_SMP_MUM;
    return INI_SMP_MUB;
  }
  
  /** Gets the default mem units. */
  public static int getMemUnits() {
    return getMemUnitsFromName(ResourceManager.getPersistentString(PROF_SECTION, INI_SMP_MEMUNIT));
  }
  
  /** Gets the default mem units from their name. */
  public static int getMemUnitsFromName(String name) {
    if (INI_SMP_MUB.equals(name)) return 1;
    if (INI_SMP_MUK.equals(name)) return 1000;
    if (INI_SMP_MUM.equals(name)) return 1000000;
    return 1;
  }
  
  /** Sets state of the radio buttons of the default displayed mem units. */
  public void setMemUnits(String name) {
    if (INI_SMP_MUB.equals(name)) mubRB.setSelected(true);
    if (INI_SMP_MUK.equals(name)) mukRB.setSelected(true);
    if (INI_SMP_MUM.equals(name)) mumRB.setSelected(true);
  }
  
  /** Gets the default time precision name. */
  public String getTimePrcName() {
    if (tp1RB.isSelected()) return INI_SMP_TP1;
    if (tp2RB.isSelected()) return INI_SMP_TP2;
    if (tp3RB.isSelected()) return INI_SMP_TP3;
    if (tp4RB.isSelected()) return INI_SMP_TP4;
    return INI_SMP_TP2;
  }
  
  /** Gets the default time precision. */
  public static int getTimePrc() {
    return getTimePrcFromName(ResourceManager.getPersistentString(PROF_SECTION, INI_SMP_TIMEPRC));
  }
  
  /** Gets the default time precision from their name. */
  public static int getTimePrcFromName(String name) {
    if (INI_SMP_TP1.equals(name)) return 2;
    if (INI_SMP_TP2.equals(name)) return 3;
    if (INI_SMP_TP3.equals(name)) return 4;
    if (INI_SMP_TP4.equals(name)) return 5;
    return 3;
  }
  
  /** Sets state of the radio buttons of the default displayed time precision. */
  public void setTimePrc(String name) {
    if (INI_SMP_TP1.equals(name)) tp1RB.setSelected(true);
    if (INI_SMP_TP2.equals(name)) tp2RB.setSelected(true);
    if (INI_SMP_TP3.equals(name)) tp3RB.setSelected(true);
    if (INI_SMP_TP4.equals(name)) tp4RB.setSelected(true);
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == okBtn) {
      okButton_actionPerformed();
    } else if (src == cancelBtn) {
      cancelButton_actionPerformed();
    } else {
      super.actionPerformed(e);
    }
  }
  
  protected void performEnterAction(KeyEvent e) {
    okButton_actionPerformed();
  }
  
  protected void okButton_actionPerformed() {
    returned = RETURN_OK;
    save();
    setVisible(false);
  }
  
  protected void cancelButton_actionPerformed() {
    returned = RETURN_CANCEL;
    dispose();
  }
  
  public void doHelp() {
    HelpManager.showHelp("profiler.gui.dialogs.PreferencesDialog");
  }
  
  //  Load / Save
  
  public void load() {
    if (isDialog) setDefaultTab   (ResourceManager.getPersistentString(PROF_SECTION, INI_TAB));
    if (isDialog) setDefaultDialog(ResourceManager.getPersistentString(PROF_SECTION, INI_DIALOG));
    if (isDialog) saveposChk.setSelected(ResourceManager.getPersistentBoolean(PROF_SECTION, INI_SAVEPOS, true));
    
    setSamplerPrecision(ResourceManager.getPersistentString (PROF_SECTION, INI_SMP_PRECSN));
    setShowZeroInsts   (ResourceManager.getPersistentBoolean(PROF_SECTION, INI_SMP_NILINST, false));
    setMemUnits        (ResourceManager.getPersistentString (PROF_SECTION, INI_SMP_MEMUNIT));
    setTimePrc         (ResourceManager.getPersistentString (PROF_SECTION, INI_SMP_TIMEPRC));
    setTopLines        (ResourceManager.getPersistentInt    (PROF_SECTION, INI_SMP_TOPLINE, 40));
    setMemColumns();
    setCpuColumns();
  }
  
  public void save() {
    if (isDialog) ResourceManager.setPersistentString (PROF_SECTION, INI_TAB,     getDefaultTabName());
    if (isDialog) ResourceManager.setPersistentString (PROF_SECTION, INI_DIALOG,  getDefaultDialogName());
    if (isDialog) ResourceManager.setPersistentBoolean(PROF_SECTION, INI_SAVEPOS, saveposChk.isSelected());
    
    ResourceManager.setPersistentString (PROF_SECTION, INI_SMP_PRECSN , getSamplerPrecisionName());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_SMP_NILINST, zeroChk.     isSelected());
    ResourceManager.setPersistentString (PROF_SECTION, INI_SMP_MEMUNIT, getMemUnitsName());
    ResourceManager.setPersistentString (PROF_SECTION, INI_SMP_TIMEPRC, getTimePrcName());
    
    int lines = 40;
    try {
      lines = toplnTxt.getValue();
    } catch (NumberFormatException ex) { }
    ResourceManager.setPersistentInt    (PROF_SECTION, INI_SMP_TOPLINE, lines);
    
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_BYTES,   m_bytesChk.  isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_INSTS,   m_instsChk.  isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_DFBYTES, m_dfbytesChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_DFINSTS, m_dfinstsChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_THREAD,  m_threadChk. isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_MCV_METHOD,  m_methodChk. isSelected());
    
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_CCV_THREAD,  c_threadChk. isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_CCV_METHOD,  c_methodChk. isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_CCV_TIMEPRC, c_timeprcChk.isSelected());
    ResourceManager.setPersistentBoolean(PROF_SECTION, INI_CCV_TIMEMS,  c_timemsChk. isSelected());
    try {
      ResourceManager.save();
    } catch (Exception _) { }
  }
}