/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.io.File;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.pgui.utils.filechooser.PFileChooser;
import com.prosyst.pgui.utils.filechooser.impl.FileObjectImpl;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ActionManager;
import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

import com.prosyst.util.log.Log;                 // from Util project
import com.prosyst.util.io.IniFile;              // from Util project

/**
 * StartDialog for entering the profiler parameters.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class StartDialog extends CButtonsDialog implements ActionListener {
  
  /** user has pressed start button */
  public static final int RETURN_START  = 0;
  /** user has pressed ok button */
  public static final int RETURN_OK     = 1;
  /** user has pressed cancel */
  public static final int RETURN_CANCEL = 2;
  
  public  static final String PROF_SECTION  = "OPTIONS";
  private static final String CLASS_SECTION = "CLASS";
  private static final String UNTITLED_PROF = "Untitled.ini";
  private static final String JAR_FILTER    = "Jar Files, Zip Files (*.jar, *.zip)";
  private static final String JAR_EXT       = "jar,zip";
  private static final String EMPTY_STRING  = "";
  
  private static final String resTitle      = ResourceManager.getString("StartDialog.title");
  private static final String loadTitle     = ResourceManager.getString("StartDialog.title.load");
  private static final String saveTitle     = ResourceManager.getString("StartDialog.title.save");
  private static final String clsTitle      = ResourceManager.getString("StartDialog.title.class");
  private static final String dirTitle      = ResourceManager.getString("StartDialog.title.workdir");
  private static final String clspathTitle  = ResourceManager.getString("StartDialog.title.clspath");
  private static final String srcpathTitle  = ResourceManager.getString("StartDialog.title.srcpath");
  private static final String storeTitle    = ResourceManager.getString("StartDialog.title.store");
  private static final String storeMsg      = ResourceManager.getString("StartDialog.msg.store");
  
  private static final int DEFAULT_SIZE    = 0;
  private static final int DEFAULT_HEIGTH  = 1;
  private static final int TEXTAREA_HEIGTH = 2;
  
  private ActionManager actionMng;
  private PFrame        frame;
  private OptionPanel   optPanel;
  private VMUsePanel    vmusePanel;
  private PathDialog    pathDialog;

  private PLabel     classLbl   = new PLabel(ResourceManager.getString("StartDialog.label.class"));
  private PLabel     workdirLbl = new PLabel(ResourceManager.getString("StartDialog.label.workdir"));
  private PTextField classTxt, workdirTxt;
  private PButton    classBtn, workdirBtn;
  
  private PLabel     javaprmLbl = new PLabel(ResourceManager.getString("StartDialog.label.javaprm"));
  private PLabel     progprmLbl = new PLabel(ResourceManager.getString("StartDialog.label.progprm"));
  private PLabel     clspathLbl = new PLabel(ResourceManager.getString("StartDialog.label.clspath"));
  private PLabel     srcpathLbl = new PLabel(ResourceManager.getString("StartDialog.label.srcpath"));
  private PTextField javaprmTxt, progprmTxt, clspathTxt, srcpathTxt;
  private PButton    clspathBtn, srcpathBtn;
  
  private PButton    startBtn, okBtn, cancelBtn;
  private PButton    newBtn, loadBtn, reldBtn, saveBtn, saveasBtn;
  
  private Dimension  freebtnSize, freebtnWidth;
  private int        returned;
  private String     lastProfile;
  private IniFile    iniFile;
  
  private String     lastProfileB;
  private IniFile    iniFileB;
  private String     className;
  private String     workDir;
  private String     javaParam;
  private String     progParam;
  private String     classPath;
  private String     sourcePath;

  //  Constructor
  
  public StartDialog(ActionManager actionMng, PFrame frame) {
    super(frame, resTitle, true);
    this.actionMng = actionMng;
    this.frame = frame;
    setBasePanel(createMainPanel());
    createButtons();
    addHelpButton();
    getRootPane().setDefaultButton(startBtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  private PPanel createMainPanel() {
    PPanel mainPanel = new PPanel(new BorderLayout(5, 5));
    
    PPanel progPanel = new PPanel(new BorderLayout(5, 5));
    progPanel.add(createClassPanel(),  BorderLayout.NORTH);
    progPanel.add(createParamsPanel(), BorderLayout.CENTER);
    
    optPanel   = new OptionPanel();
    vmusePanel = new VMUsePanel(frame);
    
    PTabbedPane tabPane = new PTabbedPane();
    tabPane.addTab(ResourceManager.getString("StartDialog.tabPaths"),    progPanel);
    tabPane.addTab(ResourceManager.getString("StartDialog.tabOptions"),  optPanel);
    //tabPane.addTab(ResourceManager.getString("StartDialog.tabTrigger"),  actionMng.triggerPanel);
    tabPane.addTab(ResourceManager.getString("StartDialog.tabVMUse"),    vmusePanel);
    
    mainPanel.add(createSettingsPanel(), BorderLayout.NORTH);
    mainPanel.add(tabPane,               BorderLayout.CENTER);
    
    return mainPanel;
  }
  
  private PPanel createSettingsPanel() {
    PPanel setPanel = new PPanel();
    setPanel.setBorder(new EtchedBorder());
    
    setPanel.add(new PLabel(ResourceManager.getString("StartDialog.label.settings")));
    setPanel.add(newBtn    = createButton("newbtn",    DEFAULT_HEIGTH, true));
    setPanel.add(loadBtn   = createButton("loadbtn",   DEFAULT_HEIGTH, true));
    //setPanel.add(reldBtn   = createButton("reldbtn",   DEFAULT_HEIGTH, true));
    //setPanel.add(saveBtn   = createButton("savebtn",   DEFAULT_HEIGTH, true));
    setPanel.add(saveasBtn = createButton("saveasbtn", DEFAULT_HEIGTH, true));
    
    return setPanel;
  }
  
  private PPanel createClassPanel() {
    PPanel pathPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    pathPanel.setBorder(new EtchedBorder());
    
    PPanel classPnl = new PPanel(new BorderLayout(5, 5));
    classPnl.add(classTxt = createTextField(50), BorderLayout.CENTER);
    classPnl.add(classBtn = createButton("classbtn", TEXTAREA_HEIGTH), BorderLayout.EAST);
    
    PPanel workdirPnl = new PPanel(new BorderLayout(5, 5));
    workdirPnl.add(workdirTxt = createTextField(50), BorderLayout.CENTER);
    workdirPnl.add(workdirBtn = createButton("workdirbtn", TEXTAREA_HEIGTH), BorderLayout.EAST);
    
    pathPanel.add(classLbl);
    pathPanel.add(classPnl);
    pathPanel.add(workdirLbl);
    pathPanel.add(workdirPnl);
    
    return pathPanel;
  }
  
  private PPanel createParamsPanel() {
    PPanel paramPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    paramPanel.setBorder(new EtchedBorder());
    
    paramPanel.add(javaprmLbl);
    paramPanel.add(javaprmTxt = createTextField(50));
    paramPanel.add(progprmLbl);
    paramPanel.add(progprmTxt = createTextField(50));

    paramPanel.add(clspathLbl);
    PPanel clspathPnl = new PPanel(new BorderLayout(5, 5));
    clspathPnl.add(clspathTxt = createTextField(50, System.getProperty("java.class.path")), BorderLayout.CENTER);
    clspathPnl.add(clspathBtn = createButton("clspathbtn", TEXTAREA_HEIGTH), BorderLayout.EAST);
    paramPanel.add(clspathPnl);
    
    paramPanel.add(srcpathLbl);
    PPanel srcpathPnl = new PPanel(new BorderLayout(5, 5));
    srcpathPnl.add(srcpathTxt = createTextField(50), BorderLayout.CENTER);
    srcpathPnl.add(srcpathBtn = createButton("srcpathbtn", TEXTAREA_HEIGTH), BorderLayout.EAST);
    paramPanel.add(srcpathPnl);
    
    return paramPanel;
  }
  
  private void createButtons() {
    addButton(startBtn  = createButton("startbtn",  DEFAULT_SIZE));
    addButton(okBtn     = createButton("okbtn",     DEFAULT_SIZE));
    addButton(cancelBtn = createButton("cancelbtn", DEFAULT_SIZE));
  }
  
  private PButton createButton(String name, int size_mode, boolean hasMnem) {
    PButton btn = createButton(name, size_mode);
    String mnem = ResourceManager.getString("StartDialog.mnem." + name);
    if (mnem != null) btn.setMnemonic(mnem.charAt(0));
    return btn;
  }
  
  private PButton createButton(String name, int size_mode) {
    PButton btn;
         if (name.equals("okbtn"))     btn = createOKButton();
    else if (name.equals("cancelbtn")) btn = createCancelButton();
    else {
      btn = new PButton(ResourceManager.getString("StartDialog.label." + name));
      Icon icon = VisualManager.getIcon("StartDialog.icon." + name);
      if (icon != null) btn.setIcon(icon);
      if (size_mode != DEFAULT_SIZE) {
        if (size_mode == DEFAULT_HEIGTH) {
          Dimension btnSize = btn.getPreferredSize();
          if (freebtnWidth == null)                    freebtnWidth       = btnSize;
          else if (freebtnWidth.width < btnSize.width) freebtnWidth.width = btnSize.width;
          btn.setPreferredSize(freebtnWidth);
        }
        if (size_mode == TEXTAREA_HEIGTH) {
          if (freebtnSize == null) {
            freebtnSize = btn.getPreferredSize();
            freebtnSize.height = classTxt.getPreferredSize().height;
          }
          btn.setPreferredSize(freebtnSize);
        }
      }
    }
    btn.addActionListener(this);
    return btn;
  }
  
  private PTextField createTextField(int col, String text) {
    PTextField txt = createTextField(col);
    txt.setText(text);
    return txt;
  }
  
  private PTextField createTextField(int col) {
    PTextField txt = new PTextField(col);
    txt.addActionListener(this);
    return txt;
  }
  
  //  Properties
  
  /**
   * Shows Start dialog.
   *
   * @return     return code from dialog: RETURN_START, RETURN_OK, RETURN_CANCEL
   */
  public int showDialog(boolean state) {
    changeTitle();
    startEnabled(state);
    setVisible(true);
    return returned;
  }
  
  public void startEnabled(boolean state) {
    startBtn.setEnabled(state);
  }
  
  public void setVisible(boolean state) {
    if (state) {
      returned = RETURN_CANCEL;
      load();
    } else {
      vmusePanel.stopSearch();
      if (returned == RETURN_CANCEL) reload();
    }
    super.setVisible(state);
  }
  
  /**
   * Gets profile name.
   *
   * @return   profile name
   */
  public String getProfileName() {
    return lastProfile;
  }
  
  /**
   * Gets main class name.
   *
   * @return   main class name
   */
  public String getClassName() {
    return classTxt.getText();
  }
  
  /**
   * Gets working directory.
   *
   * @return   working directory
   */
  public String getWorkDir() {
    return workdirTxt.getText();
  }
  
  /**
   * Gets java parameters.
   *
   * @return   java parameters
   */
  public String getJavaParam() {
    return javaprmTxt.getText();
  }
  
  /**
   * Gets program parameters.
   *
   * @return   program parameters
   */
  public String getProgParam() {
    return progprmTxt.getText();
  }
  
  /**
   * Gets class path.
   *
   * @return   class path
   */
  public String getClassPath() {
    return clspathTxt.getText();
  }
  
  /**
   * Gets source path.
   *
   * @return   source path
   */
  public String getSourcePath() {
    return srcpathTxt.getText();
  }
  
  /**
   * Gets jdk path.
   *
   * @return   jdk path
   */
  public String getJDKPath() {
    return vmusePanel.getVMPath();
  }
  
  /**
   * Gets option panel.
   *
   * @return   option panel
   */
  public OptionPanel getOptionPanel() {
    return optPanel;
  }
  
  /**
   * Gets vm use panel.
   *
   * @return   vm use panel
   */
  public VMUsePanel getVMUsePanel() {
    return vmusePanel;
  }
  
  /**
   * Sets main class name.
   *
   * @param   classname  main class name
   */
  public void setClassName(String classname) {
    classTxt.setText(classname);
  }
  
  /**
   * Sets working directory.
   *
   * @param   workdir  working directory
   */
  public void setWorkDir(String workdir) {
    workdirTxt.setText(workdir);
  }
  
  /**
   * Sets java parameters.
   *
   * @param   javaparam  java parameteres
   */
  public void setJavaParam(String javaparam) {
    javaprmTxt.setText(javaparam);
  }
  
  /**
   * Sets program parameters.
   *
   * @param   progparam  program parameteres
   */
  public void setProgParam(String progparam) {
    progprmTxt.setText(progparam);
  }
  
  /**
   * Sets class path.
   *
   * @param   clspath  class path
   */
  public void setClassPath(String clspath) {
    clspathTxt.setText(clspath);
  }
  
  /**
   * Sets source path.
   *
   * @param   srcpath  source path
   */
  public void setSourcePath(String srcpath) {
    srcpathTxt.setText(srcpath);
  }
  
  public void setSourcePathOut(String srcpath) {
    srcpathTxt.setText(srcpath);
    sourcePath = srcpath;
  }
  
  public int getJavaVersion() {
    return vmusePanel.getJavaVersion();
  }
  
  private void changeTitle() {
    if (lastProfile != null && lastProfile != UNTITLED_PROF) {
      setTitle(resTitle + " - [" + lastProfile + "]");
    } else {
      setTitle(resTitle);
    }
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == classBtn) {
      actionClassName();
    } else if (src == workdirBtn) {
      actionWorkDir();
    } else if (src == clspathBtn) {
      actionClsPath();
    } else if (src == srcpathBtn) {
      actionSrcPath();
    } else if (src == newBtn) {
      actionNew();
    } else if (src == loadBtn) {
      actionLoad();
    } else if (src == reldBtn) {
      actionReload();
    } else if (src == saveBtn) {
      if (lastProfile != null && lastProfile != UNTITLED_PROF) {
        save(lastProfile);
      }
    } else if (src == saveasBtn) {
      actionSave();
    } else if (src == startBtn) {
      actionStart();
    } else if (src == okBtn) {
      okButton_actionPerformed();
    } else if (src == cancelBtn) {
      cancelButton_actionPerformed();
    } else {
      super.actionPerformed(e);
    }
  }
  
  private void actionClassName() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.removeFileFilter(0);
    fileChooser.addFileFilter("Class Files, Jar Files (*.class, *.jar)", "class,jar");
    String[] theFile = fileChooser.showFileDialog(clsTitle, PFileChooser.LAST_DIR, false, true);
    if (theFile != null) {
      if (theFile[0].endsWith("class")) {
        HelpClassLoader loader = new HelpClassLoader();
        String clsName = loader.resolveClassName(theFile[0]);
        if (clsName != null) {
          classTxt.setText(clsName);
          workdirTxt.setText(loader.getDirPath());
        }
      } else {
        classTxt.setText(theFile[0]);
      }
    }
  }
  
  private void actionWorkDir() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theDir = fileChooser.showDirDialog(dirTitle, PFileChooser.LAST_DIR, false, true);
    if (theDir != null) {
      workdirTxt.setText(theDir[0]);
    }
  }
  
  private void actionClsPath() {
    if (pathDialog == null) {
      pathDialog = new PathDialog(frame);
    }
    int stat = pathDialog.showDialog(clspathTitle, clspathTxt.getText(), JAR_FILTER, JAR_EXT);
    if (stat == PathDialog.RETURN_OK) {
      clspathTxt.setText(pathDialog.getPath());
    }
  }
  
  private void actionSrcPath() {
    if (pathDialog == null) {
      pathDialog = new PathDialog(frame);
    }
    int stat = pathDialog.showDialog(srcpathTitle, srcpathTxt.getText(), JAR_FILTER, JAR_EXT);
    if (stat == PathDialog.RETURN_OK) {
      srcpathTxt.setText(pathDialog.getPath());
    }
  }
  
  public boolean shouldSave() {
    if (isChanged()) {
      String lp = (lastProfile == null ? UNTITLED_PROF : new File(lastProfile).getName());
      int code = POptionPane.showConfirmDialog(frame, storeMsg + lp, storeTitle, POptionPane.YES_NO_CANCEL_OPTION);
      if (code == POptionPane.CANCEL_OPTION) return true;
      if (code == POptionPane.YES_OPTION) {
        if (lastProfile == UNTITLED_PROF && iniFile == null) actionSave();
        else save(lastProfile);
      }
    }
    return false;
  }
  
  private void actionNew() {
    if (getClassName().equals(EMPTY_STRING) && getWorkDir().equals(EMPTY_STRING)
     && getJavaParam().equals(EMPTY_STRING) && getProgParam().equals(EMPTY_STRING)
     && getClassPath().equals(EMPTY_STRING) && getSourcePath().equals(EMPTY_STRING)) return;
    if (shouldSave()) return;
    lastProfile = UNTITLED_PROF;
    iniFile = null;
    setClassName (EMPTY_STRING);
    setWorkDir   (EMPTY_STRING);
    setJavaParam (EMPTY_STRING);
    setProgParam (EMPTY_STRING);
    setClassPath (EMPTY_STRING);
    setSourcePath(EMPTY_STRING);
    changeTitle();
  }
  
  private void actionLoad() {
    if (shouldSave()) return;
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.removeFileFilter(0);
    fileChooser.addFileFilter("Settings Files (*.ini)", "ini");
    String filename = null;
    if (lastProfile != null) {
      File file = new File(lastProfile);
      if (file.getParent() != null) fileChooser.setCurrentDirectory(new FileObjectImpl(file.getParent()));
      filename = file.getName();
    }
    String[] theFile = fileChooser.showFileDialog(loadTitle, false, filename, true);
    if (theFile != null) {
      load(theFile[0]);
    }
  }
  
  private void actionReload() {
    if (shouldSave()) return;
    if (lastProfile != null && lastProfile != UNTITLED_PROF) {
      load(lastProfile);
    }
  }
  
  private void actionSave() {
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    fileChooser.removeFileFilter(0);
    fileChooser.addFileFilter("Settings Files (*.ini)", "ini");
    String filename = null;
    if (lastProfile != null) {
      File file = new File(lastProfile);
      if (file.getParent() != null) fileChooser.setCurrentDirectory(new FileObjectImpl(file.getParent()));
      filename = file.getName();
    }
    String[] theFile = fileChooser.showFileDialog(saveTitle, false, filename, false);
    if (theFile != null) {
      if (!theFile[0].endsWith(".ini")) theFile[0] += ".ini";
      save(theFile[0]);
    }
  }
  
  private void actionStart() {
    if (getClassName().equals(EMPTY_STRING)) return;
    returned = RETURN_START;
    save();
    setVisible(false);
  }
  
  protected void performEnterAction(KeyEvent e) {
    actionStart();
  }
  
  protected void okButton_actionPerformed() {
    returned = RETURN_OK;
    save();
    setVisible(false);
  }
  
  protected void cancelButton_actionPerformed() {
    reload();
    returned = RETURN_CANCEL;
    dispose();
  }
  
  private void backup() {
    lastProfileB = lastProfile;
    iniFileB     = iniFile;
    className    = getClassName();
    workDir      = getWorkDir();
    javaParam    = getJavaParam();
    progParam    = getProgParam();
    classPath    = getClassPath();
    sourcePath   = getSourcePath();
  }
  
  private void reload() {
    lastProfile = lastProfileB;
    iniFile     = iniFileB;
    setClassName(className);
    setWorkDir(workDir);
    setJavaParam(javaParam);
    setProgParam(progParam);
    setClassPath(classPath);
    setSourcePath(sourcePath);
    optPanel.reload();
    actionMng.triggerPanel.reload();
    vmusePanel.load();
  }
  
  public void doHelp() {
    HelpManager.showHelp("profiler.gui.dialogs.StartDialog");
  }
  
  //  Load / Save
  
  private void load() {
    loadFile(ResourceManager.getPersistentString(PROF_SECTION, "last.profile"));
  }
  
  public void loadFile(String file) {
    if (className == null) {
      lastProfile = file;
      load(lastProfile);
      backup();
      optPanel.backup();
      actionMng.triggerPanel.backup();
    }
    vmusePanel.load();
  }
  
  private void save() {
    if (lastProfile != null) ResourceManager.setPersistentString(PROF_SECTION, "last.profile", lastProfile);
    backup();
    optPanel.backup();
    actionMng.triggerPanel.backup();
    vmusePanel.save();
  }
  
  private void load(String prf) {
    if (prf == null || prf.length() < 1) {
      lastProfile = UNTITLED_PROF;
      iniFile = null;
      return;
    }
    lastProfile = prf;
    iniFile = new IniFile();
    try {
      iniFile.load(new File(prf));
      setClassName (iniFile.getValue(CLASS_SECTION, "class"));
      setWorkDir   (iniFile.getValue(CLASS_SECTION, "workdir"));
      setJavaParam (iniFile.getValue(CLASS_SECTION, "javaparam"));
      setProgParam (iniFile.getValue(CLASS_SECTION, "progparam"));
      setClassPath (iniFile.getValue(CLASS_SECTION, "classpath"));
      setSourcePath(iniFile.getValue(CLASS_SECTION, "sourcepath"));
      //vmusePanel.load(file); // only class section is loaded from the profile
      optPanel.load(iniFile);
      actionMng.triggerPanel.load(iniFile);
    } catch (Exception _) {
      lastProfile = UNTITLED_PROF;
      iniFile = null;
      Log.printDebug("Error reading ini file.");
    }
    changeTitle();
  }
  
  private void save(String prf) {
    lastProfile = prf;
    iniFile = new IniFile();
    backup();
    try {
      ResourceManager.setPersistentString(PROF_SECTION, "last.profile", lastProfile);
      iniFile.setValue(CLASS_SECTION, "class",      getClassName());
      iniFile.setValue(CLASS_SECTION, "workdir",    getWorkDir());
      iniFile.setValue(CLASS_SECTION, "javaparam",  getJavaParam());
      iniFile.setValue(CLASS_SECTION, "progparam",  getProgParam());
      iniFile.setValue(CLASS_SECTION, "classpath",  getClassPath());
      iniFile.setValue(CLASS_SECTION, "sourcepath", getSourcePath());
      //vmusePanel.save(file); // only class section is saved in the profile
      optPanel.save(iniFile);
      actionMng.triggerPanel.save(iniFile);
      iniFile.save(new File(prf));
    } catch (Exception _) {
      lastProfile = UNTITLED_PROF;
      iniFile = null;
      Log.printDebug("Error saving ini file.");
    }
    changeTitle();
  }
  
  private boolean isChanged() {
    if (getClassName().equals(EMPTY_STRING) && getWorkDir().equals(EMPTY_STRING)
     && getJavaParam().equals(EMPTY_STRING) && getProgParam().equals(EMPTY_STRING)
     && (getClassPath().equals(EMPTY_STRING) || getClassPath().equals(System.getProperty("java.class.path")))
     && getSourcePath().equals(EMPTY_STRING)) return false;
    if (iniFile == null) return true;
    if (!getClassName(). equals(iniFile.getValue(CLASS_SECTION, "class")))      return true;
    if (!getWorkDir().   equals(iniFile.getValue(CLASS_SECTION, "workdir")))    return true;
    if (!getJavaParam(). equals(iniFile.getValue(CLASS_SECTION, "javaparam")))  return true;
    if (!getProgParam(). equals(iniFile.getValue(CLASS_SECTION, "progparam")))  return true;
    if (!getClassPath(). equals(iniFile.getValue(CLASS_SECTION, "classpath")))  return true;
    if (!getSourcePath().equals(iniFile.getValue(CLASS_SECTION, "sourcepath"))) return true;
    return optPanel.isChanged(iniFile) || actionMng.triggerPanel.isChanged(iniFile);
  }
}