/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.util.Vector;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.table.TableModelImpl;
import com.prosyst.pgui.table.TableSelectionModel;
import com.prosyst.pgui.table.TableCellRendererImpl;
import com.prosyst.pgui.layout.GridBagConstraints2;
import com.prosyst.gui.CButtonsDialog;

import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.help.HelpManager;

public class ThreadInfoDialog extends CButtonsDialog implements ActionListener {
  
  private static final String resTitle = ResourceManager.getString("ThreadInfoDialog.title");
  private static final String[] colName  = { ResourceManager.getString("ThreadInfoDialog.column.time"),
                                             ResourceManager.getString("ThreadInfoDialog.column.description")};
  
  private PButton closebtn;
  
  private PLabel runLabel   = new PLabel();
  private PLabel blockLabel = new PLabel();
  private PLabel waitLabel  = new PLabel();
  private PLabel sleepLabel = new PLabel();
  private PLabel runTimeL   = new PLabel();
  private PLabel blockTimeL = new PLabel();
  private PLabel waitTimeL  = new PLabel();
  private PLabel sleepTimeL = new PLabel();
  private PTable eventTable;
  private TableModelImpl tableModel = new TableModelImpl(colName, 0);
  
  //  Constructor
  
  public ThreadInfoDialog(PFrame frame) {
    super(frame, resTitle, true);
    setBasePanel(createMainPanel());
    closebtn = createCloseButton();
    closebtn.addActionListener(this);
    addButton(closebtn);
    addHelpButton();
    getRootPane().setDefaultButton(closebtn);
    pack();
    setLocationRelativeTo(frame);
  }
  
  private PPanel createMainPanel() {
    PPanel mainPnl = new PPanel(new BorderLayout());
    PPanel labelPnl = new PPanel(new GridBagLayout());
    labelPnl.setBorder(new EtchedBorder());
    
    runLabel.  setText(ResourceManager.getString("ThreadInfoDialog.label.runtime"));
    blockLabel.setText(ResourceManager.getString("ThreadInfoDialog.label.blocktime"));
    waitLabel. setText(ResourceManager.getString("ThreadInfoDialog.label.waittime"));
    sleepLabel.setText(ResourceManager.getString("ThreadInfoDialog.label.sleeptime"));
    labelPnl.add(runLabel,
        new GridBagConstraints2(0, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 8, 2, 0), 0, 0));
    labelPnl.add(runTimeL,
        new GridBagConstraints2(1, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 5, 0, 5), 0, 0));
    labelPnl.add(new PLabel(" "),
        new GridBagConstraints2(2, 0, 1, 1, 1.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    
    labelPnl.add(blockLabel,
        new GridBagConstraints2(0, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 8, 2, 0), 0, 0));
    labelPnl.add(blockTimeL,
        new GridBagConstraints2(1, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    
    labelPnl.add(waitLabel,
        new GridBagConstraints2(0, 2, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 8, 2, 0), 0, 0));
    labelPnl.add(waitTimeL,
        new GridBagConstraints2(1, 2, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    
    labelPnl.add(sleepLabel,
        new GridBagConstraints2(0, 3, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 8, 5, 0), 0, 0));
    labelPnl.add(sleepTimeL,
        new GridBagConstraints2(1, 3, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 2, 5), 0, 0));
    
    eventTable = new PTable(tableModel);
    eventTable.setEditable(false);
    eventTable.setRowSelectionAllowed(true);
    eventTable.setSelectionMode(TableSelectionModel.SINGLE_SELECTION);
    TableCellRendererImpl tcri = new TableCellRendererImpl();
    tcri.setHorizontalAlignment(RIGHT);
    eventTable.getColumn(0).setCellRenderer(tcri);
    eventTable.getColumn(1).setPreferredWidth(250);
    ToolTipManager.sharedInstance().registerComponent(eventTable);
    PScrollPane scrollPane = new PScrollPane(eventTable);
    scrollPane.setPreferredSize(new Dimension(300, 200));
    
    mainPnl.add(labelPnl, BorderLayout.NORTH);
    mainPnl.add(scrollPane, BorderLayout.CENTER);
    
    return mainPnl;
  }
  
  public void clear() {
    tableModel.removeAllRows();
  }
  
  public void addEvent(int time, String description) {
    Vector data = new Vector();
    data.addElement(Integer.toString(time, 10) + " ");
    data.addElement(" " + description);
    tableModel.addRow(data);
  }
  
  public void showDialog(String threadName, int runTime, int blockTime, int waitTime, int sleepTime) {
    setTitle(resTitle + threadName);
    runTimeL.  setText(Integer.toString(runTime   < 0 ? 0 : runTime,   10));
    blockTimeL.setText(Integer.toString(blockTime < 0 ? 0 : blockTime, 10));
    waitTimeL. setText(Integer.toString(waitTime  < 0 ? 0 : waitTime,  10));
    sleepTimeL.setText(Integer.toString(sleepTime < 0 ? 0 : sleepTime, 10));
    setVisible(true);
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent ev) {
    if (ev.getSource() == closebtn) {
      okButton_actionPerformed();
    } else {
      super.actionPerformed(ev);
    }
  }
  
  protected void performEnterAction(KeyEvent ke) {
    setVisible(false);
  }
  
  protected void okButton_actionPerformed() {
    setVisible(false);
  }
  
  public void doHelp() {
    HelpManager.showHelp("profiler.gui.dialogs.ThreadInfoDialog");
  }
}