/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.gui.dialogs;

import java.io.File;

import java.util.Vector;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.TextListener;
import java.awt.event.TextEvent;

import com.prosyst.pgui.*;
import com.prosyst.pgui.border.EtchedBorder;
import com.prosyst.pgui.event.ListSelectionListener;
import com.prosyst.pgui.event.ListSelectionEvent;
import com.prosyst.pgui.list.ListModel;
import com.prosyst.pgui.list.ListSelectionModel;
import com.prosyst.pgui.layout.VerticalFlowLayout;
import com.prosyst.pgui.utils.filechooser.PFileChooser;

import com.prosyst.jpd.profiler.Parser;
import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.jpd.profiler.gui.VisualManager;

import com.prosyst.util.log.Log;                 // from Util project

/**
 * VMUsePanel to search, setup and choose between different VMs.
 *
 * @author Ivo Karabashev
 * @version 1.0
 */
public class VMUsePanel extends PPanel implements ActionListener, ListSelectionListener, TextListener {
  
  public static final String JVMS_SECTION = "JVMS";
  
  private static final String[] buttons        = {"    Yes    ", "Yes to All", "     No     ", "  Cancel  "};
  private static final String vmpathTitle      = ResourceManager.getString("StartDialog.title.vmpath");
  private static final String titleNotFound    = ResourceManager.getString("StartDialog.title.notfound");
  private static final String msgNotFound      = ResourceManager.getString("StartDialog.msg.notfound");
  private static final String msgFileNotFound  = ResourceManager.getString("StartDialog.msg.filenotfound");
  private static final String titleJavaUnsupp  = ResourceManager.getString("StartDialog.title.javaunsupp");
  private static final String msgJavaOne       = ResourceManager.getString("StartDialog.msg.javaone");
  private static final String msgJavaUnsupp    = ResourceManager.getString("StartDialog.msg.javaunsupp");
  private static final String titleJavaUnknown = ResourceManager.getString("StartDialog.title.javaunknown");
  private static final String msgJavaUnknown   = ResourceManager.getString("StartDialog.msg.javaunknown");
  private static final String titleNotJava     = ResourceManager.getString("StartDialog.title.notjava");
  private static final String msgNotJava       = ResourceManager.getString("StartDialog.msg.notjava");
  private static final String titleAddJava     = ResourceManager.getString("StartDialog.title.addjava");
  private static final String msgAddJava       = ResourceManager.getString("StartDialog.msg.addjava");
  
  private PFrame     frame;
  
  private PList      javapathList;
  private PTextField javapathTxt;
  
  private PButton    searchBtn;
  private PButton    addBtn;
  private PButton    modifyBtn;
  private PButton    removeBtn;
  
  private Dimension  btnSize;
  
  private Vector     verv = new Vector();
  private String     tempVM;
  
  //  Constructor
  
  public VMUsePanel(PFrame frame) {
    this.frame = frame;
    setLayout(new BorderLayout(5, 5));
    
    add(createVMListPanel(),               BorderLayout.CENTER);
    add(javapathTxt = createTextField(50), BorderLayout.SOUTH);
    add(createCmdPanel(),                  BorderLayout.EAST);
    javapathTxt.addTextListener(this);
  }
  
  private PPanel createVMListPanel() {
    PPanel javaPanel = new PPanel(new BorderLayout());
    
    javapathList = new PList();
    javapathList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    javapathList.addListSelectionListener(this);
    PScrollPane scrollPane = new PScrollPane(javapathList);
    javaPanel.add(scrollPane);
    
    return javaPanel;
  }
  
  private PPanel createCmdPanel() {
    PPanel cmdPanel = new PPanel(new VerticalFlowLayout(PGUIConstants.LEFT, 5, 5));
    cmdPanel.setBorder(new EtchedBorder());
    
    cmdPanel.add(searchBtn = createButton("searchvmbtn"));
    cmdPanel.add(addBtn    = createButton("addvmbtn"));
    cmdPanel.add(modifyBtn = createButton("modifyvmbtn"));
    cmdPanel.add(removeBtn = createButton("removevmbtn"));
    
    return cmdPanel;
  }
  
  private PButton createButton(String name) {
    PButton btn = new PButton(ResourceManager.getString("StartDialog.label." + name));
    Icon icon = VisualManager.getIcon("StartDialog.icon." + name);
    if (icon != null) btn.setIcon(icon);
    String mnem = ResourceManager.getString("StartDialog.mnem." + name);
    if (mnem != null) btn.setMnemonic(mnem.charAt(0));
    btn.setHorizontalAlignment(PButton.LEFT);
    if (btnSize == null) btnSize = btn.getPreferredSize();
    else btn.setPreferredSize(btnSize);
    btn.addActionListener(this);
    return btn;
  }
  
  private PTextField createTextField(int col) {
    PTextField txt = new PTextField(col);
    return txt;
  }
  
  //  Properties
  
  /** Gets the path to the current VM. */
  public String getVMPath() {
    Object obj = javapathList.getSelectedValue();
    return (obj == null ? null : (String) obj);
  }
  
  private String getJavaPath() {
    return javapathTxt.getText();
  }
  
  private void setJavaPath(String javaPath) {
    javapathTxt.setText(javaPath);
  }
  
  //  ActionListener
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
           if (src == addBtn) {
      actionAdd(0);
    } else if (src == modifyBtn) {
      actionModify();
    } else if (src == removeBtn) {
      actionRemove();
    } else if (src == searchBtn) {
      actionSearch();
    }
    javapathTxt.requestFocus();
  }
  
  public boolean actionAdd(String javapath) {
    if (actionAdd(0, javapath)) {
      tempVM = javapath;
      return true;
    }
    return false;
  }

  private void actionAdd(int ver) {
    actionAdd(ver, getJavaPath());
  }

  private boolean actionAdd(int ver, String javapath) {
    if (javapath.length() < 1) return false;
    if (!(new File(javapath).exists())) {
      POptionPane.showMessageDialog(frame, msgFileNotFound, titleNotFound, POptionPane.ERROR_MESSAGE);
      return false;
    }
    ListModel lm = javapathList.getModel();
    for (int i = 0; i < lm.size(); i++) {
      if (((String) lm.elementAt(i)).equalsIgnoreCase(javapath)) {
        javapathList.setSelectedValue(javapath, true);
        return true;
      }
    }
    if (ver == 0) ver = Parser.checkJavaVersion(javapath);
    switch (ver) {
      case 1:
        if (!isSearching || code != 1)
          POptionPane.showMessageDialog(frame, javapath + "\n" + msgJavaOne, titleJavaUnsupp, POptionPane.ERROR_MESSAGE);
        return false;
      case Parser.VER_UNKNOWN:
        POptionPane.showMessageDialog(frame, javapath + "\n" + msgJavaUnknown, titleJavaUnknown, POptionPane.WARNING_MESSAGE);
        break;
      case Parser.VER_UNSUPPORTED:
        if (!isSearching || code != 1)
          POptionPane.showMessageDialog(frame, javapath + "\n" + msgJavaUnsupp, titleJavaUnsupp, POptionPane.ERROR_MESSAGE);
        return false;
    }
    lm.addElement(javapath);
    verv.addElement(new Integer(ver));
    javapathList.setSelectedValue(javapath, true);
    if (lm.size() == 1) {
      modifyBtn.setEnabled(true);
      removeBtn.setEnabled(true);
    }
    return true;
  }
  
  private void actionModify() {
    int row = javapathList.getSelectedIndex();
    if (row != -1) {
      String javapath = getJavaPath();
      int ver = Parser.checkJavaVersion(javapath);
      javapathList.getModel().setElementAt(javapath, row);
      verv.setElementAt(new Integer(ver), row);
    }
  }
  
  private void actionRemove() {
    int row = javapathList.getSelectedIndex();
    if (row != -1) {
      javapathList.getModel().removeElementAt(row);
      verv.removeElementAt(row);
    }
    if (row >= javapathList.getModel().size()) row = javapathList.getModel().size() - 1;
    if (row != -1) javapathList.setSelectedIndex(row);
    else {
      setJavaPath("");
      modifyBtn.setEnabled(false);
      removeBtn.setEnabled(false);
    }
  }
  
  private void actionSearch() {
    if (isSearching) {
      stopSearch();
      return;
    }
    PFileChooser fileChooser = PFileChooser.getInstance(frame);
    fileChooser.reinit();
    String[] theFile = fileChooser.showDialog(vmpathTitle, PFileChooser.LAST_DIR, false, null, PFileChooser.BOTH, true);
    if (theFile != null) {
      String javaFile = theFile[0];
      File file = new File(javaFile);
      if (file.isDirectory()) {
        new SearchThread(file).start();
      } else {
        if (!isJava(file)) {
          if (POptionPane.showConfirmDialog(frame, msgNotJava, titleNotJava,
                             POptionPane.YES_NO_OPTION) == POptionPane.YES_OPTION) {
            setJavaPath(javaFile);
            actionAdd(0);
          }
        } else {
          setJavaPath(javaFile);
          actionAdd(0);
        }
      }
    }
  }
  
  boolean isSearching = false;
  boolean isStopped   = false;
  boolean isFound     = false;
  
  class SearchThread extends Thread {
    File root;
    SearchThread(File dir) {
      setName("SearchThread");
      setDaemon(true);
      setPriority(Thread.NORM_PRIORITY);
      root = dir;
    }
    
    public void run() {
      isSearching = true;
      searchBtn.setText(ResourceManager.getString("StartDialog.label.stopvmbtn"));
      isFound = false;
      isStopped = false;
      code = -1;
      search(root);
      if (!isFound && !isStopped) {
        POptionPane.showMessageDialog(frame, msgNotFound, titleNotFound, POptionPane.ERROR_MESSAGE);
      }
      isSearching = false;
      searchBtn.setText(ResourceManager.getString("StartDialog.label.searchvmbtn"));
    }
  }
  
  protected void stopSearch() {
    isSearching = false;
    searchBtn.setText(ResourceManager.getString("StartDialog.label.searchvmbtn"));
    isStopped = true;
  }
  
  private int code;
  
  private static final boolean ignore_jre = Boolean.getBoolean("ignore.jre");
  
  private boolean search(File dir) {
    String[] list = dir.list();
    if (list == null) return false;
    File subfile;
    for (int i = 0; i < list.length; i++) {
      if (isStopped) return true;
      subfile = new File(dir, list[i]);
      if (subfile.isFile()) {
        if (isJava(subfile)) {
          isFound = true;
          String javaFile = subfile.getPath();
          if (listContains(javaFile) == -1) {
            if (ignore_jre && javaFile.indexOf(File.separatorChar + "jre" + File.separatorChar) != -1) continue;
            if (ignore_jre && javaFile.indexOf("green_threads") != -1) continue;
            if (ignore_jre && javaFile.indexOf("native_threads") != -1) continue;
            int ver = Parser.checkJavaVersion(javaFile);
            if (ver == 1 || ver == Parser.VER_UNSUPPORTED) continue;
            if (code != 1) {
              code = POptionPane.showOptionDialog(frame, msgAddJava + javaFile + '?', titleAddJava, 
                                                  4, POptionPane.QUESTION_MESSAGE, null, buttons, buttons[0]);
              if (code > 2) return true;
            }
            if (code < 2) {
              setJavaPath(javaFile);
              actionAdd(ver);
            }
          }
        }
      } else {
        if (search(subfile)) return true;
      }
    }
    return false;
  }
  
  private boolean isJava(File file) {
    String name = file.getName();
    for (int i = 0; i < vmnames.length; i++) {
      String vmname = vmnames[i];
      if (name.equalsIgnoreCase(vmname) || name.equalsIgnoreCase(vmname + ".exe")) return true;
    }
    return false;
  }
  
  private String[] vmnames = { "java", "evm", "j9", "cvm" }; //, "chaivm", "pjava", "kaffe", "pJSCP" };
  
  private int listContains(String javapath) {
    ListModel lm = javapathList.getModel();
    for (int i = 0; i < lm.size(); i++) {
      if (((String) lm.elementAt(i)).equalsIgnoreCase(javapath)) return i;
    }
    return -1;
  }
  
  //  ListSelectionListener
  
  private boolean blockEvent = false;
  
  public void valueChanged(ListSelectionEvent ev) {
    if (blockEvent) return;
    String javapath = (String) javapathList.getSelectedValue();
    if (javapath != null) {
      setJavaPath(javapath);
    }
    actionEnabled();
  }
  
  //  TextListener
  
  public void textValueChanged(TextEvent te) {
    actionEnabled();
  }
  
  private void actionEnabled() {
    boolean isNotEmpty = javapathTxt.getText().length() != 0;
    addBtn.setEnabled(isNotEmpty);
    int sel = javapathList.getSelectedIndex();
    modifyBtn.setEnabled(sel != -1 && isNotEmpty);
    removeBtn.setEnabled(sel != -1);
  }
  
  //  Load / Save
  
  protected void load() {
    int num = ResourceManager.getPersistentInt(JVMS_SECTION, "jvms.number", 0);
    if (num == 0) {
      javapathList.getModel().removeAllElements();
      modifyBtn.setEnabled(false);
      removeBtn.setEnabled(false);
      if (tempVM != null) {
        actionAdd(0, tempVM);
      }
      return;
    } else {
      modifyBtn.setEnabled(true);
      removeBtn.setEnabled(true);
    }
    Vector v = new Vector();
    verv.removeAllElements();
    String javapath;
    for (int i = 0; i < num; i++) {
      javapath = ResourceManager.getPersistentString(JVMS_SECTION, "jvm." + i);
      if (javapath == null) {
        Log.printDebug("JVMS.load", "Error in jvm" + i);
        break;
      }
      v.addElement(javapath);
      int ver = ResourceManager.getPersistentInt(JVMS_SECTION, "jvm.ver." + i, 0);
      verv.addElement(new Integer(ver));
    }
    javapathList.setListData(v);
    if (tempVM != null) {
      actionAdd(0, tempVM);
    } else {
      int sel = ResourceManager.getPersistentInt(JVMS_SECTION, "jvm.default", -1);
      if (sel > -1) {
        blockEvent = true;
        javapathList.setSelectedIndex(sel);
        setJavaPath((String) v.elementAt(sel));
        blockEvent = false;
      }
    }
  }
  
  protected void save() {
    int old_num = ResourceManager.getPersistentInt(JVMS_SECTION, "jvms.number", 0);
    ListModel lm = javapathList.getModel();
    int num = lm.size();
    ResourceManager.setPersistentInt(JVMS_SECTION, "jvms.number", num);
    int sel = javapathList.getSelectedIndex();
    if (sel != -1) {
      ResourceManager.setPersistentInt(JVMS_SECTION, "jvm.default",   sel);
      ResourceManager.setPersistentInt(JVMS_SECTION, "jvm.vmversion", 0);
    }
    String javapath;
    for (int i = 0; i < num; i++) {
      javapath = (String) lm.elementAt(i);
      int ver = ((Integer) verv.elementAt(i)).intValue();
      ResourceManager.setPersistentString(JVMS_SECTION, "jvm." + i, javapath);
      if (ver == 0) {
        ver = Parser.checkJavaVersion(javapath);
        verv.setElementAt(new Integer(ver), i);
      }
      ResourceManager.setPersistentInt(JVMS_SECTION, "jvm.ver." + i, ver);
    }
    // removes all old virtual machines
    for (int i = num; i < old_num; i++) {
      ResourceManager.deletePersistentKey(JVMS_SECTION, "jvm." + i);
      ResourceManager.deletePersistentKey(JVMS_SECTION, "jvm.ver." + i);
    }
    try {
      ResourceManager.save();
    } catch (Exception _) { }
    tempVM = null;
  }
  
  // Check Java Version
  
  public int getJavaVersion() {
    int sel = javapathList.getSelectedIndex();
    if (sel != -1) {
      int ver = ((Integer) verv.elementAt(sel)).intValue();
      if (ver == 0) {
        ver = Parser.checkJavaVersion((String) javapathList.getModel().elementAt(sel));
        verv.setElementAt(new Integer(ver), sel);
      }
      return ver;
    }
    return 0;
  }
}