/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import java.io.IOException;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.ObjectDumpInfo;

/**
 * Denote a heap dump -- dump of a normal object.
 *
 * <PRE>
 * TAG           BODY      notes
 * ----------------------------------------------------
 * PROF_GC_INSTANCE_DUMP        dump of a normal object
 *
 *                          id         object ID
 *                          u4         stack trace serial number
 *                          id         class object ID
 *                          u4         number of bytes that follow
 *                          [vl]*      instance field values (class, followed
 *                                     by super, super's super ...)
 */
public class HeapDump_InstanceDump extends HeapDump {
  
  public int            classObjectID;
  public int            numberOfBytes;
  public ObjectDumpInfo objDumpInfo;
  
  public HeapDump_InstanceDump(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws IOException {
    objDumpInfo = new ObjectDumpInfo();
    
    objDumpInfo.objectID           = in.read_id();
    
    int         stackTraceSerialNo = in.read_u4();
    objDumpInfo.traceInfo          = namespace.lookupTrace(stackTraceSerialNo);
    
    objDumpInfo.classObjectID      = in.read_id();
    
                numberOfBytes      = in.read_u4();
    
    objDumpInfo.buffer             = in.read_bytes(numberOfBytes);
    
    bind();
    
    // id + u4 + id + u4 + numberOfBytes
    return in.sizeofID() * 2 + 8 + numberOfBytes;
  }
  
  public void bind() {
    namespace.getObjdumpTable().bind(objDumpInfo.objectID, objDumpInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_INSTANCE_DUMP");
    out.println("  object ID                 = " + Integer.toHexString(objDumpInfo.objectID));
    out.println("  stack trace               = " + objDumpInfo.traceInfo);
    out.println("  class type                = " + Integer.toHexString(objDumpInfo.classObjectID));
    if (objDumpInfo.values != null) {
      for (int i = 0; i < objDumpInfo.values.length; i++) {
        out.println("  instance field value[" + i + "]   = " + Integer.toHexString(objDumpInfo.values[i]));
      }
    }
    out.println();
  }
}