/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.ObjArrayDumpInfo;

/**
 * Denote a heap dump -- dump of an object array.
 *
 * <PRE>
 * TAG           BODY      notes
 * ----------------------------------------------------
 * PROF_GC_OBJ_ARRAY_DUMP       dump of an object array
 *
 *                          id         array object ID
 *                          u4         stack trace serial number
 *                          u4         number of elements
 *                          id         element class ID
 *                          [id]*      elements
 */
public class HeapDump_ObjArrayDump extends HeapDump {
  
  public int              numberOfElements;
  public ObjArrayDumpInfo objDumpInfo;
  
  public HeapDump_ObjArrayDump(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    objDumpInfo = new ObjArrayDumpInfo();
    
    objDumpInfo.objectID           = in.read_id();
    
    int         stackTraceSerialNo = in.read_u4();
    objDumpInfo.traceInfo          = namespace.lookupTrace(stackTraceSerialNo);
    
                numberOfElements   = in.read_u4();
    
    objDumpInfo.classObjectID      = in.read_id();
    
    objDumpInfo.values = in.read_ids(numberOfElements);
    
    bind();
    
    // id + u4 + u4 + id + numberOfElements * id
    return in.sizeofID() * (numberOfElements + 2) + 8;
  }
  
  public void bind() {
    namespace.getObjdumpTable().bind(objDumpInfo.objectID, objDumpInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_OBJ_ARRAY_DUMP");
    out.println("  array object ID           = " + Integer.toHexString(objDumpInfo.objectID));
    out.println("  stack trace               = " + objDumpInfo.traceInfo);
    out.println("  " + objDumpInfo.toString());
    out.println();
  }
}