/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.InstDumpInfo;
import com.prosyst.jpd.profiler.info.PrimArrayDumpInfo;

/**
 * Denote a heap dump -- dump of a primitive array.
 *
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------------------
 * PROF_GC_PRIM_ARRAY_DUMP      dump of a primitive array
 *
 *                          id         array object ID
 *                          u4         stack trace serial number
 *                          u4         number of elements
 *                          u1         element type
 *                                     4:  boolean array
 *                                     5:  char array
 *                                     6:  float array
 *                                     7:  double array
 *                                     8:  byte array
 *                                     9:  short array
 *                                     10: int array
 *                                     11: long array
 *                          [u1]*      elements
 */
public class HeapDump_PrimArrayDump extends HeapDump {

  public PrimArrayDumpInfo objDumpInfo;
  
  public HeapDump_PrimArrayDump(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    objDumpInfo = new PrimArrayDumpInfo();
    
    objDumpInfo.objectID           = in.read_id();
    
    int         stackTraceSerialNo = in.read_u4();
    objDumpInfo.traceInfo          = namespace.lookupTrace(stackTraceSerialNo);
    
    objDumpInfo.length             = in.read_u4();
    
    objDumpInfo.type               = in.read_u1();
    
    int bytesRead = 0;
    if (in.isHprofAgent) {
      bytesRead = objDumpInfo.length * InstDumpInfo.sizeof(objDumpInfo.type);
      in.skip(bytesRead, false); // sending values of primitive array elements is removed
    }
    
    bind();
    
    // id + u4 + u4 + u1 + objDumpInfo.length * sizeof(elementType)
    return in.sizeofID() + 9 + bytesRead;
  }
  
  public void bind() {
    namespace.getObjdumpTable().bind(objDumpInfo.objectID, objDumpInfo);
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_PRIM_ARRAY_DUMP");
    out.println("  array object ID           = " + Integer.toHexString(objDumpInfo.objectID));
    out.println("  stack trace               = " + objDumpInfo.traceInfo);
    out.println("  " + objDumpInfo.toString());
    out.println();
  }
}