/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.JavaFrameRoot;
import com.prosyst.jpd.profiler.info.Misc;

/**
 * Denote a heap dump -- Java stack frame.
 *
 * <PRE>
 * TAG           BODY      notes
 * ---------------------------------------------
 * PROF_GC_ROOT_JAVA_FRAME      Java stack frame
 *
 *                          id         object ID
 *                          u4         thread serial number
 *                          u4         frame # in stack trace (-1 for empty)
 */
public class HeapDump_RootJavaFrame extends HeapDump {
  
  JavaFrameRoot javaFrame;
  
  public HeapDump_RootJavaFrame(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    javaFrame = new JavaFrameRoot();
    
    javaFrame.objectID       = in.read_id();
    javaFrame.threadSerialNo = in.read_u4();
    javaFrame.frameNumber    = in.read_u4();
    
    namespace.getJavaFrameRootTable().addElement(javaFrame);
    
    return in.sizeofID() + 8; // id + u4 + u4
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_JAVA_FRAME");
    out.println("  object ID              = " + javaFrame.objectID);
    out.println("  thread serial number   = " + javaFrame.threadSerialNo);
    out.println("  frame # in stack trace = " + Misc.toStringFrameNumber(javaFrame.frameNumber));
    out.println();
  }
}
