/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;

/**
 * Denote a heap dump -- JNI global reference root.
 *
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------------
 * PROF_GC_ROOT_JNI_GLOBAL      JNI global ref root
 *
 *                          id         object ID
 *                          id         JNI global ref ID
 */
public class HeapDump_RootJniGlobal extends HeapDump {
  
  public int objectID;
  public int jniGlobalRefID;
  
  public HeapDump_RootJniGlobal(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    objectID       = in.read_id();
    jniGlobalRefID = in.read_id();
    
    return in.sizeofID() * 2; // id + id
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_JNI_GLOBAL");
    out.println("  object ID         = " + objectID);
    out.println("  JNI global ref ID = " + jniGlobalRefID);
    out.println();
  }
}