/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.Misc;

/**
 * Denote a heap dump -- JNI local reference root.
 *
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------
 * PROF_GC_ROOT_JNI_LOCAL       JNI local ref
 *
 *                          id         object ID
 *                          u4         thread serial number
 *                          u4         frame # in stack trace (-1 for empty)
 */
public class HeapDump_RootJniLocal extends HeapDump {
  
  public int objectID;
  public int threadSerialNo;
  public int frameNumber;
    
  public HeapDump_RootJniLocal(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    objectID       = in.read_id();
    threadSerialNo = in.read_u4();
    frameNumber    = in.read_u4();
    
    return in.sizeofID() + 8; // id + u4 + u4
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_JNI_LOCAL");
    out.println("  object ID              = " + objectID);
    out.println("  thread serial number   = " + threadSerialNo);
    out.println("  frame # in stack trace = " + Misc.toStringFrameNumber(frameNumber));
    out.println();
  }
}
