/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;
import com.prosyst.jpd.profiler.info.NativeStackRoot;

/**
 * Denote a heap dump -- Native stack.
 *
 * <PRE>
 * TAG           BODY      notes
 * -----------------------------------------
 * PROF_GC_ROOT_NATIVE_STACK    Native stack
 *
 *                          id         object ID
 *                          u4         thread serial number
 */
public class HeapDump_RootNativeStack extends HeapDump {
  
  NativeStackRoot nativeStack;
  
  public HeapDump_RootNativeStack(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    nativeStack = new NativeStackRoot();
    
    nativeStack.objectID       = in.read_id();
    nativeStack.threadSerialNo = in.read_u4();
    
    namespace.getNativeStackRootTable().addElement(nativeStack);
    
    return in.sizeofID() + 4; // id + u4
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_NATIVE_STACK");
    out.println("  object ID            = " + nativeStack.objectID);
    out.println("  thread serial number = " + nativeStack.threadSerialNo);
    out.println();
  }
}