/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;

/**
 * Denote a heap dump -- Reference from thread block.
 *
 * <PRE>
 * TAG           BODY      notes
 * --------------------------------------------------------
 * PROF_GC_ROOT_THREAD_BLOCK    Reference from thread block
 *
 *                          id         object ID
 *                          u4         thread serial number
 */
public class HeapDump_RootThreadBlock extends HeapDump {
  
  public int objectID;
  public int threadSerialNo;
  
  public HeapDump_RootThreadBlock(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    objectID       = in.read_id();
    threadSerialNo = in.read_u4();
    
    return in.sizeofID() + 4; // id + u4
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_THREAD_BLOCK");
    out.println("  object ID            = " + objectID);
    out.println("  thread serial number = " + threadSerialNo);
    out.println();
  }
}