/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.heapdump;

import com.prosyst.jpd.profiler.binbody.BinBody_HeapDump;

/**
 * Denote a heap dump -- thread object.
 *
 * <PRE>
 * TAG           BODY      notes
 * ------------------------------------------
 * PROF_GC_ROOT_THREAD_OBJ      Thread object
 *
 *                          id         thread object ID  (may be 0 for a
 *                                     thread newly attached through JNI)
 *                          u4         thread sequence number
 *                          u4         stack trace sequence number
 */
public class HeapDump_RootThreadObj extends HeapDump {
  
  public int threadObjectID;
  public int threadSequenceNo;
  public int stackTraceSequenceNo;
    
  public HeapDump_RootThreadObj(BinBody_HeapDump parent) {
    super(parent);
  }
  
  public int read() throws java.io.IOException {
    threadObjectID       = in.read_id();
    threadSequenceNo     = in.read_u4();
    stackTraceSequenceNo = in.read_u4();
    
    return in.sizeofID() + 8; // id + u4 + u4
  }
  
  // Dumpable
  
  public void dump(java.io.PrintStream out) {
    out.println("  sub-record type = PROF_GC_ROOT_THREAD_OBJ");
    out.println("  thread object ID            = " + threadObjectID);
    out.println("  thread sequence number      = " + threadSequenceNo);
    out.println("  stack trace sequence number = " + stackTraceSequenceNo);
    out.println();
  }
}