/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import com.prosyst.jpd.profiler.storage.ObjectStorageInt;
import com.prosyst.jpd.profiler.storage.ObjectStorageIntDefaultImpl;

/**
 * Alloc site information group.
 *
 * @author Ivo Karabashev
 */
public class AllocSiteGroup {
  
  private static final String EMPTY_STRING = "";
  
  public int              bytesLive;
  public int              instsLive;
  public int              bytesAlloc;
  public int              instsAlloc;
  public int              bytesLiveMark;
  public int              instsLiveMark;
  public int              bytesAllocMark;
  public int              instsAllocMark;
  public String           className;
  public ObjectStorageInt storage;
  public boolean          isOpened;
  
  public AllocSiteGroup(AllocSiteInfo asi, boolean isMarking) {
    storage    = new ObjectStorageIntDefaultImpl(3);
    className  = asi.className;
    bytesLive  = asi.bytesLive;
    instsLive  = asi.instsLive;
    bytesAlloc = asi.bytesAlloc;
    instsAlloc = asi.instsAlloc;
    if (isMarking) {
      bytesLiveMark  = asi.bytesLive;
      instsLiveMark  = asi.instsLive;
      bytesAllocMark = asi.bytesAlloc;
      instsAllocMark = asi.instsAlloc;
    }
  }
  
  public void addFrom(AllocSiteInfo asi, boolean isMarking) {
    bytesLive  += asi.bytesLive;
    instsLive  += asi.instsLive;
    bytesAlloc += asi.bytesAlloc;
    instsAlloc += asi.instsAlloc;
    if (isMarking) {
      bytesLiveMark  += asi.bytesLive;
      instsLiveMark  += asi.instsLive;
      bytesAllocMark += asi.bytesAlloc;
      instsAllocMark += asi.instsAlloc;
    }
  }
  
  public void startMark() {
    bytesAllocMark = 0;
    instsAllocMark = 0;
    bytesLiveMark  = 0;
    instsLiveMark  = 0;
  }
  
  public String getClassName() {
    return className;
  }
  
  public String toTabbedString() {
    StringBuffer sb = new StringBuffer();
    sb.append(Misc.formatTabbed(bytesAlloc));
    sb.append(Misc.formatTabbed(instsAlloc));
    sb.append("  ");
    sb.append(getClassName());
    return sb.toString();
  }
  
  public String toString() {
    return EMPTY_STRING; // This method is for speeding up tree and table
  }
}