/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

/**
 * Resolved alloc site information.
 *
 * @author Ivo Karabashev
 */
public class AllocSiteInfo {
  
  public static final int BOTH_INFO  = 0;
  public static final int ALLOC_INFO = 1;
  public static final int DIFF_INFO  = 2;
  
  private static final String EMPTY_STRING = "";
  
  public int       stackTraceSerialNo;
  public int       bytesLive;
  public int       instsLive;
  public int       bytesAlloc;
  public int       instsAlloc;
  public int       bytesLiveMark;
  public int       instsLiveMark;
  public int       bytesAllocMark;
  public int       instsAllocMark;
  public String    className;
  public TraceInfo traceInfo;
  
  public void copyFrom(AllocSiteInfo asi) {
    bytesLive  = asi.bytesLive;
    instsLive  = asi.instsLive;
    bytesAlloc = asi.bytesAlloc;
    instsAlloc = asi.instsAlloc;
  }
  
  public void copyMark() {
    bytesLiveMark  = bytesLive;
    instsLiveMark  = instsLive;
    bytesAllocMark = bytesAlloc;
    instsAllocMark = instsAlloc;
  }
  
  public void startMark() {
    bytesAllocMark = 0;
    instsAllocMark = 0;
    bytesLiveMark  = 0;
    instsLiveMark  = 0;
  }
  
  public String getClassName() {
    return className;
  }
  
  public String getThreadName() {
    if (traceInfo == null) return TraceInfo.UNKNOWN_TRACE;
    return traceInfo.getThreadName();
  }
  
  public String getFrameName() {
    if (traceInfo == null) return TraceInfo.UNKNOWN_TRACE;
    return traceInfo.getFrameName();
  }
  
  public ThreadInfo getThread() {
    if (traceInfo == null) return null;
    return traceInfo.threadInfo;
  }
  
  public String toTabbedString(int mode) {
    StringBuffer sb = new StringBuffer();
    if (mode < DIFF_INFO) {
      sb.append(Misc.formatTabbed(bytesAlloc));
      sb.append(Misc.formatTabbed(instsAlloc));
      sb.append(Misc.formatTabbed(bytesLive));
      sb.append(Misc.formatTabbed(instsLive));
    }
    if (mode != ALLOC_INFO) {
      sb.append(Misc.formatTabbed(bytesAllocMark));
      sb.append(Misc.formatTabbed(instsAllocMark));
      sb.append(Misc.formatTabbed(bytesLiveMark));
      sb.append(Misc.formatTabbed(instsLiveMark));
    }
    sb.append("  ");
    sb.append(getClassName());
    sb.append("  ");
    sb.append((traceInfo == null ? TraceInfo.UNKNOWN_TRACE : traceInfo.toString()));
    return sb.toString();
  }
  
  public String toString() {
    return EMPTY_STRING; // This method is for speeding up tree and table
  }
}