/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

/**
 * Resolved cpu sample information.
 *
 * @author Ivo Karabashev
 */
public class CpuSampleInfo {
  
  private static final String EMPTY_STRING = "";
  
  public int       stackTraceCost;
  public int       stackTraceSerialNo;
  public TraceInfo traceInfo;
  
  public String    threadName;
  public String    frameName;
  
  public void resolve() {
    threadName = getThreadName();
    frameName  = getFrameName();
  }
  
  private String getThreadName() {
    if (traceInfo == null) return TraceInfo.UNKNOWN_TRACE;
    return traceInfo.getThreadName();
  }
  
  private String getFrameName() {
    if (traceInfo == null) return TraceInfo.UNKNOWN_TRACE;
    return traceInfo.getFrameName();
  }
  
  public ThreadInfo getThread() {
    if (traceInfo == null) return null;
    return traceInfo.threadInfo;
  }
  
  public String toAsciiString() {
    return (traceInfo == null ? TraceInfo.UNKNOWN_TRACE : traceInfo.toString() + ' ' + stackTraceCost);
  }
  
  public String toTabbedString(int totalNumberOfSamples, int endTicks) {
    StringBuffer sb = new StringBuffer();
    String prc = Misc.formatPercent(100F * stackTraceCost / totalNumberOfSamples, 3);
    sb.append(Misc.COL_TAB.substring(0, Misc.COL_SIZE - prc.length()));
    sb.append(prc);
    sb.append(Misc.formatTabbed(endTicks * stackTraceCost / totalNumberOfSamples));
    sb.append("  ");
    sb.append((traceInfo == null ? TraceInfo.UNKNOWN_TRACE : traceInfo.toString()));
    return sb.toString();
  }
  
  public String toString() {
    return EMPTY_STRING; // This method is for speeding up tree and table
  }
}