/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import com.prosyst.jpd.profiler.gui.ResourceManager;

/**
 * Resolved frame information.
 *
 * @author Ivo Karabashev
 */
public class FrameInfo {
  
  public static final String UNKNOWN_FRAME  = ResourceManager.getString("InfoClass.unknown.frame");
  public static final String UNKNOWN_METHOD = ResourceManager.getString("InfoClass.unknown.method");
  
  public int       stackFrameID;
  public String    methodName;
  //public String    methodSignature; // not needed for now
  public String    sourceFileName;
  public String    className;
  public int       lineNumber;
  private String   frameName;
  
  public String getSourceFileName() {
    if (className == null) return null;
    String name = className;
    int index = name.lastIndexOf('.');
    if (index != -1) {
      name = name.substring(0, index + 1).replace('.', '/') + sourceFileName;
    } else {
      name = sourceFileName;
    }
    return name;
  }
  
  public boolean compareMethodFrame(FrameInfo fi) {
    if (className == fi.className && methodName == fi.methodName) return true;
    return false;
  }
  
  public String toString() {
    if (frameName == null) {
      StringBuffer sb = new StringBuffer();
      sb.append((className  == null ? Misc.UNKNOWN_CLASS : className));
      sb.append('.');
      sb.append((methodName == null ? UNKNOWN_METHOD     : methodName));
      sb.append(':');
      sb.append(toStringLineNumber(lineNumber));
      frameName = sb.toString();
    }
    return frameName;
  }
  
  public String toStringWithSource() {
    if (frameName == null) {
      StringBuffer sb = new StringBuffer();
      sb.append((className  == null ? Misc.UNKNOWN_CLASS : className));
      sb.append('.');
      sb.append((methodName == null ? UNKNOWN_METHOD     : methodName));
      sb.append('(');
      sb.append(sourceFileName);
      sb.append(':');
      sb.append(toStringLineNumber(lineNumber));
      sb.append(')');
      frameName = sb.toString();
    }
    return frameName;
  }
  
  public static String toStringLineNumber(int lineNumber) {
    switch (lineNumber) {
      case -1:
        return "unknown";
      case -2:
        return "compiled method";
      case -3:
        return "native method";
      default:
        return Integer.toString(lineNumber, 10);
    }
  }
}