/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import com.prosyst.jpd.profiler.BinInput;
import com.prosyst.util.elements.ComparableElement;

/**
 * Resolved instance dump interface.
 *
 * @author Ivo Karabashev
 */
public abstract class InstDumpInfo implements ComparableElement {
  
  /** Object type constants. */
  public static final byte TYPE_OBJECT  = 2;
  public static final byte TYPE_BOOLEAN = 4;
  public static final byte TYPE_CHAR    = 5;
  public static final byte TYPE_FLOAT   = 6;
  public static final byte TYPE_DOUBLE  = 7;
  public static final byte TYPE_BYTE    = 8;
  public static final byte TYPE_SHORT   = 9;
  public static final byte TYPE_INT     = 10;
  public static final byte TYPE_LONG    = 11;
  public static final byte TYPE_ARRAY   = 20;
  
  public int            objectID;
  public TraceInfo      traceInfo;
  
  public abstract byte getType();
  
  public abstract String toString();
  
  public static int readInstance(byte type, BinInput in) throws java.io.IOException {
    if (type == TYPE_OBJECT) return in.read_id();
    else {
      in.skip(sizeof(type), false);
      return 0;
    }
  }
  
  public static int sizeof(byte type) {
    switch (type) {
      case TYPE_OBJECT:
        return 4; // sizeofID
      case TYPE_BOOLEAN:
        return 1;
      case TYPE_CHAR:
        return 2;
      case TYPE_FLOAT:
        return 4;
      case TYPE_DOUBLE:
        return 8;
      case TYPE_BYTE:
        return 1;
      case TYPE_SHORT:
        return 2;
      case TYPE_INT:
        return 4;
      case TYPE_LONG:
        return 8;
    }
    throw new IllegalArgumentException("Unsupported value type = " + type);
  }
  
  public static String toStringType(byte type) {
    switch (type) {
      case TYPE_OBJECT:
        return "object";
      case TYPE_BOOLEAN:
        return "boolean";
      case TYPE_CHAR:
        return "char";
      case TYPE_FLOAT:
        return "float";
      case TYPE_DOUBLE:
        return "double";
      case TYPE_BYTE:
        return "byte";
      case TYPE_SHORT:
        return "short";
      case TYPE_INT:
        return "int";
      case TYPE_LONG:
        return "long";
    }
    throw new IllegalArgumentException("Unsupported value type = " + type);
  }
  
  public int compareTo(ComparableElement obj) {
    InstDumpInfo info = (InstDumpInfo) obj;
    int i = (objectID >>> 1) - (info.objectID >>> 1);
    if (i != 0) return i;
    return objectID - info.objectID;
  }
}