/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import java.util.Vector;

import com.prosyst.jpd.profiler.binbody.BinRecord;
import com.prosyst.jpd.profiler.gui.ResourceManager;
import com.prosyst.util.elements.ComparableElement;

public class Misc {
  
  /* Thread status. */
  public static final int JVMPI_THREAD_RUNNABLE     = 1;
  public static final int JVMPI_THREAD_MONITOR_WAIT = 2;
  public static final int JVMPI_THREAD_CONDVAR_WAIT = 3;
  
  public static final int JVMPI_THREAD_SUSPENDED    = 0x8000;
  public static final int JVMPI_THREAD_INTERRUPTED  = 0x4000;
  
  /* Object type constants. */
  public static final int JVMPI_NORMAL_OBJECT       = 0;
  public static final int JVMPI_CLASS               = 2;
  public static final int JVMPI_BOOLEAN             = 4;
  public static final int JVMPI_CHAR                = 5;
  public static final int JVMPI_FLOAT               = 6;
  public static final int JVMPI_DOUBLE              = 7;
  public static final int JVMPI_BYTE                = 8;
  public static final int JVMPI_SHORT               = 9;
  public static final int JVMPI_INT                 = 10;
  public static final int JVMPI_LONG                = 11;
  
  public static final String UNKNOWN_CLASS       = ResourceManager.getString("InfoClass.unknown.class");
  public static final String SYSTEM_CLASS_LOADER = ResourceManager.getString("InfoClass.system_class_loader");
  public static final String EMPTY_STRING        = "";
  
  public static String toStringAllocSiteType(byte type) {
    switch (type) {
      case  0: return "normal object";
      case  2: return "object []";
      case  4: return "boolean []";
      case  5: return "char []";
      case  6: return "float []";
      case  7: return "double []";
      case  8: return "byte []";
      case  9: return "short []";
      case 10: return "int []";
      case 11: return "long []";
    }
    return UNKNOWN_CLASS;
  }
  
  public static String toStringObjectType(byte type, String className) {
    switch (type) {
      case  0: return (className == null ? UNKNOWN_CLASS : className);
      case  2: return (className == null ? UNKNOWN_CLASS : className) + "[]";
      case  4: return "boolean[]";
      case  5: return "char[]";
      case  6: return "float[]";
      case  7: return "double[]";
      case  8: return "byte[]";
      case  9: return "short[]";
      case 10: return "int[]";
      case 11: return "long[]";
    }
    return UNKNOWN_CLASS;
  }
  
  public static String toStringMonitorEvent(int type) {
    switch (type) {
      case BinRecord.PROF_MONITOR_ENTER:   return "ENTER";
      case BinRecord.PROF_MONITOR_ENTERED: return "ENTERED";
      case BinRecord.PROF_MONITOR_EXIT:    return "EXIT";
      case BinRecord.PROF_MONITOR_WAIT:    return "WAIT";
      case BinRecord.PROF_MONITOR_WAITED:  return "WAITED";
    }
    return "unknown monitor event";
  }
  
  public static String toStringFrameNumber(int frameNumber) {
    return (frameNumber == -1 ? "empty" : Integer.toString(frameNumber, 10));
  }
  
  public static String toStringThreadStatus(int threadStatus) {
    StringBuffer sb = new StringBuffer();
    if ((threadStatus & JVMPI_THREAD_SUSPENDED) != 0) {
      sb.append("SUSPENDED|");
      threadStatus &= ~JVMPI_THREAD_SUSPENDED;
    }
    if ((threadStatus & JVMPI_THREAD_INTERRUPTED) != 0) {
      sb.append("INTERRUPTED|");
      threadStatus &= ~JVMPI_THREAD_INTERRUPTED;
    }
    switch (threadStatus) {
      case JVMPI_THREAD_RUNNABLE:
        sb.append("RUNNING");
        break;
      case JVMPI_THREAD_MONITOR_WAIT:
        sb.append("MONITOR WAIT");
        break;
      case JVMPI_THREAD_CONDVAR_WAIT:
        sb.append("CONDVAR WAIT");
        break;
    }
    return sb.toString();
  }
  
  // compares two strings ignoring case (JDK 1.1 compatibility)
  public static int compareToIgnoreCase(String s1, String s2) {
    if (s1 == s2) return 0;
    int n = Math.min(s1.length(), s2.length());
    for (int i = 0; i < n; i++) {
      char c1 = s1.charAt(i);
      char c2 = s2.charAt(i);
      if (c1 != c2) {
        c1 = Character.toUpperCase(c1);
        c2 = Character.toUpperCase(c2);
        if (c1 != c2) {
          c1 = Character.toLowerCase(c1);
          c2 = Character.toLowerCase(c2);
          if (c1 != c2) return c1 - c2;
        }
      }
    }
    return s1.length() - s2.length();
  }
  
  // formats bytes and insts so that they are right aligned in columns
  public static final String COL_TAB  = "        ";
  public static final int    COL_SIZE = COL_TAB.length();
  
  public static String formatTabbed(int value) {
    String temp = COL_TAB + value;
    return temp.substring(temp.length() - COL_SIZE);
  }
  
  // formats memory units in bytes, kilobytes and megabytes
  //public static int memUnits = 1;
  
  public static String formatMemUnit(int value, int memUnits) {
    if (memUnits == 1)       return Integer.toString(value,           10);
    if (memUnits == 1000)    return Integer.toString(value / 1000,    10) + 'K';
    if (memUnits == 1000000) return Integer.toString(value / 1000000, 10) + 'M';
    return Integer.toString(value, 10);
  }
  
  // formats cpu time percentage with given number of digits after the point
  private static final String ZEROS = "000";  // max precision is 0.0000
  //public static int timePercentPrecision = 3; // default precision is 0.00, max is 5
  
  public static String formatPercent(float value, int timePercentPrecision) {
    String temp = Float.toString(value) + ZEROS;
    return temp.substring(0, temp.indexOf('.') + timePercentPrecision);
  }
  
  // shows or not the packages in classes
  public static boolean showPackages = true;
  
  public static String formatClass(String classname) {
    if (showPackages) return classname;
    int i = classname.lastIndexOf('.');
    if (i == -1) return classname;
    return classname.substring(i + 1);
  }
  
  public static void sort(Vector v, int lo0, int hi0) {
    if (lo0 >= hi0) return;
    int lo = lo0;
    int hi = hi0;
    ComparableElement mid = (ComparableElement) v.elementAt((lo + hi) / 2);
    while (lo <= hi) {
      while ((lo < hi0) && (((ComparableElement) v.elementAt(lo)).compareTo(mid) < 0)) lo++;
      while ((lo0 < hi) && (((ComparableElement) v.elementAt(hi)).compareTo(mid) > 0)) hi--;
      if (lo <= hi) {
        Object temp = v.elementAt(lo);
        v.setElementAt(v.elementAt(hi), lo);
        v.setElementAt(temp, hi);
        lo++;
        hi--;
      }
    }
    if (lo0 < hi) sort(v, lo0, hi);
    if (lo < hi0) sort(v, lo, hi0);
  }
}