/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import com.prosyst.jpd.profiler.binbody.BinRecord;

/**
 * Resolved monitor dump information.
 *
 * @author Ivo Karabashev
 */
public class MonitorInfo {
  
  public int          ticks;
  public byte         monitorType; // java or raw
  public byte         monitorObjectType; // object, array or object array
  public String       monitorName;
  public int          monitorID;
  public TraceInfo    traceInfo;
  public ThreadInfo   t_ownerInfo;
  public int          entryCount;
  public long         timeout;
  public ThreadInfo[] t_enterInfo;
  public ThreadInfo[] t_notifyInfo;
  
  public boolean isSleep() {
    if (monitorObjectType == Misc.JVMPI_NORMAL_OBJECT && monitorName == null && monitorID == 0
      && (monitorType == BinRecord.PROF_MONITOR_WAIT || monitorType == BinRecord.PROF_MONITOR_WAITED)) return true;
    return false;
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(Misc.toStringObjectType(monitorObjectType, monitorName));
    sb.append('@');
    sb.append(Integer.toHexString(monitorID));
    if (monitorType == BinRecord.JVMPI_MONITOR_RAW) sb.append(" (raw monitor)");
    if (entryCount > 0) {
      sb.append("\n  owner: ");
      sb.append((t_ownerInfo == null ? ThreadInfo.UNKNOWN_THREAD : t_ownerInfo.toString()));
      sb.append(", ");
      sb.append(entryCount);
      sb.append(" entries");
    } else {
      sb.append(" unowned");
    }
    if (traceInfo != null) {
      sb.append("\n  allocated:");
      for (int i = 0; i < traceInfo.numberOfFrames; i++) {
        sb.append("\n    at ");
        sb.append(traceInfo.frameInfo[i].toStringWithSource());
      }
    }
    if (t_enterInfo.length > 0) sb.append("\n  waiting to enter:");
    for (int i = 0; i < t_enterInfo.length; i++) {
      sb.append("\n    ");
      sb.append((t_enterInfo[i] == null ? ThreadInfo.UNKNOWN_THREAD : t_enterInfo[i].toString()));
    }
    if (t_notifyInfo.length > 0) sb.append("\n  waiting to be notified:");
    for (int i = 0; i < t_notifyInfo.length; i++) {
      sb.append("\n    ");
      sb.append((t_notifyInfo[i] == null ? ThreadInfo.UNKNOWN_THREAD : t_notifyInfo[i].toString()));
    }
    return sb.toString();
  }
  
  public String toStringMonitorName() {
    StringBuffer sb = new StringBuffer();
    sb.append(Misc.toStringObjectType(monitorObjectType, monitorName));
    sb.append('@');
    sb.append(Integer.toHexString(monitorID));
    return sb.toString();
  }
  
  public String toStringMonitorEventShort() {
    StringBuffer sb = new StringBuffer();
    sb.append(ticks);
    sb.append(' ');
    if (isSleep()) {
      sb.append((monitorType == BinRecord.PROF_MONITOR_WAIT ? "SLEEP" : "SLEEPED"));
    } else {
      sb.append(Misc.toStringMonitorEvent(monitorType));
    }
    sb.append(' ');
    sb.append((t_ownerInfo == null ? ThreadInfo.UNKNOWN_THREAD : t_ownerInfo.toString()));
    return sb.toString();
  }
  
  public String toStringMonitorEventDescr() {
    StringBuffer sb = new StringBuffer();
    switch (monitorType) {
      case BinRecord.PROF_MONITOR_ENTER:   sb.append("began blocked on "); break;
      case BinRecord.PROF_MONITOR_ENTERED: sb.append("finished blocked on "); break;
      case BinRecord.PROF_MONITOR_WAIT:    sb.append(isSleep() ? "began sleeping" : "began waiting on "); break;
      case BinRecord.PROF_MONITOR_WAITED:  sb.append(isSleep() ? "finished sleeping" : "finished waiting on "); break;
    }
    if (!isSleep()) {
      sb.append(Misc.toStringObjectType(monitorObjectType, monitorName));
      sb.append('@');
      sb.append(Integer.toHexString(monitorID));
    }
    return sb.toString();
  }
  
  public String toStringMonitorEvent() {
    StringBuffer sb = new StringBuffer();
    if (isSleep()) {
      sb.append((monitorType == BinRecord.PROF_MONITOR_WAIT ? "SLEEP" : "SLEEPED"));
    } else {
      sb.append(Misc.toStringMonitorEvent(monitorType));
      sb.append(' ');
      sb.append(Misc.toStringObjectType(monitorObjectType, monitorName));
      sb.append('@');
      sb.append(Integer.toHexString(monitorID));
      if (traceInfo != null) {
        sb.append("\n  allocated at:");
        for (int i = 0; i < traceInfo.numberOfFrames; i++) {
          sb.append("\n    " + traceInfo.frameInfo[i]);
        }
      }
    }
    sb.append("\n  owner: ");
    sb.append((t_ownerInfo == null ? ThreadInfo.UNKNOWN_THREAD : t_ownerInfo.toString()));
    sb.append(", timeout:");
    sb.append(timeout);
    return sb.toString();
  }
}