/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

import com.prosyst.util.log.Log;      // from Util project

/**
 * Resolved object dump information.
 *
 * @author Ivo Karabashev
 */
public class ObjectDumpInfo extends InstDumpInfo {
  
  public int            classObjectID;
  public ClassDumpInfo  classDumpInfo;
  public byte[]         buffer;
  public int[]          values;
  
  public byte getType() {
    return TYPE_OBJECT;
  }
  
  public String toString() {
    return (classDumpInfo == null ? Misc.UNKNOWN_CLASS : classDumpInfo.getClassName())
           + '@' + Integer.toHexString(objectID);
  }
  
  public void readValues() {
    ClassDumpInfo classdump;
    
    int numberOfFields = 0;
    classdump = classDumpInfo;
    while (classdump != null) {
      numberOfFields += classdump.instanceFields.length;
      classdump = classdump.superClassInfo;
    }
    
    int bufIndex = 0;
    int valIndex = 0;
    values = new int[numberOfFields];
    classdump = classDumpInfo;
    while (classdump != null) {
      ClassDumpInfo.InstanceField[] fields = classdump.instanceFields;
      for (int i = 0; i < fields.length; i++) {
        if (fields[i].type == TYPE_OBJECT) values[valIndex] = readIdFromBuf(bufIndex);
        bufIndex += sizeof(fields[i].type);
        valIndex++;
      }
      classdump = classdump.superClassInfo;
    }
    
    if (bufIndex != buffer.length) Log.printDebug("BIN_INPUT.skip", (buffer.length - bufIndex) + " bytes skipped");
    buffer = null;
  }
  
  private int readIdFromBuf(int pos) {
    int ch1 = buffer[pos]     & 0xff;
    int ch2 = buffer[pos + 1] & 0xff;
    int ch3 = buffer[pos + 2] & 0xff;
    int ch4 = buffer[pos + 3] & 0xff;
    return (ch1 << 24) + (ch2 << 16) + (ch3 << 8) + ch4;
  }
}