/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.info;

/**
 * Method entry or exit event.
 *
 * @author Ivo Karabashev
 */
public class TriggerEvent {
  
  public static final byte PROF_TRIGGER_ENTRY = 0x00;
  public static final byte PROF_TRIGGER_EXIT  = 0x01;
  
  public static final int  PROF_TRGACT_BEGIN     = 0x00;
  public static final int  PROF_TRGACT_CONT      = 0x01;
  public static final int  PROF_TRGACT_END       = 0x02;
  public static final int  PROF_TRGACT_MEM_BEGIN = 0x04;
  public static final int  PROF_TRGACT_MEM_CONT  = 0x05;
  public static final int  PROF_TRGACT_MEM_END   = 0x06;
  public static final int  PROF_TRGACT_CPU_BEGIN = 0x08;
  public static final int  PROF_TRGACT_CPU_CONT  = 0x09;
  public static final int  PROF_TRGACT_CPU_END   = 0x0A;
  
  public static final int  PROF_TRGACT_EXPORT    = 0x10;
  public static final int  PROF_TRGACT_PAUSE     = 0x11;
  public static final int  PROF_TRGACT_STOP      = 0x12;
  
  public boolean enabled;
  public byte    type;
  public String  className;
  public String  methodName;
  public int     action;
  
  public boolean equals(TriggerEvent te) {
    if (type != te.type) return false;
    if (!className.equals(te.className)) return false;
    if (methodName == null) if (te.methodName != null) return false;
    else if (!methodName.equals(te.methodName)) return false;
    return true;
  }
  
  public boolean equalsFull(TriggerEvent te) {
    if (enabled != te.enabled) return false;
    if (type != te.type) return false;
    if (action != te.action) return false;
    if (!className.equals(te.className)) return false;
    if (methodName == null) if (te.methodName != null) return false;
    else if (!methodName.equals(te.methodName)) return false;
    return true;
  }
  
  public TriggerEvent copy() {
    TriggerEvent trg_copy = new TriggerEvent();
    trg_copy.enabled    = enabled;
    trg_copy.type       = type;
    trg_copy.className  = className;
    trg_copy.methodName = methodName;
    trg_copy.action     = action;
    return trg_copy;
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(type == PROF_TRIGGER_ENTRY ? "ENTRY " : "EXIT ");
    sb.append(className);
    sb.append(" ");
    sb.append((methodName == null ? "*" : methodName));
    return sb.toString();
  }
  
  public boolean fromStringRepresentation(String triggerAsString) {
    int indx = triggerAsString.indexOf(',');
    if (indx == -1) return false;
    enabled    = (triggerAsString.charAt(0) == '1');
    type       = (triggerAsString.charAt(1) == 'N' ? PROF_TRIGGER_ENTRY : PROF_TRIGGER_EXIT);
    action     = (triggerAsString.charAt(2) - 'A');
    className  = triggerAsString.substring(3, indx);
    methodName = triggerAsString.substring(indx + 1);
    if ("*".equals(methodName)) methodName = null;
    return true;
  }
  
  public String toStringRepresentation() {
    StringBuffer sb = new StringBuffer();
    sb.append(enabled ? '1' : '0');
    sb.append(type == PROF_TRIGGER_ENTRY ? 'N' : 'X');
    sb.append((char) ('A' + action));
    sb.append(className);
    sb.append(',');
    sb.append((methodName == null ? "*" : methodName));
    return sb.toString();
  }
}
