/*
 * Copyright (c) 2000 by Prosyst Software AG.,
 * http://www.prosyst.com
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Prosyst Software AG. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Prosyst.
 */

package com.prosyst.jpd.profiler.storage;

import java.util.Vector;

import com.prosyst.jpd.profiler.info.TraceInfo;
import com.prosyst.jpd.profiler.info.ThreadInfo;
import com.prosyst.jpd.profiler.info.FrameInfo;
import com.prosyst.jpd.profiler.info.ClassDumpInfo;
import com.prosyst.jpd.profiler.info.InstDumpInfo;

import com.prosyst.util.log.Dumpable; // from Util project

/**
 * @author Vladislav Doshev (VladiSD)
 */
public class ProfilerNamespaceDefaultImpl extends ProfilerNamespace
  implements Dumpable {
  
  public static final int NAMETABLE_ISIZE             =  20009;
  public static final int THREADTABLE_ISIZE           =    101;
  public static final int TRACETABLE_ISIZE            =  80009;
  public static final int FRAMETABLE_ISIZE            =  40009;
  public static final int CLASSTABLE_ISIZE            =   5003;
  public static final int CLSDUMPTABLE_ISIZE          =   1001;
  public static final int OBJDUMPTABLE_ISIZE          =  10001;
  
  protected ObjectStorageInt nameTable
    = createObjectStorage(NAMETABLE_ISIZE);
  
  protected ObjectStorageInt threadTable
    = createObjectStorage(THREADTABLE_ISIZE);
  
  protected ObjectStorageInt traceTable
    = createObjectStorage(TRACETABLE_ISIZE);
  
  protected ObjectStorageInt frameTable
    = createObjectStorage(FRAMETABLE_ISIZE);
  
  protected ObjectStorageInt classTable
    = createObjectStorage(CLASSTABLE_ISIZE);
  
  protected ObjectStorageInt classFromObjectIDTable
    = createObjectStorage(CLASSTABLE_ISIZE);
  
  protected ObjectStorageInt clsdumpTable
    = createObjectStorage(CLSDUMPTABLE_ISIZE);
  
  protected ObjectStorageInt objdumpTable
    = createObjectStorage(OBJDUMPTABLE_ISIZE);
  
  protected Vector javaFrameTable = new Vector();
  
  protected Vector nativeStackTable = new Vector();
  
  protected static ObjectStorageInt createObjectStorage(int size) {
    return new ObjectStorageIntDefaultImpl(size);
  }
  
  // ProfilerNamespace implementation
  
  public ObjectStorageInt getNameTable() {
    return nameTable;
  }
  
  public ObjectStorageInt getThreadTable() {
    return threadTable;
  }
  
  public ObjectStorageInt getTraceTable() {
    return traceTable;
  }
  
  public ObjectStorageInt getFrameTable() {
    return frameTable;
  }
  
  public ObjectStorageInt getClassTable() {
    return classTable;
  }
  
  public ObjectStorageInt getClassFromObjectIDTable() {
    return classFromObjectIDTable;
  }
  
  public ObjectStorageInt getClsdumpTable() {
    return clsdumpTable;
  }
  
  public ObjectStorageInt getObjdumpTable() {
    return objdumpTable;
  }
  
  // Lookup
  
  public String lookupName(int nameID) {
    return (String) nameTable.lookup(nameID);
  }
  
  public ThreadInfo lookupThread(int threadID) {
    return (ThreadInfo) threadTable.lookup(threadID);
  }
  
  public TraceInfo lookupTrace(int traceID) {
    return (TraceInfo) traceTable.lookup(traceID);
  }
  
  public FrameInfo lookupFrame(int frameID) {
    return (FrameInfo) frameTable.lookup(frameID);
  }
  
  public String lookupClass(int classID) {
    return (String) classTable.lookup(classID);
  }
  
  public String lookupClassFromObjectID(int classObjectID) {
    return (String) classFromObjectIDTable.lookup(classObjectID);
  }
  
  public ClassDumpInfo lookupClassDump(int classID) {
    return (ClassDumpInfo) clsdumpTable.lookup(classID);
  }
  
  public InstDumpInfo lookupObjectDump(int objectID) {
    return (InstDumpInfo) objdumpTable.lookup(objectID);
  }
  
  // Dumpable
  
  public void dump() {
    dump(System.out);
  }
  
  public void dump(java.io.PrintStream out) {
    dump(out, nameTable,             "Name table");
    dump(out, threadTable,           "Thread table");
    dump(out, traceTable,            "Trace table");
    dump(out, frameTable,            "Frame table");
    dump(out, classTable,            "Class table");
    dump(out, clsdumpTable,          "Class dump table");
    dump(out, objdumpTable,          "Object dump table");
  }
  
  public void dump(java.io.PrintStream out, ObjectStorageInt storage, String tableName) {
    if (storage instanceof Dumpable) {
      out.println(tableName);
      out.println();
      ((Dumpable) storage).dump(out);
    } else {
      out.println(tableName + " is not dumpable");
    }
    out.println();
  }
  
  public void loadData(java.io.DataInputStream input) throws java.io.IOException {
    int count;
    // Threads
    count = input.readInt();
    ThreadInfo threadInfo;
    for (int i = 0; i < count; i++) {
      threadInfo = new ThreadInfo();
      threadInfo.threadSerialNo = input.readInt();
      threadInfo.threadName     = input.readUTF();
      threadTable.bind(threadInfo.threadSerialNo, threadInfo);
    }
    // Frames
    count = input.readInt();
    FrameInfo frameInfo;
    for (int i = 0; i < count; i++) {
      frameInfo = new FrameInfo();
      frameInfo.stackFrameID   = input.readInt();
      frameInfo.methodName     = input.readUTF();
      frameInfo.sourceFileName = input.readUTF();
      frameInfo.className      = input.readUTF();
      frameInfo.lineNumber     = input.readInt();
      frameTable.bind(frameInfo.stackFrameID, frameInfo);
    }
    // Traces
    count = input.readInt();
    int traceSerialNo;
    TraceInfo traceInfo;
    for (int i = 0; i < count; i++) {
      traceInfo = new TraceInfo();
      traceSerialNo = input.readInt();
      traceInfo.threadInfo = (ThreadInfo) threadTable.lookup(input.readInt());
      traceInfo.numberOfFrames = input.readInt();
      traceInfo.frameInfo = new FrameInfo[traceInfo.numberOfFrames];
      for (int j = 0; j < traceInfo.numberOfFrames; j++) {
        traceInfo.frameInfo[j] = (FrameInfo) frameTable.lookup(input.readInt());
      }
      traceTable.bind(traceSerialNo, traceInfo);
    }
  }
  
  public void saveData(java.io.DataOutputStream output) throws java.io.IOException {
    int[]    keys;
    Object[] values;
    // Threads
    output.writeInt(threadTable.size());
    values = threadTable.elements();
    ThreadInfo threadInfo;
    for (int i = 0; i < values.length; i++) {
      if (values[i] != null) {
        threadInfo = (ThreadInfo) values[i];
        output.writeInt(threadInfo.threadSerialNo);
        output.writeUTF(threadInfo.threadName);
      }
    }
    // Frames
    output.writeInt(frameTable.size());
    values = frameTable.elements();
    FrameInfo frameInfo;
    for (int i = 0; i < values.length; i++) {
      if (values[i] != null) {
        frameInfo = (FrameInfo) values[i];
        output.writeInt(frameInfo.stackFrameID);
        output.writeUTF(frameInfo.methodName);
        output.writeUTF(frameInfo.sourceFileName);
        output.writeUTF(frameInfo.className);
        output.writeInt(frameInfo.lineNumber);
      }
    }
    // Traces
    output.writeInt(traceTable.size());
    keys   = traceTable.keys();
    values = traceTable.elements();
    TraceInfo traceInfo;
    for (int i = 0; i < keys.length; i++) {
      if (values[i] != null) {
        traceInfo = (TraceInfo) values[i];
        output.writeInt(keys[i]);
        output.writeInt(traceInfo.threadInfo.threadSerialNo);
        output.writeInt(traceInfo.numberOfFrames);
        for (int j = 0; j < traceInfo.numberOfFrames; j++) {
          output.writeInt(traceInfo.frameInfo[j].stackFrameID);
        }
      }
    }
  }
  
  public Vector getJavaFrameRootTable() {
    return javaFrameTable;
  }
  
  public Vector getNativeStackRootTable() {
    return nativeStackTable;
  }
}